       /* Program CH15EX07.PRO */

DATABASE - indexes
 determ lockindex(bt_selector)
 determ index(bt_selector)
 determ mark(real)

DOMAINS
 my_dom = f(string)
 db_selector = dba


PREDICATES
 nondeterm repeat
 wr_err(integer)  
 
 % List texts and their status
 list
 list_texts(bt_selector,bt_selector)
 show_textname(string,bt_selector)

CLAUSES
 show_textname(Key,LockIndex):-
  key_search(dba,LockIndex,Key,_),!,
  write("\n*",Key).
  show_textname(Key,_):-
  write("\n ",Key).
  
 list_texts(Index, LockIndex) :-
  key_current(dba, Index, Key, _),
  show_textname(Key,LockIndex),
  key_next(dba, Index, _), !,
  list_texts(Index, LockIndex).
 list_texts(_,_).
  
 list:-
  shiftwindow(2),
  clearwindow,
  index(Index),
  lockindex(LockIndex),		
  key_first(dba,Index,_),!,
  list_texts(Index, LockIndex).
 list.

 repeat.
 repeat:-repeat.

 wr_err(E):-
  errormsg("PROLOG.ERR",E,Errormsg,_),
  write(Errormsg),readchar(_).

PREDICATES
 %Logical locking of files
 lock(string,bt_selector,bt_selector)

CLAUSES
 lock(Name,Index,LockIndex) :-
  not(key_search(dba, LockIndex, Name, _)), !,
  key_search(dba,Index,Name,Ref),
  key_insert(dba, LockIndex, Name, Ref).
 lock(Name,_,_):-
  db_endtransaction(dba),
  write(Name," is being updated by another user.\n Accessdenied"),
  fail.

PREDICATES
 ed(db_selector, bt_selector, bt_selector, string)
 ed1(db_selector, bt_selector, bt_selector, string)

CLAUSES
 % The ed predicates ensure that the edition will never fail. 
 ed(dba, Index,LockIndex, Name) :-
  ed1(dba, Index,LockIndex, Name), !.
 ed(_, _,_, _).

  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * There are two choices:                                     *
  *                                                            *
  * 1) The name already exists - modify the contents of the    *
  *  file                                                      *
  * 2) The name is a new name - create a new file              *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 %Old file
 ed1(dba, Index,LockIndex, Name) :-
  db_begintransaction(dba,readwrite),
  key_search(dba, Index, Name, Ref), !,
  ref_term(dba, string, Ref, Str),
  lock(Name,Index,LockIndex),
  list,
  db_endtransaction(dba),
  shiftwindow(4),
  edit(Str, Str1, "Edit", NAME, "", 0, "PROLOG.HLP",1,1,1,1,_,_),
  clearwindow,
  db_begintransaction(dba,readwrite),
  term_replace(dba, string, Ref, Str1),
  key_delete(dba, LockIndex, Name, Ref), %unlock
  list,
  db_endtransaction(dba).
 %New file
 ed1(dba, Index,LockIndex, Name):-
  chain_insertz(dba, file_chain, string, "", Ref),
  key_insert(dba, Index, Name, Ref),
  list,
  db_endtransaction(dba),
  ed1(dba,Index,LockIndex, Name).

PREDICATES
 main(db_selector, bt_selector, bt_selector)
 interpret(char, bt_selector, bt_selector)
 check_update_view
 update_view
 get_command(char)

CLAUSES
 % Loop until 'Q' is pressed
 main(dba, Index, LockIndex) :-
  check_update_view,
  get_command(Command),
  trap(interpret(Command,Index, LockIndex),E,wr_err(E)),!,
  main(dba,Index,LockIndex).
 main(_, _,_).

 check_update_view:-
  mark(T),timeout(T),!,
  db_begintransaction(dba,read),
  update_view,
  db_endtransaction(dba),
  marktime(100,Mark),
  retractall(mark(_)),
  assert(mark(Mark)).
  check_update_view.

 update_view:-
  shiftwindow(3),
  clearwindow,
  write("COMMAND>"),
  db_updated(dba),!,
  list.
 update_view.
  
 get_command(Command):-
  inkey(C),!,
  upper_lower(Command,C),
  write(Command).
  get_command(' ').
   
 %interpret commandlineinput
 interpret(' ',_,_):-!.
 interpret('Q',_,_):-!,fail.
 interpret('E',Index,LockIndex):-!,
  write("\nFile Name: "),
  readln(Name),
  ed(dba,Index,LockIndex,Name).
 interpret('V',Index,_):-
  write("\nFile Name: "),
  readln(Name),
  db_begintransaction(dba,read),
  key_search(dba,Index,Name,Ref),!,
  ref_term(dba,string,Ref,Str),
  db_endtransaction(dba),
  shiftwindow(4),
  edit(Str,_, "VIEW", Name, "", 0, "PROLOG.HLP", 0,1,1,1,_,_),
  clearwindow.
 interpret('V',_,_):-!,db_endtransaction(dba).
 interpret('D',Index,LockIndex):-
  write("\nDelete file: "),
  readln(Name),
  db_begintransaction(dba,readwrite),
  key_search(dba,Index,Name,Ref),
  not(key_search(dba,LockIndex,Name,_)),!,
  key_delete(dba,Index,Name,Ref),
  term_delete(dba,file_chain,Ref),
  list,
  db_endtransaction(dba).
 interpret('D',_,_):-!,db_endtransaction(dba).
 interpret(_,_,_):-beep.

PREDICATES
 open_dbase(bt_selector,bt_selector)

CLAUSES
 open_dbase(INDEX,LOCKINDEX):-
  existfile("share.dba"),!,
  db_open(dba, "share.dba",readwrite,denynone),
  db_begintransaction(dba,readwrite),
  bt_open(dba, "locks", LOCKINDEX),
  bt_open(dba, "ndx", INDEX),
  db_endtransaction(dba).
 open_dbase(INDEX,LOCKINDEX):-
  db_create(dba,"share.dba" , in_file),
  bt_create(dba, "locks",TEMPLOCKINDEX,20, 4),
  bt_create(dba, "ndx",TEMPINDEX , 20, 4),
  bt_close(dba, TEMPINDEX),
  bt_close(dba, TEMPLOCKINDEX),
  db_close(dba),
 open_dbase(INDEX,LOCKINDEX).

GOAL
  makewindow(2,23,23,"TEXTS (*=Locked)",0,50,25,30),
  makewindow(4,7,7,"EDIT",0,0,21,50),
  makewindow(3,13,13,"COMMANDS E:Edit V:View D:Delete Q:Quit",21,0,4,50),
  open_dbase(INDEX,LOCKINDEX),
  assert(index(INDEX)),
  assert(lockindex(LOCKINDEX)),
  marktime(10,Mark),
  assert(mark(Mark)),
  db_setretry(dba,5,20),
  db_begintransaction(dba,read),
  list,
  db_endtransaction(dba),
  main(dba, INDEX,LOCKINDEX),
  db_begintransaction(dba,read),
  bt_close(dba, INDEX),
  bt_close(dba, LOCKINDEX),
  db_endtransaction(dba),
  db_close(dba),
  removewindow,
  removewindow,
  removewindow.

