/*
===========================================================================
	SEMANTIC.PRO   

	semantic analysis for CHEMICAL project

	Copyright (c) 1986, 1990 by Prolog Development Center
===========================================================================
*/

code=3999
project "chemical"

include "chemglob.pro"
include "chemutil.pro"


   /* Classifying Groups */

PREDICATES
    classify_list(List,List,List,List,List,List,List).
    classify_listx(List,List,List,List,List,List,List).
    classify_hal(List,List,List). classify_ane_ene_yne(List,List).
    classify_en_ynyl(List,List,List). 
    classify_enyl(List,List,List).
    classify_spcl_yl(List,List,List). classify_yl(List,List,List).
    extract_upto_bra(List,List,List). 
    classify_btwnbra(List,List).
   

CLAUSES

   classify_list(List,Enynyl,Enyl,Spclyl,Yl,H,AEYne):-
    classify_listx(List,Enynyl,Enyl,Spclyl,Yl,H,Listx),
    classify_ane_ene_yne(Listx,AEYne).
         
   classify_listx(List,Enynyl,Enyl,Spcl,Yl,H,Rlist):-
    classify_hal(List,H1,L1),
    classify_en_ynyl(L1,Enynyl1,L2),
    classify_spcl_yl(L2,Spcl1,L3),
    classify_enyl(L3,Enyl1,L4),
    classify_yl(L4,Yl1,L5),not(equal(List,L5)),
    classify_listx(L5,Enynyl2,Enyl2,Spcl2,Yl2,H2,Rlist),
    append(H1,H2,H),
    append(Spcl1,Spcl2,Spcl),append(Yl1,Yl2,Yl),
    append(Enyl1,Enyl2,Enyl),
    append(Enynyl1,Enynyl2,Enynyl),!.
             
   classify_listx(L,[],[],[],[],[],L).
      

   classify_hal([Tok|T],H,Rlist):-
    check(Tok),classify_hal(T,H,Rlist),!.

   classify_hal([Tok|T],[Intstr,H1|H2],Rlist):-
    check_int(Tok),sieve_int([Tok|T],Intstr,[HnAl|List]),
    halogen(Hn,_),find(Hn,HnAl,Al,Ps),
    str_len(Hn,Lhn),
    L=Lhn +Ps-1, frontstr(L,HnAl,H1,_),
    classify_hal([Al|List],H2,Rlist),!.

   classify_hal([HnAl|List],[H1|H2],Rlist):-
    halogen(Hn,_),find(Hn,HnAl,Al,Ps),
    str_len(Hn,Lhn),
    L=Lhn + Ps - 1, frontstr(L,HnAl,H1,_),
    classify_hal([Al|List],H2,Rlist),!.
     
   classify_hal(L,[],L).
                              
 
  classify_en_ynyl([Tok|T],Enynyl,List):-
    check(Tok),classify_en_ynyl(T,Enynyl,List),!.
 
   classify_en_ynyl([Tok|T],Enynyl,List):-
    check_int(Tok),sieve_int([Tok|T],Intstr,["("|List1] ),
    extract_upto_bra(List1,Extract,Rlist),
    classify_btwnbra([Intstr,"("|Extract],NwExtr),
    classify_en_ynyl(Rlist,Enyl,List),
    append(NwExtr,Enyl,Enynyl),!.

   classify_en_ynyl([Tok|T],Enynyl,List):-
    check_int(Tok),sieve_int([Tok|T],Intstr,[Pfx,"("|List1] ),
    prefix(Pfx,_),extract_upto_bra(List1,Extract,Rlist),
    classify_btwnbra([Intstr,Pfx,"("|Extract],NwExtr),
    classify_en_ynyl(Rlist,Enyl,List),
    append(NwExtr,Enyl,Enynyl),!.
              
   classify_en_ynyl(L,[],L).                        
     
   extract_upto_bra([")"|L],[")"],L):-!.
   extract_upto_bra([Tok|L],[Tok|L1],L2):-
    extract_upto_bra(L,L1,L2).

   classify_btwnbra([],[]).
   classify_btwnbra([I,P,"(",T|L],[I,P,"(",I2|L2]):-
    check_int(T),sieve_int([T|L],I2,Lx),
    classify_btwnbra(Lx,L2),!.

   classify_btwnbra([I,"(",T|L],[I,"(",I2|L2]):-
    check_int(T),sieve_int([T|L],I2,Lx),
    classify_btwnbra(Lx,L2),!.
                 
   classify_btwnbra([T|L],[I|L2]):-
    check_int(T),sieve_int([T|L],I,Lx),
    classify_btwnbra(Lx,L2),!.

   classify_btwnbra([T|L],[T|L2]):-
    not(check_int(T)),classify_btwnbra(L,L2).
                                
   classify_spcl_yl([Tok|T],Yl,List):-
    check(Tok),classify_spcl_yl(T,Yl,List),!. 

   classify_spcl_yl([Tok|T],[Intstr,Front|SpYl],List):-
    check_int(Tok),sieve_int([Tok|T],Intstr,[AlkylRst|Rlist]),
    find(yl,AlkylRst,Rst,_),
    sp_stem(Alk,_),concat(Alk,yl,Alkyl),
    find(Alkyl,AlkylRst,_,N),
    str_len(Alkyl,Lal),L=(N + Lal -1),
    frontstr(L,AlkylRst,Front,_),
    classify_spcl_yl([Rst|Rlist],SpYl,List),!.
             
   classify_spcl_yl(L,[],L).


  classify_enyl([Tok|T],Yl,List):-
   check(Tok),
   classify_enyl(T,Yl,List),!. 

  classify_enyl([Tok|T],[Intstr,Front|EYnyl],List):-
   check_int(Tok),sieve_int([Tok|T],Intstr,[AlkenylRst|Rlist]),
   suffix_yl(Enl,S),S>1,
   find(Enl,AlkenylRst,Rst,_),
   stem(Alk,_),concat(Alk,Enl,Alkenyl),
   find(Alkenyl,AlkenylRst,_,N),
   str_len(Alkenyl,Lal),L=(N + Lal -1),
   frontstr(L,AlkenylRst,Front,_),       
   classify_enyl([Rst|Rlist],EYnyl,List),!.
             
  classify_enyl(L,[],L).


  classify_yl([Tok|T],Yl,List):-
   check(Tok),
   classify_yl(T,Yl,List),!. 

  classify_yl([Tok|T],[Intstr,Front|Yl],List):-
   check_int(Tok),sieve_int([Tok|T],Intstr,[AlkylRst|Rlist]),
   find(yl,AlkylRst,Rst,_),
   stem(Alk,_),concat(Alk,yl,Alkyl),
   find(Alkyl,AlkylRst,_,N),
   str_len(Alkyl,Lal),L=(N + Lal -1),
   frontstr(L,AlkylRst,Front,_),
   classify_yl([Rst|Rlist],Yl,List),!.
        
  classify_yl(L,[],L).
    
    
  classify_ane_ene_yne([Tok|T],AEYne):-
   check(Tok),
   classify_ane_ene_yne(T,AEYne),!.
             
  classify_ane_ene_yne([Tok],[Tok]):-
   right_str(3,Tok,Ane),suffix(Ane,_),
   stem(Nm,_),concat(Nm,Ane,NmAne),
   str_len(NmAne,L),right_str(L,Tok,Rtstr),
   equal(Rtstr,NmAne),!.


  classify_ane_ene_yne([Tok|T],[Intstr,Alkenyne]):-
   check_int(Tok),sieve_int([Tok|T],Intstr,[Alkenyne]),
   right_str(3,Alkenyne,Enyne),suffix(Enyne,N),N>1,
   stem(Nm,_),concat(Nm,Enyne,NmEnyne),
   equal(Alkenyne,NmEnyne),!.


  classify_ane_ene_yne([Tok|T],[Intstr,AlkPx2enyne]):-
   check_int(Tok),sieve_int([Tok|T],Intstr,[AlkPx2enyne]),
   right_str(3,AlkPx2enyne,Enyne),suffix(Enyne,N),N>1,
   stem(Alk,_),match(Alk,AlkPx2Enyne,Px2Enyne),
   prefix2(Px2,_),match(Px2,Px2Enyne,EnyneRst),
   equal(EnyneRst,Enyne),!.
             

  classify_ane_ene_yne([Tok|T],[Intstr,Alken,Intstr2,Yne]):-
   check_int(Tok),sieve_int([Tok|T],Intstr,[Alken,Num|R]),
   stem(Alk,_),match(Alk,Alken,En),
   equal(En,en),
   check_int(Num),sieve_int([Num|R],Intstr2,[Yne]),
   equal(Yne,yne),!.

  classify_ane_ene_yne([Tok|T],[Intstr,Alken,Intstr2,Px3Yne]):-
   check_int(Tok),sieve_int([Tok|T],Intstr,[Alken,Num|R]),
   stem(Alk,_),match(Alk,Alken,En),
   equal(En,en),
   check_int(Num),sieve_int([Num|R],Intstr2,[Px3Yne]),
   prefix(Px3,_),match(Px3,Px3Yne,Yne),
   equal(Yne,yne),!.


  classify_ane_ene_yne([Tok|T],[Intstr,AlkPx2en,Intstr2,Yne]):-
   check_int(Tok),sieve_int([Tok|T],Intstr,[AlkPx2en,Num|R]),
   stem(Alk,_),match(Alk,AlkPx2en,Px2en),
   prefix2(Px2,_),match(Px2,Px2en,En),
   equal(En,en),
   check_int(Num),sieve_int([Num|R],Intstr2,[Yne]),
   equal(Yne,yne),!.

  classify_ane_ene_yne([Tok|T],[Intstr,AlkPx2en,Intstr2,Px3Yne]):-
   check_int(Tok),sieve_int([Tok|T],Intstr,[AlkPx2en,Num|R]),
   stem(Alk,_),match(Alk,AlkPx2en,Px2en),
   prefix2(Px2,_),match(Px2,Px2en,En),
   equal(En,en),
   check_int(Num),sieve_int([Num|R],Intstr2,[Px3Yne]),
   prefix(Px3,_),match(Px3,Px3Yne,Yne),
   equal(Yne,yne),!.

   
 
  /*********** Some semantic check and Formatting the namelist*******/
   
  Predicates
   format_AEYne_list(List,LList).  format_hal_list(List,LList,LList). 
   format_YL_list(List,LList).format_Spcl_Yl_list(List,LList).
   format_EYnyl_list(List,LList).  
   format_EnYnyl_list(List,LList).
   
  Clauses
  
   format_AEYne_list([AEYne],[sp_name(Front),nc(N),db([]),trb([])]):-
    right_str(3,AEYne,R),equal(R,ane),
    stem(Nm,N),find(Nm,AEYne,_,L),L1=L-1,
    frontstr(L1,AEYne,Front,_),!.
   
  format_AEYne_list([EYne],[sp_name(""),nc(N),db([1]),trb([])]):-
   right_str(3,EYne,R),equal(R,ene),
   stem(Nm,N),find(Nm,EYne,_,_),!.

  format_AEYne_list([EYne],[sp_name(""),nc(N),db([]),trb([1])]):-
   right_str(3,EYne,R),equal(R,yne),
   stem(Nm,N),find(Nm,EYne,_,_),!.
   
  format_AEyne_list([Num,AlkEYne],[sp_name(""),nc(N),db(I),trb([])]):-
   right_str(3,AlkEYne,R),equal(R,ene),
   scanx(Num,IntL),filter(IntL,InL),
   str_int_list(InL,I),list_len(I,L),L=1,
   stem(Nm,N),match(Nm,AlkEYne,EYne),equal(EYne,ene),!.
   
  format_AEyne_list([Num,AlkEYne],[sp_name(""),nc(N),db([]),trb(I)]):-
   right_str(3,AlkEYne,R),equal(R,yne),
   scanx(Num,IntL),filter(IntL,InL),
   str_int_list(InL,I),list_len(I,L),L=1,
   stem(Nm,N),match(Nm,AlkEYne,EYne),equal(EYne,yne),!.

  format_AEyne_list([Num,AlkPfxEYne],[sp_name(""),nc(N),db(I),trb([])]):-
   right_str(3,AlkPfxEYne,R),equal(R,ene),
   scanx(Num,IntL),filter(IntL,InL),
   str_int_list(InL,I),list_len(I,L),L>1,
   stem(Nm,N),match(Nm,AlkPfxEYne,PfxEYne),
   prefix2(Pfx,L),match(Pfx,PfxEYne,EYne),equal(EYne,ene),!.
   
  format_AEyne_list([Num,AlkPfxEYne],[sp_name(""),nc(N),db([]),trb(I)]):-
   right_str(3,AlkPfxEYne,R),equal(R,yne),
   scanx(Num,IntL),filter(IntL,InL),
   str_int_list(InL,I),list_len(I,L),L>1,
   stem(Nm,N),match(Nm,AlkPfxEYne,PfxEYne),
   prefix2(Pfx,L),match(Pfx,PfxEYne,EYne),equal(EYne,yne),!.
               
   format_AEyne_list([Num,AlkEn,Num2,yne],[sp_name(""),nc(N),db(I),trb(J)]):-
        scanx(Num,IntL),filter(IntL,InL),
        str_int_list(InL,I),list_len(I,L),L=1,
        stem(Nm,N),match(Nm,AlkEn,En),equal(En,en),
        scanx(Num2,IntJ),filter(IntJ,InJ),
        str_int_list(InJ,J),list_len(J,M),M=1,!.
        
   format_AEyne_list([Num,AlkEn,Num2,Pxyne],[sp_name(""),nc(N),db(I),trb(J)]):-
        right_str(3,Pxyne,Yne),equal(Yne,yne),
        scanx(Num,IntL),filter(IntL,InL),
        str_int_list(InL,I),list_len(I,L),L=1,
        stem(Nm,N),match(Nm,AlkEn,En),equal(En,en),
        scanx(Num2,IntJ),filter(IntJ,InJ),
        str_int_list(InJ,J),list_len(J,M),M>1,
        prefix(Px,M),match(Px,Pxyne,_),!.
        


   format_AEyne_list([Num,AlkPx2En,Num2,yne],[sp_name(""),nc(N),db(I),trb(J)]):-
        scanx(Num,IntL),filter(IntL,InL),
        str_int_list(InL,I),list_len(I,L),L>1,
        stem(Nm,N),match(Nm,AlkPx2En,Px2En),
        prefix2(Pfx2,L),match(Pfx2,Px2En,_),
        scanx(Num2,IntJ),filter(IntJ,InJ),
        str_int_list(InJ,J),list_len(J,M),M=1,!.
       
        

   format_AEyne_list([Num,AlkPx2En,Num2,Pxyne],[sp_name(""),nc(N),db(I),trb(J)]):-
        right_str(3,Pxyne,Yne),equal(Yne,yne),
        scanx(Num,IntL),filter(IntL,InL),
        str_int_list(InL,I),list_len(I,L),L>1,
        stem(Nm,N),match(Nm,AlkPx2En,Px2En),
        prefix2(Pfx2,L),match(Pfx2,Px2En,_),
        scanx(Num2,IntJ),filter(IntJ,InJ),
        str_int_list(InJ,J),list_len(J,M),M>1,
        prefix(Px,M),match(Px,Pxyne,_),!.
        
      
      format_hal_list([H],[sp_name(""),nc(N),db([]),trb([])],[sp_name(Fm),loc([1])]):-
      N<3, halogen(H,Fm),!.
      
      format_hal_list([H],[sp_name(""),nc(2),db([1]),trb([])],[sp_name(Fm),loc([1])]):-
       halogen(H,Fm),!.
      
      format_hal_list([H],[sp_name(""),nc(2),db([]),trb([1])],[sp_name(Fm),loc([1])]):-
       halogen(H,Fm),!.
      
      format_hal_list([PxH],[sp_name(""),nc(1),db([]),trb([])],[sp_name(Fm),loc(I)]):-
       prefix(Px,P),match(Px,PXH,H),halogen(H,Fm),
       make_list(1,P,I),!.
      
      format_hal_list([PxH],[sp_name(""),nc(2),db([1]),trb([])],[sp_name(Fm),loc(I)]):-
       prefix(Px,P),match(Px,PXH,H),halogen(H,Fm),
       P>2,Q=P-2,make_list(1,2,I1),make_list(2,Q,I2),
       append(I1,I2,I),!.
      
      format_hal_list([DiH],[sp_name(""),nc(2),db([]),trb([1])],[sp_name(Fm),loc([1,2])]):-
       match(di,DiH,H),halogen(H,Fm),!.

     
      format_hal_list([],_,[]):-!.
      format_hal_list([Num,Hal|List],_,[sp_name(Fm),loc(I)|NwList]):-
       scanx(Num,IntL),filter(IntL,InL),
       List_len(InL,L),L=1,
       halogen(Hal,Fm),
       str_int_list(InL,I),format_hal_list(List,[],NwList),!.
       
      format_hal_list([Num,PxHal|List],_,[sp_name(Fm),loc(IL)|NwList]):-
       scanx(Num,IntL),filter(IntL,InL),
       List_len(InL,L),L>1,
       prefix(Px,L),match(Px,PxHal,Hal),
       halogen(Hal,Fm),
       str_int_list(InL,IL),format_hal_list(List,[],NwList),!.

      
      format_Yl_list([],[]):-!.
      format_YL_list([Num,AlkYl|List],[sp_name(""),nc(N),loc(I)|NwList]):-
       scanx(Num,IntL),filter(IntL,InL),
       list_len(InL,L),L=1,
       stem(Nm,N),match(Nm,AlkYl,_),
       max_alkyl(M),N<=M,
       str_int_list(InL,I),
       format_Yl_list(List,NwList),!.
       
      format_YL_list([Num,IsoAlkYl|List],[sp_name(Iso),nc(N),loc(I)|NwList]):-
       scanx(Num,IntL),filter(IntL,InL),
       list_len(InL,L),L=1,
       prefix_yl(Iso,P),match(Iso,IsoAlkYl,AlkYl),
       stem(Nm,N),match(Nm,AlkYl,_),
       max_alkyl(M),Q=P+M,N<=Q,
       str_int_list(InL,I),
       format_Yl_list(List,NwList),!.
      
      format_YL_list([Num,PfxAlkYl|List],[sp_name(""),nc(N),loc(I)|NwList]):-
       scanx(Num,IntL),filter(IntL,InL),
       list_len(InL,L),L>1,
       prefix(Pfx,L),match(Pfx,PfxAlkYl,AlkYl),
       stem(Nm,N),match(Nm,AlkYl,_),
       max_alkyl(M),N<=M,
       str_int_list(InL,I),
       format_Yl_list(List,NwList),!.

      format_YL_list([Num,PfxIsoAlkYl|List],[sp_name(Iso),nc(N),loc(I)|NwList]):-
       scanx(Num,IntL),filter(IntL,InL),
       list_len(InL,L),L>1,
       prefix(Pfx,L),match(Pfx,PfxIsoAlkYl,IsoAlkYl),
       prefix_yl(Iso,P),match(Iso,IsoAlkYl,AlkYl),
       stem(Nm,N),match(Nm,AlkYl,_),
       max_alkyl(M),Q=P+M,N<=Q,
       str_int_list(InL,I),
       format_Yl_list(List,NwList),!.
     


  format_Spcl_Yl_list([],[]):-!.
  format_Spcl_Yl_list([Num,Spcl|List],[sp_name(Nm),loc(I)|NwList]):-
   scanx(Num,IntL),filter(IntL,InL),
   list_len(InL,L),L=1,
   sp_stem(Nm,_),match(Nm,Spcl,_),
   str_int_list(InL,I),format_Spcl_Yl_list(List,NwList),!.
   
  format_Spcl_Yl_list([Num,PfxSpcl|List],[sp_name(Nm),loc(I)|NwList]):-
   scanx(Num,IntL),filter(IntL,InL),
   list_len(InL,L),L>1,
   prefix(Pfx,L),match(Pfx,PfxSpcl,Spcl),
   sp_stem(Nm,_),match(Nm,Spcl,_),
   str_int_list(InL,I),format_Spcl_Yl_list(List,NwList),!.
      
   

  format_EYnyl_list([],[]):-!.
  format_EYnyl_list([Num,EYnyl|List],[nc(N),db([1]),trb([]),loc(I) |NwList]):-
   right_str(4,EYnyl,Enyl),equal(Enyl,enyl),
   scanx(Num,IntL),filter(IntL,InL),
   list_len(InL,L),L=1,
   stem(Nm,N),match(Nm,EYnyl,_),
   str_int_list(InL,I),
   format_EYnyl_list(List,NwList),!.

  format_EYnyl_list([Num,EYnyl|List],[nc(N),db([]),trb([1]),loc(I) |NwList]):-
   right_str(4,EYnyl,Ynyl),equal(Ynyl,ynyl),
   scanx(Num,IntL),filter(IntL,InL),
   list_len(InL,L),L=1,
   stem(Nm,N),match(Nm,EYnyl,_),
   str_int_list(InL,I),
   format_EYnyl_list(List,NwList),!.

  format_EYnyl_list([Num,PxEYnyl|List],[nc(N),db([1]),trb([]),loc(I) |NwList]):-
   right_str(4,PxEYnyl,Enyl),equal(Enyl,enyl),
   scanx(Num,IntL),filter(IntL,InL),
   list_len(InL,L),L>1,
   prefix(Px,L),match(Px,PxEYnyl,EYnyl),
   stem(Nm,N),match(Nm,EYnyl,_),
   str_int_list(InL,I),
   format_EYnyl_list(List,NwList),!.

  format_EYnyl_list([Num,PxEYnyl|List],[nc(N),db([]),trb([1]),loc(I) |NwList]):-
   right_str(4,PxEYnyl,Ynyl),equal(Ynyl,ynyl),
   scanx(Num,IntL),filter(IntL,InL),
   list_len(InL,L),L>1,
   prefix(Px,L),match(Px,PxEYnyl,EYnyl),
   stem(Nm,N),match(Nm,EYnyl,_),
   str_int_list(InL,I),
   format_EYnyl_list(List,NwList),!.
 


  format_EnYnyl_list([],[]):-!. 

  format_EnYnyl_list([Num,"(",Num2,AlkEYnyl,")"|List],[nc(N),db(D),trb([]),loc(I)|NwList]):-
   right_str(4,AlkEYnyl,Enyl),equal(Enyl,enyl),
   scanx(Num,IntL),filter(IntL,InL),
   list_len(InL,L),L=1,
   scanx(Num2,IntL2),filter(IntL2,InL2),
   list_len(InL2,M),M=1,
   stem(Nm,N),match(Nm,AlkEYnyl,EYnyl),
   max_alkyl(K),N<=K,equal(EYnyl,enyl),
   str_int_list(InL,I),str_int_list(InL2,D),
   format_EnYnyl_list(List,NwList),!.

  format_EnYnyl_list([Num,Px,"(",Num2,AlkEYnyl,")"|List],[nc(N),db(D),trb([]),loc(I)|NwList]):-
   right_str(4,AlkEYnyl,Enyl),equal(Enyl,enyl),
   scanx(Num,IntL),filter(IntL,InL),
   list_len(InL,L),L>1,prefix(Px,L),
   scanx(Num2,IntL2),filter(IntL2,InL2),
   list_len(InL2,M),M=1,
   stem(Nm,N),match(Nm,AlkEYnyl,EYnyl),
   max_alkyl(K),N<=K,equal(EYnyl,enyl),
   str_int_list(InL,I),str_int_list(InL2,D),
   format_EnYnyl_list(List,NwList),!.



  format_EnYnyl_list([Num,"(",Num2,AlkPx2EYnyl,")"|List],[nc(N),db(D),trb([]),loc(I)|NwList]):-
   right_str(4,AlkPx2EYnyl,Enyl),equal(Enyl,enyl),
   scanx(Num,IntL),filter(IntL,InL),
   list_len(InL,L),L=1,
   scanx(Num2,IntL2),filter(IntL2,InL2),
   list_len(InL2,M),M>1,
   stem(Nm,N),match(Nm,AlkPx2EYnyl,Px2EYnyl),
   max_alkyl(K),N<=K,
   prefix2(Px2,M),match(Px2,Px2EYnyl,EYnyl),equal(EYnyl,enyl),
   str_int_list(InL,I),str_int_list(InL2,D),
   format_EnYnyl_list(List,NwList),!.

  format_EnYnyl_list([Num,Px,"(",Num2,AlkPx2EYnyl,")"|List],[nc(N),db(D),trb([]),loc(I)|NwList]):-
   right_str(4,AlkPx2EYnyl,Enyl),equal(Enyl,enyl),
   scanx(Num,IntL),filter(IntL,InL),
   list_len(InL,L),L>1,prefix(Px,L),
   scanx(Num2,IntL2),filter(IntL2,InL2),
   list_len(InL2,M),M>1,
   stem(Nm,N),match(Nm,AlkPx2EYnyl,Px2EYnyl),
   max_alkyl(K),N<=K,
   prefix2(Px2,M),match(Px2,Px2EYnyl,EYnyl),equal(EYnyl,enyl),
   str_int_list(InL,I),str_int_list(InL2,D),
   format_EnYnyl_list(List,NwList),!.


  format_EnYnyl_list([Num,"(",Num2,AlkEYnyl,")"|List],[nc(N),db([]),trb(D),loc(I)|NwList]):-
   right_str(4,AlkEYnyl,Ynyl),equal(Ynyl,ynyl),
   scanx(Num,IntL),filter(IntL,InL),
   list_len(InL,L),L=1,
   scanx(Num2,IntL2),filter(IntL2,InL2),
   list_len(InL2,M),M=1,
   stem(Nm,N),match(Nm,AlkEYnyl,EYnyl),
   max_alkyl(K),N<=K,equal(EYnyl,ynyl),
   str_int_list(InL,I),str_int_list(InL2,D),
   format_EnYnyl_list(List,NwList),!.

  format_EnYnyl_list([Num,Px,"(",Num2,AlkEYnyl,")"|List],[nc(N),db([]),trb(D),loc(I)|NwList]):-
   right_str(4,AlkEYnyl,Ynyl),equal(Ynyl,ynyl),
   scanx(Num,IntL),filter(IntL,InL),
   list_len(InL,L),L>1,prefix(Px,L),
   scanx(Num2,IntL2),filter(IntL2,InL2),
   list_len(InL2,M),M=1,
   stem(Nm,N),match(Nm,AlkEYnyl,EYnyl),
   max_alkyl(K),N<=K,equal(EYnyl,ynyl),
   str_int_list(InL,I),str_int_list(InL2,D),
   format_EnYnyl_list(List,NwList),!.

  format_EnYnyl_list([Num,"(",Num2,AlkPx2EYnyl,")"|List],[nc(N),db([]),trb(D),loc(I)|NwList]):-
   right_str(4,AlkPx2EYnyl,Ynyl),equal(Ynyl,ynyl),
   scanx(Num,IntL),filter(IntL,InL),
   list_len(InL,L),L=1,
   scanx(Num2,IntL2),filter(IntL2,InL2),
   list_len(InL2,M),M>1,
   stem(Nm,N),match(Nm,AlkPx2EYnyl,Px2EYnyl),
   max_alkyl(K),N<=K,
   prefix2(Px2,M),match(Px2,Px2EYnyl,EYnyl),equal(EYnyl,ynyl),
   str_int_list(InL,I),str_int_list(InL2,D),
   format_EnYnyl_list(List,NwList),!.

  format_EnYnyl_list([Num,Px,"(",Num2,AlkPx2EYnyl,")"|List],[nc(N),db([]),trb(D),loc(I)|NwList]):-
   right_str(4,AlkPx2EYnyl,Ynyl),equal(Ynyl,ynyl),
   scanx(Num,IntL),filter(IntL,InL),
   list_len(InL,L),L>1,prefix(Px,L),
   scanx(Num2,IntL2),filter(IntL2,InL2),
   list_len(InL2,M),M>1,
   stem(Nm,N),match(Nm,AlkPx2EYnyl,Px2EYnyl),
   max_alkyl(K),N<=K,
   prefix2(Px2,M),match(Px2,Px2EYnyl,EYnyl),equal(EYnyl,ynyl),
   str_int_list(InL,I),str_int_list(InL2,D),
   format_EnYnyl_list(List,NwList),!.
   
   

   format_EnYnyl_list([Num,"(",Num2,Alken,Num3,ynyl,")"|List],[nc(N),db(D),trb(T),loc(I)|NwList]):-
    scanx(Num,IntL),filter(IntL,InL),list_len(InL,L),L=1,
    scanx(Num2,IntL2),filter(IntL2,InL2),list_len(InL2,M),M=1,
    stem(Nm,N),match(Nm,Alken,En),max_alkyl(K),N<=K,equal(En,en),
    scanx(Num3,IntL3),filter(IntL3,InL3),list_len(InL3,P),P=1,
    str_int_list(InL,I),str_int_list(InL2,D),str_int_list(InL3,T),
    format_EnYnyl_list(List,NwList),!.

   format_EnYnyl_list([Num,Px,"(",Num2,Alken,Num3,ynyl,")"|List],[nc(N),db(D),trb(T),loc(I)|NwList]):-
    scanx(Num,IntL),filter(IntL,InL),list_len(InL,L),L>1,prefix(Px,L),
    scanx(Num2,IntL2),filter(IntL2,InL2),list_len(InL2,M),M=1,
    stem(Nm,N),match(Nm,Alken,En),max_alkyl(K),N<=K,equal(En,en),
    scanx(Num3,IntL3),filter(IntL3,InL3),list_len(InL3,P),P=1,
    str_int_list(InL,I),str_int_list(InL2,D),str_int_list(InL3,T),
    format_EnYnyl_list(List,NwList),!.

   format_EnYnyl_list([Num,"(",Num2,AlkPx2en,Num3,ynyl,")"|List],[nc(N),db(D),trb(T),loc(I)|NwList]):-
    scanx(Num,IntL),filter(IntL,InL),list_len(InL,L),L=1,
    scanx(Num2,IntL2),filter(IntL2,InL2),list_len(InL2,M),M>1,
    stem(Nm,N),match(Nm,AlkPx2en,Px2En),max_alkyl(K),N<=K,
    prefix2(Px2,M),match(Px2,Px2En,En),equal(En,en),
    scanx(Num3,IntL3),filter(IntL3,InL3),list_len(InL3,P),P=1,
    str_int_list(InL,I),str_int_list(InL2,D),str_int_list(InL3,T),
    format_EnYnyl_list(List,NwList),!.
    

   format_EnYnyl_list([Num,Px,"(",Num2,AlkPx2en,Num3,ynyl,")"|List],[nc(N),db(D),trb(T),loc(I)|NwList]):-
    scanx(Num,IntL),filter(IntL,InL),list_len(InL,L),L>1,prefix(Px,L),
    scanx(Num2,IntL2),filter(IntL2,InL2),list_len(InL2,M),M>1,
    stem(Nm,N),match(Nm,AlkPx2en,Px2En),max_alkyl(K),N<=K,
    prefix2(Px2,M),match(Px2,Px2En,En),equal(En,en),
    scanx(Num3,IntL3),filter(IntL3,InL3),list_len(InL3,P),P=1,
    str_int_list(InL,I),str_int_list(InL2,D),str_int_list(InL3,T),
    format_EnYnyl_list(List,NwList),!.
    

   format_EnYnyl_list([Num,"(",Num2,Alken,Num3,Px3ynyl,")"|List],[nc(N),db(D),trb(T),loc(I)|NwList]):-
    scanx(Num,IntL),filter(IntL,InL),list_len(InL,L),L=1,
    scanx(Num2,IntL2),filter(IntL2,InL2),list_len(InL2,M),M=1,
    stem(Nm,N),match(Nm,Alken,En),max_alkyl(K),N<=K,equal(En,en),
    scanx(Num3,IntL3),filter(IntL3,InL3),list_len(InL3,P),P>1,
    prefix(Px3,P),match(Px3,Px3ynyl,Ynyl),equal(Ynyl,ynyl),
    str_int_list(InL,I),str_int_list(InL2,D),str_int_list(InL3,T),
    format_EnYnyl_list(List,NwList),!.
    

   format_EnYnyl_list([Num,Px,"(",Num2,Alken,Num3,Px3ynyl,")"|List],[nc(N),db(D),trb(T),loc(I)|NwList]):-
    scanx(Num,IntL),filter(IntL,InL),list_len(InL,L),L>1,prefix(Px,L),
    scanx(Num2,IntL2),filter(IntL2,InL2),list_len(InL2,M),M=1,
    stem(Nm,N),match(Nm,Alken,En),max_alkyl(K),N<=K,equal(En,en),
    scanx(Num3,IntL3),filter(IntL3,InL3),list_len(InL3,P),P>1,
    prefix(Px3,P),match(Px3,Px3ynyl,Ynyl),equal(Ynyl,ynyl),
    str_int_list(InL,I),str_int_list(InL2,D),str_int_list(InL3,T),
    format_EnYnyl_list(List,NwList),!.
    

   format_EnYnyl_list([Num,"(",Num2,AlkPx2en,Num3,Px3ynyl,")"|List],[nc(N),db(D),trb(T),loc(I)|NwList]):-
    scanx(Num,IntL),filter(IntL,InL),list_len(InL,L),L=1,
    scanx(Num2,IntL2),filter(IntL2,InL2),list_len(InL2,M),M>1,
    stem(Nm,N),match(Nm,AlkPx2en,Px2En),max_alkyl(K),N<=K,
    prefix2(Px2,M),match(Px2,Px2En,En),equal(En,en),
    scanx(Num3,IntL3),filter(IntL3,InL3),list_len(InL3,P),P>1,
    prefix(Px3,P),match(Px3,Px3ynyl,Ynyl),equal(Ynyl,ynyl),
    str_int_list(InL,I),str_int_list(InL2,D),str_int_list(InL3,T),
    format_EnYnyl_list(List,NwList),!.
    

   format_EnYnyl_list([Num,Px,"(",Num2,AlkPx2en,Num3,Px3ynyl,")"|List],[nc(N),db(D),trb(T),loc(I)|NwList]):-
    scanx(Num,IntL),filter(IntL,InL),list_len(InL,L),L>1,prefix(Px,L),
    scanx(Num2,IntL2),filter(IntL2,InL2),list_len(InL2,M),M>1,
    stem(Nm,N),match(Nm,AlkPx2en,Px2En),max_alkyl(K),N<=K,
    prefix2(Px2,M),match(Px2,Px2En,En),equal(En,en),
    scanx(Num3,IntL3),filter(IntL3,InL3),list_len(InL3,P),P>1,
    prefix(Px3,P),match(Px3,Px3ynyl,Ynyl),equal(Ynyl,ynyl),
    str_int_list(InL,I),str_int_list(InL2,D),str_int_list(InL3,T),
    format_EnYnyl_list(List,NwList),!.
    

/****************************************************************************/
   
PREDICATES
     check_hal_loc(LList,integer).
     check_alk_loc(LList,integer).
     extract_loc(LList,int_list).
     check_db_trb_loc(LList).
     check_max_C(List,List,List,List,List,LList).
     accum_loc(LList,LList,LList,LList,LList,Int_list).
     accum_db_trb_loc(LList,Int_list).
     check_C_bonds(Int_list,integer).



CLAUSES

    extract_loc(LList,IntList):-
     member(loc(I),LList,NwList),extract_loc(NwList,List),
     append(I,List,IntList),!.
    extract_loc(_,[]). 

      accum_loc(EnYnyl,EYnyl,SpYl,Yl,H,IntList):-
        extract_loc(H,Hlist),extract_loc(Yl,Yllist),
        extract_loc(SpYl,Splist),extract_loc(EYnyl,EYlist),
        extract_loc(EnYnyl,EnYnlist),
        append(Hlist,Yllist,HY),append(HY,Splist,HYS),
        append(HYS,EYlist,HYSEY),append(HYSEY,EnYnlist,IntList),!.
      
      accum_db_trb_loc(AEYne,DxTxx):-
        member(db(D),AEYne,_),
        member(trb(T),AEYne,_),append(D,T,DT),
        make_db_trb_list(DT,DxTx),make_db_trb_list(T,Tx),
        append(DxTx,Tx,DxTxx),!.

     check_hal_loc([],_):-!. 
     check_hal_loc(LList,N):-
      extract_loc(LList,IntL),not(check_list(IntL)),
      ourmin(IntL,L),0<L, ourmax(IntL,M),
      M<=N,!.

     check_hal_loc(LList,_):-
      extract_loc(LList,IntL),check_list(IntL),!.

     check_alk_loc([],_):-!.
     check_alk_loc(LList,N):-
      extract_loc(LList,IntL),not(check_list(IntL)),
      ourmax(IntL,H),ourmin(IntL,L),
      1<L,H<N,!.

     check_alk_loc(LList,_):-
      extract_loc(LList,IntL),check_list(IntL),!.
     
      
    check_db_trb_loc(AEYne):-
     member(db(D),AEYne,_),member(trb(T),AEYne,_),
     append(D,T,DT),not(check_list(DT)),ourmax(DT,M),
     member(nc(N),AEYne,_),M<N,!.

    check_db_trb_loc(AEYne):-
     member(db(D),AEYne,_),member(trb(T),AEYne,_),
     append(D,T,DT),check_list(DT),!.

          

     check_max_C([],[],[],[],_,AEyne):-
     member(nc(N),AEYne,_),
     max_C_wob(M),N<=M,!.
    
     
    check_max_C(L1,L2,L3,L4,_,AEyne):-
     append(L1,L2,L12),append(L3,L4,L34),
     append(L12,L34,L1234),not(check_list(L1234)),
     member(nc(N),AEYne,_),
     max_C_wb(M),N<=M,!.


        

    check_C_bonds([],_).

    check_C_bonds([I|IntList],N):-
      count_N(I,[I|IntList],M),
      I<>1,I<>N,M<3,delete(I,IntList,List),
      check_C_bonds(List,N),!.

    check_C_bonds([I|IntList],N):-
      count_N(I,[I|IntList],M),
      I=1,M<4,delete(I,IntList,List),
      check_C_bonds(List,N),!.

    check_C_bonds([I|IntList],N):-
      count_N(I,[I|IntList],M),
      I=N,M<4,delete(I,IntList,List),
      check_C_bonds(List,N),!.
       
    check_C_bonds([I|IntList],N):-
      count_N(I,[I|IntList],M),
      I=1,N=1,M<=4,delete(I,IntList,List),
      check_C_bonds(List,N),!.
   
    



   /*......... semantic check for bi/tri valence radicals ............*/

   Predicates 
      semantic_check_of(LList).
      check_db_trb_loc_in(LList).check_C_bonds_in(Int_list,integer).
      accum_db_trb_in(LList,int_list).    

  Clauses   

   check_db_trb_loc_in([]):-!.
  
   check_db_trb_loc_in(EnYnyl):-
    member(db(D),EnYnyl,_),member(trb(T),EnYnyl,_),
    append(D,T,DT),make_db_trb_list(DT,DxTx),
    not(check_list(DxTx)),ourmax(DxTx,M),member(nc(N),EnYnyl,_),
    M<=N,!.


          


   accum_db_trb_in(EnYnyl,DxTxx):-
    member(db(D),EnYnyl,_),member(trb(T),EnYnyl,_),
    append(D,T,DT),make_db_trb_list(DT,DxTx),
    make_db_trb_list(T,Tx),append(DxTx,Tx,DxTxx),!.    
 
     check_C_bonds_in([],_).     
     check_C_bonds_in([I|IntList],N):-
      count_n(I,[I|IntList],M),
      I=N,M<4, delete(I,IntList,List),
      check_C_bonds_in(List,N),!.

     check_C_bonds_in([I|IntList],N):-
      count_n(I,[I|IntList],M),
      I<N,M<3, delete(I,IntList,List),
      check_C_bonds_in(List,N),!.

            
  semantic_check_of([]):-!.
 
  semantic_check_of(Enynyl):-
   member(loc(_),Enynyl,List),append(En,List,Enynyl),
   check_db_trb_loc_in(En),
   accum_db_trb_in(En,IntList),   
   member(nc(N),En,_),check_C_bonds_in(IntList,N),
   semantic_check_of(List),!.
            
   


   /*******************  semantic checks  main body ********************/
      

     semantic_check(List,EnYnlf,EYnlf,Splf,Ylf,Hf,AEYnef,no_error):-
       classify_list(List,EnYnyl,EYnyl,SpYl,Yl,H,AEYne),
       format_AEYne_list(AEYne,AEYnef),member(nc(N),AEYnef,_),
       check_max_C([],EYnyl,SpYl,Yl,H,AEYnef),
       check_db_trb_loc(AEYnef),
       format_hal_list(H,AEYnef,Hf), check_hal_loc(Hf,N),
       format_Yl_list(Yl,Ylf), check_alk_loc(Ylf,N),
       format_Spcl_Yl_list(SpYl,Splf),check_alk_loc(Splf,N),
       format_EYnyl_list(EYnyl,EYnlf),check_alk_loc(EYnlf,N),
       format_EnYnyl_list(EnYnyl,EnYnlf),write("#91"),check_alk_loc(EnYnlf,N),
       accum_loc([],EYnlf,Splf,Ylf,Hf,IntL),
       accum_db_trb_loc(AEYnef,IList),append(IntL,IList,IntList),
       check_C_bonds(IntList,N),
       semantic_check_of(EnYnlf),!.


     semantic_check(List,[],[],[],[],[],[],error):-
       classify_list(List,EnYnyl,EYnyl,SpYl,Yl,H,AEYne),
       format_AEYne_list(AEYne,AEYnef),member(nc(N),AEYnef,_),
       check_max_C([],EYnyl,SpYl,Yl,H,AEYnef),
       check_db_trb_loc(AEYnef),
       format_hal_list(H,AEYnef,Hf),check_hal_loc(Hf,N),
       format_Yl_list(Yl,Ylf),check_alk_loc(Ylf,N),
       format_Spcl_Yl_list(SpYl,Splf),check_alk_loc(Splf,N),
       format_EYnyl_list(EYnyl,EYnlf),check_alk_loc(EYnlf,N),
       format_EnYnyl_list(EnYnyl,EnYnlf),check_alk_loc(EnYnlf,N),
       accum_loc([],EYnlf,Splf,Ylf,Hf,IntL),
       accum_db_trb_loc(AEYnef,IList),append(IntL,IList,IntList),
       check_C_bonds(IntList,N),
       error(gm,"SEMANTIC ERROR IN A RADICAL IN BRACKETS: IT MAY BE..",""),
       error(gm," IMPOSSIBLE LOCATION FOR double/triple BOND - OR - ",""),
       error(gm," TOO MANY VALENCIES FOR A C ATOM",""),!.

     semantic_check(List,[],[],[],[],[],[],error):-
       classify_list(List,EnYnyl,EYnyl,SpYl,Yl,H,AEYne),
       format_AEYne_list(AEYne,AEYnef),member(nc(N),AEYnef,_),
       check_max_C([],EYnyl,SpYl,Yl,H,AEYnef),
       check_db_trb_loc(AEYnef),
       format_hal_list(H,AEYnef,Hf),check_hal_loc(Hf,N),
       format_Yl_list(Yl,Ylf),check_alk_loc(Ylf,N),
       format_Spcl_Yl_list(SpYl,Splf),check_alk_loc(Splf,N),
       format_EYnyl_list(EYnyl,EYnlf),check_alk_loc(EYnlf,N),
       format_EnYnyl_list(EnYnyl,EnYnlf),check_alk_loc(EnYnlf,N),
       error(gm,"TOO MANY VALENCEIES FOR A C ATOM IN THE MAIN CHAIN",""),!.
              
     semantic_check(List,[],[],[],[],[],[],error):-
       classify_list(List,[],EYnyl,SpYl,Yl,H,AEYne),
       format_AEYne_list(AEYne,AEYnef),member(nc(N),AEYnef,_),
       check_max_C([],EYnyl,SpYl,Yl,H,AEYnef),
       check_db_trb_loc(AEYnef),
       format_hal_list(H,AEYnef,Hf), check_hal_loc(Hf,N),
       error(gm,"ERROR IN AN alk(yl/enyl/ynyl/enynyl) RADICAL: IT MAY BE..",""),
       error(gm," AN IMPOSSIBLE LOCATION FOR IT - OR - ",""),
       error(gm," NUMERAL(S) AND PREFIX (di/tri etc.) ARE NOT COMPATIBLE - OR - ",""),
       error(gm," NO OF C ATOMS EXCEEDS THE MAX IN IT. ",""),!.
       

     semantic_check(List,[],[],[],[],[],[],error):-
       classify_list(List,[],EYnyl,SpYl,Yl,H,AEYne),
       format_AEYne_list(AEYne,AEYnef),
       check_max_C([],EYnyl,SpYl,Yl,H,AEYnef),
       check_db_trb_loc(AEYnef),
       error(gm,"ERROR IN HALOGEN : IT MAY BE...",""),
       error(gm," LOCATION IS NOT OBVIOUS - OR - ",""),
       error(gm," IMPOSSIBLE LOCATION FOR IT - OR - ",""),
       error(gm," NUMERAL(S) AND PREFIX (di/tri etc) ARE NOT COMPATIBLE",""),!.
       
    semantic_check(_,[],[],[],[],[],[],error):-
       error(gm,"ERROR IN THE MAIN CHAIN :IT MAY BE..",""),
       error(gm," NO OF C ATOMS EXCEEDS THE MAX. - OR - ",""),
       error(gm," IMPOSSIBLE LOCATION FOR DOUBLE/TRIPLE BOND - OR - ",""),
       error(gm," NUMERAL(S) PREFIX (di/tri etc.) NOT COMPATIBLE",""),!.
