/*
===========================================================================
	SYNTAX.PRO   

	syntax check for CHEMICAL project

	Copyright (c) 1986, 1990 by Prolog Development Center
===========================================================================
*/
code=3999
project "chemical"

include "chemglob.pro"
include "chemutil.pro"

    
PREDICATES
       syntax_checkA(List,Symbol). 
       syntax_checkAx(List,Symbol).
       syntax_checkB(List,Symbol). 
       syntax_checkC(List,Symbol). 
       syntax_checkD(List,Symbol). 
       syntax_checkE(List,Symbol). 

       syntax_checkABC(List,Symbol).
       syntax_checkABCE(List,Symbol). 
       syntax_checkBCE(List,Symbol). 

       check_ene_yne(List,Symbol). 
       check_yne(List,Symbol).

       check_en_ynyl(List,Symbol).
       check_ynyl(List,Symbol).     



CLAUSES
                        
           check_ene_yne([Tok|T],no_error):-
                     frontstr(3,Tok,Fs,Bs),suffix(Fs,N),N>1,
                     check(Bs),check_list(T),!.
          
           check_ene_yne([Tok|_],error):-
                     frontstr(3,Tok,Fs,Bs),suffix(Fs,N),N>1,
                     check(Bs),error(ea,"Nothing",Tok),!.

           check_ene_yne([Tok|_],error):-
                     frontstr(3,Tok,Fs,_),suffix(Fs,N),N>1,
                     error(ea,"Nothing",Fs),!.


               check_yne([Tok|T],no_error):-
                     match(yne,Tok,Bs),
                     check(Bs),check_list(T),!.
           
               check_yne([Tok|_],error):-
                     match(yne,Tok,Bs),
                     check(Bs),error(ea,"Nothing",Tok),!.
 
               check_yne([Tok|_],error):-
                     match(yne,Tok,_),
                     error(ea,"Nothing",yne),!.
                 
               check_ynyl([Ynyl|_],error):-
                     match(ynyl,Ynyl,Nl),
                     not(check(Nl)),
                     error(ei,")",Nl),!.
               
                check_ynyl([Ynyl,B|_],error):-
                      match(ynyl,Ynyl,_),
                      not(equal(")",B)),
                      error(ei,")",B),!.
                      
                check_ynyl([Ynyl,")"|L],Reply):-
                      match(ynyl,Ynyl,_),
                      syntax_checkBCE(L,Reply),!.
                        
             
             check_en_ynyl([EnYnyl|_],error):-
                      suffix_yl(Sfx,Sn),Sn>1,
                      match(Sfx,EnYnyl,Nl),
                      not(check(Nl)),
                      error(ei,")",Nl),!.
             
             check_en_ynyl([EnYnyl,B|_],error):-
                      suffix_yl(Sfx,Sn),Sn>1,
                      match(Sfx,EnYnyl,_),
                      not(equal(")",B)),
                      error(ei,")",B),!.
                                             
             check_en_ynyl([EnYnyl,")"|L],Reply):-
                      suffix_yl(Sfx,Sn),Sn>1,
                      match(Sfx,EnYnyl,_),
                      syntax_checkBCE(L,Reply),!.
                       
             
             check_en_ynyl([En|_],error):-
                      not(match(en,En,_)),
                      error(mso,"enyl/en/ynyl",""),
                      error(gm," NAME IS NOT COMPLETE",""),!.
                              
             check_en_ynyl([En|_],error):-
                      match(en,En,Nl),
                      not(check(Nl)),
                      error(uea,Nl,en),!.
 
             check_en_ynyl([en,Num|_],error):-
                      not(check_int(Num)),
                      error(ea,"Numeral(s)",en),!.
 
             check_en_ynyl([en,Num|R],Reply):-
                      check_int(Num),
                      sieve_int([Num|R],Intstr,Ynyl),
                      scanx(Intstr,IntL),filter(IntL,IL),
                      list_len(IL,L),L=1,
                      check_ynyl(Ynyl,Reply),!.
 
             check_en_ynyl([en,Num|R],error):-
                      check_int(Num),sieve_int([Num|R],_,[Ynyl|_]),
                      check_int(Ynyl),
                      error(gm,"NAME IS NOT COMPLETE",""),!.
                  
             check_en_ynyl([en,Num|R],error):-
                      check_int(Num),sieve_int([Num|R],Intstr,[Ynyl|_]),
                      scanx(Intstr,IntL),filter(IntL,IL),
                      list_len(IL,L),L=1,
                      error(ei,ynyl,Ynyl),!.

             check_en_ynyl([en,Num|R],Reply):-
                      check_int(Num),
                      sieve_int([Num|R],Intstr,[PxYnyl|Alk]),
                      scanx(Intstr,IntL),filter(IntL,IL),
                      list_len(IL,L),L>1,
                      prefix(Px,_),match(Px,PxYnyl,Ynyl),
                      check_ynyl([Ynyl|Alk],Reply),!.

             check_en_ynyl([en,Num|R],error):-
                      check_int(Num),sieve_int([Num|R],Intstr,[Ynyl|_]),
                      scanx(Intstr,IntL),filter(IntL,IL),
                      list_len(IL,L),L>1,
                      error(ei,"prefix di,tri,etc. [ upto deca ]",Ynyl),!.
            

                    
               syntax_checkA([Tok|T],Reply):-
                      check(Tok),syntax_checkA(T,Reply),!.
                       
               syntax_checkA([Tok|T],no_error):-
                      stem(Nm,_),match(Nm,Tok,Bstr),
                      match(ane,Bstr,Bs),
                      check(Bs),check_list(T),!.

               syntax_checkA([Tok|_],error):-
                      stem(Nm,_),match(Nm,Tok,Bstr),
                      match(ane,Bstr,Bs),
                      check(Bs),error(ea,"Nothing",Tok),!.

               syntax_checkA([Tok|_],error):-
                      stem(Nm,_),match(Nm,Tok,Bstr),
                      match(ane,Bstr,_),
                      error(ea,"Nothing",ane),!.

       syntax_checkAx([Tok|T],Reply):-
              check(Tok),syntax_checkAx(T,Reply),!.
                                                                                           
       syntax_checkAx([H|T],no_error):-
              prefix1(Px,Np),match(Px,H,Tok),
              stem(Nm,N),match(Nm,Tok,Bstr),
              Np11=Np+1,N>Np11,
              match(ane,Bstr,Bs),
              check(Bs),check_list(T),!.

  
       syntax_checkAx([H|_],error):-
              prefix1(Px,Np),match(Px,H,Tok),
              stem(Nm,N),match(Nm,Tok,Bstr),
              Np11=Np+1,N>Np11,
              match(ane,Bstr,Bs),
              check(Bs),error(ea,"Nothing",Tok),!.

       syntax_checkAx([H|_],error):-
              prefix1(Px,Np),match(Px,H,Tok),
              stem(Nm,N),match(Nm,Tok,Bstr),
              Np11=Np+1,N>Np11,
              match(ane,Bstr,_),
              error(ea,"Nothing",ane),!.
                                                                            
       syntax_checkAx([H|_],error):-
              prefix1(Px,Np),match(Px,H,Tok),
              stem(Nm,N),match(Nm,Tok,_),
              Np11=Np+1,N>Np11,
              error(eao,"ane",Nm),error(sp,"Numeral",Px),!.

       syntax_checkAx([H|_],error):-
              prefix1(Px,Np),match(Px,H,Tok),
              stem(Nm,N),match(Nm,Tok,_),
              Np11=Np+1,N=Np11,
              error(sp,"Numeral",Px),error(ea,"yl/enyl",Nm),!.

       syntax_checkAx([H|_],error):-
              prefix1(Px,Np),match(Px,H,Tok),
              stem(Nm,N),match(Nm,Tok,_),
              N<=Np,
              error(uea,Nm,Px),!.

       syntax_checkAx([H|_],error):-
              prefix1(Px,_),match(Px,H,R),
              error(spo,"Numerals",Px),
              error(mso,R,""),error(out,"",""),!.

     
        syntax_checkB([Tok|T],Reply):-
               check(Tok),syntax_checkB(T,Reply),!.
                      
        syntax_checkB([Tok|T],Reply):-
               stem(Nm,N),match(Nm,Tok,Bs),
               1<N,N<4,
               check_ene_yne([Bs|T],Reply),!.
              
        syntax_checkB([Tok|_],error):-
               stem(Nm,N),match(Nm,Tok,_),
               1<N,
               error(sp,"NUMERAL",Nm),
               error(eao,"AND  ene/yne/yl SUFFIX",Nm),
               error(ea,"\nane",Nm),!.
 
        syntax_checkB([Tok|_],error):-
               match(meth,Tok,_),
               error(sp ,"NUMERAL",meth),
               error(eao,"and   yl",meth),
               error(ea,ane,meth),!.
              

      syntax_checkC([Tok|T],Reply):-
             check(Tok),syntax_checkC(T,Reply),!.
     
      syntax_checkC([Tok|T],Reply):-
             check_int(Tok),sieve_int([Tok|T],_,[Tok2|Rlist]),
             stem(Nm,N),match(Nm,Tok2,Bs),
             1<N,
             check_ene_yne([Bs|Rlist],Reply),!.

     
      syntax_checkC([Tok|T],Reply):-
             check_int(Tok),sieve_int([Tok|T],_,[Tok2|Rlist]),
             stem(Nm,N),match(Nm,Tok2,Bs),
             1<N,
             prefix2(Px,_),match(Px,Bs,Bbs),
             check_ene_yne([Bbs|Rlist],Reply),!.
                   
                  
             
      syntax_checkC([Tok|T],Reply):-
             check_int(Tok),sieve_int([Tok|T],_,[Tok2,Tok3|Rlist]),
             stem(Nm,N), match(Nm,Tok2,Bs),
             1<N,
             equal(Bs,en),
             check_int(Tok3),sieve_int([Tok3|Rlist],_,Rest),
             check_yne(Rest,Reply),!.
             
      syntax_checkC([Tok|T],Reply):-
             check_int(Tok),sieve_int([Tok|T],_,[Tok2,Tok3|Rlist]),
             stem(Nm,N), match(Nm,Tok2,Bs),
             1<N,
             prefix2(Px,_),match(Px,Bs,Bbs), equal(Bbs,en),
             check_int(Tok3),sieve_int([Tok3|Rlist],_,Rest),
             check_yne(Rest,Reply),!.

      syntax_checkC([Tok|T],error):-
             check_int(Tok),sieve_int([Tok|T],_,[Tok2,Tok3|Rlist]),
             stem(Nm,N),match(Nm,Tok2,Bs),
             1<N,
             equal(Bs,en),
             check_int(Tok3),sieve_int([Tok3|Rlist],_,[Tok4|_]),
             check_int(Tok4),
             error(gm," NAME IS NOT COMPLETE",""),!.

      syntax_checkC([Tok|T],error):-
             check_int(Tok),sieve_int([Tok|T],_,[Tok2,Tok3|Rlist]),
             stem(Nm,N),match(Nm,Tok2,Bs),
             1<N,
             prefix2(Px,_),match(Px,Bs,Bbs),equal(Bbs,en),
             check_int(Tok3),sieve_int([Tok3|Rlist],_,[Tok4|_]),
             check_int(Tok4),
             error(gm," NAME IS NOT COMPLETE",""),!.

      syntax_checkC([Tok|T],Reply):-
             check_int(Tok),sieve_int([Tok|T],_,[Tok2,Tok3|Rlist]),
             stem(Nm,N),match(Nm,Tok2,Bs),
             1<N,
             equal(Bs,en),
             check_int(Tok3),sieve_int([Tok3|Rlist],_,[Tok4|Rest]),
             prefix(P,_),match(P,Tok4,Btok4),
             check_yne([Btok4|Rest],Reply),!.
             
      syntax_checkC([Tok|T],Reply):-
             check_int(Tok),sieve_int([Tok|T],_,[Tok2,Tok3|Rlist]),
             stem(Nm,N),match(Nm,Tok2,Bs),
             1<N,
             prefix2(Px,_),match(Px,Bs,Bbs),equal(Bbs,en),
             check_int(Tok3),sieve_int([Tok3|Rlist],_,[Tok4|Rest]),
             prefix(P,_),match(P,Tok4,Btok4),
             check_yne([Btok4|Rest],Reply),!.

      syntax_checkC([Tok|T],error):-
             check_int(Tok),sieve_int([Tok|T],_,[Tok2,Tok3|_]),
             stem(Nm,N),match(Nm,Tok2,Bs),
             1<N,
             equal(Bs,en),
             not(check_int(Tok3)),
             error(ea,"Numeral(s)",Tok2),!.

      syntax_checkC([Tok|T],error):-
             check_int(Tok),sieve_int([Tok|T],_,[Tok2,Tok3|_]),
             stem(Nm,N),match(Nm,Tok2,Bs),
             1<N,
             prefix2(Px,_),match(Px,Bs,Bbs),equal(Bbs,en),
             not(check_int(Tok3)),
             error(ea,"Numeral(s)",Tok2),!.

      syntax_checkC([Tok|T],error):-
             check_int(Tok),sieve_int([Tok|T],_,[Tok2,Tok3|_]),
             stem(Nm,N),match(Nm,Tok2,Bs),
             1<N,
             equal(Bs,en),
             check_int(Tok3),
             error(ea," prefix/yne ","Numeral(s)"),!.

      syntax_checkC([Tok|T],error):-
             check_int(Tok),sieve_int([Tok|T],_,[Tok2,Tok3|_]),
             stem(Nm,N),match(Nm,Tok2,Bs),
             1<N,
             prefix2(Px,_),match(Px,Bs,Bbs),equal(Bbs,en),
             check_int(Tok3),
             error(ea," prefix/yne ","Numeral(s)"),!.


      syntax_checkD([Tok|T],Reply):-
             check(Tok),syntax_checkD(T,Reply),!.
     

      syntax_checkD([HA|T],Reply):-
             halogen(Hn,_),match(Hn,HA,A),
             syntax_checkABC([A|T],Reply),!.
             
      syntax_checkD([HA|_],error):-
             halogen(Hn,_),match(Hn,HA,A),
             error(spo,"Numerals/Prefix",Hn),error(mso,A,""),
             error(out,"anes/enes/ynes",""),!.
             
             
      syntax_checkD([PHA|T],Reply):-
             prefix(Px,_),match(Px,PHA,HA),
             halogen(Hn,_),match(Hn,HA,A),
             syntax_checkABC([A|T],Reply),!.

      syntax_checkD([PHA|_],error):-
             prefix(Px,_),match(Px,PHA,HA),
             halogen(Hn,_),match(Hn,HA,A),
             error(spo,"Numerals",Px),error(mso,A,""),
             error(out,"anes/enes/ynes",""),!.

      syntax_checkD([PHA|_],error):-
             prefix(Px,_),match(Px,PHA,_),
             error(eao,"Halide",Px),
             error(sp,"Numeral(s)",Px),!.

  
     syntax_checkE([Tok|T],Reply):-
            check(Tok),syntax_checkE(T,Reply),!.
                             
      syntax_checkE([Num|T],Reply):-
             check_int(Num),sieve_int([Num|T],_,[HA|R]),
             halogen(Hn,_),match(Hn,HA,A),
             syntax_checkABCE([A|R],Reply),!.

      syntax_checkE([Num|T],Reply):-
             check_int(Num),sieve_int([Num|T],_,[PxHA|R]),
             prefix(Px,_),match(Px,PxHA,HA),
             halogen(Hn,_),match(Hn,HA,A),
             syntax_checkABCE([A|R],Reply),!.
      
      syntax_checkE([Num|T],Reply):-
             check_int(Num),sieve_int([Num|T],_,[AlkylA|R]),
             stem(Alk,_),match(Alk,AlkylA,YlA),
             match(yl,YlA,A),
             syntax_checkABCE([A|R],Reply),!.

    syntax_checkE([Num|T],Reply):-
             check_int(Num),sieve_int([Num|T],_,[PxAlkylA|R]),
             prefix(Px,_),match(Px,PxAlkylA,AlkylA),
             stem(Alk,_),match(Alk,AlkylA,YlA),
             match(yl,YlA,A),
             syntax_checkABCE([A|R],Reply),!.

      syntax_checkE([Num|T],Reply):-
             check_int(Num),sieve_int([Num|T],_,[AlkylA|R]),
             stem(Alk,N),match(Alk,AlkylA,YlA),
             1<N,N<4,
             suffix_yl(Sfx,Sx),Sx>1,
             match(Sfx,YlA,A),
             syntax_checkBCE([A|R],Reply),!.

      syntax_checkE([Num|T],Reply):-
             check_int(Num),sieve_int([Num|T],_,[PxAlkylA|R]),
             prefix(Px,_),match(Px,PxAlkylA,AlkylA),
             stem(Alk,N),match(Alk,AlkylA,YlA),
             1<N,N<4,
             suffix_yl(Sfx,Sx),Sx>1,
             match(Sfx,YlA,A),
             syntax_checkBCE([A|R],Reply),!.

      syntax_checkE([Num|T],error):-
             check_int(Num),sieve_int([Num|T],_,[PxAlkylA|_]),
             prefix(Px,_),match(Px,PxAlkylA,AlkylA),
             stem(Alk,N),match(Alk,AlkylA,_),
             1<N,N<4,
              error(ea,"yl",Alk),!.

      syntax_checkE([Num|T],error):-
             check_int(Num),sieve_int([Num|T],_,[AlkylA|_]),
             stem(Alk,N),match(Alk,AlkylA,_),
             1<N,N<4,
             error(ea," ene/yne/yl  suffix ",Alk),!.


     syntax_checkE([Num|T],error):-
             check_int(Num),sieve_int([Num|T],_,[PxAlkylA|_]),
             prefix(Px,_),match(Px,PxAlkylA,AlkylA),
             stem(Alk,N),match(Alk,AlkylA,_),N>3,
             error(ea,yl,Alk),!.

     syntax_checkE([Num|T],error):-
             check_int(Num),sieve_int([Num|T],_,[AlkylA|_]),
             stem(Alk,N),match(Alk,AlkylA,_),N>3,
             error(eao,yl,Alk),
             error(ea," ene/yne suffix ",Alk),!.

     syntax_checkE([Num|T],error):-
             check_int(Num),sieve_int([Num|T],_,[PxAlkylA|_]),
             prefix(Px,_),match(Px,PxAlkylA,AlkylA),
             stem(Alk,_),match(Alk,AlkylA,_),
             error(ea,yl,Alk),!.

     syntax_checkE([Num|T],error):-
             check_int(Num),sieve_int([Num|T],_,[AlkylA|_]),
             stem(Alk,_),match(Alk,AlkylA,YlA),
             error(ea,yl,Alk),
             error(uea,YlA,Alk),!.



      syntax_checkE([Num|T],Reply):-
             check_int(Num),sieve_int([Num|T],_,[SpclYlA|R]),
             sp_stem(Spcl,_),match(Spcl,SpclYlA,YlA),
             match(yl,YlA,A),
             syntax_checkBCE([A|R],Reply),!.

      syntax_checkE([Num|T],error):-
             check_int(Num),sieve_int([Num|T],_,[SpclYlA|_]),
             sp_stem(Spcl,_),match(Spcl,SpclYlA,YlA),
             not(match(yl,YlA,_)),
             error(ea,yl,Spcl),!.
                   
      syntax_checkE([Num|T],Reply):-
             check_int(Num),sieve_int([Num|T],_,[PxSpclYlA|R]),
             prefix(Px,_),match(Px,PxSpclYlA,SpclYlA),
             sp_stem(Spcl,_),match(Spcl,SpclYlA,YlA),
             match(yl,YlA,A),
             syntax_checkBCE([A|R],Reply),!.

      syntax_checkE([Num|T],error):-
             check_int(Num),sieve_int([Num|T],_,[PxSpclYlA|_]),
             prefix(Px,_),match(Px,PxSpclYlA,SpclYlA),
             sp_stem(Spcl,_),match(Spcl,SpclYlA,YlA),
             not(match(yl,YlA,_)),
             error(ea,yl,Spcl),!.

      syntax_checkE([Num|T],Reply):-
             check_int(Num),sieve_int([Num|T],_,[PxAlkYlA|R]),
             prefix_yl(Px,P),match(Px,PxAlkYlA,AlkYlA),
             stem(Alk,N),match(Alk,AlkYlA,YlA),
             match(yl,YlA,A),
             N>P,
             syntax_checkABCE([A|R],Reply),!.


      syntax_checkE([Num|T],Reply):-
             check_int(Num),sieve_int([Num|T],_,[PxYlPxAlkYlA|R]),
             prefix(Px,_),match(Px,PxYlPxAlkYlA,YlPxAlkYlA),
             prefix_yl(YlPx,P),match(YlPx,YlPxAlkYlA,AlkYlA),
             stem(Alk,N),match(Alk,AlkYlA,YlA),
             match(yl,YlA,A),
             N>P,
             syntax_checkABCE([A|R],Reply),!.
   

     syntax_checkE([Num|T],error):-
            check_int(Num),sieve_int([Num|T],_,[Bra,Num2|_]),
            match("(",Bra,_),
            not(check_int(Num2)),
            error(ea,"Numeral(s)",Bra),!. 

     syntax_checkE([Num|T],error):-
            check_int(Num),sieve_int([Num|T],_,[Px,Bra,Num2|_]),
            prefix(Px,_),
            match("(",Bra,_),
            not(check_int(Num2)),
            error(ea,"Numeral(s)",Bra),!. 

     syntax_checkE([Num|T],error):-
            check_int(Num),sieve_int([Num|T],_,[Bra,Num2|T2]),
            match("(",Bra,_),
            sieve_int([Num2|T2],_,[Alkynylbra|_]),
            match(meth,Alkynylbra,_),
       error(uea,meth,Bra),
       error(gm,"'BRACKETS' ARE TO BRACE  bi/tri VALENT  ALKYL GROUPS",""),!.
            
     syntax_checkE([Num|T],error):-
            check_int(Num),sieve_int([Num|T],_,[Px,Bra,Num2|T2]),
            prefix(Px,_),
            match("(",Bra,_),
            sieve_int([Num2|T2],_,[Alkynylbra|_]),
            match(meth,Alkynylbra,_),
       error(uea,meth,Bra),
       error(gm,"'BRACKETS' ARE TO BRACE  bi/tri VALENT  ALKYL GROUPS",""),!.
       
            

     syntax_checkE([Num|T],Reply):-
            check_int(Num),sieve_int([Num|T],_,[Bra,Num2|T2]),
            match("(",Bra,_),
            sieve_int([Num2|T2],_,[Alkpx2ynylbra|T3]),
            stem(Alk,_),match(Alk,Alkpx2ynylbra,Px2ynylbra),
            prefix2(Px2,_),match(Px2,Px2ynylbra,Ynylbra),
            check_en_ynyl([Ynylbra|T3],Reply),!.

     syntax_checkE([Num|T],Reply):-
            check_int(Num),sieve_int([Num|T],_,[Px,Bra,Num2|T2]),
            prefix(Px,_),
            match("(",Bra,_),
            sieve_int([Num2|T2],_,[Alkpx2ynylbra|T3]),
            stem(Alk,_),match(Alk,Alkpx2ynylbra,Px2ynylbra),
            prefix2(Px2,_),match(Px2,Px2ynylbra,Ynylbra),
            check_en_ynyl([Ynylbra|T3],Reply),!.

            
            

     syntax_checkE([Num|T],Reply):-
            check_int(Num),sieve_int([Num|T],_,[Bra,Num2|T2]),
            match("(",Bra,_),
            sieve_int([Num2|T2],_,[Alkynylbra|T3]),
            stem(Alk,_),match(Alk,Alkynylbra,Ynlbra),
            check_en_ynyl([Ynlbra|T3],Reply),!.

     syntax_checkE([Num|T],Reply):-
            check_int(Num),sieve_int([Num|T],_,[Px,Bra,Num2|T2]),
            prefix(Px,_),
            match("(",Bra,_),
            sieve_int([Num2|T2],_,[Alkynylbra|T3]),
            stem(Alk,_),match(Alk,Alkynylbra,Ynlbra),
            check_en_ynyl([Ynlbra|T3],Reply),!.

            
/***********************  The rest of Error guessing ***************************/
                   

    
      syntax_checkE([Num|T],error):-
             check_int(Num),
             sieve_int([Num|T],Intstr,[PxRest|_]),
             scanx(Intstr,IntL),filter(IntL,IL),
             list_len(IL,L),L>1,
             prefix(Px,_),match(Px,PxRest,_),
             concat(" 'BEYOND'  ",Px,M),
             error(ms,M,""),!.             
             
      syntax_checkE([Num|T],error):-
             check_int(Num),
             sieve_int([Num|T],Intstr,_),
             scanx(Intstr,IntL),filter(IntL,IL),
             list_len(IL,L),L>1,
             error(ea," prefix di/tri etc. [ upto deca ] ","Numerals"),!.
      
      syntax_checkE([Num|T],error):-
             check_int(Num),
             sieve_int([Num|T],_,[PxRest|_]),
             prefix(Px,_),match(Px,PxRest,_),
             concat(" 'BEYOND'  ",Px,M),
             error(ms,M,""),!.             
             
      syntax_checkE([Num|_],error):-
             check_int(Num),
             error(ms," BEYOND NUMERALS ",""),!.
      
      
      syntax_checkE([n|_],error):-
             error(gm,"n-  PREFIX UN-EXPECTED",""),!.
             
      syntax_checkE([Tok|_],error):-
             frontstr(1,Tok,Fr,Back),equal(Fr,n),
             stem(Nm,_),match(Nm,Back,_),
             error(gm,"n-  PREFIX UN-EXPECTED",""),!.
             
      syntax_checkE([Tok|_],error):-
              error(spo," Numeral(s) ",Tok),
              error(mso,Tok,""),
              error(gm ,"MAY BE CYCLIC OR ",""),
              error(gm,"MAY BE GROUPS OTHER THAN ane/ene/yne  OR\n",""),
              error(out,"",""),
              error(gm,"\n\n UNNECESSARY INSERTION OF A SEPARATOR MAY ALSO HAVE CAUSED THE ERROR",""),!.            
                          

      syntax_checkE([],error):-        
             error(gm," NAME IS NOT COMPLETE ",""),!.


             
               syntax_checkABC(L,Reply):-
                      syntax_checkA(L,Reply),!.
             

                   syntax_checkABC(L,Reply):-
                          syntax_checkB(L,Reply),!.

                   syntax_checkABC(L,Reply):-
                          syntax_checkC(L,Reply),!. 
             
                  
                  syntax_checkABCE(List,Reply):-
                         syntax_checkABC(List,Reply),!.
                  
                  syntax_checkABCE(List,Reply):-
                         syntax_checkE(List,Reply),!.
                  
                  syntax_checkBCE(List,Reply):-
                         syntax_checkB(List,Reply),!.
                  
                  syntax_checkBCE(List,Reply):-
                         syntax_checkC(List,Reply),!.

                  syntax_checkBCE(List,Reply):-
                         syntax_checkE(List,Reply),!.
                        
     
     syntax_check([],error):-
            error(gm," NAME IS NOT COMPLETE",""),!.

     syntax_check(List,Reply):-
            syntax_checkA(List,Reply),!.
    
     syntax_check(List,Reply):-
            syntax_checkAx(List,Reply),!.


     syntax_check(List,Reply):-
            syntax_checkB(List,Reply),!. 


     syntax_check(List,Reply):-
            syntax_checkC(List,Reply),!.

     syntax_check(List,Reply):-
            syntax_checkD(List,Reply),!.
           
     syntax_check(List,Reply):-
            syntax_checkE(List,Reply),!.

