/******************************************************************
	   PDCRUNT.H

      Declarations for miscellaneous PDC Prolog runtime functions

		Copyright (c) 1990 Prolog Development Center
*******************************************************************/

#ifndef _SIZE_T
#define _SIZE_T
typedef unsigned size_t;
#endif

#define BOOL char

#define RUN_JmpReturn RUN_JmpReturn1     /* error in manual */


typedef unsigned long STACKMARK;
STACKMARK MEM_MarkGStack(void);
void MEM_ReleaseGStack(STACKMARK);
void MEM_AdjustGStackAlloc(void *ptr,unsigned realsiz);


/* Handling of gstack */
void *MEM_AllocGStack(unsigned size);
void *MEM_AllocNormGStack(unsigned size);
unsigned MEM_GetMaxGStack(void);


/* Handling of HEAP */
void *pascal MEM_AllocHeap(unsigned size);
void pascal MEM_ReleaseHeap(void* ptr,unsigned size);
void * MEM_ReAllocHeap(void *Old, unsigned OldSize, unsigned NewSize);

/* Making a string permanent */
char * MEM_SaveStringGStack(char *);
char * MEM_SaveStringHeap(char *str);

void * MEM_NewCopyHeap(void *ptr,unsigned size);
void * MEM_NewCopyGStack(void *ptr,unsigned size);

/* Handy often used functions */
void MEM_MovMem(void *source,void *dest,unsigned len);
#define MEM_MovMemType(source,dest,len) \
				MEM_MovMem(source,dest,len*sizeof(*source))

void MEM_SetMem(void *dest,unsigned length,unsigned value);
void MEM_SetMemW(void *dest,unsigned length,unsigned value);

long PTR_PtrDiff(void *,void *);

/* Generates a fail */
void RUN_Fail(void);

/* Stack a backtrack point */
int RUN_StackBTrack(void);

/* Cut backtrack points away */
void RUN_Cut(unsigned SaveSP);

/* Stack a trap point */
int RUN_StackTrap(void);

/* Remove trap point */
void RUN_RemoveTrap(unsigned SaveSP);

/* Perform a nondeterministic return */
void RUN_JmpReturn1(void);

/* Semi-standard predicate bpoints(NoOfBPoints) */
void RUN_BPoints(unsigned *);

/* Check for stack overrun */
void RUN_StackCheck(void);

/* Record the instantiation  of a reference variable

   note that in actual use you may need to typecast the actual argument
   as (void **) to avoid a "supicious pointer conversion" warning
*/
void RUN_trail_ptr(void **);

/* Activate an error during execution */
void RUN_Error(unsigned errorno);

/* Make an exit */
void RUN_Exit(unsigned errorno);
void RUN_ExitZero(void);

/* Error code from RUN_error or exit */
extern unsigned near RUN_ExitCode;

/* Activate debugger */
void debug(void);


/* Return current stacktop (SP) */
#ifdef __TURBOC__
#define GETSP  _SP
#else
#define GETSP  MEM_GetSP()
unsigned MEM_GetSP(void);
#endif


/* Check for user interrupt */
void BRK_CheckBreak(void);

/* Enable & disable break; return old status */
BOOL BRK_Enable(BOOL newstatus);

/* Insert string in symbol table */
char *HASH_BinInsert(char *str);

/* Print symbol table */

/* Output in the PDC Prolog library; redirectable */
void IO_WrStr(char *s);
void IO_WrCh(char ch);
void IO_WrIntg(int v);
void IO_WrReal(double r);
void IO_WrRef(long ref);
void cdecl IO_Writef(char *format,...);
unsigned IO_SPrintf(char *ds,char *cs,...);

/* Direct non redirectable screen output to current window */
void SCR_WrCh(char ch);
void SCR_WrStr(char *str);
void SCR_WrChN(char ch,unsigned x);

/* String handling in the PDC Prolog library */
int  STR_StrCmp(char *,char *);
unsigned  STR_StrLen(char *);
unsigned  STR_StrCat(char *Dest,char *Source);
unsigned  STR_StrCat0(char *Dest,char *Source);
unsigned  STR_StrCat0Max(char *Dest,char *Source, unsigned MaxLen);
unsigned  STR_StrCatMax(char *Dest,char *Source,unsigned MaxLen);
unsigned  STR_UCat(char *Dest,unsigned Numb);
void  STR_ToUpper(char *str);
void  STR_ToLower(char *str);
char *  STR_ExtendStr(char *Str,unsigned NewLen);
unsigned  STR_SearchStr(char *Str,char *Srch,BOOL CaseSens);

/* Scanner functions */
int SCAN_Intg(char *str,int *val);
int SCAN_Real(char *str,double *val);
int SCAN_Uns(char *str,unsigned *valp);

