/*************************************************************************
	STRLIST2.PRO
	
	Comparison of the "nice" and "nasty" methods 
	  for concatenating a list of strings

	Defining the constant use_movmem greatly speeds up
	  the "nasty" method -- try it both ways 
	  and note the difference in timings

	Copyright (c) 1986, 1990 by Prolog Development Center	
*************************************************************************/
CONSTANTS 
	use_movmem = 1

GLOBAL PREDICATES
movmem(string,integer,integer,integer) - (i,i,i,i) language C as "_MEM_MovMem"

DOMAINS
char_list = char*
string_list = string* 


PREDICATES
mr_nice_strlist_str(string_list,string)
mr_nasty_strlist_str(string_list,string)
bytes_needed(string_list,integer,integer)
assign(string_list,integer,integer)
assign_string(integer,integer,integer,integer,integer)
chkstor
chktime(integer,integer,integer,integer,integer)

CLAUSES
mr_nice_strlist_str([H|T],S):- mr_nice_strlist_str(T,S1), concat(H,S1,S).
mr_nice_strlist_str([],"").

bytes_needed([H|T],N,S):- str_len(H,L1), N1=N+L1, bytes_needed(T,N1,S).
bytes_needed([],S,S).

mr_nasty_strlist_str(L,S):-
	bytes_needed(L,0,B),
	str_len(S,B),
	ptr_dword(S,Seg,Offs),
	assign(L,Seg,Offs).


ifdef use_movmem

  assign([H|T],Seg,Offs):-
	str_len(H,L),
	movmem(H,Offs,Seg,L),
	N_Offs=Offs+L,
	assign(T,Seg,N_Offs).
  assign([],_,_).

elsedef

  assign([H|T],Seg,Offs):-
	ptr_dword(H,Seg1,Offs1),
	assign_string(Seg1,Offs1,Seg,Offs,N_Offs),
	assign(T,Seg,N_Offs).
  assign([],_,_).

enddef


assign_string(Seg1,Offs1,Seg,Offs,Ret_Offs):-
	membyte(Seg1,Offs1,B), B<>0, !,
	membyte(Seg,Offs,B),
	N_Offs=Offs+1, N_Offs1=Offs1+1,
	assign_string(Seg1,N_Offs1,Seg,N_Offs,Ret_Offs).
assign_string(_,_,_,R,R).

chkstor:-
	storage(_,Heap1,_),
	mr_nasty_strlist_str([
	"The ","Mad ","Professor ","likes ","cutting ","tape,\n",
	"but ","when ","he ","misses ","Sandra ","gets ","Cross\n",
	"and ","Halls ","Audrey.\n"
	],Str1),
	write(Str1),
	storage(_,Heap2,_), Ha=Heap1-Heap2,
	writef("G-stack usage for nasty method: %\n",Ha),
	mr_nice_strlist_str([
	"The ","Mad ","Professor ","likes ","cutting ","tape,\n",
	"but ","when ","he ","misses ","Sandra ","gets ","Cross\n",
	"and ","Halls ","Audrey.\n"
	],Str2),
	write(Str2),
	storage(_,Heap3,_), Hb=Heap2-Heap3,
	writef("G-stack usage for nice method:  %\n",Hb).

chktime(0,T1,T1,T2,T2):- !.
chktime(N,T1,F1,T3,F2):-
	time(H1,M1,S1,C1),
	mr_nasty_strlist_str([
	"The ","Mad ","Professor ","likes ","cutting ","tape,\n",
	"but ","when ","he ","misses ","Sandra ","gets ","Cross\n",
	"and ","Halls ","Audrey.\n"
	],_),
	time(H2,M2,S2,C2), T2=T1+C2-C1+100*(S2-S1+60*(M2-M1+60*(H2-H1))),
	time(H3,M3,S3,C3),
	mr_nice_strlist_str([
	"The ","Mad ","Professor ","likes ","cutting ","tape,\n",
	"but ","when ","he ","misses ","Sandra ","gets ","Cross\n",
	"and ","Halls ","Audrey.\n"
	],_),
	time(H4,M4,S4,C4), T4=T3+C4-C3+100*(S4-S3+60*(M4-M3+60*(H4-H3))),
	NN=N-1,
	chktime(NN,T2,F1,T4,F2).
	
GOAL
	chkstor,
	write("Timing...\n"),
	chktime(2000,0,T1,0,T2),
	writef("Time for nasty method:  %\nTime for nice method: %\n",T1,T2),
	write("Press a key"), readchar(_).
