/*************************************************************************
	FWGC.PRO
	
	Farmer, Wolf, Goat Cabbage Problem: Overlay Demo		

	Copyright (c) 1986, 1990 by Prolog Development Center             
**************************************************************************/

project "overlay"

include "globals.pro"


DOMAINS
  LOC   = east ; west
  STATE = state(LOC,LOC,LOC,LOC)
  PATH  = STATE*

PREDICATES
  go(STATE,STATE)		/* Start of the algorithem		*/
  nondeterm path(STATE,STATE,PATH,PATH)	/* Finds a path from one state to another */
  nondeterm move(STATE,STATE)	/* Transfer the system from one state to another */
  opposite(LOC,LOC)	/* Gives a location on the opposite side	*/
  unsafe(STATE)		/* Determines wether a state is unsafe		*/
  member(STATE,PATH)	/* Checks if the state is already visited	*/
  write_path(PATH)
  write_move(STATE,STATE)

CLAUSES
  fwgc:-makewindow(1,7,7,"Solutions",0,0,25,80),
	go(state(east,east,east,east),state(west,west,west,west)),
	write("Press any key"),
	readchar(_),
	removewindow.

  go(S,G):-
  	path(S,G,[S],L),
  	nl,write("A solution is:"),nl,
  	write_path(L),fail.
  go(_,_).

  path(G,G,L,L).	/* The final state is reached  */
  path(S,G,L,L1):-	move(S,S1),
			not( unsafe(S1) ),
			not( member(S1,L) ),
			path( S1,G,[S1|L],L1).

  move(state(X,X,G,C),state(Y,Y,G,C)):-opposite(X,Y). /* FARMER + WOLF    */
  move(state(X,W,X,C),state(Y,W,Y,C)):-opposite(X,Y). /* FARMER + GOAT    */
  move(state(X,W,G,X),state(Y,W,G,Y)):-opposite(X,Y). /* FARMER + CABBAGE */
  move(state(X,W,G,C),state(Y,W,G,C)):-opposite(X,Y). /* ONLY FARMER	  */

  opposite(east,west).
  opposite(west,east).

  unsafe( state(F,X,X,_) ):- opposite(F,X),!.  /* The wolf eats the goat    */
  unsafe( state(F,_,X,X) ):- opposite(F,X).  /* The goat eats the cabbage */
  
  member(X,[X|_]):-!.
  member(X,[_|L]):-member(X,L).

  write_path( [H1,H2|T] ) :- write_move(H1,H2),write_path([H2|T]).
  write_path( [] ).
 
  write_move( state(X,W,G,C), state(Y,W,G,C) ) :-!,
    write("The farmer go from ",X," to ",Y),nl.
  write_move( state(X,X,G,C), state(Y,Y,G,C) ) :-!,
    write("The farmer and the wolf go from ",X," to ",Y),nl.
  write_move( state(X,W,X,C), state(Y,W,Y,C) ) :-!,
    write("The farmer and the goat go from ",X," to ",Y),nl.
  write_move( state(X,W,G,X), state(Y,W,G,Y) ) :-!,
    write("The farmer and the cabbage go from ",X," to ",Y),nl.
