/*****************************************************************
	P2CSCAN.C 

	C-based Scanner for use with PRO2C

	Copyright (c) 1986, 1990 by Prolog Development Center
*****************************************************************/

/****************************************************************
			S C A N N E R

  The C procedure scanner_0(STRING,TOKL) converts a string to a list
  of tokens with cursor positions.

  TOK		  = or_();
		    comma();
		    global_();
		    domains_();
		    predicates_();
		    database_();
		    minus();
		    id(STRING);
		    equal();
		    reference_();
		    asterisk();
		    plus();
		    lpar();
		    rpar();
		    determ_();
		    nondeterm_();
		    language_();
		    as_();
		    str(STRING);
		    period();
		    nill

  CURSOR = INTEGER
  CURSORTOK = t(TOK,CURSOR)
  TOKL = CURSORTOK*

****************************************************************/


#include "scandoms.c"
#include <pdcrunt.h>

#define MAXIDLEN 250

char	*txtptr,  	/* Current pointer to input-string */
		*start,  	/* Pointer to start of input-string */
			_ch,  	/* Current character ( = *(txtptr-1)) */
		chclass;  	/* Type of current char (one of onedigit..space below) */
int	pos;  		/* Current position */

extern char CH_ClassTab[];     /* chclass = chclasstab[_ch] */

#define onedigit         0
#define small_let        1
#define cap_let			 2
#define spec_ch          3
#define end_of_file      4
#define other_ch         5
#define nl_or_page       6
#define space            7

#define isdigit(c)      (CH_ClassTab[c]==onedigit)
#define isspace(c)      (CH_ClassTab[c]==space)

/****************************************************************
			error
****************************************************************/

void near error(void)
{ RUN_Exit(1); }


/****************************************************************
			getone
****************************************************************/

void near getone(void)
{ if ( (_ch=*txtptr++) != 0)
    chclass=CH_ClassTab[_ch];
  else {
    chclass=end_of_file;
    txtptr--;
  }
}

/****************************************************************
		getstring
****************************************************************/

#define MAXSTRINGL	64

TOK *getstring(void)
{
  TOK *tok;
  int sleng = 0;
  char string[MAXSTRINGL];
  char *strp=string;

  while ( (_ch = *txtptr++)!=0 && _ch!='"' && _ch!='\n' ) {
     *strp++ = _ch;
     if (sleng++ > MAXSTRINGL) error();
  }
  if (_ch != '"') error();
  *strp = 0;
  getone();
  (tok=(TOK*)MEM_AllocGStack(sizeof(unsigned char)+sizeof(STRING)))->fno=tok_str;
  tok->u.str=(STRING)MEM_SaveStringGStack(string);
  return(tok);
}

/****************************************************************
			str_tok
****************************************************************/

void far pascal str_tok_0(STRING str,TOK **tok)		/* (i,o) flow */
{
  int tokno;

  (*tok=(TOK*)MEM_AllocGStack(sizeof(unsigned char)))->fno=tok_nill;
  for(tokno=0; tokno<MAXTOKS; tokno++)
    if(STR_StrCmp(str,tokens[tokno].strtok)==0) {	/* Reserved word */
/*	if(RUN_StackBtrack()) continue;	*/
	(*tok)->fno=tokens[tokno].fno;
	return;
/*	RUN_JmpReturn();	*/
    }
/*  RUN_Fail();	*/
}

void far pascal str_tok_1(STRING *str,TOK **tok)	/* (o,o) flow */
{
  int tokno;

  for(tokno=0; tokno<MAXTOKS; tokno++) {
    if(RUN_StackBTrack())
      continue;
    *str=MEM_SaveStringGStack(tokens[tokno].strtok);
    (*tok=(TOK*)MEM_AllocGStack(sizeof(unsigned char)))->fno=tokens[tokno].fno;
    RUN_JmpReturn1();
  }
  RUN_Fail();
}

/*
void far pascal str_tok_2(STRING *str,TOK *tok)		/* (o,i) flow */
{
  *str=MEM_SaveStringGStack(tokens[tok->fno-1].strtok);
}
*/

/****************************************************************
			char_seq
****************************************************************/

TOK *char_seq(void)
{
  char buf[MAXIDLEN];
  char *bufp=buf;
  TOK *tok;
  int nleng=0;

  while(nleng++ < MAXIDLEN) {
#ifdef case_sensitive
      *bufp++ = _ch;
#else
      *bufp++ = _ch >='A' && _ch <= 'Z' ? _ch + 'a' - 'A' : _ch;
#endif
      if ((chclass=CH_ClassTab[_ch=*txtptr++]) > cap_let && _ch!='_') break;
  }
  if (nleng > MAXIDLEN) error();
  *bufp = 0;
  str_tok_0(buf,&tok);
  if(tok->fno==tok_nill) { /* Unrecognized alphabetic, must be id */
    (tok=MEM_AllocGStack(sizeof(unsigned char)+sizeof(STRING)))->fno=tok_id;
    tok->u.id=(STRING)MEM_SaveStringGStack(buf);
  }
  return(tok);
}

/****************************************************************
			gettoken
****************************************************************/

TOK *gettoken(void)
{
TOK *tok;
int tokkind;
/* char tokstr[2]; If str_tok_0 is to be used */

ANOTHER :
pos=(int)(txtptr-start);
switch(chclass)
  {
  case space :
  case nl_or_page :
	for(;;)
	  { getone();
	    if(!isspace(_ch)) goto ANOTHER;
	  }

  case small_let:
  case cap_let :
	id :
	return(char_seq());

  case end_of_file : return((TOK*)0);

  default:
    switch(_ch) {
     case '/' :	getone();
		if ( _ch == '*' )
		  { int comcount = 1;
		    for (;;)
		     { if (!*txtptr) error();
		       if ((*txtptr)=='*')
		         if ((*++txtptr)=='/')
		           { txtptr++;
		             if (--comcount) continue;
		             getone(); goto ANOTHER;
		           }
		         else continue;

		       if ((*txtptr)=='/') {
		         if ((*++txtptr)=='*') { txtptr++; comcount++; }
		       }
		       else txtptr++;
		    }
		  }
		else {
		  (tok=(TOK*)MEM_AllocGStack(sizeof(unsigned char)))->fno=tok_nill;
		  return(tok);
		}

     case '*': tokkind=tok_asterisk; break;
     case ';': tokkind=tok_or_; break;
     case '=': tokkind=tok_equal; break;
     case ',': tokkind=tok_comma; break;
     case '(': tokkind=tok_lpar; break;
     case ')': tokkind=tok_rpar; break;
     case '-': tokkind=tok_minus; break;
     case '+': tokkind=tok_plus; break;
     case '.': tokkind=tok_period; break;
     case '"' : return(getstring());
     case '_': goto id;
     case '%':
	while ( (_ch=*txtptr++)!=0 && _ch!='\n' ) ;
	if(_ch) getone();
	chclass=CH_ClassTab[_ch];
	goto ANOTHER;
     default :
/*  Alternative method of decoding symb chars:
	*tokstr=_ch;
	tokstr[1]=0;
	str_tok_0(tokstr,&tok);
	getone();
	return(tok);
*/
	tokkind=tok_nill; break;
     }
  }
(tok=(TOK*)MEM_AllocGStack(sizeof(unsigned char)))->fno=tokkind;
getone();
return(tok);
}


/****************************************************************
		Scanner
****************************************************************/

void far scanner_0(STRING string,TOKL** tokl)
{
  TOKL **curtokl=tokl, *toklist;
  CURSORTOK *ctok;
  TOK *tok;

  txtptr=start=string;
  getone();
  for(;;)
    { tok=gettoken();
      if(tok)
        { toklist=(TOKL*)MEM_AllocGStack(sizeof(TOKL));
          ctok=(CURSORTOK*)MEM_AllocGStack(sizeof(CURSORTOK));
          ctok->fno=cursortok_t;
          ctok->cursor=pos;
          ctok->tok=tok;
          toklist->fno=LISTFNO;
          toklist->cursortok=ctok;
          *curtokl=toklist;
          curtokl=&toklist->next;
        }
      else
        { toklist=(TOKL*)MEM_AllocGStack(sizeof(unsigned char));
          toklist->fno=NILLFNO;
          *curtokl=toklist;
          return;
        }
    }
}
