/***********************************************************************
	PEXPERT.PRO


	PROSPECTOR-ISH EXPERT SYSTEM SHELL
		ANDREW JONES 

	Copyright (c) 1986, 90 by Prolog Development Center
***********************************************************************/
nowarnings
code=3000

domains
  KEY    = cr ; esc ; break ; tab ; btab ; del ; bdel ; ins ;
  	    end ; home ; ftast(INTEGER) ; up ; down ; left ; right ;
  	    tegn(CHAR) ; otherspec

domains
	valueinfo = sym(symbol); num(real)
	symbollist = symbol*
	stringlist = string*
	contextentry = context(symbol,real,real)
	continfo = contextentry*
	bayescontrib = contrib(symbol,real,real)
	bayeslist = bayescontrib*
	numpair = nummap(real,real)
/* nummap(value,prob) */
	nummaplist = numpair*
	sympair = symmap(symbol,real)
/* symmap(symbolval,prob) */
	symmaplist = sympair*
	bodyinfo = not_rule(symbol);
and_rule(symbollist); or_rule(symbollist); bayes_rule(bayeslist);
cert_rule; yesno_rule; sym_value_rule(symbollist); 
num_value_rule(real,real); sym_map_rule(symbol,symmaplist); 
num_map_rule(symbol,nummaplist)
	rule = cl(symbol,symbol,real,bodyinfo,string,string,continfo)

database
	val(symbol,symbol,valueinfo)
	level(real)
	context(symbol,symbol)
	rule_met(symbol)
	cl(symbol,symbol,real,bodyinfo,string,string,continfo)
	failed
	endconsulting
	reevaluate
	
predicates
	space(integer)
	frontstrtrail(integer,string,string)
	frontstrtrail(integer,symbol,symbol)
	delay(integer)
	expert
	initlevel
	dofunction(symbol)
	evaluate
	retractforreevaluate
	retractrundata
	reportevaluating(symbol,symbol,symbol)
	choosehypothesis(symbol)
	gethypothesis(symbol)
	hypchoice(symbol,symbol)
	answer_menu
	modify_answers(symbollist)
	makeansmenu(symbollist,symbollist)
	modify_answer(symbol)
	mustreevaluate
	display_histogram
	display_clauses
/* Last argument (Mode) can be one of: concluding,running,asking */
	eval(symbol,symbol,symbol,real,valueinfo,symbollist,symbol)
	evalbody(bodyinfo,string,real,valueinfo,symbollist,symbol)
	combinebayes(real,real,bayescontrib,real,real)
	bayesfactor(real,real,real,real,real)
	calcsymrule(bodyinfo,real,valueinfo,real)
	calcnumrule(bodyinfo,real,valueinfo,real)
	evalcontext1(symbol,continfo,real,symbol,symbollist,symbol)
	modifykb
	retractkb
	repeat
	member(symbol,symbollist)
	member(string,stringlist)
	defaultval(bodyinfo,real,valueinfo)
	checkok
	writeconc(symbol,string,valueinfo,real,symbol)
	compatible(symbol,bodyinfo)
	getval(symbol,valueinfo,symbollist,symbol)
	evalcontext(symbol,continfo,symbol,symbollist,symbol)
	valuestring(symbol,string)
	contextstring(symbol,string)
	displayrulesmet
	displayrule(symbol)
	displayruledef(symbol)
	probtocert(real,real,real)
	certtoprob(real,real,real)
	probtoodds(real,real)
	oddstoprob(real,real)
	min(real,real,real)
	max(real,real,real)
	parsefile(string)
	parsefile1(string)
	askuser(bodyinfo,string,valueinfo,symbollist,symbol)
	askuser1(bodyinfo,string,string,valueinfo,symbollist,symbol)
	runtimemenu(symbollist)
	runtimemenu1(symbol,symbollist)
	displayancestors(symbollist)

include "mymenu.pro"

predicates
	menuchoicesubst(stringlist,stringlist,string,string)
	menuchoicesubst(symbollist,symbollist,symbol,symbol)
	menuchoice(stringlist,string,string)
	menuchoice(symbollist,symbol,symbol)
	listel(integer,stringlist,string)
	listel(integer,symbollist,symbol)

goal
	expert.

clauses

menuchoice(L,_,S):-
	menu(1,79,L,N),
	listel(N,L,S1),!,S=S1.

menuchoice(_,Default,Default).

/* The following clauses are for a choice made from a displayed list,
   which is then used to select a corresponding choice from the ChoiceList.
   This allows you to have a different list displayed from the list whence
   the choice is actually returned. */

menuchoicesubst(DisplList,ChoiceList,_,Choice):-
	menu(1,79,DisplList,N),
	listel(N,ChoiceList,Ch1),!,Choice=Ch1.

menuchoicesubst(_,_,Default,Default).

listel(1,[H|_],H):-!.

listel(N,[_|T],S):-
	M=N-1,
	listel(M,T,S).

clauses

space(0):-!.

space(N):-
	write(' '),N1=N-1,space(N1).

frontstrtrail(Size,S,NewStr):-
	frontstr(Size,S,NewStr,_),!.
frontstrtrail(Size,S,NewStr):-
	concat(S,"                    ",S1),
	frontstrtrail(Size,S1,NewStr).

delay(0):-!.
delay(N):-
	N2=N+1,X=ln(N2),N1=N-1,delay(N1).

repeat.
	
repeat:-repeat.
	
member(X,[X|_]):-!.
	
member(X,[_|T]):-
	member(X,T).

expert:-
	makewindow(1,7,7,"expert system",0,0,25,80),
	initlevel,
	repeat,
	menuchoice(["Choose kb","Query kb","Exit system"],"Other",F),
	dofunction(F).

initlevel:-
	retract(level(_)),fail.		
initlevel:-
	assert(level(2)).

dofunction("Query kb"):-
	evaluate,fail.
	
dofunction("Choose kb"):-
	modifykb,fail.
	
dofunction("Exit system"):-
	menuchoice(["Continue","Really quit"],"Continue","Really quit"),!.

evaluate:-
	retractrundata,
	level(Lev),
	nl,nl,write("REPORTING CONCLUSIONS OF LEVEL ",Lev," AND OVER"),
	choosehypothesis(Heading),
	eval(entry_rule,hypothesis,Heading,Prior,Value,[],running),
	cl(hypothesis,Heading,_,_,Desc,_,_),
	writeconc(Heading,Desc,Value,Prior,running).

checkok:-
	reevaluate,!,
	retractforreevaluate.

checkok:-
	not(failed),
	not(endconsulting).

retractforreevaluate:-
	retract(val("hypothesis",_,_)),fail.
retractforreevaluate:-
	retract(val("intermediate",_,_)),fail.
retractforreevaluate:-
	retract(context(_,_)),fail.
retractforreevaluate:-
	retract(reevaluate),fail.
retractforreevaluate.

retractrundata:-	
	retract(val(_,_,_)),fail.
retractrundata:-	
	retract(context(_,_)),fail.
retractrundata:-	
	retract(rule_met(_)),fail.
retractrundata:-
	retract(failed),fail.
retractrundata:-
	retract(endconsulting),fail.
retractrundata:-
	retract(reevaluate),fail.
retractrundata.

choosehypothesis(_):-
	not(checkok),!,fail.

choosehypothesis(Heading):-
	gethypothesis(Heading).

choosehypothesis(Heading):-
	choosehypothesis(Heading).

/* This menu also will allow you to do things like add info etc. */
gethypothesis(Heading):-
	menuchoice(["Try next hypothesis",
	"Choose hypothesis","Set level",
	"Display conclusions","Summary of conclusions",
	"Edit replies",
	"Quit consultation"],"Esc",Choice),
	hypchoice(Choice,Heading).

hypchoice("Choose hypothesis",Heading):-
	findall(Clname,cl("hypothesis",Clname,_,_,_,_,_),L),
	menuchoice(L,"Esc",Heading),
	Heading <> "Esc",
	!.

hypchoice("Quit consultation",_):-
	assert(endconsulting),!,fail.

hypchoice("Try next hypothesis",Heading):-
	cl("hypothesis",Heading,_,_,_,_,Clist),
	not(val(_,Heading,_)),
	not(evalcontext(Heading,Clist,out,[],concluding)),
	!.

hypchoice("Try next hypothesis",_):-
	nl,nl,write("Nothing left to evaluate"),
	!,fail.

hypchoice("Set level",_):-
	nl,
	askuser(cert_rule,"NEW LEVEL:",num(Lev),[],asking),
	retract(level(_)),
	assert(level(Lev)),!,fail.

hypchoice("Display conclusions",_):-
	cl(hypothesis,Heading,Prior,_,Desc,_,_),
	eval(entry_rule,hypothesis,Heading,Prior,Value,[],concluding),
	writeconc(Heading,Desc,Value,Prior,concluding),
	fail.

hypchoice("Summary of conclusions",_):-
	display_histogram,!,fail.

hypchoice("Edit replies",_):-
	answer_menu,!,fail.

answer_menu:-
	findall(Rule,cl(data,Rule,_,_,_,_,_),RuleList),
	modify_answers(RuleList).

modify_answers(RList):-
	repeat,
	makeansmenu(RList,DList),
	menuchoicesubst(DList,RList,"Esc",Rule),
	modify_answer(Rule),!.

makeansmenu([],[]):-!.
makeansmenu([Rule|RTail],[Displ|DisplTail]):-
	frontstrtrail(20,Rule,RString),
	valuestring(Rule,VString),
	concat(RString," ",RString1),
	concat(RString1,VString,Displ),!,
	makeansmenu(RTail,DisplTail).

modify_answer("Esc"):-!.
modify_answer(Rule):-
	val(_,Rule,Val),!,
	cl(Type,Rule,Prior,RuleStuff,Desc,Help,CList),
	evalbody(RuleStuff,Desc,Prior,NewVal,[],asking),
	retract(val(_,Rule,Val)),
	assert(val("data",Rule,NewVal)),
	mustreevaluate,!,fail.
modify_answer(Rule):-
	cl(Type,Rule,Prior,RuleStuff,Desc,Help,CList),
	evalbody(RuleStuff,Desc,Prior,NewVal,[],asking),
	assert(val("data",Rule,NewVal)),!,fail.

mustreevaluate:-
	reevaluate,!.
mustreevaluate:-
	assert(reevaluate).


display_histogram:-
	makewindow(2,7,7,"Summary - press the space bar to continue",0,0,25,80),
	space(56),write("Certainty"),nl,
	space(44),write(" -5 -4 -3 -2 -1  0  1  2  3  4  5"),nl,
	display_clauses,
	readchar(_),removewindow.

display_clauses:-
	cl(hypothesis,Heading,Prior,_,Desc,_,Clist),
	eval(entry_rule,hypothesis,Heading,Prior,num(Value),[],concluding),
	evalcontext(Heading,Clist,in,[],concluding),
	probtocert(Value,Prior,Cert),
	frontstrtrail(46,Heading,Hd),
	write(Hd),
	Spc = (round(Cert) + 5) * 3,
	space(Spc),write('*'),nl,
	frontstrtrail(41,Desc,Ds),
	write(Ds),nl,
	fail.
	
display_clauses.

modifykb:-
	nl,write("What kb do you wish to consult? "),
	readln(KbName),
	retractrundata,
	retractkb,
 	parsefile(KbName).
		
retractkb:-
	retract(cl(_,_,_,_,_,_,_)),
	fail.
retractkb.

eval(Calltype,Cltype,Clname,Prior,Val,Anceslst,Mode):-
	cl(Cltype,Clname,Prior,Bodinf,_,Expl,Continf),
	compatible(Calltype,Bodinf),
	asserta(rule_met(Clname)),!,
	getval(Clname,Val,Anceslst,Mode).

eval(_,_,Clname,_,_,_,running):-
	nl,nl,write("Rule "),write(Clname),
	write(" not found or wrong type."),nl,
	assert(failed),fail.

/* Just fails if doesn't work nicely for e.g. concluding */

compatible("sym_map_rule",sym_value_rule(_)):-!.
compatible("sym_map_rule",_):-!,fail.
compatible("num_map_rule",num_value_rule(_,_)):-!.
compatible("num_map_rule",_):-!,fail.
compatible(_,num_value_rule(_,_)):-!,fail.
compatible(_,sym_value_rule(_)):-!,fail.
compatible(_,_).
/* Everything else that got through the compiling stage should be OK */

writeconc(Heading,Desc,num(Value),Prior,Mode):-
	context(Heading,in),
	probtocert(Value,Prior,Cert),
	level(Lev),
	Lev <= Cert,!,
	nl,nl,write("Conclusion..."),nl,
	write("Hypothesis ",Heading),nl,
	write(Desc),nl,
	write("Certainty: "),write(Cert).

writeconc(Heading,_,num(Value),Prior,running):-
	context(Heading,in),
	!,
	nl,nl,
	write("Hypothesis ",Heading," does not reach required certainty level").

writeconc(Heading,_,_,_,running):-
	!,context(Heading,out),
	nl,nl,write("Hypothesis ",Heading," is out of context").

writeconc(_,_,_,_,_).
/* This does nothing in order that I can simply list out the conclusions
that are relevant in non-running mode */

getval(Clname,Val,Anceslst,Mode):-
	cl(Cltype,Clname,Prior,Bodinf,_,Expl,Continf),
	context(Clname,out),
	defaultval(Bodinf,Prior,Val),!.

getval(Clname,Val,_,_):-
	context(Clname,in),val(_,Clname,Val),!.

getval(Clname,Val,Anceslst,Mode):-
	cl(Cltype,Clname,Prior,Bodinf,Desc,Expl,Continf),
	reportevaluating(Cltype,Clname,Mode),
	evalcontext(Clname,Continf,out,[Clname|Anceslst],Mode),
	defaultval(Bodinf,Prior,Val),!.

/* This clause is repeated since context will have been asserted in previous
   clause, if successful, and so now we can use val if it exists */
getval(Clname,Val,_,_):-
	context(Clname,in),val(_,Clname,Val),!.

getval(Clname,Val,Anceslst,Mode):-
	cl(Cltype,Clname,Prior,Bodinf,Desc,Expl,Continf),
	context(Clname,in),
	!,
	evalbody(Bodinf,Desc,Prior,Val,[Clname|Anceslst],Mode),
	assert(val(Cltype,Clname,Val)),!.

evalcontext(Clname,Continf,Context,Anceslst,Mode):-
	context(Clname,Co),!,Co=Context.
	
evalcontext(Clname,[],Con,Anceslst,Mode):-
	assert(context(Clname,in)),!,Con=in.
/* You know it's in if you got this far. */

evalcontext(Clname,[context(Cl,Least,Greatest)|T],Con,Anceslst,Mode):-
	eval(context_rule,_,Cl,Prior,num(Val),Anceslst,Mode),
	probtocert(Val,Prior,Cert),
	evalcontext1(Clname,[context(Cl,Least,Greatest)|T],
		Cert,Cont,Anceslst,Mode),
	!,Con=Cont.

evalcontext1(Clname,[context(Cl,Least,Greatest)|T],Cert,Con,Anceslst,Mode):-
	Least <= Cert, Cert <= Greatest,
	!,evalcontext(Clname,T,Con,Anceslst,Mode).

evalcontext1(Clname,_,_,Con,_,_):-
	Con=out,assert(context(Clname,out)).

reportevaluating(hypothesis,Name,Mode):-
	Mode=running,nl,nl,write("evaluating hypothesis ", Name),nl,!.

reportevaluating(_,_,_).

evalbody(not_rule(Clname),Desc,Prior,num(Val),Anceslst,Mode):-
	eval(not_rule,_,Clname,_,num(Val1),Anceslst,Mode),Val=1-Val1.

evalbody(and_rule([]),_,_,num(1),_,_):-!.

evalbody(and_rule([Clname|T]),Desc,Prior,num(Val),Anceslst,Mode):-
	eval(and_rule,_,Clname,_,num(Val1),Anceslst,Mode),
	evalbody(and_rule(T),Desc,Prior,num(Val2),Anceslst,Mode),
	min(Val1,Val2,Val).

evalbody(or_rule([]),_,_,num(0),_,_):-!.

evalbody(or_rule([Clname|T]),Desc,Prior,num(Val),Anceslst,Mode):-
	eval(or_rule,_,Clname,_,num(Val1),Anceslst,Mode),
	evalbody(or_rule(T),Desc,Prior,num(Val2),Anceslst,Mode),
	max(Val1,Val2,Val).

evalbody(bayes_rule([]),_,Prior,num(Prior),_,_):-!.

evalbody(bayes_rule([contrib(Clname,Inc,Dec)|T]),Desc,Prior,num(Val),Anceslst,Mode):-
	eval(bayes_rule,_,Clname,Pri,num(Val1),Anceslst,Mode),
	evalbody(bayes_rule(T),Desc,Prior,num(Val2),Anceslst,Mode),
	combinebayes(Pri,Val1,contrib(Clname,Inc,Dec),Val2,Val).

evalbody(sym_map_rule(Clname,L),Desc,Prior,num(Val),Anceslst,Mode):-
	eval(sym_map_rule,_,Clname,_,Val1,Anceslst,Mode),
	calcsymrule(sym_map_rule(Clname,L),Prior,Val1,Val).

evalbody(num_map_rule(Clname,L),Desc,Prior,num(Val),Anceslst,Mode):-
	eval(num_map_rule,_,Clname,_,Val1,Anceslst,Mode),
	calcnumrule(num_map_rule(Clname,L),Prior,Val1,Val).

evalbody(cert_rule,Prompt,Prior,num(Value),Anceslst,Mode):-
	askuser(cert_rule,Prompt,num(Cert),Anceslst,Mode),
	certtoprob(Cert,Prior,Value).

evalbody(num_value_rule(Lo,Hi),Prompt,Prior,num(Value),Anceslst,Mode):-
	askuser(num_value_rule(Lo,Hi),Prompt,num(Value),Anceslst,Mode).

evalbody(yesno_rule,Prompt,Prior,num(Value),Anceslst,Mode):-
	askuser(yesno_rule,Prompt,num(Cert),Anceslst,Mode),
	certtoprob(Cert,Prior,Value).

evalbody(sym_value_rule(L),Prompt,Prior,sym(Symb),Anceslst,Mode):-
	askuser(sym_value_rule(["don't know"|L]),Prompt,sym(Symb),Anceslst,Mode).

askuser(_,_,_,_,concluding):-!,fail.

askuser(cert_rule,Prompt,num(Val),Anceslst,Mode):-
	nl,write(Prompt,"  "),
	readln(Str),!,
	askuser1(cert_rule,Prompt,Str,num(Val),Anceslst,Mode).

askuser(num_value_rule(Lo,Hi),Prompt,num(Val),Anceslst,Mode):-
	nl,write(Prompt,"  "),
	readln(Str),!,
	askuser1(num_value_rule(Lo,Hi),Prompt,Str,num(Val),Anceslst,Mode).

askuser(yesno_rule,Prompt,num(Val),Anceslst,Mode):-
	nl,write(Prompt),
	menuchoice(["yes","no","don't know"],"Esc",Str),
	Str <> "Esc",!,
	write("  ",Str),nl,
	askuser1(yesno_rule,Prompt,Str,num(Val),Anceslst,Mode).

askuser(sym_value_rule(L),Prompt,sym(Symb),Anceslst,Mode):-
	nl,write(Prompt),
	menuchoice(L,"Esc",Symb),
	Symb <> "Esc",!,
	write("  ",Symb),nl.

askuser(Type,Prompt,Val,Anceslst,running):-
	runtimemenu(Anceslst),
	!,
	askuser(Type,Prompt,Val,Anceslst,running).

askuser1(cert_rule,Prompt,Str,num(Val),Anceslst,Mode):-
	str_real(Str,Val),-5<=Val,Val<=5,!.

askuser1(cert_rule,Prompt,Str,num(Val),Anceslst,Mode):-
	nl,write("Input a real number in the range -5 to +5!"),
	askuser(cert_rule,Prompt,num(Val),Anceslst,Mode).

askuser1(num_value_rule(Lo,Hi),Prompt,Str,num(Val),Anceslst,Mode):-
	str_real(Str,Val),Lo<=Val,Val<=Hi,!.

askuser1(num_value_rule(Lo,Hi),Prompt,Str,num(Val),Anceslst,Mode):-
	nl,write("Input a real number in the range ",Lo," to ",Hi," !"),
	askuser(num_value_rule(Lo,Hi),Prompt,num(Val),Anceslst,Mode).

askuser1(yesno_rule,Prompt,"no",num(-5),Anceslst,Mode):-!.
askuser1(yesno_rule,Prompt,"yes",num(5),Anceslst,Mode):-!.
askuser1(yesno_rule,Prompt,"don't know",num(0),Anceslst,Mode):-!.


runtimemenu(Anceslst):-
	menuchoice(["Help","Rules met",
		"Backtrace","Continue","Quit"],"Continue",Choice),
	runtimemenu1(Choice,Anceslst).

runtimemenu1("Continue",_):-!.
runtimemenu1("Quit",_):-!,fail.
runtimemenu1("Help",[Cl|_]):-
	cl(_,Cl,_,_,_,Help,_),
	!,nl,nl,
	write(Help),
	nl.
runtimemenu1("Rules met",_):-
	nl,!,
	makewindow(2,7,7,"Rules starting at most recent - press space bar to terminate",0,0,25,80),
	displayrulesmet,
	nl,write("PRESS SPACE BAR TO RETURN TO CONSULTATION"),
	readchar(X),removewindow.
runtimemenu1("Backtrace",Alist):-
	nl,
	makewindow(2,7,7,"Current rule and ancestors - press space bar to terminate",0,0,25,80),
	displayancestors(Alist),
	nl,write("PRESS SPACE BAR TO RETURN TO CONSULTATION"),
	readchar(X),removewindow.

displayancestors([]):-nl,!.
displayancestors([H|T]):-
	displayruledef(H),delay(20),not(keypressed),displayancestors(T).
displayancestors(_):-readchar(X).

defaultval(sym_value_rule(_),_,sym("don't know")):-!.
defaultval(num_value_rule(_,_),_,sym("don't know")):-!.
defaultval(Bodinf,Prior,num(Prior)).
/*Less of a fiddle*/

combinebayes(ClPrior,ClVal,contrib(_,Inc,Dec),RestVal,Ans):-
	probtoodds(RestVal,RestOdds),
	bayesfactor(ClVal,ClPrior,Dec,Inc,F),
	Odds=F*RestOdds,
	oddstoprob(Odds,Ans).

bayesfactor(Prob,Prior,Dec,Inc,F):-
	Prior >= Prob,!,
	probtocert(Prob,Prior,Cert),
	F = ((1 - Dec) * (Cert + 5) / 5) + Dec.
	
bayesfactor(Prob,Prior,Dec,Inc,F):-
	probtocert(Prob,Prior,Cert),
	F = 1 + (Inc-1) * Cert / 5.

certtoprob(Cert,Prior,Value):-
	Cert <= 0, Value = Prior * (Cert + 5) / 5,!;
	Value = Prior + (1 - Prior) * Cert / 5.

calcsymrule(sym_map_rule(_,[]),Prior,_,Prior):-!.
calcsymrule(sym_map_rule(_,[symmap(S,V)|_]),_,sym(S),V):-!.
calcsymrule(sym_map_rule(Clname,[_|T]),Prior,Val,Ans):-
	calcsymrule(sym_map_rule(Clname,T),Prior,Val,Ans).

calcnumrule(_,Prior,sym("don't know"),Prior):-!.
calcnumrule(num_map_rule(_,[nummap(V,P)|_]),_,num(X),P):-
	X<V,!.
calcnumrule(num_map_rule(_,[nummap(_,P)]),_,_,P):-!.
calcnumrule(num_map_rule(_,[nummap(V1,P1),nummap(V2,P2)|_]),_,num(X),P):-
	V1 <= X,X < V2,!,
	P = P1 + (P2 - P1) * (X - V1) / (V2 - V1).
calcnumrule(num_map_rule(Clname,[_|T]),Prior,Val,Ans):-
	calcnumrule(num_map_rule(Clname,T),Prior,Val,Ans).

displayrulesmet:-
	rule_met(Clname),
	displayrule(Clname),
	delay(20),keypressed,!,readchar(X).

displayrulesmet.

displayrule(Clname):-
	displayruledef(Clname),
	valuestring(Clname,Val),
	write(Val),nl,
	contextstring(Clname,Con),
	write(Con),nl.

displayruledef(Clname):-
	cl(Type,Clname,_,_,Desc,_,_),
	!,nl,
	concat(Type," ",Str1),concat(Str1,Clname,Str2),
	frontstrtrail(30,Str2,Str),
	write(Str,"\"",Desc,"\""),nl.

valuestring(Clname,"unevaluated"):-
	cl(_,Clname,_,_,_,_,_),
	not(val(_,Clname,_)),!.

valuestring(Clname,Str):-
	cl(_,Clname,Prior,_,_,_,_),
	val(_,Clname,sym(Str1)),!,
	Str2=Str1,concat("Value ",Str2,Str).

valuestring(Clname,Str):-
	cl(_,Clname,Prior,num_value_rule(_,_),_,_,_),
	val(_,Clname,num(R)),
	!,str_real(Str1,R),
	concat("Value ",Str1,Str).

valuestring(Clname,"Value no"):-
	cl(_,Clname,Prior,yesno_rule,_,_,_),
	val(_,Clname,num(0)),!.

valuestring(Clname,"Value yes"):-
	cl(_,Clname,Prior,yesno_rule,_,_,_),
	val(_,Clname,num(1)),!.

valuestring(Clname,"Value don't know"):-
	cl(_,Clname,Prior,yesno_rule,_,_,_),
	val(_,Clname,num(_)),!.

valuestring(Clname,Str):-
	cl(_,Clname,Prior,_,_,_,_),
	val(_,Clname,num(R)),
	!,probtocert(R,Prior,Cert),
	str_real(Str1,Cert),
	concat("Cert ",Str1,Str).

contextstring(Cl,"context unknown"):-
	not(context(Cl,_)),!.

contextstring(Cl,"in context"):-
	context(Cl,"in"),!.

contextstring(Cl,"out of context"):-
	context(Cl,"out").

probtocert(_,0,-5):-!.
probtocert(_,1,5):-!.
probtocert(Prob,Prior,Cert):-
	Prob <= Prior, Cert = 5 * Prob / Prior - 5,!;
	Cert = 5 * (Prob - Prior) / (1 - Prior).

oddstoprob(Odds,Prob):-
	Prob=Odds/(1+Odds).

probtoodds(1,10000):-!.

probtoodds(Prob,Odds):-
	Odds=Prob/(1-Prob).

min(V1,V2,V1):-
	V1 <= V2,!.

min(_,V2,V2).

max(V1,V2,V1):-
	V1 >= V2,!.

max(_,V2,V2).

/* the parser */
predicates
	wfronttoken(string,symbol,string)
	writediff(string,string)
	parsekb(string,string)
	parsecomment(string,string)
	findrightbrace(string,string)
	parserule(string,string,rule)
	parsehypish(string,string,rule)
	parseheading(string,string,symbol,real,string,string)
	parsestring(string,string,string)
	formparsedstring(string,string,string)
	parsehelpentry(string,string,string)
	parsepriorentry(string,string,real)
	parsenumber(string,string,real)
	parsehypishbody(string,string,bodyinfo)
	parsemap(string,string,symbol,bodyinfo)
	parsenummap(string,string,nummaplist)
	parsesymmap(string,string,symmaplist)
	parsenamelist(string,string,symbollist)
	parsebayeslist(string,string,bayeslist)
	parsedata(string,string,rule)
	parseevbody(string,string,bodyinfo)
	parsevalue(string,string,bodyinfo)
	parsecontextentry(string,string,continfo)
	parsecontextlist(string,string,continfo)	

clauses

wfronttoken(Str,Tok,Rem):-
	fronttoken(Str,Tok,Rem),
	writediff(Str,Rem).

writediff(Str1,Str2):-
	concat(S,Str2,Str1),write(S).

parsefile(Fname):-
	existfile(Fname),!,parsefile1(Fname).

parsefile(Fname):-
	concat(Fname,".kbs",FullFname),
	existfile(FullFname),!,parsefile1(FullFname).

parsefile(Fname):-
	nl,nl,
	write("CAN'T FIND THE FILE!"),
	nl,nl,fail.

parsefile1(Fname):-
	file_str(Fname,Str),
	nl,nl,
	parsekb(Str,Left),writediff(Left,""),!,nl,nl,
	write("COMPILATION SUCCESSFUL"),nl,nl.

parsefile1(_):-
	nl,nl,write("ERROR!"),nl,nl,fail.

parsekb(Text,Text):-
	not(fronttoken(Text,_,_)),!.

parsekb(Text,Left):-
	parsecomment(Text,R1),
	!,
	parsekb(R1,Left).

parsekb(Text,Left):-
	parserule(Text,Remains,cl(Type,Name,Prior,RuleStuff,Desc,Help,CList)),
	assert(cl(Type,Name,Prior,RuleStuff,Desc,Help,CList)),
	!,parsekb(Remains,Left).

parsecomment(Text,Remains):-
	fronttoken(Text,"{",R1),
	writediff(Text,R1),
	findrightbrace(R1,Remains).

findrightbrace(Text,Remains):-
	frontchar(Text,'}',Remains),
	!,
	write('}').

findrightbrace(Text,Remains):-
	frontchar(Text,C,R1),
	write(C),
	findrightbrace(R1,Remains).

parserule(Str,Remains,cl(Type,Name,Prior,RuleStuff,Desc,Help,CList)):-
	fronttoken(Str,Type,R1),
	member(Type,["hypothesis","intermediate"]),
	!,
	writediff(Str,R1),
	parsehypish(R1,Remains,cl(Type,Name,Prior,RuleStuff,Desc,Help,CList)).

parserule(Str,Remains,cl("data",Name,Prior,RuleStuff,Desc,Help,CList)):-
	fronttoken(Str,"data",R1),
	!,
	writediff(Str,R1),
	parsedata(R1,Remains,cl("data",Name,Prior,RuleStuff,Desc,Help,CList)).

parsehypish(Str,Remains,cl(Type,Name,Prior,RuleStuff,Desc,Help,CList)):-
	parseheading(Str,R1,Name,Prior,Desc,Help),
	parsehypishbody(R1,R2,RuleStuff),
	parsecontextentry(R2,Remains,CList).

parseheading(Str,Remains,Name,Prior,Desc,Help):-
	wfronttoken(Str,Name,R1),
	wfronttoken(R1,"display",R2),
	parsestring(R2,R3,Desc),
	parsehelpentry(R3,R4,Help),
	parsepriorentry(R4,Remains,Prior).

parsestring(Str,Remains,S):-
	wfronttoken(Str,"\"",R1),
	formparsedstring(R1,Remains,S).

formparsedstring(Str,Remains,""):-
	frontchar(Str,'"',Remains),
	!,
	write("\"").

formparsedstring(Str,Remains,S):-
	frontchar(Str,Ch,R1),
	write(Ch),
	formparsedstring(R1,Remains,S1),
	frontchar(S,Ch,S1).

parsehelpentry(Str,Remains,Help):-
	wfronttoken(Str,"help",R1),
	!,
	parsestring(R1,Remains,Help).

parsehelpentry(Str,Str,"").

parsepriorentry(Str,Remains,Prior):-
	wfronttoken(Str,"prior",R1),
	!,
	parsenumber(R1,Remains,Prior).

parsepriorentry(Str,Str,0.5).

parsenumber(Str,Remains,N):-
	wfronttoken(Str,"-",R1),
	!,
	parsenumber(R1,Remains,N1),
	N=-N1.

parsenumber(Str,Remains,N):-
	wfronttoken(Str,T,Remains),
	str_real(T,N).

parsehypishbody(Str,Remains,not_rule(Rule)):-
	wfronttoken(Str,"not",R1),
	!,
	wfronttoken(R1,Rule,Remains).

parsehypishbody(Str,Remains,and_rule(NameList)):-
	wfronttoken(Str,"and",R1),
	!,
	parsenamelist(R1,Remains,NameList).

parsehypishbody(Str,Remains,or_rule(NameList)):-
	wfronttoken(Str,"or",R1),
	!,
	parsenamelist(R1,Remains,NameList).

parsehypishbody(Str,Remains,bayes_rule(BayesList)):-
	wfronttoken(Str,"bayes",R1),
	!,
	parsebayeslist(R1,Remains,BayesList).

parsehypishbody(Str,Remains,RuleStuff):-
	wfronttoken(Str,"map",R1),
	!,
	wfronttoken(R1,Rule,R2),
	parsemap(R2,Remains,Rule,RuleStuff).

parsemap(Str,Remains,Rule,num_map_rule(Rule,NMList)):-
	fronttoken(Str,"(",R1),
	fronttoken(R1,"-",_),
	!,
	parsenummap(Str,Remains,NMList).

parsemap(Str,Remains,Rule,num_map_rule(Rule,NMList)):-
	fronttoken(Str,"(",R1),
	fronttoken(R1,T,_),
	str_real(T,N),
	!,
	parsenummap(Str,Remains,NMList).

parsemap(Str,Remains,Rule,sym_map_rule(Rule,SMList)):-
	fronttoken(Str,"(",_),
	parsesymmap(Str,Remains,SMList).

parsenummap(Str,Str,[]):-
	not(fronttoken(Str,_,_)),!.

parsenummap(Str,Str,[]):-
	fronttoken(Str,X,_),
	member(X,["{","context","hypothesis","intermediate","data"]),!.
	
parsenummap(Str,Remains,[nummap(Val,Prob)|T]):-
	wfronttoken(Str,"(",R1),
	parsenumber(R1,R2,Val),
	parsenumber(R2,R3,Prob),
	wfronttoken(R3,")",R4),
	parsenummap(R4,Remains,T).

parsesymmap(Str,Str,[]):-
	not(fronttoken(Str,_,_)),!.

parsesymmap(Str,Str,[]):-
	fronttoken(Str,X,_),
	member(X,["{","context","hypothesis","intermediate","data"]),!.
	
parsesymmap(Str,Remains,[symmap(Sym,Prob)|T]):-
	wfronttoken(Str,"(",R1),
	wfronttoken(R1,Sym,R2),
	parsenumber(R2,R3,Prob),
	wfronttoken(R3,")",R4),
	parsesymmap(R4,Remains,T).
/* This clause takes anything as a symbol once you've decided that
that's the sort of map you're parsing, even a number! The error report
occurs when the inference engine tries to find rule "43.25" or whatever,
and fails */

parsenamelist(Str,Str,[]):-
	not(fronttoken(Str,_,_)),!.

parsenamelist(Str,Str,[]):-
	fronttoken(Str,X,_),
	member(X,["{","context","hypothesis","intermediate","data"]),!.

parsenamelist(Str,Remains,[Name|T]):-
	wfronttoken(Str,Name,R1),
	parsenamelist(R1,Remains,T).

parsebayeslist(Str,Str,[]):-
	not(fronttoken(Str,_,_)),!.

parsebayeslist(Str,Str,[]):-
	fronttoken(Str,X,_),
	member(X,["{","context","hypothesis","intermediate","data"]),!.

parsebayeslist(Str,Remains,[contrib(Name,Inc,Dec)|T]):-
	wfronttoken(Str,Name,R1),
	wfronttoken(R1,"inc",R2),
	parsenumber(R2,R3,Inc),
	wfronttoken(R3,"dec",R4),
	parsenumber(R4,R5,Dec),
	parsebayeslist(R5,Remains,T).

parsedata(Str,Remains,cl("data",Name,Prior,RuleStuff,Desc,Help,CList)):-
	parseheading(Str,R1,Name,Prior,Desc,Help),
	parseevbody(R1,R2,RuleStuff),
	parsecontextentry(R2,Remains,CList).

parseevbody(Str,Remains,cert_rule):-
	wfronttoken(Str,"certainty",Remains),!.

parseevbody(Str,Remains,yesno_rule):-
	wfronttoken(Str,"yesno",Remains),!.

parseevbody(Str,Remains,RuleStuff):-
	wfronttoken(Str,"value",R1),
	!,
	parsevalue(R1,Remains,RuleStuff).

parsevalue(Str,Remains,num_value_rule(Lo,Hi)):-
	fronttoken(Str,"-",_),
	!,parsenumber(Str,R1,Lo),
	parsenumber(R1,Remains,Hi).

parsevalue(Str,Remains,num_value_rule(Lo,Hi)):-
	fronttoken(Str,T,_),
	str_real(T,N),
	!,parsenumber(Str,R1,Lo),
	parsenumber(R1,Remains,Hi).

parsevalue(Str,Remains,sym_value_rule(SList)):-
	parsenamelist(Str,Remains,SList).

parsecontextentry(Str,Str,[]):-
	not(fronttoken(Str,"context",_)),!.

parsecontextentry(Str,Remains,CList):-
	wfronttoken(Str,"context",R1),
	parsecontextlist(R1,Remains,CList).

parsecontextlist(Str,Str,[]):-
	not(fronttoken(Str,_,_)),!.

parsecontextlist(Str,Str,[]):-
	fronttoken(Str,X,_),
	member(X,["{","hypothesis","intermediate","data"]),!.

parsecontextlist(Str,Remains,[context(Rule,Lo,Hi)|T]):-
	wfronttoken(Str,Rule,R1),
	parsenumber(R1,R2,Lo),
	parsenumber(R2,R3,Hi),
	parsecontextlist(R3,Remains,T).

