%
%
%project "pro2c"
%  /|\
%   |
% NOTE: to compile with the c-scanner, enable this as a project
%       and the corresponding constant below

code = 4000

CONSTANTS
% menubased = 1
% c_scanner = 1
scantokname = "tok"	% This name will trigger gen. of C-scanner initializers

GLOBAL DOMAINS
ifdef c_scanner
  CURSORTOK = t(TOK,CURSOR)
  TOKL = CURSORTOK*
enddef

  CURSOR = INTEGER
  PROCID = STRING

  BASISDOM = STRING
  NAME = STRING
  FLOWPAT = SYMBOL
  FILE = out

ifdef c_scanner
  GLOBAL PREDICATES
  scanner(STRING,TOKL) - (i,o) language C
  nondeterm str_tok(STRING,TOK) - (i,o),(o,o) language pascal % Could also be C
enddef

GLOBAL DATABASE
  source(STRING)
  filename(STRING)
  error(STRING,CURSOR)

include "tdoms.pro"
include "tpreds.pro"
include "pro2c.dom"

/***************************************************************************
	Reserved variable names and C keywords
***************************************************************************/
DATABASE - resnames
determ pro_resnames	% Flag indicating if reserved var names can occur

PREDICATES
  keyword(SYMBOL)

CLAUSES
  keyword("fno"):- pro_resnames.
  keyword("next"):- pro_resnames.  % Variable id's used in structs and unions
  keyword("refvar"):- pro_resnames.
 
  keyword("asm").
  keyword("auto").
  keyword("break").
  keyword("case").
  keyword("cdecl").
  keyword("char").
  keyword("const").
  keyword("continue").
  keyword("default").
  keyword("do").
  keyword("double").
  keyword("else").
  keyword("enum").
  keyword("extern").
  keyword("far").
  keyword("float").
  keyword("for").
  keyword("goto").
  keyword("huge").
  keyword("if").
  keyword("int").
  keyword("interrupt").
  keyword("long").
  keyword("near").
  keyword("pascal").
  keyword("register").
  keyword("return").
  keyword("short").
  keyword("signed").
  keyword("sizeof").
  keyword("static").
  keyword("struct").
  keyword("switch").
  keyword("typedef").
  keyword("union").
  keyword("unsigned").
  keyword("void").
  keyword("volatile").
  keyword("while").

/***************************************************************************
	Assertion of domain declarations
***************************************************************************/

DATABASE - declarations
  dom(NAME,RIGHTHAND)
  refdom(NAME)
  cpred(STRING,PARMS,FLOWPATTERNS,LANGUAGE,EXTNAME)

PREDICATES
  assertdecls(PROLOGDECL)
  assertdecl(DECLSECTION)
  assertdomlist(DOMLIST)
  assertdomdecl(NAMELIST,REFERENCE,RIGHTHAND)
  check_assert_reference(NAME,REFERENCE)

  assert_external_decls(GLOBPREDLIST)
  assert_external_decl(GLOBPRED)

  % conversion from dbasedef to domain declaration
  convert_dbadef_to_domdef(DBALIST,D_ALTERNATIVES)
  convert_single(DBA,D_ALTERNATIVE)
  parml_doml(PARMLIST,NAMELIST)


CLAUSES
  assertdecls([H|T]):-
	assertdecl(H),
	assertdecls(T).
  assertdecls([]).

  assertdecl(domdecl(DOMLIST)):-!,
	assertdomlist(DOMLIST).
  assertdecl(dbadecl1(NAME,DBALIST)):-!,
	convert_dbadef_to_domdef(DBALIST,RIGHTHAND),
	assertdomdecl([NAME],none,alternatives(RIGHTHAND)).
  assertdecl(dbadecl(DBALIST)):-!,
	convert_dbadef_to_domdef(DBALIST,RIGHTHAND),
	assertdomdecl(["dbasedom"],none,alternatives(RIGHTHAND)).
  assertdecl(globpreddecl(GLOBPREDLIST)):-!,
	assert_external_decls(GLOBPREDLIST).
  assertdecl(_).


  % ------ Assertion of domain declarations -------
  assertdomlist([dom(NAMELIST,REFERENCE,RIGHTHAND)|T]):-
	assertdomdecl(NAMELIST,REFERENCE,RIGHTHAND),
	assertdomlist(T).
  assertdomlist([]).

  assertdomdecl([NAME|T],REFERENCE,RIGHTHAND):-
	check_assert_reference(NAME,REFERENCE),
	assert(dom(NAME,RIGHTHAND)),
	assertdomdecl(T,REFERENCE,RIGHTHAND).
  assertdomdecl([],_,_).

  check_assert_reference(NAME,reference_):- !,
	assert(refdom(NAME)).
  check_assert_reference(_,_).


  % ------- Assertion of external predicate declaration -------
  assert_external_decls([H|T]):-
	assert_external_decl(H),
	assert_external_decls(T).
  assert_external_decls([]).

  assert_external_decl(pred(_,NAME,PARMS,FLOW,language_(LINGO),EXTNAME)):- !,
	assert(cpred(NAME,PARMS,FLOW,language_(LINGO),EXTNAME)).
  assert_external_decl(nullarg(_,NAME,language_(LINGO),EXTNAME)):-!,
	assert(cpred(NAME,none,[],language_(LINGO),EXTNAME)).
  assert_external_decl(_).

  % ------- Conversion between database declaration and domain definition -----
  convert_dbadef_to_domdef([H|T],[H1|T1]):-
	convert_single(H,H1),
	convert_dbadef_to_domdef(T,T1).
  convert_dbadef_to_domdef([],[]).

  convert_single(dba(_,PREDNAME,none),atom(FUNCNAME)):-
	FUNCNAME=PREDNAME.
  convert_single(dba(_,PREDNAME,parms(PARML)),cmp(FUNCNAME,DOML)):-
	FUNCNAME=PREDNAME,
	parml_doml(PARML,DOML).

  parml_doml([H|T],[H|T1]):-
	parml_doml(T,T1).
  parml_doml([],[]).


/***************************************************************************
	Generate implicitly defined reference domains
***************************************************************************/

PREDICATES
  gen_recursive_refdoms
  makereference(RIGHTHAND)
  makealtrefdoms(D_ALTERNATIVES)
  makenamelrefdom(NAMELIST)
  gennewrefdom(NAME)

CLAUSES
  gen_recursive_refdoms:-
	refdom(NAME),
	dom(NAME,RIGHTHAND),
	makereference(RIGHTHAND),
	fail.
  gen_recursive_refdoms.

  makereference(listdom(NAME)):- gennewrefdom(NAME).
  makereference(alternatives(ALTL)):- makealtrefdoms(ALTL).
  makereference(basisdom(_)).

  makealtrefdoms([cmp(_,NAMELIST)|T]):- !,
	makenamelrefdom(NAMELIST),
	makealtrefdoms(T).
  makealtrefdoms([_|T]):-
	makealtrefdoms(T).
  makealtrefdoms([]).

  makenamelrefdom([NAME|T]):-
	gennewrefdom(NAME),
	makenamelrefdom(T).
  makenamelrefdom([]).

  gennewrefdom(NAME):-refdom(NAME), !.
  gennewrefdom(NAME):-
	asserta(refdom(NAME)),
	dom(NAME,RIGHTHAND), !,
	makereference(RIGHTHAND).
  gennewrefdom(_).


/***************************************************************************
	Generate a list of all domains that need to be declared 
	for the external predicates.
***************************************************************************/

DATABASE - referred_domains
  referred_dom(NAME)

PREDICATES
  gen_recursive_referreddoms
  reg_referred(NAME)
  recursivereferred(RIGHTHAND)
  altlreferred(D_ALTERNATIVES)
  namelreferred(NAMELIST)

  nondeterm member(NAME,PARMLIST)

CLAUSES
  gen_recursive_referreddoms:-
	cpred(_,PARMS,_,_,_),
	PARMS=parms(PARML),
	member(NAME,PARML),
	reg_referred(NAME),
	fail.
  gen_recursive_referreddoms.

  recursivereferred(listdom(NAME)):- reg_referred(NAME).
  recursivereferred(basisdom(NAME)):- reg_referred(NAME).
  recursivereferred(alternatives(ALTL)):- altlreferred(ALTL).

  altlreferred([cmp(_,NAMELIST)|T]):- !,
	namelreferred(NAMELIST),
	altlreferred(T).
  altlreferred([_|T]):-
	altlreferred(T).
  altlreferred([]).

  namelreferred([NAME|T]):-
	reg_referred(NAME),
	namelreferred(T).
  namelreferred([]).

  reg_referred(NAME):- referred_dom(NAME), !.
  reg_referred(NAME):-
	assert(referred_dom(NAME)),
	dom(NAME,RIGHTHAND), !,
	recursivereferred(RIGHTHAND).
  reg_referred(_).

  member(X,[X|_]).
  member(X,[_|L]):- member(X,L).


/***************************************************************************
	TEST Output of the domain database
***************************************************************************/

PREDICATES
  writeref(NAME)
  writedom

CLAUSES
  writedom:-
	dom(NAME,RIGHTHAND),
	write("\n",NAME," = "),
	writeref(NAME),
	write("\n\t",RIGHTHAND),
	fail.
  writedom.

  writeref(NAME):-
	refdom(NAME), !,
	write("reference").
  writeref(_).


/***************************************************************************
	Generate structure pointer
***************************************************************************/

PREDICATES
  genptrname(NAME,STRING)

CLAUSES
  genptrname(NAME,PTRNAME):-
	upper_lower(NAME,LNAME),
	concat(LNAME,"_struct",PTRNAME).


/***************************************************************************
			Generation of C types

 This group of predicates does the generation of C type declarations from
 the database dom(NAME,RIGHTHAND) and the description of reference domains
 refdom(NAME).

 As long as the database predicate referred_dom is called in genctypes,
 C type-definitions will not be generated for domains not used in the
 global predicates declared to be C procedures.

***************************************************************************/

DATABASE - known
  known(NAME)
  known_selector(NAME,INTEGER)
  known_globstruct(NAME,INTEGER)

PREDICATES
  genctypes
  genctype(NAME,RIGHTHAND)
  wtype(NAME)
  wstructref(NAME)
  gen_globstructname(NAME,NAME)
  gen_sel_no(NAME,INTEGER)
  wselector(NAME)
  wdoml(STRING,NAMELIST)
  walt(D_ALTERNATIVE)
  waltlist(D_ALTERNATIVES)
  enumlist(D_ALTERNATIVES)
  asteriskneeded(NAME,RIGHTHAND)
  enum_functors(NAME,D_ALTERNATIVES,INTEGER,INTEGER)
  get_func(D_ALTERNATIVE,STRING)
  get_tabs(STRING,STRING)
  check_scanname(NAME,D_ALTERNATIVES,INTEGER)
  gen_initlist(D_ALTERNATIVES)
  comma_check(D_ALTERNATIVES)

CLAUSES
  genctypes:-
	write("\n#define VARFNO\t\t0"),
	write("\n#define CONSTFNO\t1"),
	write("\n#define LISTFNO\t\t1"),
	write("\n#define NILLFNO\t\t2"),
	write("\n\n#define INTEGER\t\tint"),
	write("\n#define CHAR\t\tchar"),
	write("\n#define REAL\t\tdouble"),
	write("\n#define STRING\t\tchar*"),
	write("\n#define SYMBOL\t\tchar*"),
	fail.

  genctypes:-
	referred_dom("ref"),
	write("\n#define REF\t\tlong"),
	fail.

  genctypes:-
	referred_dom("bt_selector"),
	write("\n#define BT_SELECTOR\tint"),
	fail.

  genctypes:-
	referred_dom("reg"),  % A less than ideal solution - should be AX,BX etc
	assert(dom("reg",alternatives([cmp("reg",
				["integer","integer","integer","integer",
				 "integer","integer","integer","integer"])]))),
	fail.

  genctypes:-
	referred_dom("file"),
	getbacktrack(TOP),
	retract(dom("file",alternatives(USERFILES))),
	assert(dom("file",alternatives(
		[atom("keyboard"),atom("screen"),atom("stdin"),atom("stdout"),
		 atom("stderr"),atom("com1"),atom("printer")|USERFILES]))),
	cutbacktrack(TOP),
	fail.

  genctypes:-
 	referred_dom("place"),
 	assert(dom("place",alternatives(
		[atom("in_memory"),atom("in_ems"),atom("in_file")]))),
	fail.

  genctypes:-
	referred_dom(NAME),
	dom(NAME,RIGHTHAND), RIGHTHAND=basisdom(_),
	nl,
	upper_lower(UNAME,NAME),
	genctype(UNAME,RIGHTHAND),
	assert(known(NAME)),
	fail.
  genctypes:-
	referred_dom(NAME),
	dom(NAME,RIGHTHAND), not(RIGHTHAND=basisdom(_)),
	nl,
	upper_lower(UNAME,NAME),
	genctype(UNAME,RIGHTHAND),
	assert(known(NAME)),
	fail.
  genctypes.


  genctype(_,_):- retractall(known_selector(_,_)), fail.

  genctype(NAME,basisdom(BASISDOM)):-
	upper_lower(NAME,LNAME),
	refdom(LNAME),!,
	gen_globstructname(BASISDOM,STRUCTNAME),
	upper_lower(UBASISDOM,BASISDOM),
	write("\ntypedef\n  struct ",STRUCTNAME," {"),
	write("\n\tunsigned char fno;"),
	write("\n\tunion {"),
	write("\n\t\t",UBASISDOM,' '),
	wselector(BASISDOM),
	write(";\n\t\tstruct ",STRUCTNAME," *refvar;"),
	write("\n\t} u;"),
	write("\n  } ",NAME,';').

  genctype(NAME,basisdom(BASISDOM)):-
	upper_lower(UBASISDOM,BASISDOM),
	write("\ntypedef ",UBASISDOM,'\t',NAME,';').

  genctype(NAME,listdom(REFDOM)):-
	genptrname(NAME,PTRNAME),
	write("\ntypedef\n  struct ",PTRNAME," {"),
	write("\n\tunsigned char fno;"),
	write("\n\t"),
	wtype(REFDOM), wselector(REFDOM), write(';'),
	write("\n\tstruct ",PTRNAME," *next;"),
	write("\n  } ",NAME,";").

  genctype(NAME,alternatives(ALTERNATIVES)):-
	enumlist(ALTERNATIVES), !,
	enum_functors(NAME,ALTERNATIVES,1,NUMBER),
	genptrname(NAME,PTRNAME),
	write("\ntypedef\n  struct ",PTRNAME," {"),
	write("\n\tunsigned char fno;"),
	write("\n  } ",NAME,";"),
	check_scanname(NAME,ALTERNATIVES,NUMBER).

  genctype(NAME,alternatives([cmp(FUNC,NAMELIST)])):- !,
	enum_functors(NAME,[cmp(FUNC,NAMELIST)],1,_),
	genptrname(NAME,PTRNAME),
	write("\ntypedef\n  struct ",PTRNAME," {"),
	write("\n\tunsigned char fno;\n\t"),
	wdoml("\n\t",NAMELIST),
	write("\n  } ",NAME,';').

  genctype(NAME,alternatives(ALTERNATIVES)):-
	enum_functors(NAME,ALTERNATIVES,1,NUMBER),
	genptrname(NAME,PTRNAME),
	write("\ntypedef\n  struct ",PTRNAME," {"),
	write("\n\tunsigned char fno;"),
	write("\n\tunion {"),
	waltlist(ALTERNATIVES),
	write("\n\t} u;"),
	write("\n  } ",NAME,";"),
	check_scanname(NAME,ALTERNATIVES,NUMBER).


  waltlist([H]):- walt(H), !.
  waltlist([H|T]):- walt(H), !, waltlist(T).
  waltlist([_|T]):- waltlist(T).
  waltlist([]).


  walt(_):- retractall(known_selector(_,_)), fail.
  walt(cmp(FUNC,[NAMETYPE])):- !,
  	write("\n\t\t"),
	wtype(NAMETYPE),
	FUNC=NAME,
	wselector(NAME),
	write(';').
  walt(cmp(FUNC,NAMELIST)):-
	write("\n\t\tstruct {\n\t\t\t"),
	wdoml("\n\t\t\t",NAMELIST),
	write("\n\t\t} "),
	retractall(known_selector(_,_)),
	FUNC=NAME,
	wselector(NAME),
	write(';').


  wdoml(_,[NAME]):- !,
	wtype(NAME),
	wselector(NAME), write(';').
  wdoml(INDENT,[NAME|T]):-
	wtype(NAME),
	wselector(NAME),
	write(';'),
	write(INDENT),
	wdoml(INDENT,T).
  wdoml(_,[]).


  wtype(TYPE):-
	dom(TYPE,RIGHTHAND),
	asteriskneeded(TYPE,RIGHTHAND),
	!,
	wstructref(TYPE),
 	write(" *").
  wtype(TYPE):-
	upper_lower(U,TYPE),
	write(U,' ').


  wstructref(TYPE):-
	known(TYPE), !,
	upper_lower(U,TYPE),
	write(U).
  wstructref(TYPE):-
	upper_lower(TYPE,L),
	write("struct ",L,"_struct").

  asteriskneeded(DOM,_):-refdom(DOM), !.
  asteriskneeded(_,basisdom(_)):- !, fail.
  asteriskneeded(_,_).


  gen_globstructname(NAME,NAME1):-
  	retract(known_globstruct(NAME,N)), !,
  	format(NAME1,"%_ref_%",NAME,N),
  	NN=N+1,
  	assert(known_globstruct(NAME,NN)).
  gen_globstructname(NAME,NAME1):-
  	concat(NAME,"_ref",NAME1),
  	assert(known_globstruct(NAME,1)).


  gen_sel_no(SEL,N):-
	retract(known_selector(SEL,N)), !,
	N1=N+1,
	assert(known_selector(SEL,N1)).
  gen_sel_no(SEL,0):-
  	assert(known_selector(SEL,1)).


  wselector(SEL):-
  	known_selector(SEL,_), !,
  	gen_sel_no(SEL,N),
  	upper_lower(SEL,L),
	write(L,'_',N).
  wselector(SEL):-
	upper_lower(SEL,L),
  	keyword(L), !,
  	gen_sel_no(SEL,N),
  	write(L,'_',N).
  wselector(SEL):-
  	assert(known_selector(SEL,0)),
  	upper_lower(SEL,L),
  	write(L).


  enumlist([atom(_)|T]):- enumlist(T).
  enumlist([]).

  enum_functors(NAME,[H|T],COUNT,NUMB):-
	get_func(H,FUNC),
	upper_lower(NAME,LNAME),
	format(LINE,"\n#define %_%",LNAME,FUNC),
	get_tabs(LINE,TABS),
	write(LINE,TABS,COUNT),
	COUNT1=COUNT+1,
	enum_functors(NAME,T,COUNT1,NUMB).
  enum_functors(_,[],NUMB,COUNT):- COUNT=NUMB-1, nl.

  get_func(cmp(FUNC,_),FUNC).
  get_func(atom(FUNC),FUNC).

  get_tabs(LINE,TABS):-
	str_len(LINE,LEN),
	TABC = 3 - (LEN-9) div 8, TABC>0, !,
	frontstr(TABC,"\t\t\t",TABS,_).
  get_tabs(_,"\t").


/*******************************************************************************
		Generate tokenlist for C-scanner:
*******************************************************************************/

  check_scanname(NAME,ALTERNATIVES,NUMB):-
  	upper_lower(NAME,scantokname), !,
  	write("\n\n\n#define MAXTOKS ",NUMB,"\n\n"),
	write("struct {\n\tSTRING strtok;\n\tint fno;\n} tokens[MAXTOKS] = {"),
	gen_initlist(ALTERNATIVES),
  	write("\n};\n").
  check_scanname(_,_,_).

  gen_initlist([H|T]):-
  	get_func(H,FUNC),
  	write("\n\t{\"\",",scantokname,'_',FUNC,'}'),
  	comma_check(T),
  	gen_initlist(T).
  gen_initlist([]).
  
  comma_check([]):- !.
  comma_check(_):- write(',').


/***************************************************************************
	Declare C procedure prototypes for the external predicates.
***************************************************************************/

PREDICATES
  gen_c_procedures
  gen_c_procedures1(STRING,PARMS,FLOWPATTERNS,LANGUAGE,EXTNAME)
  gen_c_procedures2(STRING,PARMS,FLOWPATTERNS,LANGUAGE,EXTNAME,INTEGER)

  writeparmlist(PARMLIST,FLOWPATLIST)
  writeparm(NAME,FLOWPAT)
  wlang(LANGUAGE)
  write_name(STRING,EXTNAME,INTEGER)

CLAUSES
  gen_c_procedures:-
	cpred(NAME,PARMS,FLOWPATTERNS,LANGUAGE,EXTNAME),
	gen_c_procedures1(NAME,PARMS,FLOWPATTERNS,LANGUAGE,EXTNAME),
	fail.
  gen_c_procedures.
	
  gen_c_procedures1(NAME,none,_,LANGUAGE,EXTNAME):- !,
  	wlang(LANGUAGE), write_name(NAME,EXTNAME,0), write("();").
  gen_c_procedures1(NAME,PARMS,FLOWPATTERNS,LANGUAGE,EXTNAME):-
	gen_c_procedures2(NAME,PARMS,FLOWPATTERNS,LANGUAGE,EXTNAME,0).

  gen_c_procedures2(NAME,parms(PARMS),[flowpat(FLOWLIST)|T],
						LANGUAGE,EXTNAME,SUFFIX):-
  	retractall(known_selector(_,_)),
  	wlang(LANGUAGE),
	write_name(NAME,EXTNAME,SUFFIX),
	write('('),
	writeparmlist(PARMS,FLOWLIST),
	write(");"),
	SUFFIX1=SUFFIX+1,
	gen_c_procedures2(NAME,parms(PARMS),T,LANGUAGE,EXTNAME,SUFFIX1).
  gen_c_procedures2(_,_,[],_,_,_).

  writeparmlist([PARM],[FLOWPAT]):- !,
	writeparm(PARM,FLOWPAT).
  writeparmlist([PARM|T1],[FLOWPAT|T2]):-
	writeparm(PARM,FLOWPAT),
	write(','),
	writeparmlist(T1,T2).
  writeparmlist([],_).

  writeparm(DOM,i):-
	wtype(DOM),
	wselector(DOM).
  writeparm(DOM,o):-
	wtype(DOM),
	write('*'),
	wselector(DOM).

  wlang(language_(pascal)):- write("\n\nvoid far pascal ").
  wlang(language_(c)):- write("\n\nvoid far cdecl ").

  write_name(NAME,none,SUFFIX):- write(NAME,'_',SUFFIX).
  write_name(_,extname(ENAME),_):- write(ENAME).

/***************************************************************************
	Scanner & Parser environment
***************************************************************************/

ifndef c_scanner
  PREDICATES
  scan_error(STRING,CURSOR)

  include "pro2c.sca"
enddef

DOMAINS
  FLOWPATTERNS = FLOWPATTERN*

PREDICATES
  ed(STRING,CURSOR)
  change(DBASEDOM)
  nondeterm repparse
  process(STRING)
  better_error(CURSOR)
  new_error(STRING,CURSOR)
  expect(CURSORTOK,TOKL,TOKL)
  syntax_error(STRING,TOKL)
  checkempty(TOKL)
  s_basisdom(TOKL,TOKL,BASISDOM)
  s_flowpatterns(TOKL,TOKL,FLOWPATTERNS)
  s_flowpatterns1(TOKL,TOKL,FLOWPATTERNS)

include "pro2c.par"

CLAUSES
/* User-defined production s_basisdom (from the grammar).
   If compound domains with functor-names identical to standard domain-names
   (eg string(string,integer) ) are avoided, 's_basisdom' won't have to
   check the following token ('NEXT') for this and may hence be speeded up
   somewhat by removal of the body of the clauses.
   
   Notice, by the way, how ridiculously easy it is to handle a multi-symbol
   lookahead grammar in Prolog.
*/

  s_basisdom([t(id("integer"),_),NEXT|LL],[NEXT|LL],"integer"):-
	not(NEXT=t(lpar,_)), not(NEXT=t(or_,_)), !.
  s_basisdom([t(id("string"),_),NEXT|LL],[NEXT|LL],"string"):-
	not(NEXT=t(lpar,_)), not(NEXT=t(or_,_)), !.
  s_basisdom([t(id("char"),_),NEXT|LL],[NEXT|LL],"char"):-
	not(NEXT=t(lpar,_)), not(NEXT=t(or_,_)), !.
  s_basisdom([t(id("symbol"),_),NEXT|LL],[NEXT|LL],"symbol"):-
	not(NEXT=t(lpar,_)), not(NEXT=t(or_,_)), !.
  s_basisdom([t(id("real"),_),NEXT|LL],[NEXT|LL],"real"):-
	not(NEXT=t(lpar,_)), not(NEXT=t(or_,_)), !.
  s_basisdom([t(id("ref"),_),NEXT|LL],[NEXT|LL],"ref"):-
	not(NEXT=t(lpar,_)), not(NEXT=t(or_,_)).

  s_flowpatterns(LL1,LL0,[FLOWPATTERN|FLOWPATTERNS]):-
	s_flowpattern(LL1,LL2,FLOWPATTERN),!,
	s_flowpatterns1(LL2,LL0,FLOWPATTERNS).
  s_flowpatterns(LL,LL,[]).

  s_flowpatterns1([t(comma,_)|LL1],LL2,FLOWPATTERNS):-!,
	s_flowpatterns(LL1,LL2,FLOWPATTERNS).
  s_flowpatterns1([t(lpar,POS)|LL1],LL2,FLOWPATTERNS):-!,
	s_flowpatterns([t(lpar,POS)|LL1],LL2,FLOWPATTERNS).
  s_flowpatterns1(LL,LL,[]).

  better_error(CURSOR):-
	error(_,OLDCURSOR), OLDCURSOR>=CURSOR, !, fail.
  better_error(_).

  new_error(MSG,CURSOR):-retractall(error(_,_)), assert(error(MSG,CURSOR)).

  expect(TOK,[TOK|L],L):- !.
  expect(t(id(_),_),[t(_,CURSOR)|_],_):- !,
  	better_error(CURSOR),
  	new_error("Identifier expected",CURSOR),
  	fail.
  expect(t(TOK,_),[t(_,CURSOR)|_],_):-
	better_error(CURSOR),
ifdef c_scanner
  	str_tok(STR,TOK1), TOK1=TOK, !,	% Avoid comp. flow
elsedef
  	str_tok(STR,TOK),
enddef
	concat(STR," expected",MSG),
	new_error(MSG,CURSOR),
  	fail.

  syntax_error(PROD,[t(_,CURSOR)|_]):-
	better_error(CURSOR),
	concat("Syntax error in ",PROD,MSG),
	new_error(MSG,CURSOR),fail.

ifndef c_scanner
  scan_error(MSG,CURSOR):- ed(MSG,CURSOR),fail.
enddef

  checkempty([]):- !.
  checkempty([t(_,CURSOR)|_]):-
	better_error(CURSOR),
	new_error("Syntax error",CURSOR).

  ed(MSG,CURSOR):-
	source(TXT),
	edit(TXT,TXT1,"","",MSG,CURSOR,"",RET),
	RET><1,!,
	change(source(TXT1)).

  change(source(_)):-retractall(source(_)), fail.
  change(filename(_)):-retractall(filename(_)), fail.
  change(X):- assert(X).

ifdef menubased
  repparse.
  repparse:- error(MSG,CURSOR), ed(MSG,CURSOR), !, repparse.
elsedef
  repparse.
  repparse:-
	error(MSG,CURSOR),
	nl,write(MSG),nl,
	writef("At position: % in source\n",CURSOR),
	write("Use GOTO POSITION in prolog system to locate place\n"),
	readkey(_),
	fail.
enddef

  process(OUTPUTFILE):-
	retractall(known(_)),
	retractall(known_selector(_,_)),
	retractall(dom(_,_)),
	retractall(refdom(_)),
	retractall(cpred(_,_,_,_,_)),
	retractall(referred_dom(_)),
	retractall(known_globstruct(_,_)),
	repparse,
	retractall(error(_,_)),
	source(STR1),
	write("\n--->SCAN"),
ifdef c_scanner
	scanner(STR1,L),
elsedef
	tokl(1,STR1,L),
enddef
	write("\n--->PARSE"),
	s_prologdecl(L,L1,X),
	checkempty(L1),
	!,
	write("\n--->ASSERT"),
	assertdecls(X),
	gen_recursive_refdoms,
	gen_recursive_referreddoms,
	write("\n--->GEN C TYPES"),
	openwrite(out,OUTPUTFILE),
	writedevice(out),
	assert(pro_resnames),
	genctypes,
	retractall(pro_resnames),
	gen_c_procedures,
	nl,
	closefile(out),
	write("\n",OUTPUTFILE," Generated").
  process(_):-
	write(">> Parsing aborted"),nl,beep.

ifdef menubased

include "ui\\lineinp.pro"
include "ui\\filename.pro"
include "ui\\status.pro"
include "ui\\pulldown.pro"

CLAUSES
  pdwaction(1,0):-
	shiftwindow(OLD),shiftwindow(1),
	filename(FILENAME),!,
	newext(FILENAME,"C",OUTPUTFILE),
	process(OUTPUTFILE),
	source(NEWSOURCE),!,
	window_str(NEWSOURCE),
	shiftwindow(OLD),
	refreshstatus.
  pdwaction(2,0):-
	filename(FILENAME),
	newext(FILENAME,"C",OUTPUTFILE),
	file_str(OUTPUTFILE,OUTSTR),!,
	textmode(ROWS,COLS),
	RR=ROWS-4,
	makewindow(1,23,23,"View C types",3,0,RR,COLS),
	edit(OUTSTR,_,OUTPUTFILE,"","",0,"",_),
	removewindow,
	refreshstatus.
  pdwaction(2,0):-refreshstatus.
  pdwaction(3,0):-
	shiftwindow(OLD),shiftwindow(1),
	source(TXT),
	filename(FILENAME),
	newext(FILENAME,"PRO",SOURCEFILE),
	edit(TXT,TXT1,SOURCEFILE,"","",0,"",RET),
	shiftwindow(OLD),
	RET><1,!,
	change(source(TXT1)),
	refreshstatus.
  pdwaction(3,0):-refreshstatus.
  pdwaction(4,1):-
	readfilename(5,40,66,66,pro,"",NEW),
	change(filename(NEW)),
	file_str(NEW,NEWSOURCE),!,
	change(source(NEWSOURCE)),
	shiftwindow(OLDW),
	shiftwindow(1),
	window_str(NEWSOURCE),
	shiftwindow(OLDW),
	refreshstatus.
  pdwaction(4,1).
  pdwaction(4,2):-
	source(SOURCE),
	filename(OLD),
	readfilename(5,40,66,66,pro,OLD,NEW),
	change(filename(NEW)),
	file_str(NEW,SOURCE),
	refreshstatus,!.
  pdwaction(4,2).
  pdwaction(4,3):-setdir(5,40,66,66).
  pdwaction(5,0):-
	lineinput(5,70,40,66,66,"Are you sure (y/n) ? ","",ANS),
	upper_lower(ANS,ANS1),
	ANS1><"y".

GOAL
  textmode(ROWS,COLS),
  RR=ROWS-4,
  makewindow(1,23,23,"Edit",3,0,RR,COLS),
  makestatus(112," Select with arrows or use first upper case letter"),
  pulldown(66,
	  [ curtain("Generate C types",[]),
	    curtain("View generated",[]),
	    curtain("Edit",[]),
	    curtain("Files",["Load","Save","Dir"]),
            curtain("Quit",[])
          ],_,_),
  removestatus, removewindow.

elsedef

GOAL	
	comline(L),
	filenameext(FILENAME, L, ".PRO"),
	filenameext(OUTPUTFILE, L, ".C"),
	existfile(FILENAME),
	file_str(FILENAME,SOURCE),
	change(source(SOURCE)),
	process(OUTPUTFILE)
	;
	comline(L),
	write("File ",L," not found").

enddef

CLAUSES

  filename("work.pro").
  source("").
