/**********************************************************************
	SCANNER.C

	S C A N N E R  routine 

  The C procedure scanner_0(STRING,TOKL) converts a string to a list
  of tokens with cursor positions.

DOMAINS
  TOK = comma(); plus(); minus(); mult(); div(); power(); exclmmark();
	questionmark(); colon(); id(STRING); lpar(); rpar(); int(INTEGER);
	real(REAL); str(STRING); char(CHAR); if_(); then(); else();
	while(); do(); goto(); nill
  CURSOR = INTEGER
  CURSORTOK = t(TOK,CURSOR)
  TOKL = CURSORTOK*

	Copyright (c) 1986, 1990 by Prolog Development Center
**********************************************************************/

#include "scandoms.c"
#include <pdcrunt.h>

#define MAXIDLENG 250


char	*txtptr, /* Current pointer to input-string */
	   *start,  /* Pointer to start of input-string */
	   _ch,  	/* Current character ( = *(txtptr-1) ) */
	   chclass; /* Type of current char (one of onedigit..space below) */
int	pos;  	/* Current position */


extern char CH_ClassTab[];     /* chclass = CH_ClassTab[_ch] */

#define onedigit         0
#define small_let        1
#define cap_let	       2
#define spec_ch          3
#define end_of_file      4
#define other_ch         5
#define nl_or_page       6
#define space            7

#define isdigit(c)      (CH_ClassTab[c]==onedigit)
#define isspace(c)      (CH_ClassTab[c]==space)

/****************************************************************
		Test output
****************************************************************/
/*

void near w10(void)
{ int i;
  for(i=0;i<5;i++) IO_Writef("%c",txtptr[i]);
}

void near wtok(TOK *tok)
{
  IO_WrStr("\t--> ");
  switch(tok->fno) {
    case tok_comma: IO_WrStr("comma"); break;
    case tok_plus:  IO_WrStr("plus"); break;
    case tok_mult:  IO_WrStr("mult"); break;
    case tok_div:   IO_WrStr("div"); break;
    case tok_power: IO_WrStr("power"); break;
    case tok_id:    IO_WrStr("id(%s)",tok->u.id); break;
    case tok_lpar:  IO_WrStr("lpar"); break;
    case tok_rpar:  IO_WrStr("rpar"); break;
    case tok_int:   IO_WrStr("int(%d)",tok->u.int_0); break;
    case tok_real:  IO_WrStr("real(%d)",tok->u.real); break;
    case tok_str:   IO_WrStr("str(%s)",tok->u.str); break;
    default: IO_WrStr("unknown token");
  }
  IO_WrCh('\n');
}

*/

/****************************************************************
			error
****************************************************************/

void near error(void)
{ RUN_Error(1); }


/****************************************************************
			getone
****************************************************************/

void near getone(void)
{ if ( (_ch=*txtptr++) != 0)
    chclass=CH_ClassTab[_ch];
  else {
    chclass=end_of_file;
    txtptr--;
  }
}

/****************************************************************
			getnumber
****************************************************************/

/*
; parsing of an integer number
; RET = SCAN_Intg(STR,&VAL)
;    -1 not integer
;    -2 maybe a real
; RET = SCAN_Real(STR,&VAL)
;    -1 error
; (defined in Prolog Library)
*/

TOK *getnumber(void)
{
  TOK *tok;
  int len;
  int integer;

  txtptr--;
  if((len=SCAN_Intg(txtptr,&integer)) > 0) {
    CHAR chr=txtptr[len];
    if((chr=='.'||chr=='e'||chr=='E')&&
	(isdigit(txtptr[len+1])||txtptr[len+1]=='-')) goto real;
    txtptr+=len;
    getone();
    (tok=(TOK*)MEM_AllocGStack(sizeof(unsigned char)+sizeof(INTEGER)))
								->fno=tok_int;
    tok->u.int_0=integer;
    return(tok);
  }
  real: {
    (tok=(TOK*)MEM_AllocGStack(sizeof(unsigned char)+sizeof(REAL)))->fno=tok_real;
    if((len=SCAN_Real(txtptr, &tok->u.real)) < 0) error();
    txtptr+=len;
    getone();
    return(tok);
  }
}


/****************************************************************
		one_escape_sequence	\n,\t,\123,\',\" etc
****************************************************************/

int near one_escape_sequence(void)
{
  int asc_val=0;
  getone();
  if (chclass==onedigit) {
      int n=0;
      while(chclass==onedigit && n++<3) {
	asc_val=_ch-'0'+asc_val*10; getone();
      }
      txtptr-=2; getone();
      return(asc_val);
  }
  else
    switch(_ch) {
	case 'n' : _ch='\n';break;
	case 't' : _ch='\t';break;
    }
  return(_ch);
}

/****************************************************************
		getstring
****************************************************************/

#define MAXSTRINGL	255

TOK *getstring(void)
{
  TOK *tok;
  int sleng = 0;
  char string[MAXSTRINGL];
  char *strp=string;

  while ( (_ch=*txtptr++)!=0 && _ch!='"' && _ch!='\n' ) {
     *strp++ = (_ch=='\\') ? one_escape_sequence() : _ch;
     if (sleng++ > MAXSTRINGL) error();
  }
  if (_ch != '"') error();
  *strp = 0;
  getone();
  (tok=(TOK*)MEM_AllocGStack(sizeof(unsigned char)+sizeof(STRING)))->fno=tok_str;
  tok->u.str=(STRING)MEM_SaveStringGStack(string);
  return(tok);
}

/****************************************************************
			str_tok
****************************************************************/

void far pascal str_tok_0(STRING str,TOK **tok)		/* (i,o) flow */
{
  int tokno;

  (*tok=(TOK*)MEM_AllocGStack(sizeof(unsigned char)))->fno=tok_nill;
  for(tokno=0; tokno<MAXTOKS; tokno++)
    if(STR_StrCmp(str,tokens[tokno].strtok)==0) {	/* Reserved word */
/*	if(RUN_StackBtrack()) continue;	*/
	(*tok)->fno=tokens[tokno].fno;
	return;
/*	RUN_JmpReturn1();	*/
    }
/*  RUN_Fail();	*/
}


void far pascal str_tok_1(STRING *str,TOK **tok)	/* (o,o) flow */
{
  int tokno;

  for(tokno=0; tokno<MAXTOKS; tokno++) {
    if(RUN_StackBTrack())
      continue;
    *str=MEM_SaveStringGStack(tokens[tokno].strtok);
    (*tok=(TOK*)MEM_AllocGStack(sizeof(unsigned char)))->fno=tokens[tokno].fno;
    RUN_JmpReturn1();
  }
  RUN_Fail();
}

/********************
void far pascal str_tok_2(STRING *str,TOK *tok)		/* (o,i) flow */
{
  /* Return the string in tokens indicated by the functor number.
     Subtract 1 from this to counteract Prologs offset of 1 */

  *str=MEM_SaveStringGStack(tokens[tok->fno-1].strtok);
}
*********************/


/****************************************************************
			char_seq
****************************************************************/

TOK *char_seq(void)
{
  char buf[MAXIDLENG];
  char *bufp=buf;
  TOK *tok;
  int nleng=0;

  while(nleng++ < MAXIDLENG) {
#ifdef case_sensitive
      *bufp++ = _ch;
#else
      *bufp++ = _ch >='A' && _ch <= 'Z' ? _ch + 'a' - 'A' : _ch;
#endif
      if ((chclass=CH_ClassTab[_ch=*txtptr++]) > cap_let && _ch!='_') break;
  }
  if (nleng > MAXIDLENG) error();
  *bufp = 0;
  str_tok_0(buf,&tok);
  if(tok->fno==tok_nill) { /* Unrecognized alphabetic, must be id */
    (tok=MEM_AllocGStack(sizeof(unsigned char)+sizeof(STRING)))->fno=tok_id;
    tok->u.id=(STRING)MEM_SaveStringGStack(buf);
  }
  return(tok);
}

/****************************************************************
			gettoken
****************************************************************/

TOK *gettoken(void)
{
  int tokkind;
  TOK *tok;


ANOTHER :
pos=(int)(txtptr-start);
switch(chclass) {
  case space :
  case nl_or_page :
    for(;;) {
	getone();
	if(!isspace(_ch)) goto ANOTHER;
    }
  case onedigit : return(getnumber());
  case small_let:
  case cap_let :
       id :
	return(char_seq());

  case end_of_file : return((TOK*)0);

  case other_ch :
  case spec_ch :
    switch(_ch) {
      case '\'':getone();
		switch(chclass) {
		  char charval;

		  case nl_or_page:
		  case end_of_file: error();
		  default :
		    charval= (_ch=='\\') ? one_escape_sequence() : _ch;
		    getone();
		    if(_ch != '\'') error();
		    else getone();
		    tok=(TOK*)MEM_AllocGStack(sizeof(unsigned char)+sizeof(char));
		    tok->fno=tok_char;
		    tok->u.char_0=charval;
		    return(tok);
		}
     case '"' : return(getstring());

     case '/' :	getone();
		if ( _ch == '*' )
		  { int comcount = 1;
		    for (;;)
		     { if (!*txtptr) error();
		       if ((*txtptr)=='*')
		         if ((*++txtptr)=='/')
		           { txtptr++;
		             if (--comcount) continue;
		             getone(); goto ANOTHER;
		           }
		         else continue;

		       if ((*txtptr)=='/')
		         { if ((*++txtptr)=='*') { txtptr++; comcount++; } }
		       else txtptr++;
		    }
		  }
		else
		  tokkind=tok_div;
		break;

     case '_':	goto id;
     case '(': tokkind=tok_lpar; break;
     case ')': tokkind=tok_rpar; break;
     case ',': tokkind=tok_comma; break;
     case '*': tokkind=tok_mult; break;
     case ':': tokkind=tok_colon; break;
     case '-': tokkind=tok_minus; break;
     case '+': tokkind=tok_plus; break;
     case '^': tokkind=tok_power; break;
     case '%':
	while ( (_ch=*txtptr++)!=0 && _ch!='\n' ) ;
	if(_ch) getone();
	chclass=CH_ClassTab[_ch];
	goto ANOTHER;
     default :
	tokkind=tok_nill; break;
     }
  }
  (tok=(TOK*)MEM_AllocGStack(sizeof(unsigned char)))->fno=tokkind;
  getone();
  return(tok);
}


/****************************************************************
		Scanner
****************************************************************/

void far scanner_0(STRING string,TOKL** tokl)
{
  TOKL **curtokl=tokl, *toklist;
  CURSORTOK *ctok;
  TOK *tok;

  txtptr=start=string;
  getone();
  for(;;) {
      tok=gettoken();
      if(tok) {
	(ctok=(CURSORTOK*)MEM_AllocGStack(sizeof(CURSORTOK)))->fno=cursortok_t;
	ctok->cursor=pos;
	ctok->tok=tok;
	(toklist=(TOKL*)MEM_AllocGStack(sizeof(TOKL)))->fno=LISTFNO;
	toklist->cursortok=ctok;
	*curtokl=toklist;
	curtokl=&toklist->next;
      }
      else {
        (*curtokl=(TOKL*)MEM_AllocGStack(sizeof(unsigned char)))->fno=NILLFNO;
	return;
      }
  }
}

