/**************************************************************************
	COINS_D.PRO

	Deterministic version of the coin changing problem

	Copyright (c) 1986,1990 by Prolog Development Center
**************************************************************************/


DOMAINS
coinlist = integer*

PREDICATES
make(integer,integer,coinlist,coinlist)
max_in(coinlist, integer)
delete(integer,coinlist,coinlist)
addcoins(integer,integer,coinlist)
append(coinlist,coinlist,coinlist)

CLAUSES
make(0,0,[],[]):- !.
make(Amount,Count,From,With):- 
	max_in(From,Coin),
	delete(Coin,From,NewFrom),
	Max_coin_no = Amount div Coin,
	New_Amount = Amount-(Max_coin_no*Coin),
	make(New_Amount,SubCount,NewFrom,NewWith),
	addcoins(Max_Coin_No,Coin,UsedCoinlist),
	Count = SubCount+Max_coin_no,
	append(UsedCoinlist,NewWith,With).

max_in([X],X):- !.
max_in([H|T],H):- max_in(T,R), R<H, !.
max_in([H|T],R):- max_in(T,R), H<=R.

delete(_,[],[]).
delete(X,[X|Y],Y):- !.
delete(X,[A|Y],[A|Z]):- delete(X,Y,Z).

append([],X,X).
append([A|X],Y,[A|Z]):- append(X,Y,Z).

addcoins(0,_,[]):- !.
addcoins(No,Value,[Value|Z]):- No1=No-1, addcoins(No1,Value,Z).

GOAL
	make(573,N,[100,50,20,10,5,2,1],CL),
	write(N," coins:\nCL=",CL,"\n\n"),
	fail. % Force backtracking in an attempt to find more then one sol'n
