/**************************************************************************
	PLANNAST.PRO
	
	Planning problem using the "nasty" procedural approach

	Copyright (c) 1986,1990 by Prolog Development Center
**************************************************************************/

DOMAINS
process  = string
path     = process*
time     = real
info     = p(time,path)
infolist = info*

PREDICATES

nondeterm follows(process,process)  /* which process is followed by which */
duration(process,time)              /* the duration of each process */
start2(process,time,path)
getinfolist(path,infolist)
greedy(infolist,info,info)

CLAUSES

start2(P,Time,[P|Path]):-
      findall(P1, follows(P1,P), Plist),
      getinfolist(Plist,InfoList),
      greedy(InfoList,p(0,[]),p(Time,Path)), !.
start2(P,0,[P]).

getinfolist( [], [] ):-!.
getinfolist([P|Rest], [p(Tstop,List)|RestInfo ]):-
      start2(P,Tstart,List), /* P is the head of List*/
      duration(P,T),
      Tstop=Tstart+T,
      getinfolist( Rest, RestInfo ).

greedy([],Info,Info):-!.
greedy([p(T1,P1)|InfoL],p(Time,_),BestInfo):-
        T1>Time,!,
        greedy(InfoL,p(T1,P1),BestInfo).
greedy([_|InfoL],OldInfo,BestInfo):-
        greedy(InfoL,OldInfo,BestInfo).

follows(make_wine,store_wine).
follows(store_wine,get_opener).
follows(get_opener,open_wine).
follows(open_wine,say_toast).
follows(say_toast,drink_wine).
follows(make_food,serve_food).
follows(send_invitations,wait_for_friends).
follows(serve_food,say_toast).
follows(wait_for_friends,say_toast).

duration(make_wine,100).
duration(store_wine,300).
duration(get_opener,2).
duration(open_wine,1).
duration(say_toast,1).
duration(drink_wine,2).
duration(prepare_food,10).
duration(serve_food,10).
duration(send_invititations,10).
duration(wait_for_friends,5).
