/**************************************************************************
	PLANNICE.PRO

	Planning Example using the "nice" declarative approach

	Copyright (c) 1986,1990 by Prolog Development Center
**************************************************************************/

DOMAINS
process  = string
path     = process*
time     = real

PREDICATES
nondeterm follows(process,process)  /* which process is followed by which*/
duration(process,time)              /* the duration for each process */
start(process,time,path)
stop(process,time,path)
later_start_needed(process,time)

CLAUSES
start(Process,0,[Process]):-
     not(follows(_,Process)), !.
start(P2,ShiftTime, [P2 | Path_to_P1] ):-
     follows(P1,P2),
     stop(P1,ShiftTime,Path_to_P1),
     not(later_start_needed(P2,ShiftTime)), !.

stop(P,Tstop,List):-
     start(P,Tstart,List),
     duration(P,Duration),
     Tstop = Tstart + Duration.

later_start_needed(Pcheck,CheckTime):-
     follows(P,Pcheck),
     stop(P,FinishP,_),
     FinishP > CheckTime, !.

follows(make_wine,store_wine).
follows(store_wine,get_opener).
follows(get_opener,open_wine).
follows(open_wine,say_toast).
follows(say_toast,drink_wine).
follows(make_food,serve_food).
follows(send_invitations,wait_for_friends).
follows(serve_food,say_toast).
follows(wait_for_friends,say_toast).

duration(make_wine,100).
duration(store_wine,300).
duration(get_opener,2).
duration(open_wine,1).
duration(say_toast,1).
duration(drink_wine,2).
duration(prepare_food,10).
duration(serve_food,10).
duration(send_invititations,10).
duration(wait_for_friends,5).
