/**************************************************************************
	SORTTEST.PRO

	Program to Compare quicksort with "reference" sort

	Copyright (c) 1986,1990 by Prolog Development Center
**************************************************************************/

        /************************************************/
        /*               QUICKSORT                      */
        /************************************************/

DOMAINS
  element=integer
  list=element*

PREDICATES
  quicksort(list,list)
  split(element,list,list,list)
  append(list,list,list)

CLAUSES
  quicksort([],[]).
  quicksort( [H | Tail],Sorted):-
        split(H,Tail,Before,After),
        quicksort(Before,BeforeSorted),
        quicksort(After,AfterSorted),
        append(BeforeSorted,[H|AfterSorted],Sorted).

  split(_,[],[],[]).
  split(H,[A|Tail],[A|Before],After):-
        A < H, !,
        split(H,Tail,Before,After).
  split(H,[A|Tail],Before,[A|After]):-
        split(H,Tail,Before,After).

  append([],L,L).
  append([X|L1],L2,[X|L3]):-
          append(L1,L2,L3).


        /************************************************/
        /*                  RSORT                       */
        /************************************************/

DOMAINS
  tree    = reference t(element,tree,tree)

PREDICATES
  lookup(element,tree)
  nondeterm element(element,tree)
  convert(list,tree)
  rsort(list,list)

CLAUSES
  lookup(Element,Tree):-   /* Insert Element as a new node, when at new leaf*/
        free(Tree), !, Tree=t(Element,_,_).

  lookup(Actual,t(Element,LeftSubTree,_)):-
        bound(Element),
        Actual <= Element, !,
        lookup(Actual,LeftSubTree).

  lookup(Actual,t(_,_,RightSubTree)):-
        lookup(Actual,RightSubTree).

  convert([H|Tail],Tree):-
        lookup(H,Tree),
        convert(Tail,Tree).
  convert([],_).

  element(_,Tree):-
        free(Tree), !, fail. /* no more on this level */

  element(X,t(_,Left,_)):-
        element(X,Left).  /* search left sub tree before the right sub tree */

  element(X,t(X,_,_)).    /* done left tree - now do element */

  element(X,t(_,_,Right)):-
        element(X,Right). /* search right sub tree */

  rsort(L,Sorted):-
        convert(L,Tree),
        findall(Elem,element(Elem,Tree),Sorted).


        /************************************************/
        /*               QUICKSORT                      */
        /************************************************/

PREDICATES
  genl(integer,list)
  nondeterm test(string,list)
  report(string,real,real,real,real,real,real,integer,integer,integer,integer,
         integer,integer,integer,integer)
  runtest

CLAUSES
  genl(0,[]):- !.
  genl(N,[Numb|Rest]):- 
        random(32767,Numb),
        N1=N-1,
        genL(N1,Rest).

  runtest:-
        write("Number of elements to be sorted? "),
        readint(N), nl,
        writef("%10  %12   %12   %12  %12\n","",
               "Stack","Heap","Trail","Time used\n"),
        genl(N,L),
        test(_,L),
        fail.
  runtest:- nl, runtest.

  test(quicksort,L):-
        storage(S,H,T),
        time(Hour,Min,Sec,Hun),
        quicksort(L,_),
        time(Hour2,Min2,Sec2,Hun2),
        storage(S2,H2,T2),
        report(quicksort,S,H,T,S2,H2,T2,Hour,Min,Sec,Hun,Hour2,Min2,Sec2,Hun2).

  test(rsort,L):-
        storage(S,H,T),
        time(Hour,Min,Sec,Hun),
        rsort(L,_),
        time(Hour2,Min2,Sec2,Hun2),
        storage(S2,H2,T2),
        report(rsort,S,H,T,S2,H2,T2,Hour,Min,Sec,Hun,Hour2,Min2,Sec2,Hun2).

  report(Name,S,H,T,S2,H2,T2,Hour,Min,Sec,Hun,Hour2,Min2,Sec2,Hun2):-
        Stack=S-S2,Heap=H-H2,Trail=T-T2,
        Secs=((Hour2-Hour)*60+Min2-Min)*60+Sec2-Sec+(Hun2-Hun)/100,
        writef("%10: %8 bytes %8 bytes %8 bytes %6.2 secs\n",
               Name,Stack,Heap,Trail,Secs), nl.

GOAL runtest.
