/**************************************************************************
  SORT_R.PRO

  Sort based on reference variables
	
  A binary sorting predicate using REFERENCE variables. Notice that rsort
  may be changed so that duplicates are NOT inserted in the sorted tree or
  list. This will improve speed as well as decrease heap requirements.
  To make this 'optimization' replace the first two lookup clauses by:

        lookup(Element,t(Element,_,_):- !.
        lookup(Actual,t(Element,LeftSubTree,_)):-
                bound(Element),
                Actual <= Element, !,
                lookup(Actual,LeftSubTree).


	Copyright (c) 1986,1990 by Prolog Development Center
*********************************************************************/

diagnostics
DOMAINS
  tree          = reference t(element,tree,tree)
  element = integer
  list    = element*

PREDICATES
  lookup(element,tree)
  nondeterm element(element,tree)
  convert(list,tree)
  rsort(list,list)

CLAUSES
  lookup(Element,Tree):-   /* Insert Element as a new node, when at new leaf*/
        free(Tree), !, Tree=t(Element,_,_).

  lookup(Actual,t(Element,LeftSubTree,_)):-
        bound(Element),
        Actual <= Element, !,
        lookup(Actual,LeftSubTree).

  lookup(Actual,t(_,_,RightSubTree)):-
        lookup(Actual,RightSubTree).

  convert([H|Tail],Tree):-
        lookup(H,Tree),
        convert(Tail,Tree).
  convert([],_).

  element(_,Tree):-
        free(Tree), !, fail. /* no more on this level */

  element(X,t(_,Left,_)):-
        element(X,Left).  /* search left sub tree before the right sub tree */

  element(X,t(X,_,_)).    /* done left tree - now do element */

  element(X,t(_,_,Right)):-
        element(X,Right). /* search right sub tree */

  rsort(L,Sorted):-
        convert(L,Tree),
        findall(Elem,element(Elem,Tree),Sorted).
