/*
============================================================================
	THEO.PRO 

	Resolution-based Theorum prover

	Copyright (c) 1986, 1990 by Prolog Development Center
============================================================================
*/

code = 2000
DOMAINS

STR		= reference STRING
CLAUSELIST	= CLAUSE*
CLAUSE		= reference LITERAL*
LITERAL		= atom(STR,TERMLIST) ; 
		  negatom(STR,TERMLIST)
TERMLIST	= TERM*
TERM		= var(STR) ; cmp(STR,TERMLIST)


STERMLIST	= STERM*
STERM		= var(STRING) ; 
		  cmp(STRING,STERMLIST)

FORMULA	=	  atomformula(STRING,STERMLIST); 
		  negformula(FORMULA);
		  impl(FORMULA,FORMULA); 
		  iden(FORMULA,FORMULA);
		  and_(FORMULA,FORMULA); 
		  or_(FORMULA,FORMULA);
		  all(STERM,FORMULA); 
		  ex(STERM,FORMULA)	

TOKLIST	= TOK*

TOK		= name(STRING)	; lpar()	; rpar()	;
		negsign()	; implsign()	; identsign() 	;
		andsign()	; or_sign()	; allsign()	;
		exsign()	; comma()	; dot()
		
CONTINF	= cont() ; fini()

/******************************************************************************
		DATABASE MANAGEMENT
******************************************************************************/

DATABASE
source(STRING)
clause(CLAUSE)

PREDICATES
nondeterm nd_retract(DBASEDOM)
makedb(CLAUSELIST)
cleardb
retract_once(DBASEDOM)

CLAUSES
source("").

nd_retract(P):- retract_once(P).
nd_retract(P):- assert(P).

makedb([C|T]):- assert(clause(C)), makedb(T).
makedb([]).

cleardb:- retract(clause(_)), fail.
cleardb.

retract_once(P):- retract(P), !.

/**************************************************************
			SCANNER
**************************************************************/
PREDICATES
make_tokl(TOKLIST)
make_tokl1(STRING,TOKLIST)
make_token(STRING,TOK,STRING,STRING)
rest_name(STRING,TOK)
exitcheck(CHAR)

CLAUSES
make_tokl(Tl):-
	retract(source(L)), !,
	edit(L,L1), assert(source(L1)), make_tokl1(L1,Tl).

make_tokl1(L,[Tok|Tokl]):-
	fronttoken(L,T,L1),
	make_token(T,Tok,L1,L2), !,
	make_tokl1(L2,Tokl).
make_tokl1(_,[]).

make_token("(",lpar(),L,L):- !.
make_token(")",rpar(),L,L):- !.
make_token("~",negsign(),L,L):- !.
make_token("&",andsign(),L,L):- !.
make_token("#",or_sign(),L,L):- !.
make_token(",",comma(),L,L):- !.
make_token(".",_,_,_):- !, fail.
make_token("<",identsign(),L,L0):-
	fronttoken(L,"=",L1), fronttoken(L1,">",L0), !.
make_token("=",implsign(),L,L0):- fronttoken(L,">",L0), !.
make_token(S,Tok,L,L):- isname(S), !, rest_name(S,Tok).
make_token(S,_,_,_):-
	writef("\nUnrecognized symbol, parsing stopped at %\n",S), fail.

rest_name(All,allsign()):- upper_lower(All,"all"),!.
rest_name(Exists, exsign()):- upper_lower(Exists,"exists"),!.
rest_name(S,name(S)):- !.

exitcheck('q'):- !, exit(0).
exitcheck(_).

/******************************************************************
			SYNTAX ANALYSIS
******************************************************************/

PREDICATES
s_formula0(TOKLIST,TOKLIST,FORMULA)
s_formula1(TOKLIST,TOKLIST,FORMULA)
s_formula2(TOKLIST,TOKLIST,FORMULA)
s_formula3(TOKLIST,TOKLIST,FORMULA)
s_formula4(TOKLIST,TOKLIST,FORMULA)
s_formula5(TOKLIST,TOKLIST,FORMULA)
rest_formula0(TOKLIST,TOKLIST,FORMULA,FORMULA)
rest_formula1(TOKLIST,TOKLIST,FORMULA,FORMULA)
rest_formula2(TOKLIST,TOKLIST,FORMULA,FORMULA)
rest_formula3(TOKLIST,TOKLIST,FORMULA,FORMULA)
rest_s_terml(TOKLIST,TOKLIST,STERMLIST)
s_termlist(TOKLIST,TOKLIST,STERMLIST)
s_term(TOKLIST,TOKLIST,STERM)
s_all_var(TOKLIST,TOKLIST,STRING)
s_ex_var (TOKLIST,TOKLIST,STRING)


CLAUSES

s_formula0(Tokl1,Tokl2,Formula):-
	s_formula1(Tokl1,Tokl3,Form1),
	rest_formula0(Tokl3,Tokl2,Form1,Formula).

s_formula1(Tokl1,Tokl2,Formula):-
	s_formula2(Tokl1,Tokl3,Form1),
	rest_formula1(Tokl3,Tokl2,Form1,Formula).

s_formula2(Tokl1,Tokl2,Formula):-
	s_formula3(Tokl1,Tokl3,Form1),
	rest_formula2(Tokl3,Tokl2,Form1,Formula).

s_formula3(Tokl1,Tokl2,Formula):-
	s_formula4(Tokl1,Tokl3,Form1),
	rest_formula3(Tokl3,Tokl2,Form1,Formula).

s_formula4([negsign()|Tokl1],Tokl2,negformula(Formula)):-
	!,
	s_formula5(Tokl1,Tokl2,Formula).
s_formula4(Tokl1,Tokl2,Formula):-
	s_formula5(Tokl1,Tokl2,Formula).


s_formula5(Tokl1,Tokl3, all(var(Var),Formula)):-
	s_all_var(Tokl1,Tokl2,Var), !,
	s_formula5(Tokl2,Tokl3,Formula).
s_formula5(Tokl1,Tokl3, ex(var(Var),Formula)):-
	s_ex_var(Tokl1,Tokl2,Var), !,
	s_formula5(Tokl2,Tokl3,Formula).
s_formula5([name(Pid)|Tokl1],Tokl2,atomformula(Pid,Terml)):-
	!,
	s_termlist(Tokl1,Tokl2,Terml).
s_formula5([lpar()|Tokl1],Tokl3,Formula):-
	s_formula0(Tokl1,Tokl2,Formula),
	Tokl2=[rpar()|Tokl3].

rest_formula0([identsign()|Tokl1], Tokl2, Forma,iden(Forma,Formb)):-
	!,
	s_formula0(Tokl1,Tokl2,Formb).
rest_formula0(Tokl,Tokl,Form,Form).

rest_formula1([implsign()|Tokl1], Tokl2, Forma,impl(Forma,Formb)):-
	!,
	s_formula1(Tokl1,Tokl2,Formb).
rest_formula1(Tokl,Tokl,Form,Form).

rest_formula2([or_sign()|Tokl1], Tokl2, Forma,or_(Forma,Formb)):-
	!,
	s_formula2(Tokl1,Tokl2,Formb).
rest_formula2(Tokl,Tokl,Form,Form).

rest_formula3([andsign()|Tokl1], Tokl2, Forma,and_(Forma,Formb)):-
	!,
	s_formula3(Tokl1,Tokl2,Formb).
rest_formula3(Tokl,Tokl,Form,Form).

s_termlist([lpar()|Tokl1], Tokl4, [Termh|Termt]):-
	s_term(Tokl1,Tokl2,Termh), !,
	rest_s_terml(Tokl2,Tokl3,Termt), Tokl3=[rpar()|Tokl4].
s_termlist(Tokl,Tokl,[]).

rest_s_terml([comma()|Tokl1],Tokl3,[Term|Terml]):-
	!, s_term(Tokl1,Tokl2,Term), rest_s_terml(Tokl2,Tokl3,Terml).
rest_s_terml(Tokl,Tokl,[]).

s_term([name(Fid)|Tokl],Tokl0,cmp(Fid,Termlist)):-
	frontchar(Fid,C,_), C>='a', C<='z', !,
	s_termlist(Tokl,Tokl0,Termlist).
s_term([name(Vid)|Tokl],Tokl,var(Vid)).


s_all_var([allsign(),name(Vid)|Tokl], Tokl, Vid):- !.
s_all_var([lpar(),allsign(),name(Vid),rpar()|Tokl], Tokl, Vid).


s_ex_var([exsign(),name(Vid)|Tokl], Tokl, Vid):- !.
s_ex_var([lpar(),exsign(),name(Vid),rpar()|Tokl], Tokl, Vid).


/******************************************************************/
/*                                                                */
/*                 TRANSFORMATION TO CLAUSE FORM                  */
/*                                                                */
/******************************************************************/


/******************************************************************/
/*				STEP 1				  */
/*								  */
/*  MOVE NEGATIONS INWARD, TRANSFORM IMPLICATIONS AND IDENTITIES  */
/******************************************************************/


PREDICATES

negin_impldel(FORMULA,FORMULA)
neg_impldel(FORMULA,FORMULA)

CLAUSES

negin_impldel(negformula(Form), Form_tr):- !,
	neg_impldel(Form,Form_tr).
negin_impldel(impl(Forma,Formb), or_(Forma_neg_tr,Formb_tr)):- !,
	neg_impldel(Forma,Forma_neg_tr),
	negin_impldel(Formb,Formb_tr).
negin_impldel( iden(Forma,Formb),
		and_( or_(Forma_neg_tr,Formb_tr),
		   or_(Forma_tr,Formb_neg_tr) ) ) :- !,
	negin_impldel(Forma,Forma_tr),
	negin_impldel(Formb,Formb_tr),
	neg_impldel  (Forma,Forma_neg_tr), 
	neg_impldel  (Formb,Formb_neg_tr).
negin_impldel( and_(Forma,Formb), and_(Forma_tr,Formb_tr)):- !,
	negin_impldel(Forma,Forma_tr), 
	negin_impldel(Formb,Formb_tr).
negin_impldel( or_(Forma,Formb),  or_(Forma_tr,Formb_tr)) :- !,
	negin_impldel(Forma,Forma_tr), 
	negin_impldel(Formb,Formb_tr).
negin_impldel(all(Vid,Form), all(Vid,Form_tr)):- !,
	negin_impldel(Form,Form_tr).
negin_impldel(ex(Vid,Form), ex(Vid,Form_tr)):- !,
	negin_impldel(Form,Form_tr).
negin_impldel( atomformula(Str,Terml), atomformula(Str,Terml)).


neg_impldel(negformula(Form), Form_tr)  :- !,
	negin_impldel(Form,Form_tr).
neg_impldel(iden(Forma,Formb), 
	   or_( and_(Forma_neg_tr,Formb_tr),
	        and_(Forma_tr,Formb_neg_tr))):- !,
	negin_impldel(Forma,Forma_tr),
	negin_impldel(Formb,Formb_tr),
	neg_impldel  (Forma,Forma_neg_tr),
	neg_impldel  (Formb,Formb_neg_tr).
neg_impldel(impl(Forma,Formb),and_(Forma_tr,Formb_neg_tr)):- !,
	negin_impldel(Forma,Forma_tr),
	neg_impldel(Formb,Formb_neg_tr).
neg_impldel(and_(Forma,Formb), or_(Forma_neg_tr,Formb_neg_tr)):- !,
	neg_impldel(Forma,Forma_neg_tr),
	neg_impldel(Formb,Formb_neg_tr).
neg_impldel(or_(Forma,Formb), and_(Forma_neg_tr,Formb_neg_tr)):- !,
	neg_impldel(Forma,Forma_neg_tr),
	neg_impldel(Formb,Formb_neg_tr).
neg_impldel(all(Vid,Form), ex(Vid,Form_neg_tr)):- !,
	neg_impldel(Form,Form_neg_tr).
neg_impldel(ex(Vid,Form), all(Vid,Form_neg_tr)):- !,
	neg_impldel(Form,Form_neg_tr).
neg_impldel(atomformula(Str,Terml), negformula(atomformula(Str,Terml))).


/******************************************************************/
/*				STEP 2				  */
/*								  */
/*			    SKOLEMIZE			  	  */
/******************************************************************/

PREDICATES

skolem(FORMULA,FORMULA,STERMLIST,INTEGER,INTEGER)
subst(STERM,STERM,FORMULA,FORMULA)
subst_terml(STERM,STERM,STERMLIST,STERMLIST)
subst_term (STERM,STERM,STERM,STERM )

CLAUSES

skolem(all(Vid,Form), all(Vid,Form_tr), Ext_var,Ii,Io):- !, 
	skolem(Form,Form_tr,[Vid|Ext_var],Ii,Io).
skolem( ex(Vid,Form), Form_tr2, Ext_var,Ii,Io):- !,
	str_int(S,Ii),
	concat("skol",S,F),
	subst(Vid,cmp(F,Ext_var),Form,Form_tr1),
	Inext = Ii+1,
	skolem(Form_tr1,Form_tr2,Ext_var,Inext,Io).
skolem(and_(Forma,Formb), and_(Forma_tr,Formb_tr), Ext_var,Ii,Io):- !,
	skolem(Forma,Forma_tr,Ext_var,Ii,Io1),
	skolem(Formb,Formb_tr,Ext_var,Io1,Io).
skolem(or_(Forma,Formb), or_(Forma_tr,Formb_tr), Ext_var,Ii,Io2):- !,
	skolem(Forma,Forma_tr,Ext_var,Ii,Io1),
	skolem(Formb,Formb_tr,Ext_var,Io1,Io2).
skolem(F,F,_,I,I).


subst(Vid,F,atomformula(Str,Terml),atomformula(Str,Terml_su)):- !,
	subst_terml(Vid,F,Terml,Terml_su).
subst(Vid,F,negformula(Form),negformula(Form_su)):- !,
	subst(Vid,F,Form,Form_su).
subst(Vid,F,and_(Forma,Formb),and_(Forma_su,Formb_su)):- !,
	subst(Vid,F,Forma,Forma_su), subst(Vid,F,Formb,Formb_su).
subst(Vid,F, or_(Forma,Formb), or_(Forma_su,Formb_su)):- !,
	subst(Vid,F,Forma,Forma_su), subst(Vid,F,Formb,Formb_su).
subst(Vid,F,all(Var,Form),all(Var,Form_su)):- !,
	subst(Vid,F,Form,Form_su).
subst(Vid,F, ex(Var,Form), ex(Var,Form_su)):-
	subst(Vid,F,Form,Form_su).
 

subst_terml(_,_,[],[]).
subst_terml(Vid,F,[Th|Tt],[Th_su|Tt_su]):-
	subst_term(Vid,F,Th,Th_su), 
	subst_terml(Vid,F,Tt,Tt_su).

subst_term(Vid,F,Vid,F):- !.
subst_term(_,_,var(V),var(V)).
subst_term(Vid,F,cmp(Str,Terml),cmp(Str,Terml_su)):-
	subst_terml(Vid,F,Terml,Terml_su).


/******************************************************************/
/*				STEP 3				  */
/*								  */
/*		REMOVE UNIVERSAL QUANTIFIERS		  	  */
/******************************************************************/

PREDICATES
univout(FORMULA,FORMULA)

CLAUSES

univout(all(_,Fa),Fb):- !,
	univout(Fa,Fb).
univout(and_(Fa1,Fb1), and_(Fa2,Fb2)):- !,
	univout(Fa1,Fa2),
	univout(Fb1,Fb2).
univout( or_(Fa1,Fb1), or_(Fa2,Fb2)):- !,
	univout(Fa1,Fa2),
	univout(Fb1,Fb2).
univout(F,F).


/******************************************************************/
/*				STEP 4				  */
/*								  */
/*			DISTRIBUTE & OVER or		  	  */
/******************************************************************/

PREDICATES

conjn(FORMULA,FORMULA)
conjnl(FORMULA,FORMULA)

CLAUSES

conjn(or_(Fp,Fq), Fr):- !,
	conjn(Fp,Fp1),
	conjn(Fq,Fq1),
	conjnl(or_(Fp1,Fq1),Fr).
conjn(and_(Fp,Fq), and_(Fp1,Fq1)) :- !,
	conjn(Fp,Fp1),
	conjn(Fq,Fq1).
conjn(F,F).

conjnl(or_( and_(Fp,Fq), Fr), and_(Fp1,Fq1)):- !,
	conjn(or_(Fp,Fr),Fp1),
	conjn(or_(Fq,Fr),Fq1).
conjnl(or_( Fp, and_(Fq,Fr)), and_(Fp1,Fq1)):- !,
	conjn(or_(Fp,Fq),Fp1),
	conjn(or_(Fp,Fr),Fq1).
conjnl(F,F).


/******************************************************************/
/*				STEP 5				  */
/*								  */
/*		TRANSFER FORMULA TO CLAUSE DOMAIN	  	  */
/******************************************************************/

PREDICATES

tr_term(STERM,TERM)
tr_terml(STERMLIST,TERMLIST)
inclauses(FORMULA,CLAUSELIST,CLAUSELIST)
list_lit(FORMULA,CLAUSE,CLAUSE)

CLAUSES
tr_terml([Term1|T1],[Term2|T2]):- tr_term(Term1,Term2), tr_terml(T1,T2).
tr_terml([],[]).

tr_term(cmp(S_Id,S_Tl),cmp(Id,Tl)):- Id=S_Id, tr_terml(S_Tl,Tl).
tr_term(var(S_Id),var(Id)):- Id=S_Id.

inclauses(and_(Fp1,Fq1),Cl,Cl2):- !,
	inclauses(Fp1,Cl ,Cl1),
	inclauses(Fq1,Cl1,Cl2).    
inclauses(or_(Fp1,Fp2),Cl,[Litl2|Cl]):- !,
	list_lit(Fp1,[]   ,Litl1),
	list_lit(Fp2,Litl1,Litl2).
inclauses(atomformula(S,T),Cl,[[atom(S1,T1)]|Cl]):- !,
	S1=S, tr_terml(T,T1).
inclauses(negformula(atomformula(S,T)),Cl,[[negatom(S1,T1)]|Cl]):-
	S1=S, tr_terml(T,T1).

list_lit(or_(Fp,Fq),Litl,Litl2):- !,
	list_lit(Fp,Litl ,Litl1),
	list_lit(Fq,Litl1,Litl2).
list_lit(negformula(atomformula(Str,Terml)), L,[negatom(Str1,Terml1)|L]):- !,
	Str1=Str, tr_terml(Terml,Terml1).
list_lit(atomformula(S,T),L,[atom(S1,T1)|L]):-
	S1=S, tr_terml(T,T1).


/******************************************************************************
		ENVIRONMENT MANAGEMENT
******************************************************************************/

DOMAINS
ENV_MEMBER = e(STR,TERM)
ENV = reference ENV_MEMBER*

PREDICATES
member(STR,TERM,ENV)

CLAUSES
member(Id,Term,[e(Id,Term)|_]):- !.
member(Id,Term,[_|Tail]):- member(Id,Term,Tail).


/******************************************************************************
		RESOLUTION
******************************************************************************/

PREDICATES
simplify
res_itself(CLAUSE,ENV)
nondeterm resolve(ENV)
nondeterm res_clauses(CLAUSE,CLAUSE,CLAUSE,CLAUSE,ENV)
efface_neg(LITERAL,CLAUSE,CLAUSE,ENV)
neg_unify(LITERAL,LITERAL,ENV)
unify_termlist(TERMLIST,TERMLIST,ENV)
unify_term(TERM,TERM,ENV)
clean_up

CLAUSES
simplify:-
	free(E),
	clause(C),
	res_itself(C,E),
	retract(clause(C)),
	fail.
simplify.

res_itself([Atom|T],Env):- efface_neg(Atom,T,_,Env), !.
res_itself([_|T],Env):- res_itself(T,Env).

resolve(Env):- 
	clause(C1), clause(C2), not(C1=C2),
	res_clauses(C1,C2,R_C1,R_C2,Env),
	nd_retract(clause(C1)), nd_retract(clause(C2)),
	assert(clause(R_C1)), assert(clause(R_C2)),
	clean_up,
	resolve(Env).
resolve(_):-
	clause(_), !, write("\nNo; unable to resolve to empty clause.\n").
resolve(_):- write("\nYes; empty clause produced.\n").

res_clauses([Atom|T],Cl,T,E_Cl,Env):- efface_neg(Atom,Cl,E_Cl,Env).
res_clauses([Atom|T],Cl,[Atom|T1],E_Cl,Env):- res_clauses(T,Cl,T1,E_Cl,Env).

efface_neg(Atom,[Atom1|T1],T1,Env):- neg_unify(Atom,Atom1,Env), !.
efface_neg(Atom,[Atom1|T1],[Atom1|T2],Env):- efface_neg(Atom,T1,T2,Env).

neg_unify(atom(Id,TL1),negatom(Id,TL2),Env):- !, unify_termlist(TL1,TL2,Env).
neg_unify(negatom(Id,TL1),atom(Id,TL2),Env):- unify_termlist(TL1,TL2,Env).

unify_termlist([Term1|T1],[Term2|T2],Env):- !,
	unify_term(Term1,Term2,Env), unify_termlist(T1,T2,Env).
unify_termlist([],[],_).

unify_term(cmp(Id,TL1),cmp(Id,TL2),Env):- !, unify_termlist(TL1,TL2,Env).
unify_term(var(Id1),var(Id2),Env):- !,
	member(Id1,Term1,Env), member(Id2,Term2,Env), Term1=Term2.
unify_term(var(Id),Term,Env):- !, member(Id,T,Env), T=Term.
unify_term(Term,var(Id),Env):- member(Id,T,Env), T=Term.

clean_up:- retract(clause([])), fail.
clean_up.


/******************************************************************/
/*			MAIN LOOP  and	PRINTOUT		  */
/*								  */
/******************************************************************/

PREDICATES

print_clauses(CLAUSELIST,INTEGER)
print_clause(CLAUSE)
print_lit(LITERAL)
print_terml(TERMLIST)
print_terml1(TERMLIST)
print_term (TERM)
write_env(ENV)
wenv(ENV)
run


CLAUSES

print_clauses([],_).
print_clauses([Ch|Ct],N):- 
	writef("%3: ",N), NN=N+1, print_clause(Ch), print_clauses(Ct,NN).

print_clause([]):- !, nl.
print_clause([Lh|Lt]):-
	print_lit(Lh), write("  "), print_clause(Lt).

print_lit(atom(Str,Terml)):- !, write(Str), print_terml(Terml).
print_lit(negatom(Str,Terml)):- write('~',Str), print_terml(Terml).

print_terml([]):- !.
print_terml(TL):- write('('), print_terml1(TL), write(')').

print_terml1([Th]):- print_term(Th), !.
print_terml1([Th|Tt]):- print_term(Th), write(','), print_terml1(Tt).

print_term(T):- free(T), !, write('_').
print_term(var(Str)):- !, write(Str).
print_term(cmp(Str,Terml)):- write(Str), print_terml(Terml).

write_env(E):- free(E), !.
write_env(E):- write("\nVariable instantiations:\n"), wenv(E).

wenv([e(Id,Term)|T]):- free(T), !, write(Id,'='), print_term(Term), nl.
wenv([e(Id,Term)|T]):-
	write(Id,'='), print_term(Term), write(", "), wenv(T).


run:-	make_tokl(Tokl1),
	s_formula0(Tokl1,Tokl2,Form1),
	Tokl2 = [],				/* all parsed */
	!,
	negin_impldel(Form1,Form2),		/* steps 1,2,3 */
	skolem(   Form2, Form3,[],1,_),		/* step 4 */
	univout(  Form3, Form4),		/* step 5 */
	conjn(    Form4, Form5),		/* step 6 */
	inclauses(Form5,[],Cl),			/* step 6 */
	nl, source(Source),
	write(Source,"\nClauses to be resolved:\n\n"), print_clauses(Cl,1),
	cleardb,
	makedb(Cl),
	simplify,
	free(Env),
	resolve(Env), !,
	write_env(Env),
	write("\nPress q to quit; SPACEBAR to continue."),
	readchar(C), exitcheck(C),
	!,
	run.
run:-
	write("Syntax Error.\nPress the SPACE bar"), readchar(_), run.

GOAL	
	textmode(R,C), R1=R-1,
	makewindow(2,71,71," Theorem prover dialog ",0,0,R1,C),
	run.
