/*****************************************************************************
	CHELP.PRO
	
	Help Text Compiler for TSRHELP 
	
	Note: Must be linked with gmem.obj, which implements the Gstack
	cutting global predicates that are used to facilitate the 
	program's use of a tail-recursive loop to process the file.
	
NOTE:   You can compile & link this under the IDE by specifying 
	GMEM.OBJ on the OPTIONS/LINK OPTIONS/LIBRARIES line as
       +GMEM.OBJ+TSRINT  (the linker doesn't care if it's not a library)

USAGE:  CHELP <SOURCEFILE>
	compiles the text in source file, adding it to TSRHELP.DBA, if
        one exists, or creating a new one if not.  Any topics with the
        same name as existing topics will overwrite the old one.

	See HELPDEMO.SRC for an example of the file format

	
 	Copyright (c) 1990 Prolog Development Center 
*****************************************************************************/
include "RESIDENT.PRE"

GLOBAL PREDICATES
 gstack_setup language c   
 cut_gstack language c 

DOMAINS
 file=infile
 db_selector=db
 sl=string*
  
topic = topic(string,string, integer, buttons)
buttons = button*
button = button(region, action)
region = region(integer,integer,integer,integer)
action = 
	jump_to(ref);
	paste(Region)

key = prev; 
	next; 
	push_button; 
	exit ; 
	index; 
	scroll(integer)

state_after_key = 
	pop_down;
	continue

resizemode = on;off

DATABASE
 reg(button)
 key(string,ref)
 determ index_is(bt_selector)
 determ report_time(real,real)

DATABASE - sticky_settings
 determ win_at(integer,integer, integer, integer)
 determ color(resizemode)
      
PREDICATES
 go
 read_loop(integer,string)
 make_dummy_if_new(string, bt_selector)
 process_line(string, integer, string, real)
 save_page(integer, string)
 trim(String,string)
 rtrim(String,string)
 is_keyword(String,string,string)
 de_arity(String,string)
 pass_loop
 create_new_or_add_to
 find_column_end(real,integer,integer)
 fce(integer, integer, integer)
 check_error(string) 
 nondeterm member(string,sl)
 nondeterm gen_toks(string,string)
 tokenize(string,sl)
 progress_report
 account_for_tabs(integer, integer, string, integer, integer)

CLAUSES
go :- 	
	signalResident(tsrhelp,9),!,
	write("Can't compile.\nTSRHELP is resident, and database is open!").
go :-
	comline(Filename),
	existfile(Filename),!,
	create_new_or_add_to,
	openread(infile, Filename),
	filepos(infile, 0, 2),
	filepos(infile, Fsize, 0),
	marktime(300, NextDue),
	asserta(report_time(NextDue, Fsize)),
	filepos(infile, 0,0),
	readdevice(infile),
	write("Pass1...\n"),
	pass_loop, 
	closefile(infile),
	readdevice(keyboard),
	write("\nCopying..."),
	db_copy(db, "tsrhelp.dba", in_file),
	db_close(db),
	deletefile("(temp).dba").
go:- 	write("File not found\nusage: chelp <source filename>").
	

create_new_or_add_to:-   % add to existing help database
	existfile("tsrhelp.dba"),!,
	renamefile("tsrhelp.dba", "(temp).dba"),
	db_open(db, "(temp).dba", in_file),
	bt_open(db,"topictree",Tree),
	asserta(index_is(Tree)).
create_new_or_add_to:-   % create new help database
	db_create(db, "(temp).dba", in_file),
	bt_create(db,"topictree",Tree,30,4),
	chain_insertz(db, user_prefers, sticky_Settings,
		win_At(0,0,24,80), _),
	chain_insertz(db, user_prefers, sticky_Settings,
		color(on), _),
	asserta(index_is(Tree)).

pass_loop:- 
	eof(infile),
	!,
	filepos(infile,0,0),
	write("Pass2...\n"),
	gstack_setup,
	read_loop(0,"").
pass_loop :- 
	readln(Line),
	is_keyword(Line, _,IndexKey),
	index_is(Tree),
	make_dummy_if_new(IndexKey, Tree),
	!, pass_loop.
pass_loop :- pass_loop.	

make_dummy_if_new(IndexKey,Tree):-
	key_Search(db, Tree, IndexKey, _),!.
make_dummy_if_new(IndexKey,Tree):-
	 % make dummy entry
	chain_insertz(db, topics, topic, topic(IndexKey, "", 0, []), Ref),
	 % and index entry
	key_insert(db, Tree, IndexKey, Ref).
		

read_loop(Row, Page) :-
	progress_report,
	not(eof(infile)),
	filepos(infile, LineStart,0),
	readln(Line),
	!,process_line(Line, Row, Page, LineStart).
read_loop(Row,PAge):- save_page(Row,Page).

	
process_line(Line,Row, Page, _) :- 
	is_keyword(Line, _, IndexKey),
	index_is(Tree),
	key_search(db, Tree, IndexKey, Ref),
	save_page(Row, Page), 	% save the last page
	asserta(key(IndexKey,Ref)),  % and start a new one
	!,
	cut_gstack, % kids, don't try this at home (without reading your PUG!)		
	read_loop(0,"").
process_line(Line,Row, _,_) :-
	fronttoken(Line,_,_),   % don't bother with empty lines
	gen_toks(Line, KeyWord), 
	str_len(KeyWord,Len), 
	Len>1,
	not(key(KeyWord,_)),  	% eg, don't reference self
	index_is(Tree),
	key_search(db, Tree, KeyWord, Ref),
	upper_lower(Line, LowLine),
	searchstring(LowLine,KeyWord,FoundPos),
	account_for_tabs(1, FoundPos, Line, 0, Col),
	str_len(KeyWord, Wide),
	assertz(reg(button(region(Row, Col, 1, Wide), jump_to(Ref)))),
	fail.
process_line(Line,Row, _, Restart) :-
	searchstring(Line, ">|", Pos), 
	account_for_tabs(1, Pos, Line, 0, ColTemp), 
	ColStart=ColTemp+2,
	find_column_end(Restart, ColEnd, High),
	
	Wide=ColEnd-ColStart,
	assertz(reg(button(region(Row, ColStart, High, Wide), 
		            paste(region(Row,ColStart,High,Wide))))),
	fail.  		            
process_line(Line,Row, Page,_) :- 
	Row2=Row+1, 
	rtrim(Line, ClippedLine),
	format(NewPage, "%s%s\n", Page, ClippedLine),
	!,read_loop(Row2, NewPage).


%% look-ahead to find the column-end marker
find_column_end(Restart, End, High) :-
	filepos(infile, Restart, 0),
	fce(1, End, High),
	filepos(infile, Restart, 0), %resync with sequential process
	readln(_).		
	
fce(N, Col, N) :-
	readln(Line),
	check_error(Line),
	searchstring(Line, "|<", Pos),!,% 1 based
	account_for_tabs(1, Pos, Line, 0, Col). 	% 1 past last column
fce(N, Col, High) :- N2=N+1, fce(N2, Col, High).


check_error(Line) :-
	is_keyword(Line,_,_),!,
	write("Error: Non-terminated paste block preceeding\n"),
	write(Line),
	exit(1).
check_error(_).
	
save_page(Row, Page) :-
	retract(key(Key, Ref)),!,
	findall(X, retract(reg(X)), Buttons),
	term_replace(db, topic, Ref, topic(Key, Page, Row, Buttons)).
save_page(_,_).
		
  		
%% remove leading & trailing blanks from a string
trim(A,B) :-
	frontchar(A,32, A2),
	!,trim(A2,B).
trim(A,B) :-
	frontchar(A,9, A2),
	!,trim(A2,B).
trim(A,B) :- rtrim(A,B).
  
rtrim(A, B) :-
	concat(A2, " ", A),
	!,rtrim(A2, B).
rtrim(A, B) :-
	concat(A2, "\t", A),
	!,rtrim(A2, B).
rtrim(A,A).


is_keyword(Line,KeyWord, IndexEntry) :-
	upper_lower(Line, LowLine),
	fronttoken(LowLine, "section", T1),
	fronttoken(T1, ":", Rest),
	trim(Rest, IndexEntry),
	de_arity(IndexEntry, Keyword).

de_arity(String, Pname) :-
	searchchar(String, '/', Pos),!,
	P=Pos-1,
	frontstr(P, String, Pname, _).
de_arity(Same, Same).  	
	
gen_toks(Line, Tok) :-
	upper_lower(Line,LowLine),
	tokenize(LowLine,List),
	member(Tok,List).

tokenize(String, [H|T]) :-
	fronttoken(String, H, Rest),
	!, tokenize(Rest, T).
tokenize(_,[]).

member(X,[Name,"/",Arity|_]) :- format(X,"%s/%s",Name,Arity).
member(X,[_,"/",_|Y]) :- !, member(X,Y). 	
member(X,[X|_]).
member(X,[_|Y]) :- member(X,Y).

progress_report :-
	report_time(TimeMark,Fsize),
	timeout(TimeMark),!,
	filepos(infile, Pos, 0),
	Pct = 100*(Pos/Fsize),
	writef("\r %2.0 %% complete", Pct),
	retract(report_time(_,_)),
	marktime(100, NextDue),
	asserta(report_time(NextDue, Fsize)).
progress_report.

account_for_tabs(Pos, Pos, _, Col, Col):-!. 
account_for_tabs(Pos, Limit, String, Col, Final) :-
	subchar(String, Pos, Char),
	Char=9,
	Col2=((Col+8) div 8) * 8,
	P2=Pos+1,
	!, account_for_tabs(P2, Limit, String, Col2, Final).
account_for_tabs(Pos, Lim, String, Col , F) :-
	P2=Pos+1, C2=Col+1,
	account_for_tabs(P2,Lim, String, C2, F).
	
goal
 go.  	
       		    
 
