/************************************************************************
	paster.pro

	Illustrates the facility to paste keystrokes back through
          the keyboard buffer to the foreground application

	Note that for pasting into an application like PDC Prolog
   	 that reads ahead into its own internal keyboard buffer,
	 the "keys_consumed" signal is received when Prolog has 
         emptied the DOS buffer, which can be slightly in advance 
         of it actually being done using the keys. (This is more
         noticeable the higher the paste speed).  

	 Also note that if pasting into the Prolog editor, you
         may wish to turn text mode off or the automatic indent
	 feature will cause double indenting (just as if, after
         pressing the enter key you tabbed or "spaced" your way
	 across prior to typing text, as is actually in the file).

     =>  If your editor "beeps" or otherwise complains while the paste
         is in progress, you may wish to slow down the paste speed	

	Copyright (C) 1990, Prolog Development Center
************************************************************************/

heap = $1000  

include "RESIDENT.PRE"
include "KEYSCAN.PRE"

CONSTANTS
 progname=paster

PREDICATES
 best_swap
 nl_to_cr(String,string)
 nl_to_cr_Aux(integer,integer)
 check_allStuffed 

GOAL
	enableResident(progname,0),
	best_Swap,
	installHotKey(1, key_p, leftAlt, needsDos_cantReenter),
	installSignal(keys_consumed, needsDos),
	nl,write("load an editor (any editor) and try this..."),
	nl,write("Left-Alt-P to activate"),
	stayResident.
	
CLAUSES
	hotkey(1) :-
		textmode(Rows, Cols),
		saveVideo(Rows, Cols, _,_),
		makewindow(1, 7,7, "File to paste", 2,2, 10,25),
		setKeyboardSense(on), %!!!!! dont forget this !!!
		dir("","*.*", File),!,
		file_Str(File,Str),
		nl_to_cr(Str, ReadyForPaste),
		removewindow,
		stuffBiosKeys(ReadyForPaste, 40). % 40 per sec
	hotkey(1) :-
		removewindow.
		
	signal(keys_consumed) :-
		textmode(Rows,Cols),		
		saveVideo(Rows,Cols,_,_),
		makewindow(1, 7,7,"press a key to continue", 5,5, 5, 41),
		write("received notice that keys are consumed"),
		check_allStuffed,	
		setKeyboardsense(on),
		readchar(_),
		removewindow.

	check_allstuffed:- allStuffed,!.
	check_allStuffed :- write("\n Stuff aborted by user keypress!").
	
	%% do this in place to save memory usage
	nl_to_cr(Text,Text) :-
		ptr_dword(Text, Seg, Off),
		nl_to_Cr_Aux(Seg, Off).

	nl_to_cr_Aux(Seg, Off) :- 
		membyte(Seg,Off,Byte), Byte=0,!.
	nl_to_cr_Aux(Seg, Off) :-
		membyte(Seg,Off,Byte), 
		Byte='\10',!,
		membyte(Seg, Off, '\13'),
		Off2=Off+1,
		nl_to_cr_Aux(Seg, Off2).
	nl_to_cr_Aux(Seg, Off) :-
		Off2=Off+1,
		nl_to_cr_Aux(Seg, Off2).
		

	best_Swap :- 
		enableSwap(swapToEms, "","", Status),
		Status=0,!,
		write(progname, " installed for EMS swap").
	best_Swap :- 
		enableSwap(swapToFile, "paster.swp","", Status),
		Status=0,!,
		write(progname, " installed for disk swap").
	best_Swap:- write(progname, " installed with no swapping").
		
				
