/*****************************************************************************
 tsrhelp.pro

 Help Utility based on the TSR facility of the Professional User's Guide.

 NOTE: be sure to turn auto-indent off if pasting into the Prolog 
       editor!

 This demo implements a very crude form of hypertext as a means to illustrate
  possible uses of the resident programming tools in the Professional User's
  Guide.  The upcoming Hypertext Toolbox features true state-of-the-art
  hypertext, with mouse support, fully integrated with a special
  object-oriented version of the Prolog editor.

 USAGE:  TSRHELP <topic>  
	      displays non-resident help on a given topic
	 TSRHELP /res
	      load utility as a resident help system
	      where topic lookup is based on the text at the cursor position
	 TSRHELP /unload 
	      unloads the resident help utility. 

 Copyright (C) 1990, Prolog Development Center
******************************************************************************/
heap = $2000

include "resident.pre"
include "keyscan.pre"

CONSTANTS
  progname = tsrhelp
  
  mono_avail = 15
  mono_select= 112
  mono_normal= 7

  color_avail = 121
  color_select= 23
  color_normal= 113

  ru_resident = 9 %dummy signal 
DOMAINS
   file=infile
   db_selector=db
   sl=string*

   % a topic is a keyword, some text, and a collection
   %   of associated buttons
  topic = topic(string, string, integer, buttons)

   % a collection of buttons is a list of objects of type button
  buttons = button*

   % a button is an association of a region of the topic text
   %  and an action to be executed when the button is "pressed"
  button = button(region, action)

  region = region(integer, integer, integer, integer)

  action =
	jump_to(ref);
	paste(region)

  key = prev;
	next;
	push_button;
	exit ;
	index;
	scroll(integer);
	move(integer);
	size(integer);
	mode_toggle

  state_after_key =
	pop_down;
	continue

  resizemode = on;off

DATABASE
 determ button_state(buttons,buttons)
 determ row_offset(integer)
 determ text(string,integer)
 determ index_is(bt_Selector)
 determ resize_is(resizemode)

DATABASE - sticky_settings
 determ win_at(integer,integer, integer, integer)
 determ color(resizemode)

PREDICATES
 start_from(string)
 lookup(bt_selector,string,string,ref)
 enter_topic(ref)
 highlight(buttons,integer,integer)
 reverse_list(buttons,buttons,buttons)
 topic_process(key, state_after_key)
 push_button(buttons, integer, integer, state_after_key)
 unmark_current(buttons, integer, integer)
 mark_current(buttons,integer,integer)
 mark_current_adjust_scroll(buttons,integer,integer)

 do(action, state_after_key)
 button_shift(key, buttons, buttons, buttons, buttons)
 get_key(key)
 get_key2(resizemode,char, key)
 get_key3(resizemode,char, key)
 change_offset(integer)
 update_window
 in_window(integer,integer, integer,integer, integer,integer)
 lessor_of(integer,integer,integer)
 greater_of(integer,integer,integer)
 box_Attr(integer,integer,integer,integer,integer)
 extract_region(region, string)

 resize_move(integer)
 resize_size(integer)
 may_adjust_offset(integer,integer)

 fwrite(integer, string)
 frame_advise(action)
 bottom_frame
 win_attr(integer)
 select_attr(integer)
 avail_attr(integer)
 remove_schmutz(string, string)
 is_schmutz(char)
 nl_to_Cr(string,string)
 nl_to_Cr_Aux(integer,integer)
 nondeterm repeat

 try_it(string,string)
 do_comline(string)
 explain
 save_prefs
 window_size(integer,integer,integer,integer)
 get_lookup_text(integer,integer,string)
 db_Setup
 db_Set2
 best_swap

CLAUSES
%% initialize hypertext given a topic key;
%%   key from screen read at cursor,
%%   or from command-line startup in non-resident...
start_from(TopicKey) :-
	cursorstate(off),
	index_is(Tree),
	lookup(Tree,TopicKey, _, Ref),
	win_attr(Attr),
	window_size(Row,Col,High,Wide),
	makewindow(1,Attr,Attr, "", Row,Col, High,Wide),
	bottom_frame,
	enter_topic(Ref),
	repeat,
	get_key(KeyPressed),
	topic_process(KeyPressed, State),
	State=pop_down,
	makewindow(_,_,_,_,Rm,Cm, Hm, Wm),
	retractall(win_at(_,_,_,_)),
	asserta(win_at(Rm,Cm,Hm,Wm)), % remember where it was, etc.
	removewindow,!.

lookup(Tree, Key, "", Ref) :-
	key_search(db, Tree, Key, Ref),
	fail.
lookup(Tree, _, FoundKey, Ref) :-
	key_current(db, Tree, FoundKey, Ref),!.
lookup(Tree, _, FoundKey, Ref) :-
	key_prev(db, Tree,_),
	key_current(db, Tree, FoundKey, Ref).


topic_process(push_button, State) :-!,
	row_offset(Start),
	makewindow(_,_,_,_,_,_,High, _),
	WinMax=High-2,
	button_state(_,Buttons),
	push_button(Buttons, Start, WinMax, State).
topic_process(scroll(Dir), continue) :- !,
	change_offset(Dir).
topic_process(exit, pop_down) :- !.
topic_process(index, continue) :-!,
	str_ref("~3", IxRef),	% index is 3rd term (first section)
	enter_topic(IxRef).     
topic_process(mode_toggle, continue) :-
	retract(resize_is(on)), !,
	asserta(resize_is(off)),
	bottom_frame.
topic_process(mode_toggle, continue) :-!,
	retract(resize_is(off)),
	asserta(resize_is(on)),
	bottom_frame.
topic_process(size(N),continue) :- !, resize_size(N).
topic_process(move(N),continue) :- !, resize_move(N).
topic_process(Shift, continue) :-
	row_offset(Start),
	makewindow(_,_,_,_,_,_,WinHigh,_),
	WinMax=WinHigh-2,
	retract(button_state(Before, Current)),
	button_shift(Shift, Before,Current, NewBefore, NewCurrent),
	asserta(button_state(NewBefore,NewCurrent)),
	unmark_current(Current, Start, WinMax),
	mark_current_adjust_scroll(NewCurrent,Start, WinMax).


%% if button was totally visible, then do it!
push_button([button(region(Row,_,High,_),Action)|_], Start, WinMax,State) :-
	in_window(Row, High, Start, WinMax,_,High),!,
	do(Action,State).
%%
%% otherwise, bring it into view first (=> must Enter twice)
push_button(Buttons, Start, WinMax, continue) :-
	mark_current_adjust_Scroll(Buttons, Start, WinMax).


%% change window size delta N
%%  redo frame & repaint contents
%%
resize_size(N) :-
	textmode(Rows,_),
	makewindow(_,_,_,_,R,C,H,W),
	NewH = H + N,
	NewH > 10,
	R + NewH <= Rows,!,
	resizewindow(R,C,NewH, W),
	may_adjust_offset(N,NewH),
	bottom_frame.
resize_size(_).

%% if window grew, and it was scrolled, "scroll up"
%%  to keep consistent
may_adjust_offset(N,H) :-
	N>0,
	row_offset(Off),
	text(_,Lrow),
	Off+H>Lrow+3,
	change_offset(-1),!.
may_adjust_offset(_,_) :-
	update_window.

%% change window position delta N;
%%  repaint frame, must repaint contents to restore highlights
resize_move(N) :-
	textmode(Rows, _),
	makewindow(_,_,_,_,R,C,H,W),
	NewR = R+N,
	NewR >= 0,
	NewR+H <= Rows,!,
	resizewindow(NewR, C, H, W),
	update_window,
	bottom_frame.
resize_move(_).


%% shift next or prev to the "next" button
button_shift(prev, [Next|Rest], Now, Rest, [Next|Now]):-!.
button_shift(prev, [], Now, Rest, [Last]) :-
	reverse_list(Now, [], [Last|Rest]).
button_shift(next, Before, [Current,Next|Rest], [Current|Before], [Next|Rest]):-!.
button_shift(next, Others, [Last], [], Reset) :-
	reverse_list(Others, [Last],Reset).

reverse_list([X|Y],Z,A) :- !, reverse_list(Y,[X|Z],A).
reverse_list([],X,X).

%% show current button as selected, may need to scroll
%%  it into view
mark_current_adjust_scroll([button(region(VRow,Col,Vhigh,Wide),Act)|_],Start, Max1) :-
	in_window(Vrow, Vhigh, Start, Max1, Arow, Ahigh),
	select_attr(SelectAttr),
	box_attr(Ahigh, Arow, Col, Wide, SelectAttr),
	frame_advise(Act),
	!
	;
	VRow<Start,			% scroll up so its visible
	Adjust=VRow-Start,
	change_offset(Adjust),!
	;
	Adjust=VRow-(Start+Vhigh+Max1-2),   % scroll down to make visible
	Adjust>0,
	change_offset(Adjust).


%% highlight a set of buttons where within window
highlight([button(region(VRow,Col,VHigh,Wide),_)|Other_Buttons],First,Max) :-
	in_window(VRow, Vhigh, First,Max, Row, High),
	avail_attr(AvailAttr),
	box_attr(High,Row,Col,Wide,AvailAttr),
	!, highlight(Other_Buttons,First, Max).
highlight([_|Rest],First,Max) :- !,highlight(Rest,First,Max).
highlight([],_,_).


%% succeed if Virtual Row is in window, yielding actual window
%%  row and actual height visible
in_window(VRow, Vhigh, First,MaxRow1, Arow, Ahigh ) :-
	ARow = Vrow - First,
	Arow >= 0, Arow < MaxRow1,	     % ie, it's in the window
	After = Arow + VHigh,
	lessor_of(After, MaxRow1, MinAfter), % = 1 row past what fits in window
	Ahigh=MinAfter-Arow.		     % = actual height to show

lessor_of(A,B,A) :- A<B,!.
lessor_of(_,B,B).

greater_of(A,B,A) :- A>B,!.
greater_of(_,B,B).


%%  start a new topic
enter_topic(Ref) :-
	retractall(row_offset(_)),
	asserta(row_offset(0)),
	retractall(text(_,_)),
	ref_term(db, topic, Ref, topic(Title, Text, NumLines, Buttons)),
	asserta(text(Text,NumLines)),
	concat(Title," ",T1),  % pad with space on
	frontchar(T2, 32, T1), % either side
	win_Attr(Attr),
	framewindow(Attr, T2, 5, "ɻȼͺ"),
	bottom_frame,
	retractall(button_state(_,_)),
	asserta(button_state([],Buttons)),
	clearwindow,
	update_window.


% repaint the window, with highlights & current
update_window :-
	row_offset(FirstRow),
	makewindow(_,_,_,_,_,_,WinRows,_),WinMax=WinRows-2,
	text(Text,_),
	window_Str(FirstRow,0,Text),
	button_state(A,B),
	highlight(A,FirstRow,WinMax),
	highlight(B,FirstRow,WinMax),
	mark_current(B,FirstRow,WinMax).


%% show the current button as highlighted
mark_current([button(region(VRow,Col,VHigh,Wide),Act)|_],First,WinHigh) :-
	in_window(VRow,Vhigh, First,WinHigh, Arow, Ahigh),!,
	select_attr(SelectAttr),
	box_attr(AHigh,ARow,Col,Wide,SelectAttr),
	frame_advise(Act).
mark_current(_,_,_).

%% set it back to "available"
unmark_current([button(region(VRow,Col,VHigh,Wide),_)|_],First, WinHigh) :-
	in_window(VRow,Vhigh, First,WinHigh, Arow, Ahigh),!,
	avail_attr(AvailAttr),
	box_attr(AHigh,ARow,Col,Wide,AvailAttr).
unmark_current(_,_,_).


%% set a rectangular region to an attribute
box_attr(0,_,_,_,_):-!.
box_attr(High,Row,Col,Wide,Attr) :-
	field_attr(Row,Col,Wide,Attr),
	H2=High-1,
	R2=Row+1,
	box_attr(H2, R2,Col,Wide,Attr).

%% dont just fail if Dir too large, etc.min/max nop if same
change_offset(Delta):-
	retract(row_offset(Current)),
	TempNew=Current+Delta,
	makewindow(_,_,_,_,_,_,WinHigh,_),
	text(_,LastRow),
	MaxAllowed=LastRow-(WinHigh-3),
	lessor_of(TempNew, MaxAllowed, TempNew1),
	greater_of(0, TempNew1, New),
	asserta(row_offset(New)),
	update_window.

do(jump_to(Ref),  continue) :-
	enter_topic(Ref).
do(paste(Region), pop_down) :-
	extract_region(Region, Subtext),
	stuffBiosKeys(SubText,0).

get_key(Returned):-
	readchar(First),
	resize_is(Mode),
	get_key2(Mode,First, Returned).

get_key2(Mode, 0, Returned) :-
	readchar(Second),
	get_key3(Mode, Second, Returned).
get_key2(off,13,push_button).	% enter
get_key2(off,9, next).		% tab
get_key2(off,27,exit).		% escape
get_key2(on, '2', move(1)).	% shift - dn
get_key2(on, '8', move(-1)).	% shift - up

get_key3(off,';',index).	% F1
get_key3(off,75,prev).		% left
get_key3(off,77,next).		% right
get_key3(off,15,prev).		% backtab
get_key3(off,80, scroll(1)).	% dn-arrow
get_key3(on, 80, size(1)).

get_key3(off,72, scroll(-1)).	% up-arrow
get_key3(on, 72, size(-1)).


get_key3(_ , 68, mode_toggle).	% F10

get_key3(off,81, scroll(Lines)):-	% pgdn
	makewindow(_,_,_,_,_,_,High,_),
	Lines=High-2.
get_key3(off,73, scroll(Lines)):-	% pgup
	makewindow(_,_,_,_,_,_,High,_),
	Lines= 0-(High-2).

resize_is(off).

repeat.
repeat:- repeat.

bottom_frame:-
	resize_is(off), !,
	fwrite(64,"F10:Resize"), 
		str_ref("~3", Foo),
		frame_advise(jump_to(Foo));
	fwrite(64,"F10:End Resize").

fwrite(Col, String) :-
	expandwindow,
	makewindow(_,_,_,_,_,_,High,_),
	Last=High-1,
	cursor(Last,Col), write(String),
	compresswindow.

frame_Advise(jump_to(_)) :-!,
	fwrite(2, " Enter:Jump TabShift-Tab\024\025PgUpPgDnF1:Index").
frame_Advise(_):-
	fwrite(2, " Enter:Paste TabShift-Tab\024\025PgUpPgDnF1:Index").


%% here we cheat and read the region from the screen, since
%%  you cant "push" the button until the button region
%%  is totally visible ...
extract_region(region(Row,Col,High,Wide), Result):-
	row_offset(Off),
	makewindow(_,_,_,_,Wrow,Wcol,_,_),
	Rrow=Wrow+Row+1-Off,
	Rcol=Wcol+Col+1,
	makewindow(100, 7,0,"", Rrow, Rcol, High, Wide,0,0,""),
	window_str(Temp),
	removewindow(100,0),
	nl_to_cr(Temp, Result).

%% do this in place to save memory usage
nl_to_cr(Text,Text) :-
	ptr_dword(Text, Seg, Off),
	nl_to_Cr_Aux(Seg, Off).

nl_to_cr_Aux(Seg, Off) :-
	membyte(Seg,Off,Byte), Byte=0,!.
nl_to_cr_Aux(Seg, Off) :-
	membyte(Seg,Off,Byte),
	Byte='\10',!,
	membyte(Seg, Off, '\13'),
	Off2=Off+1,
	nl_to_cr_Aux(Seg, Off2).
nl_to_cr_Aux(Seg, Off) :-
	Off2=Off+1,
	nl_to_cr_Aux(Seg, Off2).

window_size(Row,Col, High, Wide) :-
	win_at(Row,Col,High,Wide),
	textmode(Rows,Cols),
	Row+High<=Rows,
	Col+Wide<=Cols,!.
window_size(0,0, Rows, Cols) :-
	textmode(Rows,Cols),
	retractall(win_at(_,_,_,_)),
	asserta(win_At(0,0,Rows,Cols)).

save_prefs:-
	chain_delete(db, user_prefers),
	retract(Setting, sticky_settings),
	chain_insertz(db, user_prefers, sticky_settings, Setting,_),
	fail.
save_prefs.

select_attr(color_select) :- color(on),!.
select_attr(mono_select).

avail_attr(color_avail) :- color(on),!.
avail_attr(mono_avail).

win_attr(color_normal) :- color(on),!.
win_attr(mono_normal).

hotkey(1) :-
	not(wasGraphics),
	setKeyboardSense(on),
	textmode(Rows,Cols),
	saveVideo(Rows, Cols, R,C),
	R>=0,C>=0, R<Rows, C<Cols,  % curor "off" ?
	get_lookup_text(R,C, Text),
	start_from(Text).

signal(kill_Signal) :-
	save_prefs,
	db_close(db),
	cursorstate(on).

db_setup :-
	trap(db_set2, _, fail),!.
db_setup :- write("Error opening help database."), exit(0).

db_set2 :-
	syspath(SysPath,_),
	filenamepath(FullName,SysPath,"tsrhelp.dba"),
	db_open(db,FullName, in_file),
	bt_open(db,"topictree", Tree),
	asserta(index_is(Tree)),
	chain_terms(db, user_prefers, sticky_settings, Term, _),
	asserta(Term),
	fail.
db_set2.

%% choose best method for swapping
best_Swap :-
	enableSwap(swapToEms, "","", Status),
	Status=0,!,
	write(progname, " installed for EMS swap").
best_Swap :-
	enableSwap(swapToFile, "tsrhelp.swp","", Status),
	Status=0,!,
	write(progname, " installed for disk swap").
best_Swap:- write(progname, " installed with no swapping").

try_it("/",Rest) :- !, do_comline(Rest).
try_it(_,_) :-   %
	signalResident(progname,ru_resident),  % see if it's loaded resident
	write("* Can't use this non-resident while it's also loaded resident *\n\n"),
	!,
	explain.
try_it(OtherText,_) :-
	db_setup,
	start_from(OtherText),
	signal(0).

do_comline(res) :-!,
	enableResident(progname,0),
	best_swap,
	db_setup,
	installHotKey(1, key_F1, eitherCtrl, needsDos_CantReenter),
	installSignal(kill_Signal, needsDos_CantReenter),
	installSignal(9, 0),
	write("\nPress Ctrl-F1 to activate"),
	stayResident.
do_comline(unload) :-!,
	removeResident(progname).
do_comline(_) :- explain.

explain :-
	write("tsrhelp <topic>   - displays non-resident help on topic\n"),
	write("        /res      - loads system as resident help utility\n"),
	write("        /unload   - unloads resident utility").
	
get_lookup_text(Row, _, Key) :-
	textmode(_, Cols),
	makewindow(1, 7,0, "", Row, 0, 1, Cols,0,0,""),
	window_Str(Stuff),
	remove_schmutz(Stuff, Temp),
	removewindow(1,0),
	fronttoken(Temp, Key,_),!.
get_lookup_text(_,_,"").

remove_schmutz(In, Out ) :-  %being the stuff like window borders
	frontchar(In, C, Rest),
	is_schmutz(C),
	!,remove_schmutz(Rest, Out).
remove_schmutz(X,X).

is_schmutz(Char) :- Char<32,! ; Char > 126.
GOAL
	comline(Text),
	upper_lower(Text,LowText),
	fronttoken(LowText, Tok, Rest),!,
	try_it(Tok,Rest)
	;
	explain.


