/*****************************************************************************
			Prolog Inference Engine
			=======================

  Copyright (c) 1986, 90 by Prolog Development Center

  Module PIE.PRO: The declarations and main program

  This program implements the Prolog inference engine as described in
  appendix I (META-PROGRAMMING) in the PDC Prolog reference manual.

Note: This version of pie differs from the version described in the manual
in that	the constant use_extdbase controls whether or not to store the
rules in the internal or external database system.

*****************************************************************************/

code = 3999

% config "pie.cfg"

CONSTANTS
  implement_trace = 1
%  use_extdbase    = 1
 
DOMAINS
  TERM  = reference
		var(VID);
		cmp(REFSYMB,TERML);list(TERM,TERM); nill;
		atom(REFSYMB); int(REFINT);
		str(REFSTR); char(REFCHAR)

  TERML = reference TERM*

  REFINT	= reference INTEGER
  REFSYMB	= reference STRING
  REFSTR	= reference STRING
  REFCHAR	= reference char
  VID		= reference STRING

/*---------- INTERPRETER ENVIRONMENT ------------------*/
  E	= e(VID,TERM)
  ENV	= reference E*

/*---------- MISC HANDY DOMAIN DECLARATIONS ------------------*/
  OP	= STRING	% Storing of operators
  XFY	= STRING	% xfy; yfx; xfx; yfy; fx; fy; xf; yf
  ASSOC = x; y		% The associavity to the left or the right
  PRIOR = INTEGER	% Priority of operators
  MODE	= SYMBOL	% Display mode for term writer

  FILE = temp; seeing; telling

  ifdef use_extdbase
    DB_SELECTOR = clause_db
    CLAUSE = clause(STERM,STERM)
  enddef

/*---------- SCANNER DOMAINS ------------------*/
  TOK		= lbrack; rbrack; lpar; rpar; lcurly; rcurly;
		  var(STRING);	  atom(STRING);
		  int(INTEGER); str(STRING); char(CHAR);
		  comma; bar; dot

  CURSORTOK	= t(TOK,CURSOR)
  CURSOR	= INTEGER
  TOKL		= CURSORTOK*

/*---------- PARSER DOMAINS ------------------*/
  STERM            = var(STRING);
		     cmp(STRING,STERML);
		     list(STERM,STERM); nill;
		     atom(STRING);
		     int(INTEGER);
		     str(STRING);
		     char(CHAR)

  STERML           = STERM*

PREDICATES % Need to be compiles first due to memory limitations
  nondeterm handle_retract(TERM)
  nondeterm call(STRING,TERML)

/* ---------- The clause and operator database ------------------- */
DATABASE
   determ traceflag
   op(PRIOR,XFY,OP)
ifndef use_extdbase
   clause(STERM,STERM)
enddef

CLAUSES
  op(1200,xfx,":-").	op(1100,xfy,";").	op(1000,xfy,",").
  op(900,fy,"not").

  op(700,xfx,"=").	op(700,xfx,"\\=").
  op(700,xfx,"is").	op(700,xfx,"<").	op(700,xfx,"=<").
  op(700,xfx,">").	op(700,xfx,">=").	op(700,xfx,"==").
  op(700,xfx,"\\==").	op(700,xfx,"=..").
  op(500,yfx,"+").	op(500,yfx,"-").
  op(400,yfx,"*").	op(400,yfx,"/").
  op(300,xfx,"mod").
  op(200,fx,"+").	op(200,fx,"-").


/*****************************************************************************
	Include files	
*****************************************************************************/

include "pie.sca"	% The scanner
include "pie.par"	% The operator precedence parser
include "pie.out"	% The term
include "pie.inf"	% The inference engine

/*****************************************************************************
	USER INTERFACE
*****************************************************************************/

DATABASE - counter
  determ counter(INTEGER)

PREDICATES
  run
  scan_parse(STRING,STERM)
  init_counter
  count
  wsol(INTEGER)
  wenv(ENV)
  handle_usergoal(STERM)
  errorhnd(INTEGER)

CLAUSES
  run:- 
  	ifdef use_extdbase
  	  db_create(clause_db,"clauses.bin",in_memory),
  	enddef
  	textmode(ROWS,COLS),
  	RR=ROWS-1,
  	makewindow(3,7,7,"Prolog Inference Engine Trace",2,20,20,60),
  	makewindow(1,112,0,"",RR,0,1,COLS),
  	write(" halt: Aborts     help: help information     list:list clauses     edit:editor"),
  	makewindow(2,7,7,"Prolog Inference Engine Dialog",0,0,RR,COLS),
	repeat,
	write("\nGoal: "),readln(L),L><"",
	init_counter,
  	scan_parse(L,TERM),
	
%	write(TERM),nl,
	
	trap(handle_usergoal(TERM),E,errorhnd(E)),
	ifdef implement_trace
	  traceflag, % If trace is turned on show the trace window
	  retractall(traceflag),
	  shiftwindow(3),
	  write("Press any key\n"),
	  readchar(_),
	  shiftwindow(2),
	enddef
	fail.

  scan_parse(STR,TERM):-
	tokl(0,Str,TOKL),s_term(TOKL,_,TERM),!.
  scan_parse(_,_):-write(">> Syntax error"),fail.

  handle_usergoal(cmp(":-",[HEAD,BODY])):-!,
	convhead(HEAD,HEAD1),
	ascla('z',HEAD1,BODY),
	write("Asserted").
  handle_usergoal(TERM):-
  	free(ENV),
	getbacktrack(BTOP),
	unify_body(TERM,ENV,BTOP),
	wenv(ENV),nl,
	count,
	free(ENV), % Give only one solution when there are no variables
	cutbacktrack(BTOP),
	fail.
  handle_usergoal(_):-
	closefile(seeing),closefile(telling),
	counter(X),wsol(X)
	ifdef use_extdbase
	  ,db_statistics(clause_db,NoOfRecords,MemSize,DbaSize,FreeSize),
	  writef("\nNoOfRecords=%, MemSize=%, DbaSize=%, Freesize=%",NoOfRecords,MemSize,DbaSize,FreeSize),
	  storage(SS,HH,TT),
	  writef("\nStack=%, Heap=%, Trail=%",SS,HH,TT)
	enddef
	.

  errorhnd(0):-!.
  errorhnd(-1):-!,exit.
  errorhnd(N):-
  	syspath(PrologPath,_),
  	filenamepath(ErrorFile,PrologPath,"prolog.err"),
	errormsg(ErrorFile,N,A,_),
	write(A).

  init_counter:-
	retractall(_,counter),assert(counter(0)).

  count:-retract(counter(N)),N1=N+1,assert(counter(N1)).

  wsol(0):-!,write("No solutions").
  wsol(1):-!,write("1 Solution").
  wsol(N):-write(N," Solutions").

  wenv(L):-free(L),!,write("True").
  wenv([e(VAR,TERM)|T]):-free(T),!,write(VAR,"= "),wterm("write",TERM).
  wenv([e(VAR,TERM)|T]):-write(VAR,"= "),wterm("write",TERM),write(", "),wenv(T).

GOAL run.
