include "dbremote.pre"

DATABASE - client
determ servhandle(integer)
determ syswind(integer)
determ flow_stopped
determ no_flowcontrol

PREDICATES
main
rmw(integer)
loopy
ehnd(integer)
rxehnd(integer)
do_errors
process_err(integer,integer,integer,integer)
understand(dbcontrol)
ctrl_Q
ctrl_S
check_congestion(integer)
min(integer,integer,integer)
bar(integer,integer,integer)
critcheck(integer,integer,integer,string)

CLAUSES
do_errors:-
	msgerror(Origin,Handle,SeqNo,RCode), !,
	process_err(Origin,Handle,SeqNo,RCode),
	do_errors.
do_errors.

process_err(senderr_orig,Handle,SeqNo,RCode):-
	shiftwindow(Old),
	syswind(Sysw),
	shiftwindow(Sysw),
	write("Error ",RCode," sending msg ",SeqNo," on handle ",Handle,'\n'),
	shiftwindow(Old).
process_err(recverr_orig,Handle,_,RCode):-
	shiftwindow(Old),
	syswind(Sysw),
	shiftwindow(Sysw),
	write("Error ",RCode," from rxdaemon ",Handle," - closing.\n"),
	shiftwindow(Old),
	msgclose(Handle,purge,filter).

rxehnd(E):-
	E <> err_pipeclosed, E <> err_pipebroken, !,
	exit(E).		% Not msg-related error.
rxehnd(_):-
	do_errors,
	exit.


ehnd(E):- write("Error ",E," in readterm.\n").

understand(advice(_)):- !.
understand(Msg):-
	write("Received ",Msg,'\n').

loopy:-
	gotowindow(2),
	trap(msgrecv(dbcontrol,H,Msg),E,rxehnd(E)),
	understand(Msg),
	Msg = logoff(), !,
	msgclose(H,purge,filter),
	exit.
loopy:-
	keypressed,
	ctrl_S,
	trap(readterm(dbcontrol,Action),E,ehnd(E)),
	ctrl_Q,
	servhandle(H),
	msgsend(dbcontrol,H,Action),
	Action = logoff(),
	assert(no_flowcontrol),	% Don't want anything sent after logoff.
	fail.
loopy:-
	gotowindow(1),
	msgqueued(TXSize,RXSize,NowTXing,TXed,_),
	check_congestion(RXSize),
	str_int(STXSize,TXSize),
	str_int(SRXSize,RXSize),
	field_str(0,0,3,"TX"),
	field_str(0,3,6,STXSize),
	bar(0,4000,TXSize),
	field_str(1,0,3,"RX"),
	field_str(1,3,6,SRXSize),
	bar(1,2000,RXSize),
	format(S,"Now sending %6  Last sent %6",NowTXing,TXed),
	field_str(2,0,50,S),
	loopy.

ctrl_Q:-
	flow_stopped, not(no_flowcontrol), !,
	retractall(flow_stopped),
	servhandle(H),
	msgsend(dbcontrol,H,start()).
ctrl_Q.

ctrl_S:-
	not(flow_stopped), not(no_flowcontrol), !,
	assert(flow_stopped),
	servhandle(H),
	msgsend(dbcontrol,H,stop()).
ctrl_S.

check_congestion(QSize):- QSize < 15, !, ctrl_Q.
check_congestion(_):- ctrl_S.

min(A,B,A):- A <= B, !.
min(_,B,B).

critcheck(Size,Crit,112,""):- Size < Crit, !.
critcheck(_,_,240,"Critical").

bar(Row,Crit,Size):-
	Barlen = Size div 2,
	min(Barlen,68,MaxBar),
	critcheck(Size,Crit,Attr,Msg),
	field_attr(Row,10,MaxBar,Attr),
	field_str(Row,10,8,Msg),
	MaxBar < 68, !,
	ClrCol = MaxBar + 10,
	ClrLen = 68 - MaxBar,
	field_attr(Row,ClrCol,ClrLen,7).
bar(_,_,_).

rmw(E):- removewindow(1,1), removewindow(2,1), write("Exit with ",E).

main:-
	shiftwindow(Sysw), assert(syswind(Sysw)),
	msgopen("\\pipe\\club",Handle,10000,100),
	assert(servhandle(Handle)),
	makewindow(1,7,7,"Msg Status",3,0,5,80),
	makewindow(2,7,7,"Activity",7,0,6,80),
	msgsend(dbcontrol,Handle,logon("per")),
	trap(loopy,E,rmw(E)), fail.
main.

GOAL
	retractall(_,client),
	main.
