include "dbremote.pre"

DATABASE - server
person(Name,Address,Age,Sex,Interests)
active_handle(integer,string)
ready(integer)
handle(integer)
determ accumstate(string)
lostconnection(integer)

PREDICATES
known_name(string)
sendall(integer)
action(dbcontrol,integer)
broadcast(dbcontrol)
upd_handles(integer,dbcontrol)
loopy
rxehnd(integer)
upd_lostconnections(integer)
do_errors
write_err(integer,integer,integer,integer)
do_lostconnections
rmw(integer)
min(integer,integer,integer)
bar(integer,integer,integer)
critcheck(integer,integer,integer,string)
get_name(integer,string)
handlestate(string)
userstate(integer,string)
readystate(integer,string)
advicestate(string)
main

CLAUSES
known_name("per").
known_name("leo").
known_name("dan").

do_errors:-
	msgerror(Origin,Handle,SeqNo,RCode), !,
	write_err(Origin,Handle,SeqNo,RCode),
	upd_lostconnections(Handle),
	do_errors.
do_errors.

write_err(senderr_orig,Handle,SeqNo,RCode):-
	write("Error ",RCode," sending msg ",SeqNo," on handle ",Handle,".\n").
write_err(recverr_orig,Handle,_,RCode):-
	write("Error ",RCode," from rxdaemon ",Handle,".\n").

upd_lostconnections(H):- lostconnection(H), !.
upd_lostconnections(H):- assert(lostconnection(H)).

do_lostconnections:-
	retract(lostconnection(H)),
	write("Close and purge ",H),
	msgclose(H,purge,filter),
	retractall(active_handle(H,_)),
	retractall(handle(H)),
	retractall(ready(H)),
	write(", and re-open.\n"),
	msgcreate("\\pipe\\club",NewH),
	assert(handle(NewH)),
	fail.
do_lostconnections.

rxehnd(E):-
	E <> err_pipeclosed, E <> err_pipebroken, !,
	exit(E).		% Not msg-related error.
rxehnd(_):-
	shiftwindow(Old),
	gotowindow(3),
	do_errors,
	do_lostconnections,
	gotowindow(Old).

broadcast(Happening):-
	active_handle(H,_),
	write("Send ",Happening," to ",H,'\n'),
	msgsend(dbcontrol,H,Happening),
	fail.
broadcast(_).

sendall(Handle):-
	person(Name,Address,Age,Sex,Interests),
	msgsend(dbcontrol,Handle,
			advice(person(Name,Address,Age,Sex,Interests))),
	fail.
sendall(_).

get_name(Handle,Name):- active_handle(Handle,Name), !.

action(get(Name),Handle):-
	person(Name,Address,Age,Sex,Interests), !,
	msgsend(dbcontrol,Handle,
			contents(person(Name,Address,Age,Sex,Interests))).
action(get(Name),Handle):- !,
	msgsend(dbcontrol,Handle,notexist(Name)).
action(stop(),Handle):- !, retractall(ready(Handle)).
action(start(),Handle):- !, assert(ready(Handle)).
action(upd(person(Name,Address,Age,Sex,Interests)),_):- !,
	retractall(person(Name,_,_,_,_)),
	assert(person(Name,Address,Age,Sex,Interests)),
	broadcast(upd(person(Name,Address,Age,Sex,Interests))).
action(del(Name),_):- !,
	retractall(person(Name,_,_,_,_)),
	broadcast(del(Name)).
action(add(Person),_):- !,
	Person = person(Name,Address,Age,Sex,Interests),
	assert(person(Name,Address,Age,Sex,Interests)),
	broadcast(add(Person)).
action(logoff(),Requestor):- !,
	retractall(active_handle(Requestor,_)),
	retractall(handle(Requestor)),
	retractall(ready(Requestor)),
	write("Send logoff to ",Requestor),
	msgsend(dbcontrol,Requestor,logoff()),
	write(", flush and close, and "),
	msgclose(Requestor,flush,filter),
	write("re-open.\n"),
	msgcreate("\\pipe\\club",NewH),
	assert(handle(NewH)).
action(_,Requestor):-
	msgsend(dbcontrol,Requestor,failure()).


loopy:-
	gotowindow(2),
	trap(msgrecv(dbcontrol,Handle,Action),E,rxehnd(E)),
	msgsetwait(0),
	upd_handles(Handle,Action),
	action(Action,Handle),
	fail.
loopy:-
	active_handle(Handle,_),
	ready(Handle),
	sendall(Handle),
	fail.
loopy:-
	gotowindow(1),
	msgqueued(TXSize,RXSize,NowTXing,TXed,LastPiped),
	str_int(STXSize,TXSize),
	str_int(SRXSize,RXSize),
	field_str(0,0,3,"TX"),
	field_str(0,3,6,STXSize),
	bar(0,4000,TXSize),
	field_str(1,0,3,"RX"),
	field_str(1,3,6,SRXSize),
	bar(1,2000,RXSize),
	str_int(SNow,NowTXing),
	field_str(2,13,5,SNow),
	str_int(STXed,TXed),
	field_str(2,32,5,STXed),
	str_int(SLastP,LastPiped),
	field_str(2,49,5,SLastP),
	handlestate(S1),
	field_str(3,0,78,S1),
	advicestate(SS),
	field_str(4,10,7,SS),
	loopy.

advicestate("blocked"):- not(ready(_)), !, msgsetwait(100).
advicestate("running").

handlestate(_):-
	assert(accumstate("")),
	handle(H),
	userstate(H,User),
	readystate(H,Ready),
	format(S,"%2: %7 %9",H,User,Ready),
	retract(accumstate(AS)),
	concat(AS,S,AAS),
	assert(accumstate(AAS)),
	fail.
handlestate(S):- retract(accumstate(S)).

userstate(H,Name):- active_handle(H,Name), !.
userstate(_,"(free)").

readystate(H,"(ready)"):- ready(H), !.
readystate(_,"(BLOCKED)").

min(A,B,A):- A <= B, !.
min(_,B,B).

critcheck(Size,Crit,112,""):- Size < Crit, !.
critcheck(_,_,240,"Critical").

bar(Row,Crit,Size):-
	Barlen = Size div 2,
	min(Barlen,68,MaxBar),
	critcheck(Size,Crit,Attr,Msg),
	field_attr(Row,10,MaxBar,Attr),
	field_str(Row,10,8,Msg),
	MaxBar < 68, !,
	ClrCol = MaxBar + 10,
	ClrLen = 68 - MaxBar,
	field_attr(Row,ClrCol,ClrLen,7).
bar(_,_,_).

upd_handles(H,_):-
	active_handle(H,_), !.
upd_handles(H,logon(Name)):-
	known_name(Name), !,
	write("Log on by ",Name," on port ",H,'\n'),
	assert(active_handle(H,Name)),
	assert(ready(H)),
	msgsend(dbcontrol,H,logon("Welcome")), fail.
upd_handles(H,logon(Name)):- !,
	write("Invalid access attempt by ",Name,'\n'),
	msgsend(dbcontrol,H,error("Access denied")), fail.
upd_handles(H,_):-
	msgsend(dbcontrol,H,error("Use 'logon(<name>)'")), fail.

rmw(E):- removewindow(1,1), removewindow(2,1), write("Exit with ",E).

main:-
	consult("xclub.dba",server),
	msgcreate("\\pipe\\club",H1),		/* Allow up to 3 clients. */
	assert(handle(H1)),
	msgcreate("\\pipe\\club",H2),
	assert(handle(H2)),
	msgcreate("\\pipe\\club",H3),
	assert(handle(H3)),
	makewindow(1,7,7,"Msg Status",3,0,7,80),
        field_str(2,0,12,"Now handling"),
	field_str(2,19,12,"Last handled"),
	field_str(2,38,10,"Last piped"),
	field_str(4,0,10,"Advice is "),
	makewindow(2,7,7,"Activity",9,0,10,80),
	makewindow(3,7,7,"Hard Errors",19,0,6,80),
	msgsetwait(100),
	trap(loopy,E,rmw(E)),
	fail.
main.


goal
	retractall(_,server),
	main.
