/*
=========================================================================
	CHART.PRO
	
	Chart display system. Supports line, bar and pie charts
	in virtual window system. Needs GRAPH.PRO to operate.
	
	Copyright (C) 1990 Prolog Development Center
=====================================================================
*/
	
/*
---------------------------------------------------------------------
	AXIS PAIRS
---------------------------------------------------------------------
*/

DATABASE - axis_pairs

	axes_(Integer,Marker,Marker)
	active_axes_(Integer)

CLAUSES

   % CREATE AN AXIS PAIR

	make_axes(Number,XMarker,YMarker,Top,Left,Bottom,Right) :-
		retractall(axes_(Number,_,_)),
		assert(axes_(Number,XMarker,YMarker)),
		retractall(active_axes_(_)),
		assert(active_axes_(Number)),
		make_inside(Left,Top,Right,Bottom).


   % SHIFT THE ACTIVE AXES

	shift_axes(Number) :- free(Number),
		active_axes_(Number), !.
	shift_axes(Number) :- bound(Number),
		retractall(active_axes_(_)),
		assert(active_axes_(Number)),
		draw_axes(Number).
		

   % SHIFT TO THE ACTIVE CENTRAL WINDOW

	shift_middle :- active_window_(Window),
		inside_(Window,Left,Top,Right,Bottom), !,
		setviewport(Left,Top,Right,Bottom,clip).
/*	shift_middle :- object_(drawlistView(Selector)),
		drawlistView_(Selector,_,_,_,_,_,_,_,_,Region),
		Region = region(Left,Top,Right,Bottom),
		setviewport(Left,Top,Right,Bottom,clip).
*/

   % SHIFT OVER TO THE BOTTOM RULER WINDOW

	shift_bottom :- 
		active_window_(Window),
		inside_(Window,Left,_,Right,T),
		window_(Window,_,_,_,_,Bottom,_), !, Top = T + 1,
		setviewport(Left,Top,Right,Bottom,0).

/*	shift_bottom :- object_(drawlistView(Selector)),
		drawlistView_(Selector,_,_,_,_,_,_,_,_,Region),
		Region = region(Left,_,Right,T),
		window_(Window,_,_,_,_,Bottom,_), !, Top = T + 1,
		setviewport(Left,Top,Right,Bottom,0).
*/

   % SHIFT OVER TO THE LEFT RULER WINDOW

	shift_left :- active_window_(Window),
		inside_(Window,R,Top,_,Bottom),
		window_(Window,_,Left,_,_,_,_), !, Right = R - 1,
		setviewport(Left,Top,Right,Bottom,0).

/*	shift_left :- object_(drawlistView(Selector)),
		drawlistView_(Selector,_,_,_,_,_,_,_,_,Region),
		Region = region(R,Top,_,Bottom),
		window_(Window,_,Left,_,_,_,_), !, Right = R - 1,
		setviewport(Left,Top,Right,Bottom,0).
*/

   % DRAW BOTH HORIZONTAL AND VERTICAL RULERS
	
	draw_axes(Number) :- 
		axes_(Number,XMarker,YMarker), !,
		horz_ruler(XMarker), 
		vert_ruler(YMarker).


   % DRAW A HORIZONTAL RULER BASED ON THE MARKER INFORMATION

	horz_ruler(marker_(Unit,Form,TextColor,LineColor,GridColor)) :-
		active_scale_(Scale), 
		scale_(Scale,XMin,XMax,YMin,YMax), !,
		horz_ruler(marker_(Unit,Form,TextColor,LineColor,GridColor),
			XMin,XMax,YMin,YMax),
		setcolor(LineColor),
		draw_line(XMin,YMin,XMax,YMin).


	horz_ruler(_,Pos,Max,_,_) :- Pos > Max, !.
	horz_ruler(marker_(Unit,Form,TextColor,LineColor,GridColor),
			Pos,XMax,YMin,YMax) :-
		Edge = YMax - (YMax - YMin) / 5, 
		shift_bottom,
		setcolor(TextColor), 
		draw_text(Pos,Edge,Form,Pos,horz_flag),
		setcolor(GridColor), 
		draw_line(Pos,Edge,Pos,YMax),
		shift_middle, 
		draw_line(Pos,YMin,Pos,YMax),
		New = Pos + Unit, !,
		horz_ruler(marker_(Unit,Form,TextColor,LineColor,GridColor),
			New,XMax,YMin,YMax).


   % DRAW A VERTICAL RULER BASED ON THE MARKER INFORMATION

	vert_ruler(marker_(Unit,Form,TextColor,LineColor,GridColor)) :-
		active_scale_(Scale), scale_(Scale,XMin,XMax,YMin,YMax), !,
		vert_ruler(marker_(Unit,Form,TextColor,LineColor,GridColor),
			XMin,XMax,YMin,YMax),
		setcolor(LineColor), 
		draw_line(XMin,YMin,XMin,YMax).


	vert_ruler(_,_,_,Pos,Max) :- Pos > Max, !.
	vert_ruler(marker_(Unit,Form,TextColor,LineColor,GridColor),
			XMin,XMax,Pos,YMax) :-
		Edge = XMax - ((XMax - XMin) / 5), 
		shift_left,
		setcolor(TextColor), 
		draw_text(Edge,Pos,Form,Pos,vert_flag),
		setcolor(GridColor), 
		draw_line(XMax,Pos,Edge,Pos),
		shift_middle, 
		draw_line(XMin,Pos,XMax,Pos),
		New = Pos + Unit, 
		!,
		vert_ruler(marker_(Unit,Form,TextColor,LineColor,GridColor),
			XMin,XMax,New,YMax).

/*
---------------------------------------------------------------------
	LINE CHART HANDLING
---------------------------------------------------------------------
*/

DATABASE - chart_line

	sample1_chart_(IntegerList)
	sample2_chart_(IntegerList)

CLAUSES

   % CHART A POINT LIST IN LINE GRAPH FORM

	chart_line([X,Y],Size,Type,Color,_) :- !,
		setcolor(Color), 
		setfillstyle(1,Color),
		draw_point(X,Y,Size,Type).
	chart_line([X1,Y1,X2,Y2|Tail],Size,Type,Color,LineColor) :-
		setcolor(Color), 
		setfillstyle(1,Color),
		draw_point(X1,Y1,Size,Type), !,
		setcolor(LineColor), 
		draw_line(X1,Y1,X2,Y2),
		chart_line([X2,Y2|Tail],Size,Type,Color,LineColor).


   % SAMPLE DATA FOR DEMO AND TESTING

	sample1_chart_([0,12,10,45,20,23,30,37,40,4,50,34,
		60,48,70,29,80,14,90,30,100,42]).
	sample2_chart_([0,10,10,20,20,5,30,20,40,35,50,40,
		60,20,70,10,80,30,90,42,100,50]).
	
/*
----------------------------------------------------------------------
	BAR CHART HANDLING
----------------------------------------------------------------------
*/

DATABASE - bar_chart

	sample_bars_(Bars)

PREDICATES

	bar_length(Bars,Integer)

CLAUSES

   % DRAW A BAR CHART

	chart_bar(List) :-
		bar_length(List,Len), active_scale_(Number),
		scale_(Number,XMin,XMax,YMin,_),
		Width = (XMax - XMin) / Len,
		chart_bar(YMin,XMin,Width,List), !.

	chart_bar(Bottom,Old,Width,
			[bar_(Height,_,Fill,Pattern,Border)|Tail]) :-
		New = Old + Width, 
		Eight = Width / 8,
		Left = Old + Eight, 
		Right = New - Eight,
		setfillstyle(Pattern,Fill), 
		setcolor(Border),
		draw_bar(Left,Height,Right,Bottom), !,
		chart_bar(Bottom,New,Width,Tail).
	chart_bar(_,_,_,_).


   % DISPLAY A LABEL RULER UNDER A BAR CHART

	label_ruler(List) :-
		bar_length(List,Len), active_scale_(Number),
		scale_(Number,XMin,XMax,YMin,YMax), !,
		Width = (XMax - XMin) / Len, shift_bottom,
		label_ruler(XMin,YMin,YMax,Width,List),
		shift_middle, draw_line(XMin,YMin,XMax,YMin).


	label_ruler(Old,YMin,YMax,Width,[bar_(_,Label,_,_,_)|Tail]) :-
		New = Old + Width, Pos = Old + (Width / 2),
		Edge = YMax - (YMax - YMin) / 5,
		draw_line(Pos,Edge,Pos,YMax),
		draw_text(Pos,Edge,"%",Label,horz_flag), !,
		label_ruler(New,YMin,YMax,Width,Tail).
	label_ruler(_,_,_,_,_).


   % LENGTH UTILITY PREDICATE

	bar_length([_|Tail],Count) :- !,
		bar_length(Tail,Old),
		Count = Old + 1.
	bar_length(_,0).
	

   % SAMPLE DATA FOR DEMO AND TESTING

	sample_bars_([	bar_(52,"A",13,1,13),
			bar_(75,"B",2,2,2),
			bar_(68,"C",3,3,3),
			bar_(14,"D",4,4,4),
			bar_(89,"E",5,5,5),
			bar_(21,"F",6,6,6),
			bar_(74,"G",7,7,7),
			bar_(12,"H",8,8,8),
			bar_(66,"I",9,1,9),
			bar_(88,"J",10,2,10),
			bar_(23,"K",11,3,11),
			bar_(35,"L",12,4,12)]).

/*
----------------------------------------------------------------------
	PIE CHART HANDLING
----------------------------------------------------------------------
*/

DATABASE - pie_chart

	sample_pie_(Slices)

PREDICATES

	label_pos(Integer,Integer,Integer,Integer,Integer,Integer,Integer)

CLAUSES

   % DRAW A PIE CHART WITH A GIVEN X AND Y RADIUS	

	chart_pie(XRadius,YRadius,List) :- 
		active_scale_(Number),
		scale_(Number,XMin,XMax,YMin,YMax),
		XMid = (XMax - XMin) / 2,
		YMid = (YMax - YMin) / 2,
		chart_pie(0,XRadius,YRadius,XMid,YMid,List), !.
	
	
	chart_pie(Old,XRadius,YRadius,XMid,YMid,
			[slice_(Percent,Label,Fill,Pattern,Border)|Tail]) :-
		New = Old + Percent, 
		Angle = Old * 3.6, 
		End = New * 3.6,
		setcolor(Border), 
		setfillstyle(Pattern,Fill),
		draw_pie(XMid,YMid,Angle,End,XRadius,YRadius),
		Pos = Angle + (End - Angle) / 2,
		label_pos(Pos,XRadius,YRadius,XMid,YMid,X,Y),
		draw_text(X,Y,"%",Label,no_flag), !,
		chart_pie(New,XRadius,YRadius,XMid,YMid,Tail).
	chart_pie(_,_,_,_,_,_).


   % USED BY PIE HANDLING TO POSITION THE LABELS PROPERLY

	label_pos(Angle,XRadius,YRadius,XMid,YMid,X,Y) :-
		Start = 270 - Angle + 180,
		X = sin(Start/57.3) * XRadius * 1.6 + XMid,
		Y = cos(Start/57.3) * YRadius * 1.2 + YMid.


   % SAMPLE DATA FOR DEMO AND TESTING

	sample_pie_([	slice_(10,"Radio",5,2,7),
			slice_(26,"Television",9,3,7),
			slice_(12,"Busboards",3,4,7),
			slice_(21,"Print",2,5,7),
			slice_(11,"Flyers",6,6,7),
			slice_(20,"Direct Mail",7,7,7)]).

