/*
=====================================================================
	DIALS.PRO

	Dial Handling Predicates

	Copyright (C) 1990 Prolog Development Center
=====================================================================
*/

/*
---------------------------------------------------------------------
	General Move Area
---------------------------------------------------------------------
*/

PREDICATES

	moveposition(Integer,Integer,String,Integer,Integer)
	decideposition(Object,Integer,Integer,Integer,
		Integer,Integer,Integer)

CLAUSES


%	Move the Top Left start of the window
	movePosition(X,Y,Image,MouseX,MouseY) :-
		mouse_get(MouseX,MouseY,Buttons),
		Buttons >< 0, !,
		mouse_off,
		putimage(X,Y,Image,xor_put),
		mouse_on.
	movePosition(X,Y,Image,Left1,Top1) :-
		mouse_get(MouseX, MouseY, _),	% same mouse position,
		MouseX=X, 			% eg hasnt moved
		MouseY=Y,!,
		movePosition(X,Y,Image,Left1,Top1).
	movePosition(X,Y,Image,Left1,Top1) :-
		mouse_get(MouseX,MouseY,_),
		mouse_off,
		putimage(X,Y,Image,xor_put),
		putimage(MouseX,MouseY,Image,xor_put),
		mouse_on,
		moveposition(MouseX,MouseY,Image,Left1,Top1).


	decidePosition(Object,Left,Top,Right,Bottom,Left1,Top1) :-
		
		getmaxX(MaxX), 
		getmaxY(MaxY),


%	Delete entire area
		getbkcolor(BkColor),
		setfillstyle(solid_fill,BkColor),
		mouse_off,
		bar(Left,Top,Right,Bottom),
		mouse_on,
		message(Object,redraw),
		mouse_off,
		setwritemode(xor_put),
		setcolor(white),
		rectangle(Left,Top,Right,Bottom),
		setwritemode(copy_put),
		getimage(Left,Top,Right,Bottom,Image),
		setfillstyle(solid_fill,BKColor),
		bar(Left,Top,Right,Bottom),
		mouse_on,
		screen_RedrawRegion(region(Left,Top,Right,Bottom)),
		mouse_off,
		putimage(Left,Top,Image,xor_put),
		mouse_on,
		mouse_waitrelease,
		MaxHor = MaxX - (Right - Left),
		mouse_set_horizontal(0,MaxHor),
		MaxVer = MaxY - (Bottom - Top),
		mouse_set_vertical(0,MaxVer),
		mouse_set(Left,Top),
		mouse_on,
		moveposition(Left,Top,Image,Left1,Top1),

/*		Right1 = Left1 + (Right - Left),
		Bottom1 = Top1 + (Bottom - Top),
		mouse_off,
		setwritemode(xor_put),
		setcolor(white),
		rectangle(Left1,Top1,Right1,Bottom1),
		setwritemode(copy_put),
		mouse_on,

*/		mouse_set_horizontal(0,MaxX),
		mouse_set_vertical(0,MaxY),
		mouse_waitrelease.

/*
---------------------------------------------------------------------
		General routines for drawing dials
---------------------------------------------------------------------
*/

PREDICATES

	setTextPosition(Integer).
	labelMark(Integer,Integer,Integer,Integer).
	needle(Integer,Integer,Integer,Integer,Integer,Integer).
	endPoint(Integer,Integer,Integer,Integer,Integer,Integer).
	redraw(Integer,Integer,Integer,Integer,Integer,Integer).
	drawNeedle(Integer,Integer,Integer,Integer).
	markDialDegrees(Integer,Integer,Integer,Integer,Integer,
		Integer,Real,Real,Real,Real).
	drawArc(Integer,Integer,Integer,Integer,Integer,Integer).
	markDialBands(DialBands,Integer,Integer,Integer,Integer,
		Integer,Integer,Integer).
	setAnyDial(Integer,Integer,Real,Real,Real,Real,Real,Integer,Integer).

CLAUSES

	setAnyDial(Col,Row,Radius,Min,Max,OldValue,Value,
			StartDegree,EndDegree) :-
		OldValue < 0, !,
		ArcDegree = EndDegree - StartDegree,
		Degree = ((Value - Min) / (Max - Min))
			* ArcDegree + StartDegree,
		needle(Col,Row,Radius,Degree,ECol,ERow),
		drawNeedle(Col,Row,ECol,ERow).

	setAnyDial(Col,Row,Radius,Min,Max,OldValue,Value,
			StartDegree,EndDegree) :-
		ArcDegree = EndDegree - StartDegree,
		OldDegree = ((OldValue - Min) / (Max - Min))
			* ArcDegree + StartDegree,
		Degree = ((Value - Min) / (Max - Min))
			* ArcDegree + StartDegree,
		needle(Col,Row,Radius,OldDegree,OldECol,OldERow),
		needle(Col,Row,Radius,Degree,ECol,ERow),
		redraw(Col,Row,OldECol,ECol,OldERow,ERow).


	redraw(_,_,OldECol,ECol,OldERow,ERow) :-
		OldECol = ECol, OldERow = ERow, !.
	redraw(Col,Row,OldECol,ECol,OldERow,ERow) :-
		setwritemode(xor_put),
		line(Col,Row,OldECol,OldERow),
		line(Col,Row,ECol,ERow),
		setwritemode(copy_put).

	drawNeedle(Col,Row,ECol,ERow) :-
		setwritemode(xor_put),
		line(Col,Row,ECol,ERow),
		setwritemode(copy_put).


	needle(Col,Row,Radius,Degree,ECol,ERow) :-
		NeedleLen = Radius * 0.75,
		endPoint(Col,Row,NeedleLen,Degree,ECol,ERow).


	endPoint(Col,Row,Radius,Degree,EndCol,EndRow) :-
		ClockWise = 360 - Degree,
	  	getAspectRatio(Xasp,Yasp),
		EndCol = Radius * sin(ClockWise/radian) + Col,
		EndRow = Radius * cos(ClockWise/radian) * Xasp/Yasp + Row.


	markDialDegrees(Counter,Col,Row,Radius,
		StartDegree,EndDegree,Fleck,Min,Mark,Inc) :-
		Angle = round(Counter * Inc + StartDegree),
		Angle <= EndDegree,
		Counter mod round(Fleck) = 0, !,
		RadStart = Radius * 0.8,
		endPoint(Col,Row,RadStart,Angle,SCol,SRow),
		endPoint(Col,Row,Radius,Angle,EndCol,EndRow),
		line(SCol,SRow,EndCol,EndRow),
		Value = (Counter div round(Fleck)) * Mark + Min,
		labelMark(Angle,Value,EndCol,EndRow),
		NCounter = Counter + 1,
		markDialDegrees(Ncounter,Col,Row,Radius,
			StartDegree,EndDegree,Fleck,Min,Mark,Inc).

	markDialDegrees(Counter,Col,Row,Radius,
			StartDegree,EndDegree,Fleck,Min,Mark,Inc) :-
		Angle = round(Counter * Inc + StartDegree),
		Angle <= EndDegree, !,
		RadStart = Radius * 0.9,
		endPoint(Col,Row,RadStart,Angle,SCol,SRow),
		endPoint(Col,Row,Radius,Angle,EndCol,EndRow),
		line(SCol,SRow,EndCol,EndRow),
		NCounter = Counter + 1,
		markDialDegrees(NCounter,Col,Row,Radius,
			StartDegree,EndDegree,Fleck,Min,Mark,Inc).
	markDialDegrees(_,_,_,_,_,_,_,_,_,_).


	labelMark(Angle,Value,EndCol, EndRow) :-
		setTextPosition(Angle),
		str_int(Val, Value),
		outtextXY(EndCol, EndRow, Val).


	setTextPosition(StartDegree) :-
		StartDegree <= 90, !,
		settextjustify(right_Text,top_Text).
	setTextPosition(StartDegree) :-
		StartDegree < 175, !,
		setTextJustify(right_Text,bottom_Text).
	setTextPosition(StartDegree) :-
		StartDegree < 185, !,
		setTextJustify(center_Text,bottom_Text).
	setTextPosition(StartDegree) :-
		StartDegree <= 270, !,
		setTextJustify(left_Text,bottom_Text).
	setTextPosition(_) :-
		setTextJustify(left_Text,top_Text).


	markDialBands([band(Color,Start,Stop)|T],Col,Row,
			Radius,Min,Max,StartDegree,EndDegree) :- !,
		ArcDegree = EndDegree - StartDegree,
		BandStart = ((Start - Min) / (Max - Min))
			* ArcDegree + StartDegree,
		BandStop = ((Stop - Min) / (Max - Min))
			* ArcDegree + StartDegree,
		drawArc(Col,Row,Color,BandStart,BandStop,Radius),
		markDialBands(T,Col,Row,Radius,Min,Max,StartDegree,EndDegree).
	markDialBands(_,_,_,_,_,_,_,_).


	drawArc(Col,Row,Color,BandStart,BandStop,Radius) :-
		NeedleLen = Radius * 0.9,
		endPoint(Col,Row,NeedleLen,BandStart,EndCol1,EndRow1),
		endPoint(Col,Row,NeedleLen,BandStop,EndCol2,EndRow2),
		endPoint(Col,Row,Radius,BandStart,EndCol3,EndRow3),
		endPoint(Col,Row,Radius,BandStop,EndCol4,EndRow4),
		ArcStart = 360 - BandStart - 90,
		ArcStop = 360 - BandStop - 90,
		arc(Col,Row,ArcStop,ArcStart,NeedleLen),
		line(EndCol1,EndRow1,EndCol3,EndRow3),
		line(EndCol2,EndRow2,EndCol4,EndRow4),
		MiddleLen = Radius * 0.95,
		MiddleBandStart = BandStart + 2,
		endPoint(Col,Row,MiddleLen,MiddleBandStart,EndCol5,EndRow5),
		setFillStyle(solid_Fill,Color),
		getColor(ForeGnd),
		floodFill(EndCol5, EndRow5, ForeGnd).

/*
---------------------------------------------------------------------
		ROUTINES FOR A FULL DIAL
---------------------------------------------------------------------
*/

PREDICATES

	setFullDial(Integer,Integer,Real,Real,Real,Real,Real).
	makeFullDial(Integer,Integer,Integer,Real,Real,Real,String,DialBands)

CLAUSES

	setFullDial(Col,Row,Radius,Min,Max,OldValue,Value):-
		StartDegree = 45, 
		EndDegree = 315,
		setAnyDial(Col,Row,Radius,Min,Max,
			OldValue,Value,StartDegree,EndDegree).


	makeFullDial(Col,Row,Radius,Min,Max,Mark,Unit,DialBands):-
		circle(Col,Row,Radius),
		StartDegree = 45, EndDegree = 315,
		DegreeRange = EndDegree - StartDegree,
		ValueRange = Max - Min,
		Marking = DegreeRange / (ValueRange / Mark),
		Fleck = ValueRange / Mark,
		Inc = Marking / Fleck,
		markDialBands(DialBands,Col,Row,Radius,
			Min,Max,StartDegree,EndDegree),
		markDialDegrees(0,Col,Row,Radius,
			StartDegree,EndDegree,Fleck,Min,Mark,Inc),
	  	getAspectRatio(Xasp,Yasp),
		TextRow = Row + Radius * (Xasp/Yasp) * 0.5,
		settextjustify(center_Text,top_Text),
		outtextxy(Col,TextRow,Unit).

/*
---------------------------------------------------------------------
		ROUTINES FOR A HALF DIAL
---------------------------------------------------------------------
*/

PREDICATES

	setHalfDial(Integer,Integer,Real,Real,Real,Real,Real).
	makeHalfDial(Integer,Integer,Integer,Real,Real,Real,
		String,DialBands).

CLAUSES

	setHalfDial(Col,Row,Radius,Min,Max,OldValue,Value) :-
		StartDegree = 110,
		EndDegree = 250,
		setAnyDial(Col,Row,Radius,Min,Max,
			OldValue,Value,StartDegree,EndDegree).

	makeHalfDial(Col,Row,Radius,Min,Max,Mark,Unit,DialBands) :-
		arc(Col,Row,0,180,Radius),
		Left = Col - Radius,
		Right = Col + Radius,
		line(Left,Row,Right,Row),		

		DegreeRange = 250 - 110,
		ValueRange = Max - Min,
		Marking = DegreeRange / (ValueRange / Mark),
		Fleck = ValueRange / Mark,
		Inc = Marking / Fleck,

		markDialBands(DialBands,Col,Row,Radius,Min,Max,110,250),
		markDialDegrees(0,Col,Row,Radius,110,250,Fleck,Min,Mark,Inc),
		setTextJustify(center_Text, top_Text),
		textHeight("H",Height),
		TextRow = Row + Height / 2,
		outtextxy(Col,TextRow,Unit).

/*
--------------------------------------------------------------------- 
	ROUTINES FOR A PEDAL DIAL
---------------------------------------------------------------------
*/

PREDICATES

	setPedalDial(Integer,Integer,Real,Real,Real,Real,Real)
	makePedalDial(Integer,Integer,Integer,Real,Real,Real,
		String,DialBands)

CLAUSES

	setPedalDial(Col,Row,Radius,Min,Max,OldValue,Value) :-
		StartDegree = 240,
		EndDegree = 270,
		NRadius = Radius * 2,
		NCol = Col - Radius,
		setAnyDial(NCol,Row,NRadius,Min,Max,
			OldValue,Value,StartDegree,EndDegree).


	makePedalDial(Col,Row,Radius,Min,Max,Mark,Unit,DialBands) :-
		SCol = Col - Radius,
		NRadius = Radius * 2,
		arc(SCol,Row,0,30,NRadius),

		DegreeRange = 270 - 240,
		ValueRange = Max - Min,
		Marking = DegreeRange / (ValueRange / Mark),
		Fleck = ValueRange / Mark,
		Inc = Marking / Fleck,

		markDialBands(DialBands,SCol,Row,NRadius,Min,Max,240,270),
		markDialDegrees(0,SCol,Row,NRadius,240,270,
			Fleck,Min,Mark,Inc),
		setTextJustify(center_Text, top_Text),
		textheight("H",Height),
		TextRow = Row + Height / 2,
		outtextxy(Col, TextRow, Unit).

/*
--------------------------------------------------------------------- 
	ROUTINES FOR A HORIZONTAL STRIP DIAL
---------------------------------------------------------------------
*/

PREDICATES

	setHStripDial(Integer,Integer,Real,Real,Real,Real).
	makeHStripDial(Integer,Integer,Integer,Real,Real,Real,
		String,DialBands).
	labelHStripMark(Integer,Integer,Integer).
	drawHStripBand(Integer,Integer,Integer,Integer,Integer).
	markHStripBands(DialBands,Integer,Integer,Integer,Integer,
		Integer,Integer).
	markHStripUnits(Integer,Integer,Integer,Integer,Integer,
		Real,Real,Real,Real).

CLAUSES

	setHStripDial(SCol,Row,Radius,Min,Max,Value) :-
		Col = SCol - Radius,
		Length = Radius * 2,
		Right = ((Value - Min) / (Max - Min)) * Length + Col,
		textheight("H",Height),
		InnerBoxTopRow = Row + Height * 2 + 1,
		InnerBoxBottomRow = Row + Height * 3,
		getcolor(Color),
		getbkcolor(BkColor),
		BoxRight = Col + Length,
		Poly = [Col,InnerBoxTopRow,
			Col,InnerBoxBottomRow,
			Right,InnerBoxBottomRow,
			Right,InnerBoxTopRow],
		Poly2 = [Right,InnerBoxTopRow,
			Right,InnerBoxBottomRow,
			BoxRight,InnerBoxBottomRow,
			BoxRight,InnerBoxTopRow],
		setfillstyle(solid_Fill,Color),
		fillpoly(Poly),
		setcolor(BkColor),
		setfillstyle(solid_Fill,BkColor),
		fillpoly(Poly2),
		setcolor(Color).


	makeHStripDial(SCol,Row,Radius,Min,Max,Mark,Unit,DialBands):-
		Length = Radius * 2,
		Col = SCol - Radius,
		textheight("H",Height),
		OuterBoxTopRow = Row + Height,
		OuterBoxBottomRow = Row + Height * 4,
		InnerBoxTopRow = Row + Height * 2,
		InnerBoxBottomRow = Row + Height * 3,
		Right = Col + Length,

		markHStripBands(DialBands,Col,Length,
			OuterBoxTopRow,OuterBoxBottomRow,Min,Max),
		rectangle(Col,OuterBoxTopRow,Right,OuterBoxBottomRow),

		rectangle(Col,InnerBoxTopRow,Right,InnerBoxBottomRow),
		Poly = [Col,InnerBoxTopRow,
			Col,InnerBoxBottomRow,
			Right,InnerBoxBottomRow,
			Right,InnerBoxTopRow],
		getbkcolor(Color),
		setfillstyle(solid_Fill,Color),
		fillpoly(Poly),
	
		ValueRange = Max - Min,
		Marking = Length / (ValueRange / Mark),
		Fleck = ValueRange / Mark,
		Inc = Marking / Fleck,

		markHStripUnits(0,Col,OuterBoxTopRow,Length,Height,
			Fleck,Min,Mark,Inc),
		settextjustify(center_Text, top_Text),
		TextRow = OuterBoxBottomRow + Height / 2,
		outtextxy(SCol, TextRow, Unit).


	markHStripBands([band(Color,Start,Stop)|Tail],
			Col,Length,Top,Bottom,Min,Max) :- !,
		BandStart = ((Start - Min) / (Max - Min)) * Length + Col,
		BandStop = ((Stop - Min) / (Max - Min)) * Length + Col,
		drawHStripBand(Top,Bottom,BandStart,BandStop,Color),
		markHStripBands(Tail,Col,Length,Top,Bottom,Min,Max).
	markHStripBands(_,_,_,_,_,_,_).


	drawHStripBand(Top,Bottom,BandStart,BandStop,Color):-
		SetFillStyle(solid_Fill,Color),
		bar(BandStart,Top,BandStop,Bottom).


	markHStripUnits(Counter,Col,Row,Length,Height,Fleck,Min,Mark,Inc):-
		VCol = round(Counter * Inc + Col),
		VCol <= Length + Col,
		Counter mod round(Fleck) = 0,!,
		EndRow = Row + Height,
		line(VCol, Row, VCol, EndRow),
		Value = (Counter div round(Fleck)) * Mark + Min,
		labelHStripMark(VCol,Row,Value),
		NCounter = Counter + 1,
		markHStripUnits(NCounter,Col,Row,Length,Height,
			Fleck,Min,Mark,Inc).

	markHStripUnits(Counter,Col,Row,Length,Height,Fleck,Min,Mark,Inc) :-
		VCol = round(Counter * Inc + Col),
		VCol <= Length + Col,!,
		EndRow = Row + Height/2,
		line(VCol, Row, VCol, EndRow),
		NCounter = Counter + 1,
		markHStripUnits(NCounter,Col,Row,Length,Height,
			Fleck,Min,Mark,Inc).
	markHStripUnits(_,_,_,_,_,_,_,_,_).


	labelHStripMark(Col,Row,Val) :-
		settextjustify(center_Text,bottom_Text),
		str_int(Value,Val),
		outtextxy(Col,Row,Value).


/*
---------------------------------------------------------------------
		ROUTINES FOR A VERTICAL STRIP DIAL
---------------------------------------------------------------------
*/

PREDICATES

	setVStripDial(Integer,Integer,Real,Real,Real,Real)
	makeVStripDial(Integer,Integer,Integer,Real,Real,Real,
		String,DialBands)
	labelVStripMark(Integer,Integer,Integer)
	drawVStripBand(Integer,Integer,Integer,Integer,Integer)
	markVStripBands(DialBands,Integer,Integer,Integer,Integer,
		Integer,Integer)
	markVStripUnits(Integer,Integer,Integer,Integer,Integer,
	  	Real,Real,Real,Real)

CLAUSES

	setVStripDial(Col,SRow,Radius,Min,Max,Value) :-
  		getaspectratio(Xasp,Yasp),
		Length = round(Radius * 2 * Xasp/Yasp),
		Row = SRow + Length / 2,
		Top = Row - ((Value - Min) / (Max - Min)) * Length,
		textwidth("H",Width),
		InnerBoxLeftCol = Width * 2 + Col + 1,
		InnerBoxRightCol = Width * 3 + Col - 1,
		BoxTop = Row - Length,
		getcolor(Color),
		getbkcolor(BkColor),
		Poly = [InnerBoxLeftCol,Top,
		        InnerBoxLeftCol,Row,
	        	InnerBoxRightCol,Row,
		        InnerBoxRightCol,Top],
		Poly2 = [InnerBoxLeftCol,Top,
	        	InnerBoxLeftCol,BoxTop,
		        InnerBoxRightCol,BoxTop,
	         	InnerBoxRightCol,Top],
		setfillstyle(solid_Fill,Color),
		fillpoly(Poly),
		setcolor(BkColor),
		setfillstyle(solid_Fill,BkColor),
		fillpoly(Poly2),
		setcolor(Color).

	makeVStripDial(Col,SRow,Radius,Min,Max,Mark,Unit,DialBands) :-
	  	getaspectratio(Xasp,Yasp),
		Length = round(Radius * 2.0 * Xasp/Yasp),
		textwidth("H",Width),
		Row = SRow + Length / 2,
		Top = Row - Length,
		OuterBoxLeftCol = Col + Width,
		OuterBoxRightCol = Col + Width * 4,
		InnerBoxLeftCol = Col + Width * 2,
		InnerBoxRightCol = Col + Width * 3,
		rectangle(OuterBoxLeftCol,Row,OuterBoxRightCol,Top),
		markVStripBands(DialBands,Row,Length,
			OuterBoxLeftCol,OuterBoxRightCol,Min,Max),
		Poly = [InnerBoxLeftCol,Top,
			InnerBoxLeftCol,Row,
			InnerBoxRightCol,Row,
			InnerBoxRightCol,Top],
		getbkcolor(Color),
		setfillstyle(solid_Fill,Color),
		fillpoly(Poly),

		ValueRange = Max - Min,
		Marking = (Length / (ValueRange / Mark)),
		Fleck = ValueRange / Mark,
		Inc = Marking / Fleck,

		markVStripUnits(0,OuterBoxLeftCol,Row,Length,Width,
			Fleck,Min,Mark,Inc),
		settextjustify(center_Text, top_Text),
		TextRow = Row + Width / 2,
		TextCol = (InnerBoxLeftCol + InnerBoxRightCol) / 2,
		outtextxy(TextCol, TextRow, Unit).


	markVStripBands([band(Color,Start,Stop)|Tail],
			Row,Length,Left,Right,Min,Max) :- !,
		BandStart = Row - ((Start - Min) / (Max - Min)) * Length,
		BandStop = Row - ((Stop - Min) / (Max - Min)) * Length,
		drawVStripBand(BandStart,BandStop,Left,Right,Color),
		markVStripBands(Tail,Row,Length,Left,Right,Min,Max).
	markVStripBands(_,_,_,_,_,_,_).


	drawVStripBand(Bottom,Top,Left,Right,Color) :-
		line(Left,Top,Right,Top),
		line(Left,Bottom,Right,Bottom),
		FillColl = (Left + Right) / 2,
		FillRow = (Top + Bottom) / 2,
		setfillstyle(solid_Fill,Color),
		getcolor(ForeGr),
		floodfill(FillColl, FillRow, ForeGr).


	markVStripUnits(Counter,Col,Row,Length,Width,Fleck,Min,Mark,Inc) :-
		VRow = round(Row - Counter * Inc),
		VRow >= Row - Length,
		Counter mod round(Fleck) = 0,
		EndCol = Col + Width,
		!,
		line(Col, VRow, EndCol, VRow),
		Value = (Counter div round(Fleck)) * Mark + Min,
		labelVStripMark(Col,VRow,Value),
		NCounter = Counter + 1,
		markVStripUnits(NCounter,Col,Row,Length,Width,
			Fleck,Min,Mark,Inc).

	markVStripUnits(Counter,Col,Row,Length,Width,Fleck,Min,Mark,Inc) :-
		VRow = round(Row - Counter * Inc),
		VRow >= Row - Length,
		EndCol = Col + Width / 2,
		!,
		line(Col, VRow, EndCol, VRow),
		NCounter = Counter + 1,
		markVStripUnits(NCounter,Col,Row,Length,Width,
			Fleck,Min,Mark,Inc).

	markVStripUnits(_,_,_,_,_,_,_,_,_).


	labelVStripMark(Col,Row,Val) :-
		settextjustify(right_Text, bottom_Text),
		str_int(Value,Val),
		outtextxy(Col,Row,Value).

/*
---------------------------------------------------------------------
	
	GetDialRadius

	Convert size to radius and set textstyle
	
---------------------------------------------------------------------
*/

PREDICATES

	getDialRadius(Integer,Integer).
%	getDialRadius(Size,Radius).

CLAUSES

	getDialRadius(5,70) :- !,
		settextstyle(default_Font,horiz_dir,1).
	getDialRadius(4,50) :- !,
		settextstyle(small_Font,horiz_dir,4).
	getDialRadius(3,40) :- !,
	  	setTextStyle(small_Font,horiz_dir,3).
	getDialRadius(2,30) :- !,
		setTextStyle(small_Font,horiz_dir,2).
	getDialRadius(1,20) :- !,
		setTextStyle(small_Font,horiz_dir,1).

/*
---------------------------------------------------------------------
	 Dials & Database
---------------------------------------------------------------------
*/

DATABASE - dials

%	dial_(Dial,Type,Col,Row,Radius,Min,Max,Mark,Unit,BandList)
	dial_(Selector,DIalType,Real,Real,Real,Real,Real,Real,
		String,Integer,DialBands).

%	Actual value of a dial
	dial_Value_(Selector,Real)

%	Gives a message to another object when set
	dial_WhenSet_(Selector,Object)

/*
---------------------------------------------------------------------
	 Find a new dial selector
---------------------------------------------------------------------
*/

PREDICATES

	dial_Selector(Selector)
	dial_Selector(Integer,Selector)

CLAUSES

	dial_Selector(Selector) :-
		dial_Selector(1,Selector).

	dial_Selector(Selector,Selector) :-
		not(dial_(Selector,_,_,_,_,_,_,_,_,_,_)), !.
	dial_Selector(Count,Selector) :-
		Next = Count + 1,
		dial_Selector(Next,Selector).

/*
---------------------------------------------------------------------
	SetTypeDialValue
---------------------------------------------------------------------
*/

PREDICATES

	setTypeDial(DialType,Integer,Integer,Integer,
		Integer,Integer,Integer,Integer)

CLAUSES

	setTypeDial(full,Col,Row,Radius,Min,Max,OldValue,NewValue) :-
		SetFullDial(Col,Row,Radius,Min,Max,OldValue,NewValue).
	setTypeDial(half,Col,Row,Radius,Min,Max,OldValue,NewValue) :-
		setHalfDial(Col,Row,Radius,Min,Max,OldValue,NewValue).
	setTypeDial(pedal,Col,Row,Radius,Min,Max,OldValue,NewValue) :-
		setPedalDial(Col,Row,Radius,Min,Max,OldValue,NewValue).
	setTypeDial(hstrip,Col,Row,Radius,Min,Max,_,NewValue) :-
		setHstripDial(Col,Row,Radius,Min,Max,NewValue).
	setTypeDial(vstrip,Col,Row,Radius,Min,Max,_,NewValue) :-
		setVStripDial(Col,Row,Radius,Min,Max,NewValue).

/*
---------------------------------------------------------------------
	
	MakeTypeDialValue
	
	Call the correct dial drawing routine
	
---------------------------------------------------------------------
*/

PREDICATES

	makeTypeDial(DialType,Integer,Integer,Real,Real,Real,Real,
		String,DialBands)

CLAUSES

	makeTypeDial(full,Col,Row,Radius,Min,Max,Mark,Unit,DialBands) :-
		makeFullDial(Col,Row,Radius,Min,Max,Mark,Unit,DialBands).
	makeTypeDial(half,Col,Row,Radius,Min,Max,Mark,Unit,DialBands) :-
		makeHalfDial(Col,Row,Radius,Min,Max,Mark,Unit,DialBands).
	makeTypeDial(pedal,Col,Row,Radius,Min,Max,Mark,Unit,DialBands) :-
		makePedalDial(Col,Row,Radius,Min,Max,Mark,Unit,DialBands).
	makeTypeDial(hstrip,Col,Row,Radius,Min,Max,Mark,Unit,DialBands) :-
		makeHStripDial(Col,Row,Radius,Min,Max,Mark,Unit,DialBands).
	makeTypeDial(vstrip,Col,Row,Radius,Min,Max,Mark,Unit,DialBands) :-
		makeVStripDial(Col,Row,Radius,Min,Max,Mark,Unit,DialBands).

/*
---------------------------------------------------------------------
	Calculate the area for a dial
---------------------------------------------------------------------
*/

PREDICATES

	calcdialbox(DialType,Integer,Integer,Integer,Integer,
		Integer,Integer,Integer,Integer,Integer)

	calcdialbox_aux(DialType,Integer,Integer,Integer,Integer,
		Integer,Integer,Integer,Integer,Integer)

	bottom_adj(dialtype, integer,integer)

CLAUSES

	calcdialbox(Type, X, Y, Rad, Aa, Ba, Left, Top, Right, Bot) :-
		calcdialbox_aux(Type, X, Y, Rad, Aa, Ba, L, T, R, B),
		getmaxx(MaxX), 
		getmaxy(MaxY),
		max(0, L, Left),
		max(0, T, Top),
		min(R, MaxX, Right),
		min(B, MaxY, Bot).
		 

	
	calcdialbox_aux(hstrip,X,Y,Radius,_,_,Left,Top,Right,Bottom) :- !,
		textheight("H", TH), 
		textwidth("100", TW),
		Right = X + Radius + TW + 4, 
		Left = X - Radius - 4,
		Top = Y - 4, 
		Bottom = Y + TH * 5 + 4. 

	calcdialbox_aux(vstrip,X,Y,Radius,_,_,Left,Top,Right,Bottom) :- !,
  		GetAspectRatio(Xasp,Yasp),
		Length = round(Radius * 2.0 * Xasp / Yasp),
		textwidth("H",TW),
		Left=X - TW - 4,
		Right=X + 6 * TW,
		textheight("DIAL",TH),
		Bottom = Y + Length / 2 + TH + 4, 
		Top = (Y - Length/2)-(TH + 4).

	calcdialbox_aux(Type,X,Y,Radius,_,_,Left,Top,Right,Bottom) :-
		textwidth("100",TWidth),
		Width = Radius + TWidth,
		Left = X - Width, 
		Right = X + Width,
	  	GetAspectRatio(Xasp,Yasp),
		textheight("100",THeigth),
		Height = 2 + THeigth + round(Radius * (Xasp / Yasp)),
		Top = Y - Height - 2 ,
		bottom_adj(Type, Height, Factor),
		Bottom = Y + 2 + Factor.
		
	bottom_adj(full, X,X):-!.	
	bottom_adj(_, _, Add):-
		textheight("LABEL",Temp), 
		Add=Temp+4.

/*
---------------------------------------------------------------------
		UTILITY ROUTINES FOR A DIAL
---------------------------------------------------------------------
*/

CLAUSES

	dial_Set(Dial,NewValue):-
		getmaxx(MaxX),
		getmaxy(MaxY),
		setviewport(0,0,MaxX,MaxY,0),
		dial_(Dial,DialType,Col,Row,Size,Min,Max,_,_,FColor,_),
		dial_Value_(Dial,OldValue),
		NewValue >< OldValue,
		max(NewValue,Min,Val1), 
		min(Val1,Max,Val),
		retract(dial_Value_(Dial,_)),
		assert(dial_Value_(Dial,Val)),!,
		getDialRadius(Size,Radius),
		getcolor(OFColor), 
		setcolor(FColor),
		mouse_off,
		setTypeDial(DialType,Col,Row,Radius,Min,Max,OldValue,Val),
		mouse_on,
		setcolor(OFColor).
	dial_Set(_,_).


	dial_Get(Dial,Value) :-
		dial_Value_(Dial,Value), !.


	dial_Create(Dial,DialType,Col,Row,Size,Min,Max,Mark,Unit,DialBands) :-
		getmaxx(MaxX), 
		getmaxy(MaxY),
		setviewport(0,0,MaxX,MaxY,0),
		dial_Selector(Dial),
		gettextsettings(Font,Dir,FSize,_,_),
		getDialRadius(Size,Radius),
		getColor(FColor),
		mouse_off,
		makeTypeDial(DialType,Col,Row,Radius,Min,Max,
			Mark,Unit,DialBands),
		mouse_on,
		assert(dial_(Dial,DialType,Col,Row,Size,Min,Max,
			Mark,Unit,FColor,DialBands)),
		assert(dial_Value_(Dial,-1)),
		dial_Set(Dial,Min),
		settextstyle(Font,Dir,FSize),
		dial_Region(Dial, Region),
		object_Install(dial(Dial), Region).


	dial_Redraw(Dial) :-
		getmaxx(MaxX), 
		getmaxy(MaxY),
		setviewport(0,0,MaxX,MaxY,0),
		dial_Get(Dial,Value),
		dial_(Dial,DialType,Col,Row,Size,Min,Max,
			Mark,Unit,FColor,DialBands), !,
		getDialRadius(Size,Radius),
		setcolor(FColor),
		mouse_off,
		makeTypeDial(DialType,Col,Row,Radius,Min,Max,
			Mark,Unit,DialBands),
		mouse_on,
		retractall(dial_Value_(Dial,_)),
		assert(dial_Value_(Dial,Value)),
		
		dial_Set(Dial,Value).


	dial_Move(Dial):-
		getmaxx(MaxX), 
		getmaxy(MaxY),
		setviewport(0,0,MaxX,MaxY,0),
		
		dial_(Dial,DialType,Col,Row,Size,Min,Max,
			Mark,Unit,FColor,DialBands), !,
		getDialRadius(Size,Radius),
		calcDialBox(DialType,Col,Row,Radius,Min,Max,
			Left,Top,Right,Bottom),
		object_Remove(dial(Dial)),
		decidePosition(dial(Dial),Left,Top,Right,Bottom,Left1,Top1),
		NewCol = Col + Left1 - Left,
		NewRow = Row + Top1 - Top,
		retractall(dial_(Dial,_,_,_,_,_,_,_,_,_,_)),
		assert(dial_(Dial,DialType,NewCol,NewRow,Size,Min,Max,
			Mark,Unit,FColor,DialBands)),
		Right1 = Right + Left1 - Left,
		Bottom1 = Bottom + Top1 - Top,
		object_Install(dial(Dial),
			region(Left1,Top1,Right1,Bottom1)),
		retract(dial_value_(Dial, Value)),
		assert(dial_value_(Dial, -1)),
		dial_redraw(Dial),
		dial_Set(Dial,Value),!.


	dial_MousePress(Selector) :- dial_Move(Selector).


	dial_Region(Dial,region(Left,Top,Right,Bottom)) :-
  		dial_(Dial,DialType,Col,Row,Size,Min,Max,_,_,_,_), !,
		getDialRadius(Size,Radius),
		calcDialBox(DialType,Col,Row,Radius,Min,Max,
			Left,Top,Right,Bottom).

/*
---------------------------------------------------------------------
	Database handling predicates
---------------------------------------------------------------------
*/

	dial_Clear :- retractall(_,dials).

	dial_Save(File) :- save(File,dials).

	dial_Consult(File) :- consult(File,dials).



