/*
=====================================================================
	DRAWVIEW.PRO
	
	Window View Predicates
	
	Copyright (C) 1990 Prolog Development Center
=====================================================================
*/

include "GRAPHICS\\GRAPH.PRO"
include "GRAPHICS\\CHART.PRO"

DATABASE - drawlistView

	drawlistView_(Selector,Selector,Selector,Selector,Selector,
		Region,Region,Region,Region,Region)
%	drawlistView_(Selector,WindowSel,SB1,SB2,SB3, 
%       	WArea,SB1Area,SB2Area,SB3Area,CircleArea)

/*
---------------------------------------------------------------------

	Find a new drawlistView selector (Local help predicate)
	Return the first unused number
	
---------------------------------------------------------------------
*/

PREDICATES

	drawlistView_Selector(Selector)
	drawlistView_Selector(Integer,Selector)

CLAUSES

	drawlistView_Selector(Selector) :-
		drawlistView_Selector(1,Selector).

	drawlistView_Selector(Selector,Selector) :-
		not(drawlistView_(Selector,_,_,_,_,_,_,_,_,_)), !.
	drawlistView_Selector(Count,Selector) :-
		Next = Count + 1,
		drawlistView_Selector(Next,Selector).

/*
---------------------------------------------------------------------

	drawlistView_Update
	
	Redraw the screen after a change

---------------------------------------------------------------------
*/

PREDICATES

	drawlistView_Update(Selector)
	adjust_XOffset(Selector)
	adjust_YOffset(Selector)
	adjust_Scale(Selector)
	bar_box(integer,integer)
	
CLAUSES

	drawlistView_Update(Selector) :-
		mouse_off,
		drawlistView_(Selector,WSelector,_,_,_,_,_,_,_,Region), !,
		Region = region(VLeft,VTop,VRight,VBottom),
		GWindow_Colors(WSelector,Colors),
		Colors = colors(WColor,_,_,_),

%	Draw background etc.
		setfillstyle(solid_fill,WColor),
		bar(VLeft,VTop,VRight,VBottom),
		gWindow_setviewport(VLeft,VTop,VRight,VBottom,clip),

		adjust_Scale(Selector),
		adjust_XOffset(Selector),
		adjust_YOffset(Selector),

		draw_set([
			style(14,9,8),
			draw_pie(500,500,0,90,800),
			style(15,12,8),
			draw_pie(500,500,180,270,800),
			style(12,5,6),
			draw_bar(300,600,450,800),
			style(11,0,1),
			draw_text(600,400,"%","Prolog",no_flag),
			style(10,0,1),
			draw_text(600,300,"%","Development",no_flag),
			style(13,0,1),
			draw_text(600,200,"%","Center",no_flag)
			]),

		maxViewPort,
		mouse_on, !.


	adjust_XOffset(Selector) :-
		drawlistView_(Selector,_,SBSel,_,_,_,_,_,_,_),
		sBar_Get(SBSel,XOffset,Right),
		active_scale_(Sel),
		scale_(Sel,XMin,XMax,YMin,YMax),
		Range = XMax - XMin,
		Unit =  XOffset / Right,
		NewMin = (Range / 2) - (Unit * Range) * 2,
		NewMax = NewMin + Range,
		retractall(scale_(Sel,_,_,_,_)), !,
		assert(scale_(Sel,NewMin,NewMax,YMin,YMax)).
	adjust_XOffset(_).


	adjust_YOffset(Selector) :-
		drawlistView_(Selector,_,_,_,SBSel,_,_,_,_,_),
		sBar_Get(SBSel,YOffset,Top),
		active_scale_(Sel),
		scale_(Sel,XMin,XMax,YMin,YMax),
		Range = (YMax - YMin),
		Unit = YOffset / Top,
		NewMin = (Range / 2) - (Unit * Range) * 2,
		NewMax = NewMin + Range,
		retractall(scale_(Sel,_,_,_,_)), !,
		assert(scale_(Sel,XMin,XMax,NewMin,NewMax)).
	adjust_YOffset(_).


	adjust_Scale(Selector) :-
		drawlistView_(Selector,_,_,SBSel,_,_,_,_,_,_),
		sBar_Get(SBSel,Scale,Top),
		active_scale_(Sel),
		scale_(Sel,XMin,_,YMin,_),
		Unit = Scale / Top,
		NewXMax = XMin + 750 + (1000 * Unit),
		NewYMax = YMin + 750 + (1000 * Unit),
		retractall(scale_(Sel,_,_,_,_)), !,
		assert(scale_(Sel,XMin,NewXMax,YMin,NewYMax)).
	adjust_Scale(_).


%% get the pixel dimensions for the bar strips
	bar_box(X, Y) :-
		getaspectratio(Xasp,Yasp),
		X=15, 			   % mouse cursor width
		Y=round(X*(Xasp/Yasp)).


/*
---------------------------------------------------------------------

	drawlistView_Remove

---------------------------------------------------------------------
*/

PREDICATES

	drawlistView_Remove(Selector)

CLAUSES

	drawlistView_Remove(Selector):-
		drawlistView_(Selector,Window,SB1,SB2,SB3,_,_,_,_,_), !,
		sBar_Remove(SB1,clear_off),
		sBar_Remove(SB2,clear_off),
		sBar_Remove(SB3,clear_off),
		gWindow_Remove(Window),
		retractall(drawlistView_(Selector,_,_,_,_,_,_,_,_,_)),
	        object_Get(drawlistView(Selector),Region),
		object_Remove(drawlistView(Selector)),
	        screen_RedrawRegion(Region), !.
	drawlistView_Remove(_).

/*
---------------------------------------------------------------------

	drawlistView_Redraw

---------------------------------------------------------------------
*/

CLAUSES

	drawlistView_Redraw(Selector):-
		drawlistView_(Selector,Window,SB1,SB2,SB3,Region,_,_,_,_), !,
		Region = region(_,_,Right,Bottom),
		gWindow_Redraw(Window),
		sBar_Redraw(SB1),
		sBar_Redraw(SB2),
		sBar_Redraw(SB3),
		
		bar_box(Window_right, Window_Bottom),
		Left = Right - Window_right,
		Top = Bottom - Window_bottom,
		setcolor(15),
		XCorner = Left + 9, 
		YCorner = Top + 6,
		mouse_off,
		rectangle(Left,Top,XCorner,YCorner),
		mouse_on,
		drawlistView_Update(Selector), !.

/*
---------------------------------------------------------------------

	drawlistView    Define and Create

---------------------------------------------------------------------
*/

PREDICATES

	drawlistView_Define(Selector,Selector,Integer,Integer,Integer)

CLAUSES

	drawlistView_Create(Selector,Colors) :-
		drawlistView_Selector(Selector),
		format(Title,"Window %",Selector),
		make_scale(1,0,1000,0,1000),
		gWindow_Define(Window,Title,Colors),
		drawlistView_Define(Selector,Window,5,0,0),
		drawlistView_Region(Selector,Region),
		object_install(drawlistView(Selector),Region),
		drawlistView_Redraw(Selector).


	drawlistView_Define(Selector,Window,Scale,XOffset,YOffset) :-
		gWindow_Region(Window,Warea),
		WArea = region(Left,Top,Right,Bottom),

		bar_box(Window_Side, Window_bottom),

		Left1 = Left + Window_Side,    % left
		Top1 = Top + window_top,
		Right1 = Right - Window_Side,  % right
		Bottom1 = Bottom - Window_bottom,

		SB1Left = Left1, 
		SB1Top = Bottom1,
		SB1Right = Right1, 
		SB1Bottom = Bottom - 1,
		SB1Area = region(SB1Left,SB1Top,SB1Right,SB1Bottom),
		sBar_Define(SB1,horz,SB1Area,scroll_bar_style,15),

		settextstyle(default_font,horiz_dir,1),
		SB2Left = Left + 1, 
		SB2Right = Left1,
		SB2Top = Top1, 
		SB2Bottom = Bottom1,
		SB2Area = region(SB2Left,SB2Top,SB2Right,SB2Bottom),
		sBar_Define(SB2,vert,SB2Area,scale_bar_style,7),

		SB3Left = Right1, 
		SB3Top = Top1,
		SB3Right = Right - 1, 
		SB3Bottom = Bottom1,
		SB3Area = region(SB3Left,SB3Top,SB3Right,SB3Bottom),
		sBar_Define(SB3,vert,SB3Area,scroll_bar_style,7),

	        sbar_get(SB1,_,XMaxvalue),
	        sbar_get(SB2,_,SMaxvalue),
	        sbar_get(SB3,_,YMaxvalue),

        	getMaxX(MaxX),
	        Sbar1_value = round(XOffset * (XMaxvalue / MaxX)),
	        getMaxY(MaxY),
        	Sbar2_value = round(Scale * (SMaxValue / MaxY)),
        	SBar3_value = round(YOffset * (YMaxvalue / MaxY)),

	        sbar_set(SB1,Sbar1_value),
        	sbar_set(SB2,Sbar2_value),
	        sbar_set(SB3,Sbar3_value),

        	Delay = 50,
		  %sBar_Install_AfterSet(SB1,drawlistView(Selector)),
		sBar_Install_WhenSet(SB1,drawlistView(Selector),Delay),
		  %sBar_Install_AfterSet(SB2,drawlistView(Selector)),
		sBar_Install_WhenSet(SB2,drawlistView(Selector),Delay),
		  %sBar_Install_AfterSet(SB3,drawlistView(Selector)),
		sBar_Install_WhenSet(SB3,drawlistView(Selector),Delay),

		LeftIn = Left1 + 1, 
		RightIn = Right1 - 1,
		TopIn = Top1 + 1, 
		BottomIn = Bottom1 - 1,
		rectangle(Left1,Top1,Right1,Bottom1),
		CircleArea = region(LeftIn,TopIn,RightIn,BottomIn),
		assert(drawlistView_(Selector,Window,SB1,SB2,SB3,
                	WArea,SB1Area,SB2Area,SB3Area,CircleArea)), 
		!.

/*
---------------------------------------------------------------------

	drawlistView_Region

---------------------------------------------------------------------
*/

CLAUSES

	drawlistView_Region(Selector,Region) :-
		drawlistView_(Selector,_,_,_,_,Region,_,_,_,_), !.

/*
---------------------------------------------------------------------

		drawlistView_Resize

---------------------------------------------------------------------
*/

PREDICATES

	drawlistView_Resize(Selector)
	drawlistView_Move(Selector)
	redraw_areas(Region,Region).

CLAUSES

	redraw_areas(Area1,Area2) :-
		screen_RedrawRegion(Area1),
		not(overlapping(Area1,Area2)), !,
		screen_RedrawRegion(Area2).
	redraw_areas(_,_).



	drawlistView_Resize(Selector) :-
		drawlistView_(Selector,Window,SB1,SB2,SB3,_,_,_,_,_), !,
		drawlistView_Region(Selector,OldWArea),
		gWindow_ReSize(Window),

        % Delete all Old information
		sBar_Remove(SB1,clear_off),
		sBar_Remove(SB2,clear_off),
		sBar_Remove(SB3,clear_off),
		retractall(drawlistView_(Selector,_,_,_,_,_,_,_,_,_)), !,

	% Create a new drawlistView
		drawlistView_Define(Selector,Window,1,1,1),
		drawlistView_Region(Selector,Area),

		object_Refresh(drawlistView(Selector),Area),

        	redraw_areas(OldWarea,Area).


	drawlistView_Move(Selector) :-
		drawlistView_(Selector,Window,SB1,SB2,SB3,_,_,_,_,_), !,
		drawlistView_Region(Selector,OldWArea),
		gWindow_Move(Window),

        % Delete all Old information
		sBar_Remove(SB1,clear_off),
		sBar_Remove(SB2,clear_off),
		sBar_Remove(SB3,clear_off),
		retractall(drawlistView_(Selector,_,_,_,_,_,_,_,_,_)), !,

	% Create a new drawlistView
		drawlistView_Define(Selector,Window,1,1,1),
		drawlistView_Region(Selector,Region),
		object_Refresh(drawlistView(Selector),Region),
        	redraw_areas(OldWarea,Region).


/*
---------------------------------------------------------------------

	drawlistView_MousePress

---------------------------------------------------------------------
*/

PREDICATES

	redraw_if_covered(Selector)

CLAUSES
	drawlistView_MousePress(Selector,MouseX,MouseY) :-
		drawlistView_(Selector,_,_,_,_,Region,_,_,_,_),
	        not(partly_covered(drawlistView(Selector))),
		Region = region(WLeft,WTop,WRight,_),
		bar_box(Window_Left,_),
		MouseX >= WLeft + Window_left, 
		MouseX <= WRight,
		MouseY >= WTop, 
		MouseY <= WTop + title_bar_height, !,
		drawlistView_Move(Selector).

	drawlistView_MousePress(Selector,MouseX,MouseY) :-
		drawlistView_(Selector,_,_,_,_,Region,_,_,_,_),
	        not(partly_covered(drawlistView(Selector))),
		
		bar_box(Window_Right, Window_Bottom),
		Region = region(_,_,WRight,WBottom),
		MouseX >= WRight - Window_right,
		MouseX <= WRight, 
		MouseY <= WBottom,
		MouseY >= WBottom - Window_bottom, !,
		drawlistView_Resize(Selector).

	drawlistView_MousePress(Selector,MouseX,MouseY) :-
		drawlistView_(Selector,_,_,_,_,Region,_,_,_,_),
	        not(partly_covered(drawlistView(Selector))),
		Region = region(WLeft,WTop,_,_),
		bar_box(Window_left,_),
		MouseX <= WLeft + Window_left,
		MouseX >= WLeft, MouseY >= WTop,
		MouseY <= WTop + window_top, !,
		drawlistView_Remove(Selector).

	drawlistView_MousePress(Selector,MouseX,MouseY) :-
		drawlistView_(Selector,_,Bar,_,_,_,Region,_,_,_),
	        not(partly_covered(drawlistView(Selector))),
		region_Inside(MouseX,MouseY,Region), !,
		sBar_MousePress(Bar,MouseX,MouseY).

	drawlistView_MousePress(Selector,MouseX,MouseY) :-
		drawlistView_(Selector,_,_,Bar,_,_,_,Region,_,_),
	        not(partly_covered(drawlistView(Selector))),
		region_Inside(MouseX,MouseY,Region), !,
		sBar_MousePress(Bar,MouseX,MouseY).

	drawlistView_MousePress(Selector,MouseX,MouseY):-
		drawlistView_(Selector,_,_,_,Bar,_,_,_,Region,_),
        	not(partly_covered(drawlistView(Selector))),
		region_Inside(MouseX,MouseY,Region), !,
		sBar_MousePress(Bar,MouseX,MouseY).

	drawlistView_MousePress(Selector,MouseX,MouseY):-
		drawlistView_(Selector,_,_,_,_,Region,_,_,_,_),
		region_Inside(MouseX,MouseY,Region),!,
        	redraw_if_covered(Selector),
		object_Refresh(drawlistView(Selector),Region).


	redraw_if_covered(Selector) :-
		partly_covered(drawlistView(Selector)),
		drawlistView_redraw(Selector), !.
	redraw_if_covered(_).

	
	
/*
---------------------------------------------------------------------

	drawlistView_SBarSet

---------------------------------------------------------------------
*/

PREDICATES

	update_if_not_same(Selector,Integer,Integer)

CLAUSES

	update_if_not_same(_,Same, Same) :-!.  % do nothing if same
	update_if_not_same(Selector,_,_) :- 
		drawlistView_Update(Selector).
	

	drawlistView_sBarSet(Selector,SBSelector,Val,Max) :-
		drawlistView_(Selector,_,SBSelector,_,_,_,_,_,_,_),
		getMaxX(MaxX),
		Off = round((1.0 * MaxX) * Val / Max),
	        max(Off,0,Offset),
	        sBar_Get(SBSelector,Old,_), !,
		update_if_not_same(Selector,Old,Offset).

	drawlistView_sBarSet(Selector,SBSelector,Val,_) :-
		drawlistView_(Selector,_,_,SBSelector,_,_,_,_,_,_),
		Sca = 2 * Val + 2, 
		max(Sca,2,Scale),
	        sBar_Get(SBSelector,Old,_), !,
		update_if_not_same(Selector,Old,Scale).

	drawlistView_sBarSet(Selector,SBSelector,Val,Max) :-
		drawlistView_(Selector,_,_,_,SBSelector,_,_,_,_,_),
	        getMaxY(MaxY),
		Off = round((1.0 * MaxY) * Val / Max),
	        max(Off,0,Offset),
	        sBar_Get(SBSelector,Old,_), !,
		update_if_not_same(Selector,Old,Offset).


	drawlistView_Consult(Filename) :- consult(Filename,drawlistView).


	drawlistView_Save(Filename) :- save(Filename,drawlistView).


	drawlistView_Clear :- retractall(_,drawlistView).




