/*
=====================================================================
	GMENU.PRO
	
	Graphic Menu Handling
	
	Copyright (C) 1990 Prolog Development Center
=====================================================================
*/

/*
---------------------------------------------------------------------
	UTILITY PREDICATES
---------------------------------------------------------------------
*/

include "GRAPHICS\\GUTIL.PRO"

PREDICATES
	maxwidth(StringList,Integer,Integer)
	listlen(StringList,Integer)
	outTextList(Integer,Integer,Integer,StringList)

CLAUSES

	maxwidth([Head|Tail],Max,Width) :-
		textwidth(Head,TWidth),
		TWidth > Max, !,
		maxwidth(Tail,TWidth,Width).
	maxwidth([_|Tail],Max,Width) :-
		maxwidth(Tail,Max,Width).
	maxwidth([],Length,Length).


	listlen([],0).
	listlen([_|T],N) :-
		listlen(T,X),
		N = X + 1.


	outTextList(_,_,_,[]).
	outTextList(X,Y,YSTEP,[H|T]):-
		outtextxy(X,Y,H),
		Y1 = Y + YSTEP,
		outTextList(X,Y1,YSTEP,T).

/*
---------------------------------------------------------------------
	MOUSE TEST AND MENU LOOP
---------------------------------------------------------------------
*/

PREDICATES

	inside_gmenu(integer,integer,integer,integer,integer,integer)
	gmenu_loop(Integer,Integer,Integer,Integer,
		Integer,Integer,Integer,Integer,Integer)

CLAUSES

	inside_gmenu(MouseX,MouseY,Left,Right,Top,Bottom) :-
		MouseX > Left, 
		MouseX < Right,
		MouseY > Top, 
		MouseY < Bottom.

	gmenu_loop(_,_,Left,Top,Right,Bottom,ElemHeight,_,Choice) :-
		mouse_get(MouseX,MouseY,BUTTONS), 
		BUTTONS >< 0,
		inside_gmenu(MouseX,MouseY,Left,Right,Top,Bottom), !,
		Choice = (MouseY - Top) div ElemHeight.

	gmenu_loop(_,_,Left,Top,Right,Bottom,_,_,-1) :-
		mouse_get(MouseX,MouseY,BUTTONS), 
		BUTTONS >< 0,
		not(inside_gmenu(MouseX,MouseY,Left,Right,Top,Bottom)), !.

	gmenu_loop(WColour,FColour,Left,Top,Right,Bottom,
			ElemHeight,OldChoice,FinalChoice) :-
		mouse_get(MouseX,MouseY,_),
		MouseX > Left, MouseX < Right,
		MouseY > Top, MouseY < Bottom,
		Choice = (MouseY - Top) div ElemHeight,
		Choice >< OldChoice, !,
		Y1A = Top + OldChoice * ElemHeight,
		Y2A = Y1A + ElemHeight,
		Y1B = Top + Choice * ElemHeight,
		Y2B = Y1B + ElemHeight,
		mouse_off,
		setcolor(WColour),
		rectangle(Left,Y1A,Right,Y2A),
		setcolor(FColour),
		rectangle(Left,Y1B,Right,Y2B),
		mouse_on,
		gmenu_loop(WColour,FColour,Left,Top,Right,Bottom,
			ElemHeight,Choice,FinalChoice).
		gmenu_loop(WColour,FColour,Left,Top,Right,Bottom,
			ElemHeight,OldChoice,Choice) :-
		gmenu_loop(WColour,FColour,Left,Top,Right,Bottom,
			ElemHeight,OldChoice,Choice).

/*
---------------------------------------------------------------------
	GRAPHIC MENU
---------------------------------------------------------------------
*/

CLAUSES

	gmenu(Left1,Top1,WColour,FColour,TColor,List,Choice) :-
		settextjustify(left_text,top_text),
		listlen(List,ListLen),
		textheight("X",THeight),
		ElemHeight = THeight + THeight div 2,
		MenuHeight = ElemHeight * (1 + ListLen),
		maxwidth(List,minwidth,MaxWidth),
		MenuWidth = MaxWidth + minwidth,
		GetMaxX(MaxX), 
		GetMaxY(MaxY),
		LeftMax = MaxX - MenuWidth, 
		TopMax = MaxY - MenuHeight,
		min(LeftMax,Left1,Left), 
		min(TopMax,Top1,Top),
		Right = Left + MenuWidth, 
		Bottom = Top + MenuHeight,
		mouse_off,
		getimage(Left,Top,Right,Bottom,Image),
		setfillstyle(solid_fill,WColour),
		setColor(FColour),
		setlinestyle(solid_line,0,2),
		bar(Left,Top,Right,Bottom),
		rectangle(Left,Top,Right,Bottom),
		ListX = Left + 10, ListY = Top + THeight,
		setColor(TColor),
		outTextList(ListX,ListY,ElemHeight,List),
		mouse_on,
		mouse_waitrelease,
		Left2 = Left + 1, 
		Right2 = Right - 1,
		Top2 = Top + ElemHeight div 2,
		Bottom2 = Bottom - ElemHeight div 2,
		Y1A = Top2,
		Y2A = Y1A + ElemHeight,
		setcolor(FColour),
		%%setwritemode(xor_put), %%%
		mouse_off,
		rectangle(Left2,Y1A,Right2,Y2A),
		mouse_on,
		gmenu_loop(WColour,FColour,Left2,Top2,Right2,Bottom2,
			ElemHeight,0,Ch),
	%%	setwritemode(copy_put), %%
		mouse_off,
		putimage(Left,Top,Image,copy_put),
		mouse_on,
		mouse_waitrelease,
		Choice = Ch + 1,
		Choice > 0.

