/*
=====================================================================
	GWINDOWS.PRO
	
	Graphic Window Handling Predicates
	
	Copyright (C) 1990 Prolog Development Center
=====================================================================
*/

/*
---------------------------------------------------------------------
	GRAPHIC WINDOWS DATABASE HANDLING
---------------------------------------------------------------------
*/

DATABASE - gWindows

	gWindow_(Selector,Colors,Label,Region)

CLAUSES

	gWindows_Clear :- retractall(_,gWindows).
	

	gWindows_Save(Name) :- save(Name,gWindows).


	gWindows_Consult(Name) :- consult(Name,gWindows).


/*
---------------------------------------------------------------------
	REPEAT WHILE MOUSE BUTTONS ARE PRESSED
---------------------------------------------------------------------
*/

CLAUSES

	gWindow_rep_mouse(_,_) :-
		mouse_get(_,_,Buttons), Buttons = 0, !, fail.
	gWindow_rep_mouse(MouseX,MouseY) :-
		gWindow_mouse_get(MouseX,MouseY,Buttons),
		Buttons >< 0.
	gWindow_rep_mouse(MouseX,MouseY) :-
		gWindow_rep_mouse(MouseX,MouseY).


/*
---------------------------------------------------------------------
	GRAPHIC WINDOW VIEWPORTS
---------------------------------------------------------------------
*/

DATABASE - gWindow_viewport

	gWindow_viewport_(Integer,Integer,Integer,Integer)

PREDICATES

	gWindow_winviewport(Integer,Integer,Integer,Integer)

CLAUSES

	gWindow_winviewport(Left,Top,Right,Bottom) :-
		gWindow_viewport_(Left,Top,Right,Bottom), !.
	gWindow_winviewport(0,0,Right,Bottom) :-
		getmaxx(Right), getmaxy(Bottom).


	gWindow_initviewport(Top,Left,Right,Bottom,Clip) :-
		retractall(gWindow_viewport_(_,_,_,_)),
		assert(gWindow_viewport_(Top,Left,Right,Bottom)),
		setviewport(Top,Left,Right,Bottom,Clip).


	gWindow_setviewport(X1,Y1,X2,Y2,Clip) :-
		gWindow_winviewport(X3,Y3,_,_), !, 
		Left = X3 + X1, Top = Y3 + Y1,
		Right = X3 + X2, Bottom = Y3 + Y2,
		setviewport(Left,Top,Right,Bottom,Clip).


	gWindow_getviewport(X1,Y1,X2,Y2,Clip) :-
		gWindow_winviewport(A,B,_,_), !, 
		getviewsettings(E,F,G,H,Clip),
		X1 = E - A, Y1 = F - B,
		X2 = G - A, Y2 = H - B.


	gWindow_setmaxviewport :-
		gWindow_winviewport(A,B,C,D), !, 
		setviewport(A,B,C,D,clip).
   
   
	gWindow_getminx(XMin) :-
		gWindow_winviewport(XMin,_,_,_), !.
    
    
	gWindow_getmaxx(XMax) :-
		gWindow_winviewport(Left,_,Right,_), !, 
		XMax = Right - Left.


	gWindow_getminy(YMin) :-
		gWindow_winviewport(_,YMin,_,_), !.


	gWindow_getmaxy(YMax) :-
		gWindow_winviewport(_,Top,_,Bottom), !,
		YMax = Bottom - Top.


	gWindow_mouse_set(X,Y) :-
		gWindow_winviewport(Left,Top, _,_), !,
		MouseX = X + Left, MouseY = Y + Top,
		mouse_set(MouseX,MouseY).


	gWindow_mouse_set_horz(X1,X2) :-
		gWindow_winviewport(A,_,_,_), !,
		C = A + X1, D = A + X2,
		mouse_set_horizontal(C, D).


	gWindow_mouse_set_vert(Y1, Y2) :-
		gWindow_winviewport(_,A,_,_), !,
		C = A + Y1, D = A + Y2,
		mouse_set_vertical(C,D).


	gWindow_setmaxmouse :-
		getmaxx(X), getmaxy(Y), !,
		mouse_set_horizontal(0,X),
		mouse_set_vertical(0,Y).


	gWindow_mouse_get(X,Y,B) :-
		gWindow_winviewport(X1,Y1,_,_), !,
		mouse_get(X2,Y2,B),
		X = X2 - X1, Y = Y2 - Y1.


	gWindow_abs_to_rel(X2,Y2,X,Y) :-
		gWindow_winviewport(X1,Y1,_,_), !,
		X = X2 - X1, Y = Y2 - Y1.
    
	gWindow_rel_to_abs(X2,Y2,X,Y) :-
		gWindow_winviewport(X1,Y1,_,_), !,
		X = X2 + X1, Y = Y2 + Y1.

/*
---------------------------------------------------------------------
	UPDATE WINDOWS LABEL
---------------------------------------------------------------------
*/

CLAUSES

	gWindow_Label(Selector,Label) :-
		retract(gWindow_(Selector,Col,_,Size)), !,
		assert(gWindow_(Selector,Col,Label,Size)).

/*
---------------------------------------------------------------------
	REDRAW A GRAPHIC WINDOW
---------------------------------------------------------------------
*/

PREDICATES
	
%	gWindow_Redraw(Selector)

CLAUSES

	gWindow_Redraw(Selector) :-
	
		%getaspectratio(Xasp,Yasp),
		gWindow_(Selector,Colors,Label,Region), !,
		Region = region(Left,Top,Right,Bottom),
		Colors = colors(WColor,TColor,FColor,BColor),
		mouse_off,
		gWindow_SetViewPort(Left,Top,Right,Bottom,clip),
		Width = Right - Left, Height = Bottom - Top,
		setfillstyle(solid_fill,WColor),
		bar(0,0,Width,Height),
		setcolor(FColor),
		rectangle(0,0,Width,Height),

%	Make top BAR and exit box
		setfillpattern([$AA,$55,$AA,$55,$AA,$55,$AA,$55],BColor),
	
		bar(0,0,Width,title_bar_height),
		setcolor(FColor), 
		
		%EboxY = 2+(title_bar_height - 4), 
		%EboxX = 5+(EboxY-3)*(Yasp/Xasp),
		
		setfillstyle(solid_fill,BColor),
		%fillpoly([5,2, 5,Eboxy, EboxX,EboxY, EboxX,2]),

		rectangle(5,2,14,8),
	
		rectangle(0,0,Width,title_bar_height),

%	Write top text
		settextjustify(center_text,center_text),
		settextstyle(default_font,0,1),
		CenterX = Width div 2,
		CenterY = title_bar_height div 2 + 1,
		textwidth(Label,TextWidth),

		LabelXStart = CenterX - TextWidth div 2 - 2,
		LabelXEnd = CenterX + TextWidth div 2 + 1,
		LabelYStart = 1,    
		LabelYEnd =  title_bar_height - 1, 

		setfillstyle(solid_fill,BColor),
		bar(LabelXStart,LabelYStart,LabelXend,LabelYend),
		setcolor(TColor),
		outtextXY(CenterX,CenterY,Label),
		gWindow_SetMaxViewPort,
		setcolor(TColor),
		settextjustify(left_text,top_text),
		mouse_on.

/*
---------------------------------------------------------------------
	CHECK OR GENERATE A GRAPHIC WINDOW SELECTOR
---------------------------------------------------------------------
*/

PREDICATES
	
%	gWindow_Selector(Integer,WindowSelector)

CLAUSES

	gWindow_Selector(Selector,Selector) :-
		not(gWindow_(Selector,_,_,_)), !.
	gWindow_Selector(Count,Selector) :-
		Count < 256, !, Next = Count + 1,
		gWindow_Selector(Next,Selector).
	gWindow_Selector(_,_) :-
		exit(287), fail. % Maximum number of windows exceeded

/*
---------------------------------------------------------------------
	MOVE THE TOP LEFT CORNER OF A WINDOW
---------------------------------------------------------------------
*/

PREDICATES

%	move_Region(Integer,Integer,Integer,Integer,Integer,Integer)

CLAUSES

%	Move the Top Left start of the window
	move_Region(Left,Top,OldX,OldY,MouseX,MouseY) :-
		gWindow_mouse_get(MouseX,MouseY,Buttons),
		Buttons >< 0, !,
		Right = Left + OldX, Bottom = Top + OldY,
		Right1 = MouseX + OldX, Bottom1 = MouseY + OldY,
		mouse_off,
		rectangle(Left,Top,Right,Bottom),
		rectangle(MouseX,MouseY,Right1,Bottom1),
		mouse_on.

	move_Region(Left,Top,OldX,OldY,Left1,Top1) :-
		gWindow_mouse_get(MouseX,MouseY,_),
		MouseX = Left, MouseY = Top, !,
		move_Region(Left,Top,OldX,OldY,Left1,Top1).

	move_Region(Left,Top,OldX,OldY,Left1,Top1) :-
		Right = Left + OldX, Bottom = Top + OldY,
		gWindow_mouse_get(MouseX,MouseY,_),
		Right1 = MouseX + OldX, Bottom1 = MouseY + OldY,
		mouse_off,
		rectangle(Left,Top,Right,Bottom),
		rectangle(MouseX,MouseY,Right1,Bottom1),
		mouse_on,
		move_Region(MouseX,MouseY,OldX,OldY,Left1,Top1).

/*
---------------------------------------------------------------------

	resizewiew

	Define Bottom Right Corner of a window
	
---------------------------------------------------------------------
*/

PREDICATES

%	resize_Region(Integer,Integer,Integer,Integer,Integer,Integer)

CLAUSES

%	Move the Bottom Right corner
	resize_Region(Left,Top,Right,Bottom,MouseX,MouseY) :-
		gWindow_mouse_get(MouseX,MouseY,Buttons),
		Buttons >< 0, !,
		mouse_off,
		rectangle(Left,Top,Right,Bottom),
		mouse_on.

	resize_Region(Left,Top,Right,Bottom,Right1,Bottom1) :-
		gWindow_mouse_get(MouseX,MouseY,_),
		MouseX = Right, MouseY = Bottom, !,
		resize_Region(Left,Top,Right,Bottom,Right1,Bottom1).

	resize_Region(Left,Top,Right,Bottom,Right1,Bottom1) :-
		gWindow_mouse_get(MouseX,MouseY,_),
		mouse_off,
		rectangle(Left,Top,Right,Bottom),
		rectangle(Left,Top,MouseX,MouseY),
		mouse_on,
		resize_Region(Left,Top,MouseX,MouseY,Right1,Bottom1).

/*
---------------------------------------------------------------------
	DEFINE THE FRAME FOR A WINDOW
---------------------------------------------------------------------
*/

PREDICATES
	
%	region_View(Integer,Region,Region)
%	region_Move(Integer,Region,Region)
%	region_Size(Integer,Region,Region)

CLAUSES

	region_View(Frame,Region,NewRegion) :-
		region_move(Frame,Region,Temp),
		Temp = region(Left,Top,Right,Bottom),
		setcolor(Frame), mouse_off,
		setwritemode(xor_put),
		rectangle(Left,Top,Right,Bottom),
		region_size(Frame,Temp,NewRegion).
		

	region_Move(Frame,Region,NewRegion) :-
		Region = region(Left,Top,Right,Bottom),
		gWindow_getmaxx(MX), MaxX = MX - 3,
	        gWindow_getmaxy(MY), MaxY = MY - 3,
		gWindow_setmaxviewport,
		setcolor(Frame),
		setwritemode(xor_put),
		mouse_waitrelease,
%	Write the rectangle
		mouse_off,
		rectangle(Left,Top,Right,Bottom),
		gWindow_mouse_set(Left,Top),
		mouse_on,
%	Define limits of the mouse during the moving of the window
		XLimit = MaxX - (Right - Left),
		YLimit = MaxY - (Bottom - Top),
		gWindow_mouse_set_horz(0,XLimit),
		gWindow_mouse_set_vert(0,YLimit),
%	Define Top left Corner
		Width = Right - Left, Height = Bottom - Top,
		move_Region(Left,Top,Width,Height,NewLeft,NewTop),
		mouse_waitrelease,
		NewRight = NewLeft + Width,
		NewBottom = NewTop + Height,
		gWindow_setmaxmouse,
		NewRegion = region(NewLeft,NewTop,NewRight,NewBottom),
		setwritemode(copy_put).


	region_Size(Frame,Region,NewRegion) :-
		Region = region(Left,Top,Right,Bottom),
		gWindow_getmaxx(MX), MaxX = MX - 3,
	        gWindow_getmaxy(MY), MaxY = MY - 3,
		gWindow_setmaxviewport,
		setcolor(Frame),
		setwritemode(xor_put),
		mouse_waitrelease,
%	Write the rectangle
		mouse_off,
		rectangle(Left,Top,Right,Bottom),
		gWindow_mouse_set(Right,Bottom),
		mouse_on,
%	Define limits of the mouse during the resizing of the window
		XLimit = Left + minGWindowWidth,
		YLimit = Top + minGWindowHeight,
		gWindow_mouse_set_horz(XLimit,MaxX),
		gWindow_mouse_set_vert(YLimit,MaxY),
%	Define Bottom right Corner
		resize_Region(Left,Top,Right,Bottom,NewRight,NewBottom),
		mouse_waitrelease,
		gWindow_setmaxmouse,
		NewRegion = region(Left,Top,NewRight,NewBottom),
		setwritemode(copy_put).

/*
---------------------------------------------------------------------

	GWindow_MouseDefine

	Defines a new BGI window and return a new window selector
	
---------------------------------------------------------------------
*/

PREDICATES
	
%	gWindow_Define(WindowSelector,WindowLabel,WindowColors)


CLAUSES

	gWindow_Define(Selector,Label,Colors):-
		getaspectratio(Xasp, Yasp),
		Colors = colors(_,_,FColor,_),
		gWindow_getmaxx(MaxX), 
		Left = MaxX div 3,
		gWindow_getmaxy(MaxY), 
		Top = MaxY div 3,
		Right = Left + minGWindowWidth * 3,
		Bottom = Top + minGWindowWidth * 3 * (Xasp/Yasp),
		region_view(FColor,region(Left,Top,Right,Bottom),Region),
		gWindow_Selector(1,Selector),
		assert(gWindow_(Selector,Colors,Label,Region)).

/*
---------------------------------------------------------------------
	DEFINE AND DRAW A NEW BGI WINDOW AND RETURN SELECTOR
---------------------------------------------------------------------
*/

PREDICATES
	
%	gWindow_Create(WindowSelector,WindowLabel,WindowColors)

CLAUSES

	gWindow_Create(Selector,Label,Colors):-
		gWindow_Define(Selector,Label,Colors),
		gWindow_ReDraw(Selector).


/*
---------------------------------------------------------------------
	MOVE AND RESIZE AN EXISTING BGI WINDOW
---------------------------------------------------------------------
*/

PREDICATES
	
%	gWindow_Resize(WindowSelector)
%	gWindow_Move(WindowSelector)

CLAUSES

	gWindow_Resize(Selector) :-
		gWindow_(Selector,Colors,Label,Region), !,
		Colors = colors(_,_,FColor,_),
		region_size(FColor,Region,NewRegion),
		retract(gWindow_(Selector,_,_,_)), !,
%	Write the background color for the old place
		setfillstyle(solid_fill,black),
		Region = region(OLeft,OTop,ORight,OBottom),
		mouse_off,
		bar(OLeft,OTop,ORight,OBottom),
		mouse_on,
%	Define and draw new window
		assert(gWindow_(Selector,Colors,Label,NewRegion)).


	gWindow_Move(Selector) :-
		gWindow_(Selector,Colors,Label,Region), !,
		Colors = colors(_,_,FColor,_),
		region_move(FColor,Region,NewRegion),
		retract(gWindow_(Selector,_,_,_)), !,
%	Write the background color for the old place
		setfillstyle(solid_fill,black),
		Region = region(OLeft,OTop,ORight,OBottom),
		mouse_off,
		bar(OLeft,OTop,ORight,OBottom),
		mouse_on,
%	Define and draw new window
		assert(gWindow_(Selector,Colors,Label,NewRegion)).


/*
---------------------------------------------------------------------

	gWindow_MousePress

---------------------------------------------------------------------
*/

PREDICATES
	
%	gWindow_MousePress(WindowSelector,MouseX,MouseY)
%	gWindow_MousePress(WindowSelector,MouseX,MmouseY,Region)

CLAUSES

	gWindow_MousePress(Selector,MouseX,MouseY) :-
		gWindow_(Selector,_,_,Region), !,
		gWindow_MousePress(Selector,MouseX,MouseY,Region).

	gWindow_MousePress(Selector,MouseX,MouseY,Region) :-
		Region = region(Left,Top,Right,Bottom),
		MouseX >= Left, 
		MouseX <= Right,
		MouseY >= Top, 
		MouseY <= Top + title_bar_height, !,
		gWindow_Resize(Selector),
%	Reinstall the object in the new position
		gWindow_(Selector,_,_,NewRegion), !,
		object_Refresh(gWindow(Selector),NewRegion),
		screen_RedrawRegion(region(Left,Top,Right,Bottom)).


	gWindow_MousePress(Selector,MouseX,MouseY,Region) :-
		Region = region(Left,Top,Right,Bottom),
		MouseX >= Left, 
		MouseX <= Right,
		MouseY >= Top, 
		MouseY <= Bottom, !,
		object_Refresh(gWindow(Selector),Region),
		gWindow_Redraw(Selector),
		mouse_waitrelease.

/*
---------------------------------------------------------------------

	gWindow_GetArea

---------------------------------------------------------------------
*/

PREDICATES

%	gWindow_Region(WindowSelector,Region)

CLAUSES

	gWindow_Region(Selector,Area) :-
		gWindow_(Selector,_,_,Area), !.

/*
---------------------------------------------------------------------

	gWindow_GetColors

---------------------------------------------------------------------
*/

PREDICATES
	
%	gWindow_Colors(Selector,Colors)


CLAUSES

	gWindow_Colors(Selector,Colors) :-
		gWindow_(Selector,Colors,_,_), !.

/*
---------------------------------------------------------------------
	REMOVE A GRAPHIC WINDOW
---------------------------------------------------------------------
*/

PREDICATES

%	gWindow_Remove(WINDOWSELECTOR)

CLAUSES

	gWindow_Remove(Selector) :-
		gWindow_(Selector,_,_,Window), !,
		Window = region(Left,Top,Right,Bottom),
		getbkColor(Background),
        	mouse_off,
		setfillstyle(solid_fill,Background),
		bar(Left,Top,Right,Bottom),
	        mouse_on,
		retractall(gWindow_(Selector,_,_,_)).

/*
---------------------------------------------------------------------

	gWindow_WriteText

	Quick and dirty, every window needs a current cursor position
	
---------------------------------------------------------------------
*/

PREDICATES

%	gWindow_Text(WindowSelector,String)
	gWindow_WriteText(Integer,String)

CLAUSES

	gWindow_Text(Selector,Text) :-
		gWindow_(Selector,Colors,_,Region), !,
		Colors = colors(_,TextColor,_,_),
		Region = region(Left,Top,Right,Bottom),
		setcolor(TextColor),
		settextstyle(default_font,horiz_dir,1),
		settextjustify(left_text,top_text),
		Left1 = Left + 3, Top1 = Top + title_bar_height + 3,
		Right1 = Right - 3, Bottom1 = Bottom - 3,
		gWindow_setviewport(Left1,Top1,Right1,Bottom1,clip),
		mouse_off,
		gWindow_WriteText(0,Text),
		mouse_on,
		gWindow_setmaxviewport.


	gWindow_WriteText(Pos,String) :-
		searchchar(String,'\n',Len),
		frontstr(Len,String,Line,Back),
		frontchar(Back,_,Tail),
		textheight(Line,Height),
		outtextxy(0,Pos,Line), !,
		Next = Pos + Height + 2,
		gWindow_WriteText(Next,Tail).
	gWindow_WriteText(Pos,String) :-
		outtextxy(0,Pos,String).

