/*
=====================================================================
	IMAGES.PRO
	
	Handle holding images in RAM, saving and loading them to
	and from disk media.
	
	Copyright (C) 1990 Prolog Development Center
=====================================================================
*/

/*
---------------------------------------------------------------------
	DATABASE TO HOLD IMAGES IN RAM
---------------------------------------------------------------------
*/

CLAUSES

   % OPEN THE DATABASE USED TO HOLD IMAGES IN RAM

	image_open :- 
		db_create(image,image,in_memory).
	

   % CLOSE THE DATABASE USED TO HOLD IMAGES IN RAM

	image_close :- 
		db_close(image).


   % HOLD AN IMAGE IN THE DATABASE INDEXED BY NAME

	image_hold(Name,Left,Top,Right,Bottom) :-
		chain_first(image, Name, Ref) ,!,% already exists
		getimage(Left,Top,Right,Bottom,Block),
		imagesize(Left,Top,Right,Bottom,Len),
		bin_replace(image,Ref,Block,Len).
	image_hold(Name,Left,Top,Right,Bottom) :-
		getimage(Left,Top,Right,Bottom,Block),
		imagesize(Left,Top,Right,Bottom,Len),
		chain_bininserta(image,Name,Block,Len,_).

   % RECALL AN IMAGE FROM THE DATABASE INDEXED BY NAME

	image_recall(Name,Left,Top,Mode) :- 
		chain_first(image, Name,Ref),
		ref_bin(image,Ref,Block,_),
		putimage(Left,Top,Block,Mode).


   % SAVE AN IMAGE TO IT'S OWN BINARY FILE

	image_save(Name,Left,Top,Right,Bottom) :-
		getimage(Left,Top,Right,Bottom,Block),
		imagesize(Left,Top,Right,Bottom,Size),
		filenameext(FileName, Name, ".GIM"),
		openwrite(blockfile,FileName),
		writedevice(blockfile),
		filemode(blockfile,1),
		int_byte(Size,High,Low),
		write(High,Low),
		filepos(blockfile,2,0),
		writeblock(Size,Block),
		closefile(blockfile), !.


   % LOAD AN IMAGE FROM IT'S OWN BINARY FILE

	image_load(Name,Left,Top,Mode) :-
		filenameext(FileName, Name, ".GIM"),
		openread(blockfile,FileName),
		readdevice(blockfile),
		filemode(blockfile,1),
		readchar(High), readchar(Low),
		int_byte(Size,High,Low),
		filepos(blockfile,2,0),
		readblock(Size,Block),
		closefile(blockfile),
		putimage(Left,Top,Block,Mode), !.


   % TRANSLATE INTEGERS TO TWO BYTES AND BACK

	int_byte(Integer,HighByte,LowByte) :-
		bound(Integer), !,
		HighByte = Integer div 256,
		LowByte = Integer mod 256.
	int_byte(Integer,HighByte,LowByte) :-
		bound(HighByte), bound(LowByte),
		Integer = HighByte * 256 + LowByte.

