/*
=====================================================================
	OBJECTS.PRO
	
	Object Handling Predicates
	
	Copyright (C) 1990 Prolog Development Center
=====================================================================
*/

/*
---------------------------------------------------------------------

		Handle Objects

---------------------------------------------------------------------
*/

include "GRAPHICS\\GUTIL.PRO"

% This database stores the objects on the screen.
% The order is important because the first found will be selected

DATABASE - objects

	object_(Object,Region)

CLAUSES

	object_Consult(Name) :- consult(Name,objects).

	object_Save(Name) :- save(Name,objects).

	object_Clear :- retractall(_,objects).

	object_Get(Object,Region) :-
		object_(Object,Region).

	object_Install(Object,Region) :-
		asserta(object_(Object,Region)).

	object_Remove(Object) :-
		retractall(object_(Object,_)).


%	Succeds if installed as an object
	object_Refresh(Object,Region) :-
		retract(object_(Object,_)), !,
		asserta(object_(Object,Region)).
	object_Refresh(_,_).


PREDICATES

	reverse_sub(ObjectList,ObjectList,ObjectList) 

CLAUSES

	objects_Reverse(L1,L2) :-
		reverse_sub(L1,[],L2).

	reverse_sub([],L,L).
	reverse_sub([H|T],Cur,Out) :-
		reverse_sub(T,[H|Cur],Out).


/*
---------------------------------------------------------------------

		region_Inside

---------------------------------------------------------------------
*/

CLAUSES

	region_Inside(MouseX,MouseY,region(Left,Top,Right,Bottom)) :-
		MouseY >= Top, 
		MouseY < Bottom,
		MouseX >= Left, 
		MouseX < Right.


/*
---------------------------------------------------------------------
	

	Redraw screen

	Either the entire screen or only the windows that overlaps a
	region can be redrawn.

	Try to ReDraw as little as possible.

	1) Don't redraw an object if it is covered by another object.
	2) In redrawing a region don't redraw an object it's part of
        the are is covered by another object.


---------------------------------------------------------------------
*/

CLAUSES

%	Redraw the whole screen

	screen_Redraw :-
		findall(Object,object_(Object,_),Objects),
		objects_Reverse(Objects,RevObjects),
		mouse_off,
		objects_Redraw(RevObjects),
		mouse_on,
		fail.
	screen_Redraw.


%	Redraw all objects that cover a part of the object

	screen_RedrawRegion(Region) :-
		findall(Object,objects_Overlap(Region,Object),Objects),
		objects_Reverse(Objects,RevObjects),
		mouse_off,
		objects_Redraw(Region,RevObjects),
		mouse_on,
		fail.
	screen_RedrawRegion(_).


%	Redraw all objects in the list that are not entirely covered
%	by another object

	objects_Redraw([]).
	objects_Redraw([Object|Tail]) :-
		object_(Object,ObjectArea),
		not(covered(ObjectArea,Tail)),
		message(Object,redraw), !,
		objects_Redraw(Tail).
	objects_Redraw([_|Tail]) :-
		objects_Redraw(Tail).

%	Redraw all objects that covers a part on the screen but where the
%	covered region are not covered by another region

	objects_Redraw(_,[]).
	objects_Redraw(Area,[Object|Tail]) :-
		object_(Object,Region),
		region_InterSection(Area,Region,Intersect),
		not(covered(Intersect,Tail)),
		message(Object,redraw), !,
		objects_Redraw(Area,Tail).
	objects_Redraw(Region,[_|Tail]) :-
		objects_Redraw(Region,Tail).


%	Return all objects that cover a given region

	objects_Overlap(Area,Object) :-
		object_(Object,ObjectArea),
	        overlapping(Area,ObjectArea).

	overlapping(region(Left,Top,Right,Bottom),
		    region(Left1,Top1,Right1,Bottom1)) :-
		Left <= Right1, 
		Right >= Left1,
		Top <= Bottom1, 
		Bottom >= Top1.


%	Is a given region covered

	covered(region(Left,Top,Right,Bottom),[Head|_]) :- 
		object_(Head,ObjectRegion),
		ObjectRegion = region(Left1,Top1,Right1,Bottom1),
		Left1 <= Left, 
		Top1  <= Top,
		Right1  >= Right, 
		Bottom1 >= Bottom, !.
	covered(Region,[_|Tail]) :- 
		covered(Region,Tail).


	region_Intersection(region(Left1,Top1,Right1,Bottom1), % Region
			    region(Left2,Top2,Right2,Bottom2), % ObjectArea,
			    region(Left3,Top3,Right3,Bottom3)) :- % Intersect
		max(Left1,Left2,Left3), 
		max(Top1,Top2,Top3),
		min(Right1,Right2,Right3), 
		min(Bottom1,Bottom2,Bottom3).


PREDICATES

	other_objects(object,objectlist)

CLAUSES

	other_objects(_,[]) :- !, fail.
	other_objects(Head,[Head|_]) :- !, fail.
	other_objects(_,_).

	
	partly_covered(Obj) :- 
		object_(Obj,Region), !,
		findall(Object,objects_Overlap(Region,Object),Objects),
        	other_objects(Obj,Objects).

