/*
=====================================================================
 	SCROLBAR.PRO
	
	Graphic Scroll Bar Predicates

 
	Copyright (C) 1990 Prolog Development Center
=====================================================================


                 MaxVal ͵

       +-----+  +--------------------------------+  +----+
       | <-  |  |                             |  | -> |
       +-----+  +--------------------------------+  +----+
                     ^
                     |
                   Curval

	BarLength, CurVal or MaxVal range is in Pixels


=====================================================================
*/

CONSTANTS
 scroll_rate = 5
 

DATABASE - scrollbar

	sBar_(Selector,SBarType,Region,Region,Region,Region,
		Style,Integer,Integer)
%	sBar_(Selector,SBarType,TotArea,LarrowArea,RarrowArea,BarArea,
%		Style,BarLength,MaxValue)

	sBar_Pos_(Selector,Integer)
%	sBar_Pos_(Selector,CurValue)

	sBar_WhenSet_(Selector,Object,Integer,Real,Integer)
%	sBar_WhenSet_(Selector,Object,Delay, DelayEnd,Old_val)                                
%	Gives a message to to an object when set

	sBar_AfterSet_(Selector,Object)
%	Gives a message to to an object after the mouse button is released


CLAUSES

   % CLEAR THE SCROLL BAR DATABASE

	sBar_Clear :- retractall(_,scrollbar).


   % SAVE THE SCROLL BAR DATABASE

	sBar_Save(Filename) :- save(Filename,scrollbar).


   % CONSULT THE SCROLL BAR DATABASE

	sBar_Consult(Filename) :- consult(Filename,scrollbar).
	

   % GENERATE A NEW SCROLL BAR SELECTOR

	sBar_Selector(Selector) :-
		sBar_Selector(1,Selector).

	sBar_Selector(Selector,Selector) :-
		not(sBar_(Selector,_,_,_,_,_,_,_,_)), !.
	sBar_Selector(Count,Selector) :-
		Next =  Count + 1,
		sBar_Selector(Next,Selector).


/*
---------------------------------------------------------------------

		create arrow

---------------------------------------------------------------------
*/

PREDICATES

	createArrow(Region,Style,Direction)
	getMiddle(Region,Direction,Integer,Integer)
	direction(Direction,SBarType)
	direction(Direction,Direction,SBarType)
	getArrowBar(Region,SBarType,Region)
	justArrow(Direction,Integer,Integer,Integer,Integer,Integer,Integer)

CLAUSES

	createArrow(Region,Style,Direction) :-
		Style = style(Color,Fill,_),
		Region = region(Left,Top,Right, Bottom), 

%	Write the background bar
		setfillstyle(solid_fill,Fill),
		bar(Left,Top,Right,Bottom),
		rectangle(Left,Top,Right,Bottom),
%	Write most of the arrow
		setfillstyle(solid_fill,Color),

		direction(Direction,HorVer),
		getArrowBar(Region,HorVer,BarRegion),
		BarRegion = region(Left1,Top1,Right1, Bottom1), 
		bar(Left1,Top1,Right1,Bottom1),
	
%              4
%              32
%        --------+
%                |1
%        --------+
%              65
%              7

		getMiddle(BarRegion,Direction,X,Y),

		justArrow(Direction,X,Y,-2,-2,X2,Y2),
		justArrow(Direction,X,Y,-3,-2,X3,Y3),
		justArrow(Direction,X,Y,-3,-3,X4,Y4),
		justArrow(Direction,X,Y,-2, 2,X5,Y5),
		justArrow(Direction,X,Y,-3, 2,X6,Y6),
		justArrow(Direction,X,Y,-3, 3,X7,Y7),

		putpixel(X,Y,Color),
		putpixel(X2,Y2,Color),
		putpixel(X3,Y3,Color),
		putpixel(X4,Y4,Color),
		putpixel(X5,Y5,Color),
		putpixel(X6,Y6,Color),
		putpixel(X7,Y7,Color).


	getArrowBar(OldRegion,horz,NewRegion) :-
		OldRegion = region(Left,Top,Right, Bottom), 
		Left1 = Left + 5, 
		Right1 = Right - 5,
		Top1 = (Top + Bottom) div 2 - 1, 
		Bottom1 = Top1 + 2,
		NewRegion = region(Left1,Top1,Right1, Bottom1). 

	getArrowBar(OldRegion,vert,NewRegion) :-
		OldRegion = region(Left,Top,Right, Bottom),
		Left1 = (Left + Right) div 2 - 1, 
		Right1 = Left1 + 2,
		Top1 = Top + 5, 
		Bottom1 = Bottom - 5,
		NewRegion = region(Left1,Top1,Right1, Bottom1).


	justArrow(right,X,Y,DX,DY,X1,Y1) :-
		X1 = X + DX, 
		Y1 = Y + DY.
	justArrow(left,X,Y,DX,DY,X1,Y1) :-
		X1 = X - DX, 
		Y1 = Y + DY.
	justArrow(top,X,Y,DX,DY,X1,Y1) :-
		X1 = X - DY, 
		Y1 = Y - DX.
	justArrow(bottom,X,Y,DX,DY,X1,Y1) :-
		X1 = X + DY, 
		Y1 = Y + DX.


	getMiddle(Region,left,X,Y) :-
		Region = region(Left,Top,_,_),
		X = Left - 1, 
		Y = Top + 1.
	getMiddle(Region,right,X,Y) :-
		Region = region(_,Top,Right,_),
		X = Right + 1,      
		Y = Top + 1.
	getMiddle(Region,top,X,Y) :-
		Region = region(Left,Top,_,_),
		X = Left + 1, 
		Y = Top - 1.
	getMiddle(Region,bottom,X,Y) :-
		Region = region(Left,_,_,Bottom),
		X = Left + 1, 
		Y = Bottom + 1.


	direction(left,horz).
	direction(right,horz).
	direction(top,vert).
	direction(bottom,vert).

	direction(left,right,horz).
	direction(top,bottom,vert).


/*
---------------------------------------------------------------------

	setHScrollBar
	Update value on screen of horizontal scrollbar.
	(Local help predicate)

---------------------------------------------------------------------
*/

PREDICATES

	setHScrollBar(Integer,Integer,Region,Integer,Style)
%	setHScrollBar(OldValue,NewValue,BarArea,BarLength,Style)

CLAUSES

	setHScrollBar(Old,New,Region,Length,Style) :-
		Style = style(Color,Fill,_),
		Region = region(Left,Top,_,Bottom),

%	Remove old Bar from screen
		OldBarLeft = Left + Old,
		OldBarRight = OldBarLeft + Length,
		setfillpattern(topBarPattern,Fill),
		bar(OldBarLeft,Top,OldBarRight,Bottom),

%	Draw the new bar
		NewBarLeft = Left + New,
		NewBarRight = NewBarLeft + Length,
		setfillpattern(topBarPattern,Color),
		bar(NewBarLeft,Top,NewBarRight,Bottom).

/*
---------------------------------------------------------------------

	setVScrollBar
	
	Update value on screen of vertical scrollbar.
	(Local help predicate)
	
---------------------------------------------------------------------
*/

PREDICATES

	setVScrollBar(Integer,Integer,Region,Integer,Style)
%	setVScrollBar(OldValue,NewValue,BarArea,BarLength,Style)

CLAUSES

	setVScrollBar(Old,New,Region,Length,Style):-
		Style = style(Color,Fill,_),
		Region = region(Left,Top,Right,_),
%	Remove old Bar from screen
		OldBarTop = Top + Old,
		OldBarBottom = OldBarTop + Length,
		setfillpattern(topBarPattern,Fill),
		bar(Left,OldBarTop,Right,OldBarBottom),
%	Draw the new bar
		NewBarTop = Top + New,
		NewBarBottom = NewBarTop + Length,
		setfillpattern(topBarPattern,Color),
		bar(Left,NewBarTop,Right,NewBarBottom).

/*
---------------------------------------------------------------------

	SBar_Draw
	
	Draw a scroll bar.  (Local help predicate )
	
---------------------------------------------------------------------
*/

PREDICATES

	sBar_Draw(SBarType,Region,Region,Region,Style)

CLAUSES

	sBar_Draw(SBarType,Region,LArrow,RArrow,Style) :-
		Style = style(Color,Fill,_),
		Region = region(Left,Top,Right,Bottom),
		getmaxx(MaxX), 
		getmaxy(MaxY),
	 	setviewport(0,0,MaxX,MaxY,0),
 		gWindow_setmaxviewport,
		mouse_off,

%	Fill whole area with FillPattern
		setfillpattern(topBarPattern,Fill),
		bar(Left,Top,Right,Bottom),

%	Create frame
		setcolor(Color),
		rectangle(Left,Top,Right,Bottom),
		direction(LADir,RADir,SBarType),
		createArrow(LArrow,Style,LADir),
		createArrow(RArrow,Style,RADir),
		mouse_on.

/*
---------------------------------------------------------------------

	setTypeScrollBar  (Local help predicate)
	
---------------------------------------------------------------------
*/

PREDICATES

	setTypeScrollBar(Integer,Integer,Region,Integer,Style,SBarType)

CLAUSES

	setTypeScrollBar(Old,New,Region,Length,Style,horz) :-
		setHScrollBar(Old,New,Region,Length,Style).
	setTypeScrollBar(Old,New,Region,Length,Style,vert) :-
		setVScrollBar(Old,New,Region,Length,Style).

/*
---------------------------------------------------------------------
	Sending messages after updating
---------------------------------------------------------------------
*/

PREDICATES

	sBar_SetWhen_Message(Selector,Integer,Integer)
	sBar_AfterSet_Message(Selector,Integer,Integer)

CLAUSES

	sBar_SetWhen_Message(Selector,NewValue,Max) :-
	        sBar_WhenSet_(Selector,Object,Delay,DelayEnd,Oldvalue),
        	NewValue <> Oldvalue,
		timeout(DelayEnd),
	        retract(sBar_WhenSet_(Selector, _,_,_,_)),
		marktime(Delay, NewDelayEnd), 
        	assert(sBar_WhenSet_(Selector,Object,Delay,NewDelayEnd,Newvalue)),
		message(Object,set(Selector,NewValue,Max)), !.
	sBar_SetWhen_Message(_,_,_).


	sBar_AfterSet_Message(Selector,NewValue,Max) :-
		sBar_AfterSet_(Selector,Object),
		message(Object,sbar_after_change(Selector,NewValue,Max)),
		fail.
	sBar_AfterSet_Message(_,_,_).

/*
---------------------------------------------------------------------

	SBar_Set

	Update the position of a scrollbar
	
---------------------------------------------------------------------
*/

PREDICATES

	test_for_draw(String,Integer,Integer)
%	sBar_Set(Selector,Integer)
	sBar_Set1(Selector,Integer,String)

CLAUSES

	test_for_draw(mandatory,_,_) :- !.
	test_for_draw(_,V1,V2) :- V1 <> V2.


	sBar_Set(Selector,NewValue) :- 
		sBar_Set1(Selector,NewValue,conditional).


	sBar_Set1(Selector,NewValue,Mode):-
		sBar_(Selector,SBarType,_,_,_,BarArea,
			Style,BarLength,MaxValue),
		sBar_Pos_(Selector,OldValue),
		NewValue >= 0,
		min(NewValue,MaxValue,Val),
		sBar_SetWhen_Message(Selector,Val,MaxValue),
	        test_for_draw(Mode,Oldvalue,Val), !,
		mouse_off,
		setTypeScrollBar(OldValue,Val,BarArea,BarLength,
			Style,SBarType),
		mouse_on,
		retractall(sBar_Pos_(Selector,_)),
		assert(sBar_Pos_(Selector,Val)).
	sBar_Set1(_,_,_).

/*
---------------------------------------------------------------------

	SBar_Install_WhenSet

	Installs an object to receive a message when the scrollbar
	is set to a new value.
	
---------------------------------------------------------------------
*/

CLAUSES

	sBar_Install_WhenSet(Selector,Object,Delay) :-
		assert(sBar_WhenSet_(Selector,Object,Delay,0,-1)).

/*
---------------------------------------------------------------------

	SBar_Install_AfterSet

	Installs an object to receive a message after the scrollbar
	is set to a new value (Slow updating)
	
---------------------------------------------------------------------
*/

CLAUSES

	sBar_Install_AfterSet(Selector,Object) :-
		assert(sBar_AfterSet_(Selector,Object)).

/*
---------------------------------------------------------------------
	
	SBar_Get

	Return the position of a scrollbar
	
---------------------------------------------------------------------
*/
/*
PREDICATES

	sBar_Get(Selector,Integer,Integer)
*/
CLAUSES

	sBar_Get(Selector,CurValue,MaxValue) :-
		sBar_(Selector,_,_,_,_,_,_,_,MaxValue),
		sBar_Pos_(Selector,CurValue), !.

/*
---------------------------------------------------------------------

	updateBar

	Update bar value while mouse buttons are pressed.
	(Local help predicate )
	
---------------------------------------------------------------------
*/

PREDICATES

	updateBar(Selector,SBarType,Region)
	calc_bar_value(SBarType,MouseX,MouseY,Integer,Integer,Integer)

CLAUSES

	updateBar(Sel,SBarType,BarArea) :-
		BarArea = region(Left,Top,Right,Bottom),
		gWindow_mouse_set_horz(Left,Right),
		gWindow_mouse_set_vert(Top,Bottom),
		gwindow_rep_mouse(MouseX,MouseY),
		calc_bar_value(SBarType,MouseX,MouseY,Left,Top,Value),
		sBar_Set(Sel,Value),
		fail.
	updateBar(Selector,_,_) :-
%		getmaxx(MaxX), getmaxy(MaxY),
%		mouse_set_horizontal(0,MaxX),
%		mouse_set_vertical(0,MaxY),
	        gwindow_setmaxmouse,
		sBar_Get(Selector,Value,MaxVal),
		sBar_AfterSet_Message(Selector,Value,MaxVal).
	
	
	calc_bar_value(vert,_,MouseY,_,Top,Value) :-
		Value = MouseY - Top.
	calc_bar_value(horz,MouseX,_,Left,_,Value) :-
		Value = MouseX - Left.

/*
---------------------------------------------------------------------

	sBar_MousePress

	Action for pressing a mouse button inside a scrollbar
	
---------------------------------------------------------------------
*/

PREDICATES

%	sBar_MousePress(Selector,MouseX,MouseY)
	sBar_MousePress1(Selector,MouseX,MouseY,SBarType,
		Region,Region,Region).

	release_or_auto_repeat
	mouse_wait(Real)
	

CLAUSES

	sBar_MousePress(Selector,MouseX,MouseY) :-
		sBar_(Selector,Type,_,LArrow,RArrow,Region,_,_,_), !,
		sBar_MousePress1(Selector,MouseX,MouseY,
			Type,LArrow,RArrow,Region).

	sBar_MousePress1(Selector,MouseX,MouseY,_,LArrow,_,_) :-
		region_inside(MouseX,MouseY,LArrow),!,    
		sBar_Get(Selector,Value,MaxVal),
		Value1 = Value - 1,
		sBar_Set(Selector,Value1),
		sBar_AfterSet_Message(Selector,Value1,MaxVal),
		release_or_auto_repeat.

	sBar_MousePress1(Selector,MouseX,MouseY,_,_,RArrow,_) :-
		region_inside(MouseX,MouseY, RArrow),!,		
		sBar_Get(Selector,Value,MaxVal),
		Value1 = Value + 1,
		sBar_Set(Selector,Value1),
		sBar_AfterSet_Message(Selector,Value1,MaxVal),
		release_or_auto_repeat.

	sBar_MousePress1(Selector,MouseX,MouseY,SBarType,_,_,BarArea) :-
		region_Inside(MouseX,MouseY,BarArea), !,
		updatebar(Selector,SBarType,BarArea).

	release_or_auto_repeat :-
		marktime(scroll_rate, WaitEnd),
		mouse_wait(WaitEnd).
		
	
	mouse_wait(_) :-
		mouse_get(_,_,Buttons),
		Buttons=0,!.
	mouse_wait(Time) :- timeout(Time),!.
	mouse_wait(Time) :- mouse_wait(Time).
	
	

/*
---------------------------------------------------------------------

	SBar_Redraw

	Redraw a scrollbar
	
---------------------------------------------------------------------
*/

CLAUSES

	sBar_Redraw(Selector) :-
		sBar_(Selector,
			Type,Region,LArrow,RArrow,BarArea,Style,Length,_), 

		
		sBar_Draw(Type,Region,LArrow,RArrow,Style),

		sBar_Pos_(Selector, Value),!,			
		setTypeScrollBar(Value,Value,BarArea,Length, Style, Type).

		%sBar_Get(Selector,Value,_),
		%retractall(sBar_Pos_(Selector,_)),
		%assert(sBar_Pos_(Selector,Value)),
		%sBar_Set1(Selector,Value,mandatory).

/*
---------------------------------------------------------------------

	SBar_Create

	Define and draw a new scrollbar
	
---------------------------------------------------------------------
*/

CLAUSES

	sBar_Define(Selector,horz,Region,Style,Length) :-
		Style = style(Color,Fill,Pattern),
		Region = region(Left,Top,Right,Bottom),
		Height = Bottom - Top,
		ArrWidth = h_arrowwidth, 
		ArrHeight = Height - 2,

		LALeft = Left + 1, 
		LATop = Top + 1,
		LARight = LALeft + ArrWidth,
		LABottom = LATop + ArrHeight,

		RATop = Top + 1,
		RARight = Right - 1,
		RALeft = RARight - ArrWidth, 
		RABottom = RATop + ArrHeight, 		

		BarLeft = LALeft + ArrWidth + 1,
		BarRight = RALeft - Length,
		BarTop = Top + 1, 
		BarBottom = Bottom - 1,
		MaxBarValue = BarRight - BarLeft - 1,
		sBar_Selector(Selector),
		Style = style(Color,Fill,Pattern),
		assert(sBar_(Selector,horz,Region,
			region(LALeft,LATop,LARight, LABottom),
			region(RALeft,RATop,RARight,RABottom),
			region(BarLeft,BarTop,BarRight,BarBottom),
				Style,Length,MaxBarValue)),
		assert(sBar_Pos_(Selector,0)).


	sBar_Define(Selector,vert,Region,Style,Length) :-
		Style = style(Color,Fill,Pattern),
		Region = region(Left,Top,Right,Bottom),
		Width = Right - Left,
		ArrWidth = Width - 2,
		ArrHeight = v_arrowheight,
		LALeft = Left + 1, 
		LATop = Top,
		LARight = LALeft + ArrWidth,
		LABottom = LATop + ArrHeight,
	
		RALeft = Left + 1, 
		RATop = Bottom - ArrHeight - 1,
		RARight = RALeft + ArrWidth,
		RABottom = RATop + ArrHeight,


		BarLeft = Left + 1, 
		BarRight = Right - 1,
		BarTop = Top + ArrHeight + 1,
		BarBottom = RATop - Length,
		MaxBarValue = BarBottom - BarTop - 1,
		sBar_Selector(Selector),
		Style = style(Color,Fill,Pattern),
		assert(sBar_(Selector,vert,Region,
			region(LALeft,LATop,LARight,LABottom),
			region(RALeft,RATop,RARight,RABottom),
			region(BarLeft,BarTop,BarRight,BarBottom),
				Style,Length,MaxBarValue)),
		assert(sBar_Pos_(Selector,1)).


	sBar_Create(Selector,Type,Region,Style,Length) :-
		sBar_Define(Selector,Type,Region,Style,Length),
		object_install(sbar(Selector),Region),
		sBar_Redraw(Selector),
		sBar_Set(Selector,0).

/*
---------------------------------------------------------------------

	SBar_Remove

	Remove a scrollbar
	
---------------------------------------------------------------------
*/

CLAUSES

	sBar_Remove(Selector,clear_on) :-
		sBar_(Selector,_,Region,_,_,_,_,_,_),
		Region = region(Left,Top,Right,Bottom),
		getbkcolor(Color),
		setfillstyle(solid_fill,Color),
		bar(Left,Top,Right,Bottom),
		screen_RedrawRegion(Region),
		fail.

	sBar_Remove(Selector,_):-
		retractall(sBar_(Selector,_,_,_,_,_,_,_,_)),
		retractall(sBar_Pos_(Selector,_)),
		retractall(sBar_WhenSet_(Selector,_,_,_,_)),
		retractall(sBar_AfterSet_(Selector,_)).

