/****************************************************************

		PRINTER TOOLS

 In order to use the printer tools:
 
  1) "tdoms.pro" and "tpreds.pro" must be included.

  2) clauses for "make_footer" and "make_header" must be defined.

  3) The symbolic filename "prn_file" must be defined.

****************************************************************/

ifndef globdecl
PREDICATES
/* init_printer(MARGIN,TOPMARGIN,PAGELENGTH,CHARCONV,INITCODE,PRINTDEST) */
  init_printer(INTEGER,INTEGER,INTEGER,STRING,STRING,STRING)
  close_printer
  write_ch(CHAR)
  write_str(STRING)
  make_footer
  make_header
  check_page(STRING,INTEGER)
  shift_page()
  get_pageno(INTEGER)
enddef

DATABASE - prn_dba
  determ print_done			/* flag for print done */
  determ prn_lineno(INTEGER)		/* Internal line number */
  determ prn_pageno(INTEGER)		/* Internal page number */
  determ prn_margin(INTEGER)		/* Left margin				*/
  determ prn_topmargin(INTEGER)	/* Noof empty toplines			*/
  determ prn_pagelength(INTEGER)	/* Pagelength				*/
  determ prn_charconv(STRING)		/* Convert characters "J" or "N"	*/

/****************************************************************
	HELPING PREDICATES
****************************************************************/

PREDICATES
  printercode(STRING)
  count_lines(STRING,INTEGER,INTEGER)
  inc_lineno
  inc_lineno1(INTEGER)
  inc_pageno
  conv_char(CHAR,CHAR) 
  mk_margin
  mk_margin(integer)
  mk_topmargin
  mk_linefeeds(INTEGER)
  setwritedevice(STRING,STRING)
  fill_page(INTEGER,INTEGER)

CLAUSES
  printercode(""):-!.
  printercode(CODE):-
	frontchar(CODE,'\\',REST),
	fronttoken(REST,NUM,RESTCODE),
	str_int(NUM,CHI),
	char_int(CH,CHI),
	write(CH),!,
	printercode(RESTCODE).
  printercode(CODE):-
	frontchar(CODE,CH,REST),
	write(CH),
	printercode(REST).

  count_lines("",N,N):-!.
  count_lines(STR,N,N1):-
	frontchar(STR,'\n',STR1),!,
	N2=N+1,
	count_lines(STR1,N2,N1).
  count_lines(STR,N,N1):-
	frontchar(STR,_,STR1),
	count_lines(STR1,N,N1).

  inc_lineno:-
	prn_lineno(N),!,
	inc_lineno1(N).
  inc_lineno.

  inc_lineno1(N):-
	prn_pagelength(N),!,
	shift_page.
  inc_lineno1(N):-
	N1=N+1,
	retract(prn_lineno(_)),
	assert(prn_lineno(N1)).

  inc_pageno:-
	retract(prn_pageno(N)),
	N1=N+1,
	assert(prn_pageno(N1)).

  mk_margin:-
	prn_margin(MARGIN),
	mk_margin(MARGIN).

  mk_margin(0):-!.
  mk_margin(N):-
	write(' '),
	N1=N-1,
	mk_margin(N1).

  conv_char('{',''):-!.
  conv_char('[',''):-!.
  conv_char('}',''):-!.
  conv_char(']',''):-!.
  conv_char('|',''):-!.
  conv_char('\\',''):-!.
  conv_char('\197',''):-!.
  conv_char(C7,C7).

  mk_topmargin:-
	prn_topmargin(Tm),
	mk_linefeeds(Tm).

  mk_linefeeds(0) :- ! .
  mk_linefeeds(N) :-
  	write_ch('\n'),
  	N1 = N - 1,
  	mk_linefeeds(N1).

  setwritedevice(PRINTDEST,_):-
	upper_lower(PRINTDEST,"con"),!,
	writedevice(screen).
  setwritedevice(PRINTDEST,INITCODE):-
	openwrite(prn_file,PRINTDEST),
	writedevice(prn_file),
	printercode(INITCODE).

  fill_page(LINENO,PAGELENGTH):-
	LINENO<PAGELENGTH,!,
	N=PAGELENGTH-LINENO,
	mk_linefeeds(N).
  fill_page(_,_).


/****************************************************************
	INITIALIZING THE PRINTER
****************************************************************/

  init_printer(MARGIN,TOPMARGIN,PAGELENGTH,CHARCONV,INITCODE,PRINTDEST):-
	retractall(_,prn_dba),
	assert(prn_margin(MARGIN)),
	assert(prn_topmargin(TOPMARGIN)),
	assert(prn_pagelength(PAGELENGTH)),
	assert(prn_charconv(CHARCONV)),
	assert(prn_lineno(0)),
	assert(prn_pageno(1)),
	setwritedevice(PRINTDEST,INITCODE).

  close_printer:-
	writedevice(DEV),DEV=screen,!.
  close_printer:-
	print_done,!,
	prn_pagelength(PAGELENGTH),
	retract(prn_lineno(LINENO)),
	fill_page(LINENO,PAGELENGTH),
	make_footer,
	write('\012'),
	closefile(prn_file),
	writedevice(screen).
  close_printer:-
	closefile(prn_file),
	writedevice(screen).

/****************************************************************
	Check page shift before output of a multilined string
	First parameter: the string
	second parameter: any extra lines.
****************************************************************/

  check_page(STRING,N1):-
	count_lines(STRING,0,N2),
	prn_pagelength(PAGELENGTH),
	prn_lineno(LINENO),
	N1+N2+LINENO<PAGELENGTH,!.
  check_page(_,_):-shift_page.


/****************************************************************
	SHIFT PAGE: The two predicates: make_header and
	make_footer must be globally defined.
****************************************************************/

  shift_page:-
	writedevice(DEV),DEV=screen,!,
	retractall(prn_lineno(_)),
	make_footer,
	write(">> Press any Key"),readkey(_),
	cursor(ROW,_),cursor(ROW,0),
	assert(prn_lineno(0)),
	inc_pageno,
	mk_topmargin,
	make_header.
  shift_page:-
	prn_pagelength(PAGELENGTH),
	retract(prn_lineno(LINENO)),
	fill_page(LINENO,PAGELENGTH),
	make_footer,
	write('\012'),
	assert(prn_lineno(0)),
	inc_pageno,
	mk_topmargin,
	make_header.

/****************************************************************
	Return current pagenumber
****************************************************************/

  get_pageno(N):-prn_pageno(N).

/****************************************************************
	WRITE STRING
****************************************************************/

  write_str(STR):-
	frontchar(STR,CH,STR1),!,
	write_ch(CH),
	write_str(STR1).
  write_str(_).

/****************************************************************
	WRITE A SINGLE CHAR
****************************************************************/

  write_ch(_):-
	not(print_done),
	assert(print_done),
	mk_topmargin,
	make_header,
	fail.
  write_ch('\n'):-!,
	nl,
	inc_lineno,
	mk_margin.
  write_ch(CH):-
  	prn_charconv("J"),
  	writedevice(Dev),Dev = prn_file,!,
	conv_char(Cch,CH),!,
  	write(CCH).
  write_ch(Ch) :- write(Ch).
