/*
=====================================================================
	MOUSE.PRO
	
	Mouse Handling Predicates 
	  via Interface to Microsoft Mouse Driver
	
	Copyright (C) 1990 Prolog Development Center
=====================================================================
*/


/*
---------------------------------------------------------------------
		Manipulation of mouse ON/OFF flag
---------------------------------------------------------------------
*/


DATABASE - mousedriver

	determ mouse_status_on

PREDICATES
	
	status_on
	status_off

CLAUSES

	status_on :- 
		status_off, 
		assert(mouse_status_on).

	status_off :- 
		retractall(mouse_status_on).


/*
---------------------------------------------------------------------
		Mouse Bios Calls
---------------------------------------------------------------------
*/

CLAUSES

	mouse_init(STATUS) :-
		bios($33,reg(0,0,0,0,0,0,0,0),reg(STATUS,_,_,_,_,_,_,_)),
		status_off.


	mouse_on :- mouse_status_on, !.
	mouse_on :-
		bios($33,reg(1,0,0,0,0,0,0,0),reg(_,_,_,_,_,_,_,_)),
		status_on.


	mouse_off :-
		mouse_status_on, !,
		bios($33,reg(2,0,0,0,0,0,0,0),reg(_,_,_,_,_,_,_,_)),
		status_off.
	mouse_off.


	mouse_get(MouseX,MouseY,BUTTONS) :-
		bios($33,reg(3,0,0,0,0,0,0,0),
			reg(_,BUTTONS,MouseX,MouseY,_,_,_,_)).
	

	mouse_set(MouseX,MouseY) :-
		bios($33,reg(4,0,MouseX,MouseY,0,0,0,0),_).


	mouse_button_pressed(BUTTON,STATUS,COUNT,MouseX,MouseY) :-
		bios($33,reg(5,BUTTON,0,0,0,0,0,0),
			reg(STATUS,COUNT,MouseX,MouseY,_,_,_,_)).


	mouse_button_released(BUTTON,STATUS,COUNT,MouseX,MouseY) :-
		bios($33,reg(6,BUTTON,0,0,0,0,0,0),
			reg(STATUS,COUNT,MouseX,MouseY,_,_,_,_)).


	mouse_set_horizontal(MIN,MAX) :-
		bios($33,reg(7,0,MIN,MAX,0,0,0,0),_).


	mouse_set_vertical(MIN,MAX) :-
		bios($33,reg(8,0,MIN,MAX,0,0,0,0),_).


	mouse_set_graphics_cursor(HOTSPOT_X,HOTSPOT_Y,SEG,OFF) :-
		bios($33,reg(9,HOTSPOT_X,HOTSPOT_Y,OFF,0,0,0,SEG),_).


	mouse_software_text_cursor(SCREEN_MASK,CURSOR_MASK) :-
		bios($33,reg(10,0,SCREEN_MASK,CURSOR_MASK,0,0,0,0),_).


	mouse_hardware_text_cursor(SCAN_LINE_START,SCAN_LINE_STOP) :-
		bios($33,reg(10,1,SCAN_LINE_START,SCAN_LINE_STOP,0,0,0,0),_).


	mouse_get_mickey_count(HORIZONTAL,VERTICAL) :-
		bios($33,reg(11,0,0,0,0,0,0,0),
			reg(_,_,HORIZONTAL,VERTICAL,_,_,_,_)).


	mouse_set_mickey_ratio(HORIZONTAL,VERTICAL) :-
		bios($33,reg(15,0,HORIZONTAL,VERTICAL,0,0,0,0),_).


	mouse_set_speed(SPEED) :-
		bios($33,reg(19,0,0,SPEED,0,0,0,0),_).


	mouse_subroutine_input_mask(MASK,SEG,OFF) :-
		bios($33,reg(12,0,MASK,OFF,0,0,0,SEG),_).


	mouse_waitrelease :- 
		mouse_get(_,_,BUTTONS), 
		BUTTONS = 0, !.
	mouse_waitrelease :- mouse_waitrelease.

