/*
=====================================================================
	READKEY.PRO

	Standard toolbox predicate readkey extensions predicate
	file. Includes optimization, pushkey extension and mouse
	support.

	Copyright (C) 1990 Prolog Development Center
=====================================================================
*/

include "MREADKEY.PRE"
include "MOUSE.PRE"
include "MOUSE.PRO"

DATABASE - mouse

	mouse_pos_(Integer,Integer)
	stat_(Integer)

PREDICATES

	determ update_stat(Integer)
	nondeterm as_long_as_no_keys_pressed
	determ singlekey(Char,Key)
	determ singlekeys_(Char,Key)
	determ doublekey(Char,Key)
	determ doublekeys_(Char,Key)

CLAUSES

   % REPEAT BUT ONLY UNTIL A KEY IS PRESSED

	as_long_as_no_keys_pressed.
	as_long_as_no_keys_pressed :-
		 not(keypressed), 
		 as_long_as_no_keys_pressed.


   % UPDATE THE LAST BUTTON PRESSED ON THE MOUSE
  
	update_stat(Button) :-
 		retractall(stat_(_)), !,
  		assert(stat_(Button)).


   % LOOP TO CHECK THE MOUSE OR PROCESS A KEYPRESS

	readkey(mouse(Button,Row,Col)) :-
 		mouse_on,
  		as_long_as_no_keys_pressed, 
		mouse_get(X,Y,Button), 
		not(stat_(Button)),
	  	update_stat(Button), 
		Button >< 0, !,
  		Col = (X / 8), 
		Row = (Y / 8).
	readkey(Key) :-
 		readchar(Char), 
		singlekey(Char,Key), !.


   % REVERSES THE READKEY FUNCTION AND PUSHES A KEY TO THE BUFFER

	pushkey(Key) :-
 		singlekeys_(Num,Key), !,
		unreadchar(Num).
	pushkey(Key) :-
 		doublekeys_(Num,Key), !,
		unreadchar(Num), 
		unreadchar(0).


   % RECOGNIZE A 0 KEY CODE AND BRANCH TO DOUBLEKEY OR PROCESS SINGLE

	singlekey(0,Key) :- !,
 		readchar(Char), 
		doublekey(Char,Key).
	singlekey(Num,Key) :-
 		singlekeys_(Num,Key), !.
	singlekey(Num,ctrl(Char)) :-
 		Num >= 1,
		Num <= 26, !, 
		Char = 96 + Num.
	singlekey(Char,char(Char)).


   % TABLE OF KEY CODES FOR SINGLE ASCII KEYS
  
	singlekeys_(13,cr).
	singlekeys_(27,esc).
	singlekeys_(3,break).
	singlekeys_(9,tab).
	singlekeys_(8,bdel).
	singlekeys_(127,ctrlbdel).


   % RECOGNIZE A DUAL CODE KEY, INCLUDING FUNCTIONS KEYS

	doublekey(Num,Key) :-
 		doublekeys_(Num,Key), !.
	doublekey(Char,fkey(Value)) :-
		Char > 58, 
		Char < 70, !, 
		Value = Char - 58.
	doublekey(Char,fkey(Value)) :-
		Char >= 84, 
		Char < 104, !, 
		Value = Char - 73.


   % TABLE OF SECOND KEY CODES IN A DOUBLE CODE SEQUENCE

	doublekeys_(15,btab).
	doublekeys_(83,del).
	doublekeys_(82,ins).
	doublekeys_(72,up).
	doublekeys_(80,down).
	doublekeys_(75,left).
	doublekeys_(77,right).
	doublekeys_(73,pgup).
	doublekeys_(81,pgdn).
	doublekeys_(79,end).
	doublekeys_(71,home).
	doublekeys_(115,ctrlleft).
	doublekeys_(116,ctrlright).
	doublekeys_(117,ctrlend).
	doublekeys_(118,ctrlpgdn).
	doublekeys_(119,ctrlhome).
	doublekeys_(132,ctrlpgup).


