/************************************************************************
 *									*
 *			Parser generator				*
 *									*
 *			 User Interface 				*
 *			 -------------- 				*
 *									*
 ************************************************************************/

check_determ
project "parser"

include "parser.glo"

include "tpreds.pro"
include "ui\\lineinp.pro"
include "ui\\filename.pro"
include "ui\\status.pro"
include "ui\\pulldown.pro"


CONSTANTS
 cols_in_readfilename_window	= 40
 cols_in_setdir_window		= 45
 display_window 		= 1
 esc				= 1
 flashing_high_white_on_black	= 143
 grammar_window 		= 2
 high_white_on_blue		= 31
 rows_in_lineinput_window	= 3
 rows_in_readfilename_window	= 3
 rows_in_setdir_window		= 3
 two_seconds			= 200
 white_on_black 		= 7
 white_on_blue			= 23
 yellow_on_black		= 14


DOMAINS
 EXTENSION	= string
 HEADER 	= string
 QUESTION	= string


PREDICATES
  determ	change_directory
  determ	change_frame_text(TEXT)
  determ	clear_db
  determ	delete_parser_output_files
  determ	display_domain_definitions
  determ	display_file(FILENAME, HEADER)
  determ	display_parsing_predicates
  determ	edit_grammar
  determ	edit_grammar(STRING, CURSOR)
  determ	generate_parser
  determ	get_file_contents(STRING,STRING)
  determ	grammar_is_safe
  determ	initialize_grammar_window
  determ	interruptable_sleep(INTEGER)
  determ	interruptable_sleep_aux(REAL)
  determ	load_grammar
  determ	load_grammar(FILENAME)
  determ	make_grammar_window
  determ	my_readfilename(EXTENSION, FILENAME, FILENAME)
  determ	prepare_to_generate_parser
  determ	quit
  nondeterm	repeat_parse
  determ	save_grammar
  determ	save_grammar(FILENAME)
  determ	shell_to_dos
  determ	update_grammar_window(FILENAME, TEXT)
  determ	user_answered_yes_to_question(QUESTION)
  determ	write_domain_definitions
  determ	write_parsing_predicates

CLAUSES
/************************************************************************
 *									*
 *	change/1							*
 *									*
 *	Replace term in internal db.					*
 *									*
 ************************************************************************/

  change(error(_, _)) :-
	retract(error(_, _)),
	fail.

  change(grammar(_)) :-
	retract(grammar(_)),
	fail.

  change(grammar_filename(_)) :-
	retract(grammar_filename(_)),
	fail.

  change(X) :-
	assert(X).


/************************************************************************
 *									*
 *	change_directory						*
 *									*
 ************************************************************************/

  change_directory :-
	textmode(ROWS, COLS),
	START_ROW = (ROWS - rows_in_setdir_window) / 2, 	% Center.
	START_COL = (COLS - cols_in_setdir_window) / 2, 	% Center.

	setdir(START_ROW,
	       START_COL,
	       white_on_black,		% window attribute
	       white_on_black). 	% frame attribute


/************************************************************************
 *									*
 *	change_frame_text/1						*
 *									*
 ************************************************************************/

  change_frame_text(NEW_TEXT) :-
	makewindow(_, _, ATTR, _, _, _, _, _, _, POSITION, BORDER_CHARS),
	framewindow(ATTR, NEW_TEXT, POSITION, BORDER_CHARS).


/************************************************************************
 *									*
 *	clear_db							*
 *									*
 ************************************************************************/

  clear_db :-
	retractall(error(_, _)),
	retractall(_, parse).


/************************************************************************
 *									*
 *	delete_parser_output_files					*
 *									*
 ************************************************************************/

/* Delete parsing predicates... 					*/

  delete_parser_output_files :-
	grammar_filename(FILENAME),
	newext(FILENAME, "PAR", NEW_FILENAME),
	existfile(NEW_FILENAME),
	deletefile(NEW_FILENAME),
	fail.

/* Delete domain definitions... 					*/

  delete_parser_output_files :-
	grammar_filename(FILENAME),
	newext(FILENAME, "DOM", NEW_FILENAME),
	existfile(NEW_FILENAME),
	deletefile(NEW_FILENAME),
	fail.

  delete_parser_output_files.


/************************************************************************
 *									*
 *	display_domain_definitions					*
 *									*
 ************************************************************************/

  display_domain_definitions :-
	grammar_filename(FILENAME),
	newext(FILENAME, "DOM", NEW_FILENAME),
	existfile(NEW_FILENAME),
	!,
	format(HEADER, " Domain Definitions (%) ", NEW_FILENAME),
	display_file(NEW_FILENAME, HEADER).

/* Unless file does not exist...					*/

  display_domain_definitions :-
	makestatus(flashing_high_white_on_black,
		   "No domain definitions (parser not generated)."),
	interruptable_sleep(two_seconds),
	removestatus.


/************************************************************************
 *									*
 *	display_file/2							*
 *									*
 ************************************************************************/

  display_file(FILENAME, HEADER) :-
	makestatus(high_white_on_blue,
		   "Use arrows, PgUp, and PgDn to browse.  Esc when done."),
	textmode(SCREEN_ROWS, SCREEN_COLS),
	DISPLAY_ROWS = SCREEN_ROWS - 1, 	% Leave room for status line.

	makewindow(display_window,		% window number
		   white_on_black,		% window attribute
		   white_on_black,		% frame attribute
		   HEADER,			% frame text
		   0,				% start row
		   0,				% start col
		   DISPLAY_ROWS,		% number of rows
		   SCREEN_COLS),		% number of cols
	file_str(FILENAME, TEXT),
	display(TEXT),
	removewindow,
	removestatus.


/************************************************************************
 *									*
 *	display_parsing_predicates					*
 *									*
 ************************************************************************/

  display_parsing_predicates :-
	grammar_filename(FILENAME),
	newext(FILENAME, "PAR", NEW_FILENAME),
	existfile(NEW_FILENAME),
	!,
	format(HEADER, " Parsing Predicates (%) ", NEW_FILENAME),
	display_file(NEW_FILENAME, HEADER).

/* Unless file does not exist...					*/

  display_parsing_predicates :-
	makestatus(flashing_high_white_on_black,
		   "No parsing predicates (parser not generated)."),
	interruptable_sleep(two_seconds),
	removestatus.


/************************************************************************
 *									*
 *	edit_grammar							*
 *									*
 ************************************************************************/

  edit_grammar :-
	edit_grammar("", 0),
	!.

/* In case user escapes out of editor...				*/

  edit_grammar.



/************************************************************************
 *									*
 *	edit_grammar/2							*
 *									*
 *	Fails if user escapes out of editor.				*
 *									*
 ************************************************************************/

  edit_grammar(MESSAGE, CURSOR) :-
	shiftwindow(OLD),
	shiftwindow(grammar_window),
	grammar(TEXT),
	edit(TEXT,			% input text
	     NEW_TEXT,			% output text
	     "",			% headstr
	     "",			% headstr2
	     MESSAGE,
	     CURSOR,
	     "",			% help file name
	     1, 			% edit mode (read/overwrite)
	     1, 			% autoindent (on)
	     1, 			% insert mode (on)
	     0, 			% textmode (off)
	     _, 			% new cursor position
	     EXIT_CODE,
	     1, 			% show top status-line.
	     1, 			% confirm esc
	     0),			% don't clear pastebuffer on exit.

	shiftwindow(OLD),
	not(EXIT_CODE = esc),
	!,
	grammar_filename(FILENAME),
	update_grammar_window(FILENAME, NEW_TEXT),
	change(grammar(NEW_TEXT)).

/* This fires if user escaped out of editor.				*/

  edit_grammar(_, _) :-
	grammar_filename(FILENAME),
	grammar(TEXT),
	update_grammar_window(FILENAME, TEXT),
	fail.

/************************************************************************
 *									*
 *	generate_parser 						*
 *									*
 ************************************************************************/

  generate_parser :-
	makestatus(yellow_on_black, "Initializing..."),
	prepare_to_generate_parser,

	repeat_parse,
	retractall(error(_, _)),	% Or deeper error may not be found.
	retractall(_, parse),

	changestatus("Scanning..."),
	scan(TOKENS),

	changestatus("Parsing..."),
	parse(TOKENS),
	!,

	changestatus("Writing Domain Definitions..."),
	write_domain_definitions,

	changestatus("Writing Parsing Predicates..."),
	write_parsing_predicates,

	removestatus.

/* This fires if parsing errors occurred and user didn't fix them
   (escaped out of the editor). 					*/

  generate_parser :-
	removestatus,
	makestatus(flashing_high_white_on_black,
		   "Parsing aborted."),
	interruptable_sleep(two_seconds),
	removestatus.


/************************************************************************
 *									*
 *	grammar_is_safe 						*
 *									*
 ************************************************************************/

/* It's safe if it's null.						*/

  grammar_is_safe :-
	grammar(""),
	!.

/* It's safe if grammar_file exists and contents of grammar_file =
   grammar.								*/

  grammar_is_safe :-
	grammar_filename(FILENAME),
	existfile(FILENAME),
	file_str(FILENAME, SOURCE),
	grammar(SOURCE),
	!.

/* Otherwise user must confirm...					*/

  grammar_is_safe :-
	QUESTION = "Grammar not saved.  Are you sure (y/n)? ",
	user_answered_yes_to_question(QUESTION).


/************************************************************************
 *									*
 *	initialize_grammar_window					*
 *									*
 ************************************************************************/

/* Look for info on previous session.					*/
  initialize_grammar_window :-
	existfile("PARSER.DB"),
	!,
	consult("PARSER.DB"),
	grammar_filename(FILENAME),
	get_file_contents(FILENAME, GRAMMAR),
	change(grammar(GRAMMAR)),
	update_grammar_window(FILENAME, GRAMMAR).

/* If there was no previous session, use WORK.GRM for the filename and
   its contents (if file exists, otherwise use "") for the grammar.	*/

  initialize_grammar_window :-
	get_file_contents("WORK.GRM", GRAMMAR),
	change(grammar_filename("WORK.GRM")),
	change(grammar(GRAMMAR)),
	update_grammar_window("WORK.GRM", GRAMMAR).

/************************************************************************
 *									*
 *	get_file_contents/2						*
 *									*
 ************************************************************************/

  get_file_contents(FILENAME, CONTENTS) :-
	existfile(FILENAME),
	!,
	file_str(FILENAME, CONTENTS).

  get_file_contents(_, "").



/************************************************************************
 *									*
 *	is_a_better_error/1						*
 *									*
 ************************************************************************/

  is_a_better_error(CURSOR) :-
	error(_, OLDCURSOR),		% Get location of last error.
	OLDCURSOR >= CURSOR,
	!,
	fail.
  is_a_better_error(_).



/************************************************************************
 *									*
 *	load_grammar							*
 *									*
 ************************************************************************/

  load_grammar :-
	DEFAULT_FILENAME = "",
	my_readfilename("GRM", DEFAULT_FILENAME, FILENAME),
	load_grammar(FILENAME),
	!.

/* If user pressed esc in my_readfilename above or unable to
   load grammar: do nothing.						*/

  load_grammar.


/************************************************************************
 *									*
 *	load_grammar/1							*
 *									*
 ************************************************************************/

  load_grammar(FILENAME) :-
	existfile(FILENAME),
	!,
	change(grammar_filename(FILENAME)),
	file_str(FILENAME, SOURCE),
	change(grammar(SOURCE)),
	update_grammar_window(FILENAME, SOURCE).

/* Unless file can't be found...					*/

  load_grammar(FILENAME) :-
	format(QUESTION, "Can't find file (%).  Create it (y/n)? ", FILENAME),
	user_answered_yes_to_question(QUESTION),
	!,
	change(grammar_filename(FILENAME)),
	change(grammar("")),
	update_grammar_window(FILENAME, "").


/************************************************************************
 *									*
 *	make_grammar_window						*
 *									*
 *	Grammar window uses all 'available' screen.  So subtract	*
 *	4 (1 for status line plus 3 for pulldown menu) from number of	*
 *	rows in screen. 						*
 *									*
 ************************************************************************/

  make_grammar_window :-
	textmode(SCREEN_ROWS, SCREEN_COLS),
	AVAILABLE_ROWS = SCREEN_ROWS - 4,
	makewindow(grammar_window,		% window number
		   white_on_blue,		% window attr
		   white_on_blue,		% frame attr
		   " Grammar ", 		% frame text
		   3,				% start row
		   0,				% start col
		   AVAILABLE_ROWS,		% number of rows
		   SCREEN_COLS).		% number of cols


/************************************************************************
 *									*
 *	my_readfilename 						*
 *									*
 ************************************************************************/

  my_readfilename(DEFAULT_EXTENSION, DEFAULT_FILENAME, FILENAME) :-
	textmode(ROWS, COLS),
	START_ROW = (ROWS - rows_in_readfilename_window) / 2 - 2,
	START_COL = (COLS - cols_in_readfilename_window) / 2 - 4,
	readfilename(START_ROW,
		     START_COL,
		     white_on_black,		% window attr
		     white_on_black,		% frame attr
		     DEFAULT_EXTENSION,
		     DEFAULT_FILENAME,
		     FILENAME).


/************************************************************************
 *									*
 *	pdwaction/2							*
 *									*
 *	Called by pulldown tool.					*
 *									*
 ************************************************************************/

  pdwaction(1, 0) :- generate_parser.

  pdwaction(2, 1) :- display_parsing_predicates.

  pdwaction(2, 2) :- display_domain_definitions.

  pdwaction(3, 0) :- edit_grammar.

  pdwaction(4, 1) :- load_grammar.

  pdwaction(4, 2) :- save_grammar.

  pdwaction(4, 3) :- change_directory.

  pdwaction(4, 4) :- shell_to_dos.

  pdwaction(5, 0) :- quit.


/************************************************************************
 *									*
 *	prepare_to_generate_parser					*
 *									*
 ************************************************************************/

  prepare_to_generate_parser :-
	delete_parser_output_files,
	clear_db.


/************************************************************************
 *									*
 *	quit								*
 *									*
 ************************************************************************/

  quit :-
	grammar_is_safe,
	retractall(error(_, _)),
	retractall(grammar(_)),
	save("PARSER.DB"),		% For next time.
	!,
	fail.

  quit.


/************************************************************************
 *									*
 *	repeat_parse							*
 *									*
 ************************************************************************/

   repeat_parse.

   repeat_parse :-
	error(MSG, CURSOR),
	edit_grammar(MSG, CURSOR),
	repeat_parse.


/************************************************************************
 *									*
 *	save_grammar							*
 *									*
 ************************************************************************/

  save_grammar :-
	grammar_filename(OLD_FILENAME),
	my_readfilename("GRM", OLD_FILENAME, NEW_FILENAME),
	save_grammar(NEW_FILENAME),
	!.

/* If user pressed esc in my_readfilename above or unable to save
   grammar: do nothing. 						*/

  save_grammar.



/************************************************************************
 *									*
 *	save_grammar/1							*
 *									*
 ************************************************************************/

  save_grammar(FILENAME) :-
	not(existfile(FILENAME)),
	!,
	grammar(SOURCE),
	file_str(FILENAME, SOURCE),
	update_grammar_window(FILENAME, SOURCE),
	change(grammar_filename(FILENAME)).

  save_grammar(FILENAME) :-
	format(QUESTION, "File (%) exists.  Overwrite it (y/n)? ", FILENAME),
	user_answered_yes_to_question(QUESTION),
	grammar(SOURCE),
	file_str(FILENAME, SOURCE),
	update_grammar_window(FILENAME, SOURCE),
	change(grammar_filename(FILENAME)).



/************************************************************************
 *									*
 *	shell_to_dos							*
 *									*
 ************************************************************************/

  shell_to_dos :- system("").


/************************************************************************
 *									*
 *	update_grammar_window/1 					*
 *									*
 ************************************************************************/

  update_grammar_window(FILENAME, SOURCE) :-
	shiftwindow(OLD_WINDOW),
	shiftwindow(grammar_window),
	format(NEW_FRAME_TEXT, " Grammar (%) ", FILENAME),
	change_frame_text(NEW_FRAME_TEXT),
	window_str(SOURCE),
	shiftwindow(OLD_WINDOW).


/************************************************************************
 *									*
 *	user_answered_yes_to_question/1 				*
 *									*
 ************************************************************************/

  user_answered_yes_to_question(QUESTION) :-
	textmode(ROWS, COLS),
	START_ROW = (ROWS - rows_in_lineinput_window) / 2,
	str_len(QUESTION, LENGTH_OF_QUESTION),
	WIDTH_OF_WINDOW = LENGTH_OF_QUESTION + 3,	% 3 allows 1 for
							% answer plus 2
							% for frame.
	START_COL = (COLS - WIDTH_OF_WINDOW) /2,
	lineinput(START_ROW,
		  START_COL,
		  WIDTH_OF_WINDOW,
		  white_on_black,		% window attribute
		  white_on_black,		% frame attribute

		  QUESTION,
		  "n",				% default answer
		  ANSWER),
	!,
	upper_lower(ANSWER, "y").


/************************************************************************
 *									*
 *	write_domain_definitions					*
 *									*
 ************************************************************************/

  write_domain_definitions :-
	grammar_filename(GRAMMAR_FILENAME),
	newext(GRAMMAR_FILENAME, "DOM", DOMAIN_FILENAME),
	openwrite(outfile, DOMAIN_FILENAME),
	writedevice(outfile),
	write_domain_definitions_aux,
	closefile(outfile).


/************************************************************************
 *									*
 *	write_parsing_predicates					*
 *									*
 ************************************************************************/

  write_parsing_predicates :-
	grammar_filename(GRAMMAR_FILENAME),
	newext(GRAMMAR_FILENAME, "PAR", PREDICATES_FILENAME),
	openwrite(outfile, PREDICATES_FILENAME),
	writedevice(outfile),
	write_parsing_predicates_aux,
	closefile(outfile).

/************************************************************************
 *									*
 *	interruptable_sleep/1						*
 *									*
 ************************************************************************/

  interruptable_sleep(DURATION) :-
	marktime(DURATION, TICKET),
	interruptable_sleep_aux(TICKET).

  interruptable_sleep_aux(TICKET) :- timeout(TICKET), !.
  interruptable_sleep_aux(_) :- keypressed, !.
  interruptable_sleep_aux(TICKET) :- interruptable_sleep_aux(TICKET).



GOAL
  make_grammar_window,
  initialize_grammar_window,
  makestatus(white_on_black,
	     "Select with arrows + Enter, or use first upper case letter."),
  pulldown(white_on_black,
	   [curtain("Generate Parser", []),
	    curtain("Display Parser", ["Parsing Predicates",
				       "Domain Definitions"	]),
	    curtain("Edit Grammar", []),
	    curtain("Files", ["Load Grammar",
			      "Save Grammar",
			      "Change Directory",
			      "DOS"			]),
	    curtain("Quit", []) 					],
	  _,
	  _),
  removestatus,
  removewindow.


