	title	SERIAL.ASM - serial interrupt handling for PDC Prolog
	page	62,132

;********************************************************************
;*
;*  SERIAL.ASM - serial interrupt handling for the PDC Prolog Toolbox
;*
;*  Copyright (C) 1990 Prolog Development Center
;*  All rights reserved worldwide.
;*
;*  Provided only that you are a licensed user of the PDC Prolog Toolbox,
;*  or other Prolog Development Center product which includes this source
;*  code:
;*
;*  You are granted a perpetual, world-wide, royalty-free license to
;*  create and distribute programs for any purpose and which include
;*  the assembled or compiled result from this source code or from
;*  any modified or translated versions of this source code.  The Prolog
;*  Development Center reserves all rights to the source code itself,
;*  and to any modified or translated versions of the source code.
;*
;********************************************************************
;*
;*  This has been assembled with Borland International's TASM version
;*  1.01, but the source deliberately doesn't do anything tricky
;*  (except for its use of text equates and two simple macros) because
;*  we hope it works with the assembler you use
;*
;********************************************************************



true	  equ	(1 eq 1)
false	  equ	(not true)



;********************************************************************
;*								    *
;*		     Misc configuration items			    *
;*								    *
;********************************************************************



MAX_PORTS equ	4		; increases have to be reflected in
				; "ports" below and in the "port1", ...
				; interrupt handler vectors "irqs" and
				; at the end of the source file in
				; added interrupt handlers

COM3_IRQ  equ	4		; IRQ for COM3, can be any from 0-15
COM4_IRQ  equ	3		; IRQ for COM4, can be any from 0-15

BREAK_TIME equ	6		; 18.2 / second ticks to hold break

ABS_DSR   equ false		; set true if DSR is required to send
				;     XON and XOFF when both DSR/DTR
				;     and XON/XOFF protocol are active
DTR_BUSY  equ false		; set true if DTR should be turned off
				;     when we're busy and DSR/DTR is active
ABS_CTS   equ false		; set true if CTS is required to send
				;     XON and XOFF when both CTS/CTR
				;     and XON/XOFF protocol are active
RTS_BUSY  equ true		; set false if RTS should not be turned off
				;     when we're busy and CTS/RTS is active



;********************************************************************
;*								    *
;*			 UART registers and bits		    *
;*								    *
;********************************************************************



DATA_REG  equ	03f8h		; COM1 UART data register
LO_DIV	  equ	03f8h		; COM1 UART baud rate divisor low
IER	  equ	03f9h		; COM1 UART interrupt enable reg
HI_DIV	  equ	03f9h		; COM1 UART baud rate divisor high
IIR	  equ	03fah		; COM1 UART interrupt id reg
LCR	  equ	03fbh		; COM1 UART line control reg
MCR	  equ	03fch		; COM1 UART modem control reg
LSR	  equ	03fdh		; COM1 UART line status reg
MSR	  equ	03feh		; COM1 UART modem status reg

DATA_READY equ	01h		; data ready bit of LSR
OVERRUN    equ	02h		; overrun error bit of LSR
PARITY	   equ	04h		; parity error bit of LSR
FRAMING    equ	08h		; framing error bit of LSR
BREAK_RCVD equ	10h		; break received bit of LSR
XMIT_EMPTY equ	20h		; xmit holding empty bit of LSR

RCV_INT    equ	01h		; data ready enable bit for IER
XMIT_INT   equ	02h		; xmit empty enable bit for IER
LSR_INT    equ	04h		; delta LSR enable bit for IER
MSR_INT    equ	08h		; delta MSR enable bit for IER

DCTS	   equ	01h		; delta CTS
DDSR	   equ	02h		; delta DSR
TERI	   equ	04h		; RI has gone from active to inactive
DDCD	   equ	08h		; delta DCD
CTS	   equ	10h		; CTS bit of MSR
DSR	   equ	20h		; DSR bit of MSR
RI	   equ	40h		; ring indicator for MSR
DCD	   equ	80h		; carrier detect for MSR

DTR	   equ	01h		; DTR bit of MCR
RTS	   equ	02h		; RTS bit of MCR
OUT1	   equ	04h		; out 1 bit of MCR
OUT2	   equ	08h		; out 2 (enable irq) bit of MCR
LOOPBACK   equ	10h		; diagnostic loopback bit of MCR

BREAK	   equ	40h		; set break bit of LCR
DLAB	   equ	80h		; divisor latch bit of LCR



;********************************************************************
;*								    *
;*			 Miscellaneous				    *
;*								    *
;********************************************************************



timer	  equ	006ch		; BIOS offset for timer low word

XOFF	  equ	13h		; (67h for XPC)
XON	  equ	11h		; (65h for XPC)



input	macro	reg,port
	in	reg,port
	jmp	short $+2
	jmp	short $+2
	endm

output	macro	port,reg
	out	port,reg
	jmp	short $+2
	jmp	short $+2
	endm



; from PDC Prolog library
	extrn	_run_fail:far
	extrn	mem_releaseheap:far
	extrn	mem_allocheap:far
	extrn	_mem_allocgstack:far



DGROUP	group	_DATA

_DATA	segment para public 'DATA'



;********************************************************************
;*								    *
;*			 Port data definition			    *
;*								    *
;********************************************************************



portrec struc
irq	  db	0		; irq for port
port	  dw	0		; physical port address, 0 unless open
ibf	  dd	0		; -> input buffer
isize	  dw	0		; input buffer size, 0 unless allocated
ihead	  dw	0		; input buffer head pointer
itail	  dw	0		; input buffer tail pointer
obf	  dd	0		; -> output buffer
osize	  dw	0		; output buffer size, 0 unless allocated
ohead	  dw	0		; output buffer head pointer
otail	  dw	0		; output buffer tail pointer
status	  dw	0		; as defined by STATUS_RS232_0
bfstatus  dw	0		; actual current INPUT_EMPTY, OUTPUT_FULL state
protocol  db	0		; bit 0=XON/XOFF, bit 1=RTS/CTS, bit2=DSR/DTR
sv_baud   dw	0		; saved baud rate divisor
sv_lcr	  db	0		; saved line control register
sv_mcr	  db	0		; saved modem control register
sv_ier	  db	0		; saved interrupt enable register
sv_imask  db	0		; saved interrupt mask bit for this port's IRQ
portrec ends



;********************************************************************
;*								    *
;*			    Local equates			    *
;*								    *
;********************************************************************


; the following bits are STATUS_RS232 bits
INPUT_LOST  equ 0001h
PARITY_ERR  equ 0002h
OVERRUN_ERR equ 0004h
FRAMING_ERR equ 0008h
WAS_BREAK   equ 0010h
XOFF_RCVD   equ 0020h
XON_RCVD    equ 0040h
XOFF_SENT   equ 0080h
XON_SENT    equ 0100h
INPUT_EMPTY equ 0200h
OUTPUT_FULL equ 0400h
RING_DETECT equ 0800h		; [added]
IS_CARRIER  equ 1000h		; [added]
SEND_XOFF   equ 2000h		; internal use (xoff needs to be sent)
SEND_XON    equ 4000h		; internal use (xon needs to be sent
IS_OUTPUT   equ 8000h		; internal use (output not empty)

; the following bits are internal-use only
RESERVED    equ (SEND_XOFF or SEND_XON or IS_OUTPUT)
; the following bits are cleared when the input buffer is flushed
CLR_INPUT   equ (not(INPUT_LOST or PARITY_ERR or OVERRUN_ERR or FRAMING_ERR))
; the following bits are cleared when the output buffer is flushed
CLR_OUTPUT  equ (not(IS_OUTPUT))
; the following bits are cleared when STATUS_RS232 is called
CLR_STATUS  equ (CLR_INPUT and not (WAS_BREAK or INPUT_EMPTY or OUTPUT_FULL))

; the following bits enable the various protocols
;     notes:
;	  xon/xoff - the code doesn't filter or notice XON's or XOFF's
;		     being sent by the application
;	  dsr/dtr  - DTR is always set regardless of dsr/dtr, DSR is
;		     required to send anything except XON or XOFF (see
;		     ABS_CTS and BUSY_DTR above)
;	  cts/rts  - RTS is set only when ready to receive, CTS is
;		     required to send anything except XON or XOFF (see
;		     ABS_CTS and BUSY_RTS above)
;
XON_XOFF    equ 01h
CTS_RTS     equ 02h
DSR_DTR     equ 04h



;********************************************************************
;*								    *
;*			    Local data				    *
;*								    *
;********************************************************************



; following are set by SetModemMode_0
ModemPort dw	0		; port for modem functions
CmdPrefix dd	0		; pointer to command prefix string for modem
PrefixLen dw	0		; size of command prefix including terminator
CmdSuffix db	2 dup(0)	; null-terminated suffix string (0 if none)
BreakTime dw	0		; hundredths of a second to hold break
RspSuffix db	0		; modem response suffix



; port data structures
;     filled out by OPEN_RS232 except for the IRQ as below
;     if you add ports, you need to add to the list below
ports	  portrec <4>,<3>,<4>,<3>
endports  equ	this byte

; the following tables are used to translate between the indices
;     passed to OPEN_RS232 and the UART register values

; baud rate divisors for 110, 150, 300, 600, 1200, 2400, 4800, 9600 baud
baudtbl   dw	1047, 768, 384, 192, 96, 48, 24, 12
	  dw	6, 3	; [added baud rates 19200, 38400]
len_baudtbl equ $-baudtbl
paritytbl db	00h,08h,18h	; parity none, odd, even
word_len  db	00h,01h,02h,03h ; word-length 5, 6, 7, 8
stop_bits db	00h,04h

; the following is used to translate from a port index to an interrupt
;     handler for the port
;     if you add ports, you need to add to the list below and
;	  add the code for the corresponding interrupt handler
irqs	  label word
	  dw	port1, port2, port3, port4	; 1 - MAX_PORTS

_DATA	ends



SERIAL_TEXT segment para public 'CODE'
	assume	cs:SERIAL_TEXT,ds:DGROUP,es:nothing,ss:nothing



	public	setModemMode_0, _setModemMode_0
	public	setModemResponse_0, _setModemResponse_0
	public	sendBreak_Modem_0, _sendBreak_Modem_0
	public	txstr_Modem_0, _txstr_Modem_0
	public	txBin_RS232_0, _txBin_RS232_0
	public	rxstr_Modem_0, _rxstr_Modem_0
	public	rxBin_RS232_0, _rxBin_RS232_0
	public	closeRS232_0, _closeRS232_0, close_RS232_0, _close_RS232_0
	public	delInBuf_RS232_0, _delInBuf_RS232_0
	public	delOutBuf_RS232_0, _delOutBuf_RS232_0
	public	openRS232_0, open_RS232_0, _openRS232_0, _open_RS232_0
	public	openRS232_1, open_RS232_1, _openRS232_1, _open_RS232_1
	public	queueSize_RS232_0, _queueSize_RS232_0
	public	rxch_RS232_0, _rxch_RS232_0
	public	set_BaudRate_0, _set_BaudRate_0, set_BaudRate_RS232_0
	public	_set_BaudRate_RS232_0
	public	status_RS232_0, _status_RS232_0
	public	txCh_RS232_0, _txCh_RS232_0
	public	sendBreak_0, _sendBreak_0, sendBreak_RS232_0
	public	_sendBreak_RS232_0



;********************************************************************
;*								    *
;*			    Public code 			    *
;*								    *
;********************************************************************



;--------------------------------------------------------------------
; setModemMode(int PortNo, char *CmdPrefix, char CmdSuffix, int BreakTime);
;     [added _setModemMode];
; PortNo = 1 - 4 else fail [added 3, 4]
; Breaktime = 0 - 65535 hundredths second
;
; establishes the command prefix and command suffix for modem commands,
; sets the time to hold break

setModemMode_0 proc far
_setModemMode_0 label far
	push	bp
	mov	bp,sp
	push	di
	push	si
	mov	ax,[bp+6]		; Port
	call	IsOpen
	jnc	setmode1
	jmp	_run_fail
setmode1:
	mov	ax,[bp+6]		; Port
	mov	[ModemPort],ax
	mov	ax,word ptr [CmdPrefix]
	or	ax,word ptr [CmdPrefix+2]
	jz	setmode2
;--------------------------------------------------------------------
; if a command prefix was previously allocated, free it before
;     allocating a new one
;--------------------------------------------------------------------
	push	word ptr [CmdPrefix+2]
	push	word ptr [CmdPrefix]
	push	[PrefixLen]
	call	mem_releaseheap
setmode2:
;--------------------------------------------------------------------
; allocate space for a command prefix
;--------------------------------------------------------------------
	les	di,[bp+8]		; measure the prefix length
	call	strlen
	push	ax			; remember the length for later
	inc	ax			; alloc space for null terminator
	push	ax
	call	mem_allocheap
	mov	word ptr [CmdPrefix],ax
	mov	word ptr [CmdPrefix+2],dx
;--------------------------------------------------------------------
; copy the command prefix into the allocated space
;--------------------------------------------------------------------
	pop	cx			; prefix length
	mov	[PrefixLen],cx
	push	ds
	lds	si,[bp+8]		; command prefix
	mov	es,dx			; from allocheap
	mov	di,ax
	inc	cx			; including null terminator
	rep movsb			; (strlen has done a cld)
	pop	ds
;--------------------------------------------------------------------
; save the command suffix and break time
;--------------------------------------------------------------------
	mov	ax,[bp+12]
	mov	word ptr [CmdSuffix],ax
	mov	ax,[bp+14]
	mov	[BreakTime],ax
	mov	[RspSuffix],0ah 	; default response suffix to line feed
	pop	si
	pop	di
	pop	bp
	ret
setModemMode_0 endp



;--------------------------------------------------------------------
; setModemResponse(char ResponseTerminator)
;
; sets response terminator character for the modem port set by
; setModemMode.  Fails if setModemMode hasn't been called.  The
; ResponseTerminator after setModemMode is called and before
; setModemResponse is called is the line feed character

setModemResponse_0 proc far
_setModemResponse_0 label far
	push	bp
	mov	bp,sp
	cmp	[ModemPort],0
	jne	setrsp1
	jmp	_run_fail
setrsp1:
	mov	al,[bp+6]
	mov	[RspSuffix],al
	pop	bp
	ret
setModemResponse_0 endp



;--------------------------------------------------------------------
; sendBreak_Modem(void);
; fails if SetModemMode hasn't been called, or the modem port isn't open
;
; sends break according to the break time established by SetModemMode

sendBreak_Modem_0 proc far
_sendBreak_Modem_0 label far
	push	bp
	mov	bp,sp
	push	[BreakTime]
	push	[ModemPort]
	push	cs			; (short far call)
	call	near ptr sendBreak_0
	add	sp,4
	pop	bp
	ret
sendBreak_Modem_0 endp



;--------------------------------------------------------------------
; txStr_Modem(char *buffer, int *sent)
; buffer is a null-terminated string
; sent count will include the prefix and suffix when returned
; fails if no port has been set with SetModemMode
;
; sends buffer prefixed by the CmdPrefix and suffixed by the CmdSuffix

txStr_Modem_0 proc far
_txStr_Modem_0 label far
	push	bp
	mov	bp,sp
	push	di
	mov	ax,[ModemPort]
	call	IsOpen
	jnc	txstr1
	jmp	_run_fail
txstr1:
;--------------------------------------------------------------------
; send modem command prefix
;--------------------------------------------------------------------
	les	di,[CmdPrefix]
	mov	cx,[PrefixLen]
	call	SendString
	les	di,[bp+10]		; sent
	stosw				; update length as actually sent
	cmp	ax,[PrefixLen]
	jne	txstrx			; ne if couldn't send it all
;--------------------------------------------------------------------
; send buffer data
;--------------------------------------------------------------------
	les	di,[bp+6]		; buffer
	call	strlen			; (includes terminating null)
	mov	cx,ax			; length of user's buffer
	push	cx
	call	SendString
	pop	cx
	les	di,[bp+10]		; sent
	add	es:[di],ax		; update length to what was sent
	cmp	ax,cx
	jne	txstrx			; ne if couldn't send it all
;--------------------------------------------------------------------
; send suffix if any
;--------------------------------------------------------------------
	cmp	[CmdSuffix],0
	je	txstrx
	push	ds
	pop	es
	mov	di,offset DGROUP:CmdSuffix
	mov	cx,1
	call	SendString
	les	di,[bp+10]		; sent
	add	es:[di],ax
txstrx:
	pop	di
	pop	bp
	ret
txStr_Modem_0 endp



;--------------------------------------------------------------------
; [added]
; txBin_RS232(int Port, char *buffer, int length)
; PortNo = 1 - 4 else fail [added 3, 4]
; send a binary block of data
;
; waits until the buffer can be sent, if it doesn't all fit in the buffer

txBin_RS232_0 proc far
_txBin_RS232_0 label far
	push	bp
	mov	bp,sp
	push	di
	mov	ax,[bp+6]		; Port
	call	IsOpen
	jnc	txbin1
	jmp	_run_fail
txbin1:
	les	di,[bp+8]		; buffer
	mov	cx,[bp+12]		; length
	call	SendString
	add	[bp+8],ax
	sub	[bp+12],ax
	jnz	txbin1
	pop	di
	pop	bp
	ret
txBin_RS232_0 endp



;--------------------------------------------------------------------
; rxStr_Modem(char **buffer)
; fails if no port has been set with SetModemMode
;
; Return received chars up to a received RspSuffix char.  If RspSuffix
; == '\0' then returns all buffered received chars.  If RspSuffix != '\0'
; then it returns a 0 length string if RspSuffix isn't in the input buffer,
; otherwise it returns the characters in the input buffer up to and
; including RspSuffix.	If no characters have been received, it fails.

rxStr_Modem_0 proc far
_rxStr_Modem_0 label far
	push	bp
	mov	bp,sp
	push	di
	push	si
	mov	ax,[ModemPort]
	call	IsOpen
	jnc	rxstr1
	jmp	_run_fail
rxstr1:
;--------------------------------------------------------------------
; if there is no RspSuffix character (it's null), then return all
; the received chars at the instant InputUsed returns
;--------------------------------------------------------------------
	call	InputUsed
	mov	cx,ax			; chars in the buffer at this instant
	mov	si,bx
	or	ax,ax
	jz	rxstr4			; z if no chars received
	mov	al,[RspSuffix]
	or	al,al
	jz	rxstr4			; z to return buffer contents
;--------------------------------------------------------------------
; if there is a RspSuffix character, look through the buffer to make
; sure it's been received - count the characters including the
; RspSuffix character to allocate the correct size string
;--------------------------------------------------------------------
	xor	cx,cx			; to count characters to be transferred
	les	di,[si].ibf
	mov	bx,[si].ihead
rxstr2:
	cmp	bx,[si].itail
	je	rxstr3			; RspSuffix hasn't been received
	inc	cx
	cmp	al,es:[di][bx]
	je	rxstr4
	inc	bx
	cmp	bx,[si].isize
	jne	rxstr2
	xor	bx,bx
	jmp	rxstr2
rxstr3:
	xor	cx,cx			; at end w/o RspSuffix, return ""
rxstr4:
;--------------------------------------------------------------------
; allocate string space on the gstack according to what we now know
;     we will need
;--------------------------------------------------------------------
	push	si
	push	cx
	inc	cx			; space for trailing null
	push	cx
	call	_mem_allocgstack
	add	sp,2
	les	di,[bp+6]		; buffer
	mov	es:[di],ax
	mov	es:[di+2],dx
;--------------------------------------------------------------------
; read the characters and transfer them to the gstack
;--------------------------------------------------------------------
	mov	es,dx
	mov	di,ax
	pop	cx
	pop	bx
	jcxz	rxstr6
	cld
rxstr5:
	push	es
	push	di
	push	cx
	call	ReadChar
	pop	cx
	pop	di
	pop	es
	stosb
	loop	rxstr5
rxstr6:
	xor	al,al			; with a trailing null
	stosb
	pop	si
	pop	di
	pop	bp
	ret
rxStr_Modem_0 endp



;--------------------------------------------------------------------
; [added]
; rxBin_RS232(int Port, char **buffer, int length)
; PortNo = 1 - 4 else fail
; receive length bytes of binary data
;
; waits until length bytes is received, if needed

rxBin_RS232_0 proc far
_rxBin_RS232_0 label far
	push	bp
	mov	bp,sp
	push	di
	mov	ax,[bp+6]		; Port
	call	IsOpen
	jnc	rxbin1
	jmp	_run_fail
rxbin1:
	push	bx
	mov	cx,[bp+12]		; length
	inc	cx
	push	cx
	call	_mem_allocgstack	; allocate space for returned string
	add	sp,2			;     on the gstack
	les	di,[bp+8]		; buffer
	mov	es:[di],ax
	mov	es:[di+2],dx
	mov	es,dx
	mov	di,ax
	pop	bx
	mov	cx,[bp+12]		; length
rxbin2:
	push	es
	push	di
	push	cx
rxbin3:
	call	ReadChar
	jc	rxbin3
	pop	cx
	pop	di
	pop	es
	stosb
	loop	rxbin2
	xor	al,al
	stosb
	pop	di
	pop	bp
	ret
rxBin_RS232_0 endp



;--------------------------------------------------------------------
; closeRS232(int PortNo)
;     [added Close_Rs232_0 and versions w/ leading underbars]
; PortNo = 1 - 4 else fail [added 3, 4]
;
; deallocates InBuf and OutBuf, cleans up interrupt handling

closeRS232_0 proc far
_closeRS232_0 label far
close_RS232_0 label far
_close_RS232_0 label far

	push	bp
	mov	bp,sp
	push	di
	push	si
	mov	ax,[bp+6]
	call	IsOpen
	jnc	close1
	jmp	_run_fail
close1:
	mov	dx,[bx].port
;--------------------------------------------------------------------
; restore saved UART registers
;--------------------------------------------------------------------
	add	dx,IER-DATA_REG
	mov	al,[bx].sv_ier
	output	dx,al
	add	dx,MCR-IER
	mov	al,[bx].sv_mcr
	output	dx,al
	add	dx,LCR-MCR
	mov	al,[bx].sv_lcr
	or	al,DLAB 		; to enable restoring baud divisor
	output	dx,al
	mov	cx,[bx].sv_baud 	; restore saved baud rate divisor
	mov	al,cl
	add	dx,LO_DIV-LCR
	output	dx,al
	add	dx,HI_DIV-LO_DIV
	mov	al,ch
	output	dx,al
	add	dx,LCR-HI_DIV
	mov	al,[bx].sv_lcr
	output	dx,al			; presumably clearing DLAB
;--------------------------------------------------------------------
; restore saved interrupt mask, if we're the last port for the irq
;--------------------------------------------------------------------
	mov	si,bx
	mov	cl,[bx].irq
close2:
	add	si,type portrec
	cmp	si,offset DGROUP:endports
	jb	close3
	mov	si,offset DGROUP:ports
close3:
	cmp	si,bx
	je	close4			; last port for the irq
	cmp	[si].port,0
	je	close2
	cmp	cl,[si].irq
	jne	close2
	jmp	close6			; at least one other for the irq
close4:
	mov	dx,21h			; master 8259 interrupt mask
	mov	di,20h			; irq0 is at INT 08H
	mov	cl,[bx].irq
	cmp	cl,8
	jb	close5
	sub	cl,8
	mov	dx,0a1h 		; slave 8259 interrupt mask
	mov	di,70h			; irq8 is at INT 70H
close5:
	input	al,dx
	mov	ah,1
	shl	ah,cl
	not	ah
	and	al,ah			; clear the bit for the IRQ
	or	al,[bx].sv_imask	; set it as it was at startup
	output	dx,al
close6:
;--------------------------------------------------------------------
; restore saved interrupt vector
;--------------------------------------------------------------------
	mov	cl,[bx].irq
	mov	si,[bp+6]		; port index
	dec	si
	shl	si,1
	mov	si,irqs[si]
	call	ResetInt
	mov	[bx].port,0		; flag it closed
;--------------------------------------------------------------------
; release output buffer
;--------------------------------------------------------------------
	mov	ax,[bx].osize
	or	ax,ax
	jz	close7			; nz if it was successfully allocated
	mov	[bx].osize,0		; mark it released
	push	bx
	push	word ptr [bx].obf+2
	push	word ptr [bx].obf
	push	ax
	call	mem_releaseheap
	pop	bx
close7:
;--------------------------------------------------------------------
; release input buffer
;--------------------------------------------------------------------
	mov	ax,[bx].isize
	or	ax,ax
	jz	close8			; nz if it was successfully allocated
	mov	[bx].isize,0		; mark it released
	push	word ptr [bx].ibf+2
	push	word ptr [bx].ibf
	push	ax
	call	mem_releaseheap
close8:
;--------------------------------------------------------------------
; invalidate setModemMode port if needed and release its command
;     prefix string
;--------------------------------------------------------------------
	mov	ax,[bp+6]
	cmp	ax,[ModemPort]
	jne	close9
	mov	[ModemPort],0
	push	word ptr [CmdPrefix+2]
	push	word ptr [CmdPrefix]
	push	[PrefixLen]
	call	mem_releaseheap
	mov	word ptr [CmdPrefix],0
	mov	word ptr [CmdPrefix+2],0
close9:
	pop	si
	pop	di
	pop	bp
	ret

closeRS232_0 endp



;--------------------------------------------------------------------
; delInBuf_RS232(int PortNo)
; PortNo = 1 - 4, else fail [added 3, 4]
;
; empties (flushes) the input buffer for PortNo

delInBuf_RS232_0 proc far
_delInBuf_RS232_0 label far

	push	bp
	mov	bp,sp
	push	di
	push	si
	mov	ax,[bp+6]
	call	IsOpen
	jnc	delinbuf1
	jmp	_run_fail
delinbuf1:
	cli
	mov	[bx].ihead,0
	mov	[bx].itail,0
	and	[bx].status,CLR_INPUT
	or	[bx].bfstatus,INPUT_EMPTY
	sti
	call	ChkSendXON
	pop	si
	pop	di
	pop	bp
	ret

delInBuf_RS232_0 endp



;--------------------------------------------------------------------
; delOutBuf_RS232(int PortNo)
; PortNo = 1 - 4, else fail [added 3, 4]
;
; empties (flushes) the output buffer for PortNo

delOutBuf_RS232_0 proc far
_delOutBuf_RS232_0 label far

	push	bp
	mov	bp,sp
	push	di
	push	si
	mov	ax,[bp+6]
	call	IsOpen
	jnc	deloutbuf1
	jmp	_run_fail
deloutbuf1:
	cli
	mov	[bx].ohead,0
	mov	[bx].otail,0
	and	[bx].status,CLR_OUTPUT
	sti
	pop	si
	pop	di
	pop	bp
	ret

delOutBuf_RS232_0 endp



;--------------------------------------------------------------------
; openRS232(int PortNo, int InputBufSize, int OutputBufSize, int BaudRate,
;	    int Parity, int WordLength, int StopBits, int Protocol)
;     [added identical open_RS232_0 and w/ leading underbars]
; PortNo = 1 - 2 else fail
; InputBufSize = 1 - 65,xxx else fail [added to whatever mem_allocheap allows]
; OutputBufSize = 1 - 65,xxx else fail [added to whatever mem_allocheap allows]
; BaudRate = 0 - 9 else fail (110, 150, 300, 600, 1200, 2400, 4800, 9600)
;     [added 8-19200, 9-38400]
; Parity = 0 - 2 else fail (none, odd, even)
; WordLength = 0 - 3 else fail (5 - 8 data bits)
; StopBits = 0 or 1 else fail (1 - 2 stop bits)
; Protocol = 0 - 7 else fail
;      (bit 0 = XON/XOFF, bit 1 = RTS/CTS [added bit 2 = DSR/DTR])

openRS232_0 proc far
open_RS232_0   label far
_openRS232_0  label far
_open_RS232_0 label far

	push	bp
	mov	bp,sp
	push	di
	push	si
;--------------------------------------------------------------------
; validate the port and establish the port address
;--------------------------------------------------------------------
	mov	ax,[bp+6]
	dec	ax
	cmp	ax,2
	jb	open1
	jmp	_run_fail		; port is invalid
open1:
	inc	ax
	call	IsOpen
	jc	open2
	jmp	_run_fail		; it's already open
open2:
	mov	di,[bp+6]
	dec	di			; 0 or 1
	mov	dx,03f8h		; com1 port address
	jz	open3
	mov	dx,02f8h		; com2 port address
open3:
	mov	ax,4
	sub	ax,di			; ax = 4 for COM1, 3 for COM2
	mov	[bx].irq,al
	shl	di,1
	mov	ax,40h			; make sure the BIOS regards the
	mov	es,ax			;     port as valid
	cmp	dx,es:[di]
	je	open4
	cmp	dx,es:[0]
	je	open4
	jmp	_run_fail		; BIOS says it's not valid
open4:
	call	DoOpen
	pop	si
	pop	di
	pop	bp
	ret

openRS232_0 endp



;--------------------------------------------------------------------
; openRS232(int PortNo, int InputBufSize, int OutputBufSize, int BaudRate,
;	    int Parity, int WordLength, int StopBits, int Protocol,
;	    int PortAddress, int IRQ)
; PortNo = 1 - 4 else fail [added 3,4]
; InputBufSize = 1 - 65,xxx else fail [added to whatever mem_allocheap allows]
; OutputBufSize = 1 - 65,xxx else fail [added to whatever mem_allocheap allows]
; BaudRate = 0 - 9 else fail (110, 150, 300, 600, 1200, 2400, 4800, 9600)
;     [added 8-19200, 9-38400]
; Parity = 0 - 2 else fail (none, odd, even)
; WordLength = 0 - 3 else fail (5 - 8 data bits)
; StopBits = 0 or 1 else fail (1 - 2 stop bits)
; Protocol = 0 - 7 else fail
;      (bit 0 = XON/XOFF, bit 1 = RTS/CTS [added bit 2 = DSR/DTR])
; PortAddress = 0 - take from BIOS for port number else is actual port address
; IRQ = 2 - 15 else fail

openRS232_1 proc far
open_RS232_1   label far
_openRS232_1  label far
_open_RS232_1 label far

	push	bp
	mov	bp,sp
	push	di
	push	si
;--------------------------------------------------------------------
; validate the port and establish the port address
;--------------------------------------------------------------------
	mov	ax,[bp+6]
	dec	ax
	cmp	ax,MAX_PORTS
	jb	openx1
	jmp	_run_fail		; port is invalid
openx1:
	inc	ax
	call	IsOpen
	jc	openx2
	jmp	_run_fail		; it's already open
openx2:
	mov	dx,[bp+22]		; PortAddr
	or	dx,dx
	jnz	openx3
	mov	di,[bp+6]		; read port from bios 40:0 if
	dec	di			;     PortAddr == 0
	shl	di,1
	mov	ax,40h
	mov	es,ax
	mov	dx,es:[di]
	or	dx,dx
	jnz	openx3
	jmp	_run_fail		; BIOS says it's not valid
openx3:
	mov	al,[bp+24]		; IRQ
	cmp	al,2
	jb	openx4
	cmp	al,15
	jbe	openx5
openx4:
	jmp	_run_fail
openx5:
	mov	[bx].irq,al
	call	DoOpen
	pop	si
	pop	di
	pop	bp
	ret

openRS232_1 endp



;--------------------------------------------------------------------
; queueSize_RS232(int PortNo, int *SizeOfInputQueue, int *SizeOfOutputQueue)
; PortNo = 1 - 4 else fail [add 3, 4]
; returns
;     SizeOfInputQueue = number of unread characters in InputQueue
;     SizeOfOutputQueue = number of unsent characters in OutputQueue

queueSize_RS232_0 proc far
_queueSize_RS232_0 label far

	push	bp
	mov	bp,sp
	push	di
	push	si
	mov	ax,[bp+6]
	call	IsOpen
	jnc	queuesize1
	jmp	_run_fail
queuesize1:
	call	PollPort
	call	InputUsed
	les	di,[bp+8]
	stosw				; returned to PDC Prolog
	call	OutputUsed
	les	di,[bp+12]
	stosw				; returned to PDC Prolog
	pop	si
	pop	di
	pop	bp
	ret

queueSize_RS232_0 endp



;--------------------------------------------------------------------
; rxch_RS232(int PortNo, char *Ch)
; PortNo = 1 - 4, else fail [added 3, 4]
; returns
;     Ch = next character from input buffer, else fail if no characters
;	   are in input buffer

rxch_RS232_0 proc far
_rxch_RS232_0 label far

	push	bp
	mov	bp,sp
	push	di
	push	si
	mov	ax,[bp+6]
	call	IsOpen
	jnc	rxch1
	jmp	_run_fail
rxch1:
	call	ReadChar
	jnc	rxch2
	or	[bx].status,INPUT_EMPTY
	jmp	_run_fail
rxch2:
	les	di,[bp+8]
	stosb
	pop	si
	pop	di
	pop	bp
	ret

rxch_RS232_0 endp



;--------------------------------------------------------------------
; sendBreak(int PortNo, int hundredths)
; PortNo = 1 - 4, else fail [added 3, 4]
;
; send break signal for hundredths .01 seconds

sendBreak_0 proc far
_sendBreak_0 label far
sendBreak_RS232_0 label far
_sendBreak_RS232_0 label far

	push	bp
	mov	bp,sp
	push	di
	push	si
	mov	ax,[bp+6]
	call	IsOpen
	jnc	brk1
	jmp	_run_fail
brk1:
	mov	ax,[bp+8]
;--------------------------------------------------------------------
; translate hundredths to ticks
;--------------------------------------------------------------------
	mov	cx,10000
	mul	cx
	add	ax,54926/2
	adc	dx,0
	mov	cx,54926		; each tick is 5.4926 hundredths
	div	cx
	mov	cx,ax
	mov	dx,[bx].port
	add	dx,LCR-DATA_REG
	input	al,dx
	or	al,BREAK
	output	dx,al
	mov	ax,40h
	mov	es,ax
	add	cx,es:[timer]
	sti				; just to be absolutely sure
brk2:
	cmp	cx,es:[timer]
	jns	brk2
	and	al,not BREAK
	output	dx,al
	pop	si
	pop	di
	pop	bp
	ret

sendBreak_0 endp



;--------------------------------------------------------------------
; set_BaudRate(int PortNo, int BaudRate)
; PortNo = 1 - 4, else fail [added 3, 4]
; BaudRate = 0 - 9, else fail (110, 150, 300, 600, 1200, 2400, 4800, 9600)
;     [added 8-19200, 9-19200]
;
; (I'm guessing - this isn't defined anywhere, or accessed by anybody, but
; it was in the serial.obj for the previous version, so it's in this code
; for the PDC Prolog 3.2 version)

set_BaudRate_0 proc far
_set_BaudRate_0 label far
set_BaudRate_RS232_0 label far
_set_BaudRate_RS232_0 label far

	push	bp
	mov	bp,sp
	push	di
	push	si
	mov	ax,[bp+6]
	call	IsOpen
	jnc	set_baudrate1
	jmp	_run_fail
set_baudrate1:
	mov	ax,[bp+8]
	mov	dx,[bx].port
	call	SetBaud
	pop	si
	pop	di
	pop	bp
	ret

set_BaudRate_0 endp



;--------------------------------------------------------------------
; status_RS232(int PortNo, int *Status)
; PortNo = 1 - 4, else fail [added 3, 4]
; returns
;     Status = 0000h ok
;	       0001h input chars lost with queue full
;	       0002h parity error detected
;	       0004h over-run detected
;	       0008h framing error detected
;	       0010h break detected
;	       0020h xoff received
;	       0040h xon received
;	       0080h xoff transmitted
;	       0100h xon transmitted
;	       0200h input buffer empty (? when trying to read)
;	       0400h output buffer full (? when trying to write)
;	       0800h ring indicator [added]
;	       1000h carrier detect [added]

status_RS232_0 proc far
_status_RS232_0 label far

	push	bp
	mov	bp,sp
	push	di
	push	si
	mov	ax,[bp+6]
	call	IsOpen
	jnc	status1
	jmp	_run_fail
status1:
	call	PollPort
	cli
	mov	ax,[bx].status
	and	[bx].status,CLR_STATUS	; clearing one-time bits like
					;     PARITY_ERR, OVERRUN_ERR, etc.
;--------------------------------------------------------------------
; supply an up-to-the-minute ring indicator status
;--------------------------------------------------------------------
	and	[bx].status,not RING_DETECT
	mov	dx,[bx].port
	add	dx,MSR-DATA_REG
	input	al,dx
	test	al,RI
	jz	status2
	or	[bx].status,RING_DETECT
status2:
	sti
	and	ax,not RESERVED 	; only the documented bits
	les	di,[bp+8]
	stosw				; returned to PDC Prolog
	pop	si
	pop	di
	pop	bp
	ret

status_RS232_0 endp



;--------------------------------------------------------------------
; txCh_RS232(int PortNo, char Ch)
; PortNo = 1 - 4, else fail [added 3, 4]
; Ch = character to send, fails if the output buffer is full

txCh_RS232_0 proc far
_txCh_RS232_0 label far

	push	bp
	mov	bp,sp
	push	di
	push	si
	mov	ax,[bp+6]
	call	IsOpen
	jnc	txch1
	jmp	_run_fail
txch1:
	mov	al,[bp+8]
	call	SendChar
	jnc	txch2
	or	[bx].status,OUTPUT_FULL
	jmp	_run_fail
txch2:
	pop	si
	pop	di
	pop	bp
	ret

txCh_RS232_0 endp



;********************************************************************
;*								    *
;*			    Local code				    *
;*								    *
;********************************************************************



;--------------------------------------------------------------------
; DoOpen - do all the work for open_RS232_0 and open_RS232_1 for bx
; -> port structure.  dx = port address, [bx].irq is already set

DoOpen	proc	near
	push	dx
;--------------------------------------------------------------------
; allocate the input buffer
;--------------------------------------------------------------------
	mov	ax,[bp+8]
	or	ax,ax
	jnz	doopen1
	jmp	_run_fail
doopen1:
	push	bx
	push	ax
	call	mem_allocheap
	pop	bx
	mov	word ptr [bx].ibf,ax
	mov	word ptr [bx].ibf+2,dx
	mov	ax,[bp+8]
	mov	[bx].isize,ax
	mov	[bx].ihead,0
	mov	[bx].itail,0
	mov	[bx].bfstatus,INPUT_EMPTY
	mov	[bx].status,0
;--------------------------------------------------------------------
; allocate the output buffer
;--------------------------------------------------------------------
	mov	ax,[bp+10]
	or	ax,ax
	jnz	doopen2
	jmp	_run_fail
doopen2:
	push	bx
	push	ax
	call	mem_allocheap
	pop	bx
	mov	word ptr [bx].obf,ax
	mov	word ptr [bx].obf+2,dx
	mov	ax,[bp+10]
	mov	[bx].osize,ax
	mov	[bx].ohead,0
	mov	[bx].otail,0

	pop	dx
	push	dx
;--------------------------------------------------------------------
; save the prior state of the UART
;--------------------------------------------------------------------
	add	dx,LCR-DATA_REG 	; save LCR register
	input	al,dx
	mov	[bx].sv_lcr,al
	or	al,DLAB
	output	dx,al
	add	dx,LO_DIV-LCR		; save baud rate divisor
	input	al,dx
	mov	cl,al
	add	dx,HI_DIV-LO_DIV
	input	al,dx
	mov	ch,al
	mov	[bx].sv_baud,cx
	add	dx,LCR-HI_DIV
	mov	al,[bx].sv_lcr
	output	dx,al
	add	dx,MCR-LCR		; save MCR register
	input	al,dx
	mov	[bx].sv_mcr,al
	add	dx,IER-MCR		; save IER register
	input	al,dx
	mov	[bx].sv_ier,al
;--------------------------------------------------------------------
; set baud rate
;--------------------------------------------------------------------
	mov	ax,[bp+12]
	pop	dx
	push	dx
	call	SetBaud
	mov	di,[bp+14]
	cmp	di,3
	jb	doopen3
	jmp	_run_fail
doopen3:
;--------------------------------------------------------------------
; set line control parameters
;--------------------------------------------------------------------
	mov	al,paritytbl[di]
	mov	di,[bp+16]
	cmp	di,4
	jb	doopen4
	jmp	_run_fail
doopen4:
	or	al,word_len[di]
	mov	di,[bp+18]
	cmp	di,2
	jb	doopen5
	jmp	_run_fail
doopen5:
	or	al,stop_bits[di]
	pop	dx
	add	dx,LCR-DATA_REG
	output	dx,al
	add	dx,DATA_REG-LCR
;--------------------------------------------------------------------
; set protocol
;--------------------------------------------------------------------
	mov	al,[bp+20]
	cmp	al,8
	jb	doopen6
	jmp	_run_fail
doopen6:
	mov	[bx].protocol,al
	mov	[bx].port,dx
;--------------------------------------------------------------------
; set interrupt vector to us, chaining for ps-2's, and enable
;     interrupt handling
;--------------------------------------------------------------------
	mov	cl,[bx].irq
	mov	si,[bp+6]		; port index
	dec	si			; 0-based
	shl	si,1			; * 2
	mov	si,irqs[si]
	call	SetInt			; setting interrupt vector
;--------------------------------------------------------------------
; see if any other port is using this irq - if so, we'll get the
;     prior state of the 8259 from that port
;--------------------------------------------------------------------
	mov	cl,[bx].irq
	mov	si,bx
doopen7:
	add	si,type portrec
	cmp	si,offset DGROUP:endports
	jb	doopen8
	mov	si,offset DGROUP:ports
doopen8:
	cmp	si,bx
	je	doopen9
	cmp	[si].port,0
	je	doopen7
	cmp	cl,[si].irq
	jne	doopen7
	mov	al,[si].sv_imask
	mov	[bx].sv_imask,al
	jmp	doopen11
doopen9:
;--------------------------------------------------------------------
; save the prior state of the interrupt mask
;--------------------------------------------------------------------
	mov	di,8			; set up for irq's < 8
	mov	dx,21h
	mov	cl,[bx].irq
	cmp	cl,8
	jb	doopen10
	sub	cl,8			; set up for irq's >= 8
	mov	di,70h
	mov	dx,0a1h
doopen10:
	input	al,dx
	mov	ch,al
	mov	ah,1			; mask out all but this irq
	shl	ah,cl
	and	al,ah
	mov	[bx].sv_imask,al
;--------------------------------------------------------------------
; enable irq at 8259
;--------------------------------------------------------------------
	mov	al,ch
	not	ah
	and	al,ah
	sti
; interrupts must be enabled here - the first output may enable a pending
; interrupt which would immediately be taken and chained to the prior
; (BIOS) interrupt handler which, on COMPAQ's at least, re-masks off the
; interrupt - the second output then successfully masks the interrupt on
	output	dx,al			; note if that was the slave, we
	output	dx,al			; assume the master has enabled irq 2
doopen11:
	mov	dx,[bx].port
	add	dx,MCR-DATA_REG
	mov	al,OUT2 or DTR or RTS
	output	dx,al
	add	dx,IER-MCR
	mov	al,RCV_INT or LSR_INT or MSR_INT
	output	dx,al
;--------------------------------------------------------------------
; set initial carrier and ring status
;--------------------------------------------------------------------
	add	dx,MSR-IER
	input	al,dx
	test	al,DCD
	jz	doopen12
	or	[bx].status,IS_CARRIER
doopen12:
;--------------------------------------------------------------------
; if XON/XOFF protocol is enabled, status is currently as though XON
; were sent
;--------------------------------------------------------------------
	test	[bx].protocol,XON_XOFF
	jz	doopen13
	or	[bx].status,XON_SENT
doopen13:
	call	PollPort
	ret
DoOpen	endp



;--------------------------------------------------------------------
; IsOpen - return NC and ds:bx -> port data structure for port index ax
;	   return C if ax is an invalid index or the port isn't open

IsOpen	proc	near
	dec	ax
	cmp	ax,MAX_PORTS
	jae	isopenx
	mov	bx,type portrec
	mul	bx
	add	ax,offset DGROUP:ports
	mov	bx,ax
	cmp	[bx].port,0
	je	isopenx
	clc
	ret
isopenx:
	stc
	ret
IsOpen	endp



;--------------------------------------------------------------------
; strlen  - return AX = length of null-terminated string starting at
; ES:DI

strlen	proc	near
	push	cx
	push	bx
	mov	bx,di			; remember start of string
	mov	cx,di
	inc	cx
	neg	cx			; can't extend past end of segment
	xor	al,al
	jcxz	strlen1
	cld
	repne scasb
	je	strlen2
strlen1:
	scasb				; check byte at offset 0ffffh
	je	strlen2
	jmp	_run_fail		; crash here instead of later if
					; the string isn't null-terminated
strlen2:
	xchg	bx,di
	sub	bx,di
	mov	ax,bx
	dec	ax			; don't count null terminator
	pop	bx
	pop	cx
	ret
strlen	endp



;--------------------------------------------------------------------
; SendString - send CX characters of string at ES:DI to BX -> port
; structure returning the number sent in AX

SendString proc near
	push	cx
	jcxz	sendstrx
sendstr1:
	mov	al,es:[di]
	push	es
	push	di
	push	cx
	call	SendChar
	pop	cx
	pop	di
	pop	es
	jc	sendstrx		; buffer must be full
	inc	di
	loop	sendstr1
sendstrx:
	pop	ax
	sub	ax,cx			; original length to send
	ret
SendString endp



;--------------------------------------------------------------------
; SendChar - buffer AL char to in output buffer of BX -> port structure
; returns C if the character couldn't be sent because the buffer is full

SendChar proc	near
	push	ax
	call	PollPort
	pop	ax
	test	[bx].bfstatus,OUTPUT_FULL
	jz	sendch1
	stc
	ret
sendch1:
	les	di,[bx].obf
	mov	dx,bx
	mov	bx,[bx].otail
	mov	es:[di][bx],al
	xchg	dx,bx
	inc	dx
	cmp	dx,[bx].osize
	jb	sendch2
	xor	dx,dx
sendch2:
	cli
	mov	[bx].otail,dx
	cmp	dx,[bx].ohead		; if head == tail, the buffer's full or
	jne	sendch3 		;     empty - since we just put in a
					;     char, it can't be empty
	or	[bx].bfstatus,OUTPUT_FULL
sendch3:
	or	[bx].status,IS_OUTPUT
	sti
	call	ChkXmit
	clc				; signal char buffered ok
	ret
SendChar endp



;--------------------------------------------------------------------
; ReadChar - return next char from input buffer in AL - return C if
; the input buffer is empty

ReadChar proc	near
	call	PollPort
	test	[bx].bfstatus,INPUT_EMPTY
	jz	rdch1
	stc
	ret
rdch1:
	les	di,[bx].ibf
	mov	dx,[bx].ihead
	xchg	dx,bx
	mov	al,es:[bx][di]
	xchg	dx,bx
	inc	dx
	cmp	dx,[bx].isize
	jb	rdch3
	xor	dx,dx
rdch3:
	cli
	mov	[bx].ihead,dx
	cmp	dx,[bx].itail		; if head == tail, the buffer's full or
	jne	rdch4			;     empty - we just took a char out
					;     so it can't be full
	or	[bx].bfstatus,INPUT_EMPTY
rdch4:
	sti
	push	ax
	call	ChkSendXON		; sent if < 1/4 full and XOFF was sent
	pop	ax
	clc
	ret
ReadChar endp



;--------------------------------------------------------------------
; InputUsed - return AX = number of chars in the input queue for BX ->
;     port structure

InputUsed proc	near
	xor	ax,ax			; size is 0 if empty
	test	[bx].bfstatus,INPUT_EMPTY
	jnz	inpuse1
	mov	ax,[bx].itail		; else size is chars from head to tail
	sub	ax,[bx].ihead
	ja	inpuse1
	add	ax,[bx].isize		; + buffer size if wrapped around
inpuse1:
	ret
InputUsed endp



;--------------------------------------------------------------------
; OutputUsed - return AX = number of chars in the output queue for BX ->
;     port structure

OutputUsed proc  near
	mov	ax,[bx].osize		; size is osize if full
	test	[bx].bfstatus,OUTPUT_FULL
	jnz	outuse1
	mov	ax,[bx].otail		; else size is chars from head to tail
	sub	ax,[bx].ohead
	jae	outuse1
	add	ax,[bx].osize		; + buffer size if wrapped around
outuse1:
	ret
OutputUsed endp



;--------------------------------------------------------------------
; ChkSendXON - send XON if the input buffer usage permits and an
; XOFF was previously sent
; ds:bx -> port data structure

ChkSendXON proc near
	push	cx
	cli
	call	InputUsed
	mov	cx,[bx].isize
	shr	cx,1
	shr	cx,1
	cmp	ax,cx
	ja	chkxonx
	and	[bx].status,not SEND_XOFF

if DTR_BUSY
	test	[bx].protocol,DSR_DTR
	jz	chkxon1
	mov	dx,[bx].port
	add	dx,MCR-DATA_REG
	input	al,dx
	or	al,DTR
	output	dx,al
chkxon1:
endif

if RTS_BUSY
	test	[bx].protocol,CTS_RTS
	jz	chkxon2
	mov	dx,[bx].port
	add	dx,MCR-DATA_REG
	input	al,dx
	or	al,RTS
	output	dx,al
chkxon2:
endif

	test	[bx].status,XOFF_SENT
	jz	chkxonx 		; no need unless actually sent XOFF
	or	[bx].status,SEND_XON
	sti
	call	ChkXmit
chkxonx:
	sti
	pop	cx
	ret
ChkSendXON endp



;--------------------------------------------------------------------
; ChkSendXOFF - send XOFF if the input buffer usage requires
; ds:bx -> port data structure

ChkSendXOFF proc near
	push	di
	push	cx
	cli
	call	InputUsed
	mov	cx,[bx].isize
	mov	di,cx
	shr	di,1
	shr	di,1
	sub	cx,di
	cmp	ax,cx
	jb	chkxoffx
	and	[bx].status,not SEND_XON

if DTR_BUSY
	test	[bx].protocol,DSR_DTR
	jz	chkxoff1
	mov	dx,[bx].port
	add	dx,MCR-DATA_REG
	input	al,dx
	and	al,not DTR
	output	dx,al
chkxoff1:
endif

if RTS_BUSY
	test	[bx].protocol,CTS_RTS
	jz	chkxoff2
	mov	dx,[bx].port
	add	dx,MCR-DATA_REG
	input	al,dx
	and	al,not RTS
	output	dx,al
chkxoff2:
endif

	test	[bx].status,XON_SENT
	jz	chkxoffx		; no need to check unless XON was sent
	or	[bx].status,SEND_XOFF
	sti
	call	ChkXmit
chkxoffx:
	sti
	pop	cx
	pop	di
	ret
ChkSendXOFF endp



;--------------------------------------------------------------------
; SetBaud - set baudrate for dx port to baud rate according to AX as
; an index into the baud rate divisor table
; 0 - 7 = 110, 150, 300, 600, 1200, 2400, 4800, 9600

SetBaud proc	near
	mov	di,ax
	shl	di,1
	cmp	di,len_baudtbl		; so additional baud rates can be added
	jb	setb1			;     by simply putting them in baudtbl
	jmp	_run_fail
setb1:
	mov	cx,baudtbl[di]
	add	dx,LCR-DATA_REG
	input	al,dx
	or	al,DLAB 		; divisor latch bit
	output	dx,al
	add	dx,LO_DIV-LCR
	mov	al,cl
	output	dx,al
	add	dx,HI_DIV-LO_DIV
	mov	al,ch
	output	dx,al
	add	dx,LCR-HI_DIV
	input	al,dx
	and	al,not DLAB
	output	dx,al
	ret
SetBaud endp



;--------------------------------------------------------------------
; InputLSR - read LSR and check and note line status error conditions
; entry with ds:bx -> port structure
; returns al = LSR

InputLSR proc	near
	mov	dx,[bx].port
	add	dx,LSR-DATA_REG
	input	al,dx
	add	dx,DATA_REG-LSR
	test	al,OVERRUN or PARITY or FRAMING or BREAK_RCVD
	jz	lsr4
	test	al,OVERRUN
	jz	lsr1
	or	[bx].status,OVERRUN_ERR
lsr1:
	test	al,PARITY
	jz	lsr2
	or	[bx].status,PARITY_ERR
lsr2:
	test	al,BREAK_RCVD
	jz	lsr3
	or	[bx].status,WAS_BREAK
lsr3:
	test	al,FRAMING
	jz	lsr4
	or	[bx].status,FRAMING_ERR
lsr4:
	ret
InputLSR endp



;--------------------------------------------------------------------
; InputMSR - read MSR and check and note ring indicator and carrier status
; entry with ds:bx -> port structure
; returns al = LSR

InputMSR proc	near
	mov	dx,[bx].port
	add	dx,MSR-DATA_REG
	input	al,dx
	add	dx,DATA_REG-MSR
	test	al,DDCD
	jz	msr4
	test	al,DCD
	jz	msr3
	or	[bx].status,IS_CARRIER
	jmp	msr4
msr3:
	and	[bx].status,not IS_CARRIER
msr4:
	ret
InputMSR endp



;--------------------------------------------------------------------
; ChkXmit - output polling
;
; send xoff if needed, or
; send xon if needed, or
; send next char from output buffer, if any, and
;     disable xmit empty interrupts if output buffer is then empty
;
; enable xmit empty interrupts if any of the above couldn't be
;     sent because the xmit holding register wasn't empty

ChkXmit proc	near
	test	[bx].protocol,XON_XOFF
	jz	xmit4
	cli
	test	[bx].status,SEND_XOFF
	jz	xmit2
;--------------------------------------------------------------------
; send XOFF
;--------------------------------------------------------------------
	mov	al,XOFF
	call	XmitChar
	jc	xmit1
	and	[bx].status,not (SEND_XOFF or SEND_XON or XON_SENT)
	or	[bx].status,XOFF_SENT
	jmp	xmit4
xmit1:
	sti
	ret
xmit2:
	test	[bx].status,SEND_XON
	jz	xmit4
;--------------------------------------------------------------------
; send XON
;--------------------------------------------------------------------
	mov	al,XON
	call	XmitChar
	jc	xmit3			; c if couldn't send it
	and	[bx].status,not (SEND_XON or SEND_XOFF or XOFF_SENT)
	or	[bx].status,XON_SENT
	jmp	xmit4
xmit3:
	sti
	ret
xmit4:
	sti
	push	es
	push	di
	push	cx
	test	[bx].protocol,XON_XOFF
	jz	xmit5
	test	[bx].status,XOFF_RCVD
	jnz	xmit10			; can't send if XOFF received
;--------------------------------------------------------------------
; XOFF wasn't received
;--------------------------------------------------------------------
xmit5:

if not ABS_CTS		; checked in XmitChar if ABS_CTS
	test	[bx].protocol,CTS_RTS
	jz	xmit6
;--------------------------------------------------------------------
; CTS/RTS protocol enabled
;--------------------------------------------------------------------
	call	InputMSR		; or if CTS isn't active and CTS_RTS
	test	al,CTS			;     protocol is enabled
	jz	xmit10
xmit6:
endif

if not ABS_DSR		; checked in XmitChar if ABS_DSR
	test	[bx].protocol,DSR_DTR
	jz	xmit6a
	call	InputMSR
	test	al,DSR
	jz	xmit10
xmit6a:
endif

	les	di,[bx].obf
	cli
	mov	cx,[bx].ohead
	test	[bx].bfstatus,OUTPUT_FULL ; if buffer isn't full & head == tail
	jnz	xmit7			;     then buffer is empty (nothing to
					;     send)
	cmp	cx,[bx].otail
	je	xmit9			; e if output buffer is empty
xmit7:
	xchg	cx,bx
	mov	al,es:[di][bx]
	xchg	cx,bx
	call	XmitChar
	jc	xmit10			; couldn't send it
	inc	cx
	cmp	cx,[bx].osize
	jb	xmit8
	xor	cx,cx
xmit8:
	mov	[bx].ohead,cx
	and	[bx].bfstatus,not OUTPUT_FULL
	cmp	cx,[bx].otail
	jne	xmit10
xmit9:
;--------------------------------------------------------------------
; nothing left to send, so turn off xmit empty interrupts
;--------------------------------------------------------------------
	and	[bx].status,not IS_OUTPUT
	mov	dx,[bx].port
	add	dx,IER-DATA_REG
	input	al,dx
	and	al,not XMIT_INT
	output	dx,al
	add	dx,DATA_REG-IER
xmit10:
	sti
	pop	cx
	pop	di
	pop	es
	ret
ChkXmit endp



;--------------------------------------------------------------------
; XmitChar - output AL = char and return no carry if xmit holding register is
; empty else return carry and enable xmit empty interrupts

XmitChar proc	near
	mov	ah,al
	mov	dx,[bx].port

if ABS_CTS				; checked in ChkXmit if not ABS_CTS
	test	[bx].protocol,CTS_RTS
	jz	xmitch1
	call	InputMSR		; or if CTS isn't active and CTS_RTS
	test	al,CTS			;     protocol is enabled
	jz	xmitch5
xmitch1:
endif

if ABS_DSR				; checked in ChkXmit if not ABS_DSR
	test	[bx].protocol,DSR_DTR
	jz	xmitch2
	call	InputMSR		; or if DSR isn't active and DSR_DTR
	test	al,DSR			;     protocol is enabled
	jz	xmitch5
xmitch2:
endif

	call	InputLSR
	test	al,XMIT_EMPTY
	jz	xmitch4
	mov	al,ah
	output	dx,al
	test	[bx].status,IS_OUTPUT
	jz	xmitch3
	call	EnableXmit
xmitch3:
	clc
	ret
xmitch4:
	call	EnableXmit
xmitch5:
	stc
	ret
XmitChar endp



;--------------------------------------------------------------------
; EnableXmit - enable Xmit empty interrupt
; ds:bx -> portrec

EnableXmit proc near
	mov	dx,[bx].port
	add	dx,IER-DATA_REG 	; enable xmit empty interrupts
	input	al,dx
	or	al,XMIT_INT
	output	dx,al
	add	dx,DATA_REG-IER
	ret
EnableXmit endp



;--------------------------------------------------------------------
; RcvChar - read the char from the port and stuff it in the input
; buffer, clearing INPUT_EMPTY - if there's no room in the input
; buffer, set INPUT_LOST and throw away the char - if the input
; buffer is more than 3/4 full, send XOFF

RcvChar proc	near
	mov	dx,[bx].port
	cli
	call	InputLSR
	test	al,DATA_READY
	jnz	rcvch0
	sti
	ret
rcvch0:
	input	al,dx
	test	[bx].protocol,XON_XOFF
	jz	rcvch2
	cmp	al,XOFF
	jne	rcvch1
	or	[bx].status,XOFF_RCVD
	and	[bx].status,not XON_RCVD
	sti
	ret
rcvch1:
	cmp	al,XON
	jne	rcvch2
	or	[bx].status,XON_RCVD
	and	[bx].status,not XOFF_RCVD
	sti
	call	ChkXmit
	ret
rcvch2:
	push	es
	push	di
	push	cx
	les	di,[bx].ibf
	mov	cx,[bx].itail
	test	[bx].bfstatus,INPUT_EMPTY
	jnz	rcvch3
	cmp	cx,[bx].ihead
	jne	rcvch3
	or	[bx].status,INPUT_LOST	; input buffer is full
	jmp	rcvch5
rcvch3:
	xchg	bx,cx
	mov	es:[di][bx],al
	xchg	bx,cx
	inc	cx
	cmp	cx,[bx].isize
	jb	rcvch4
	xor	cx,cx
rcvch4:
	mov	[bx].itail,cx
	and	[bx].bfstatus,not INPUT_EMPTY
	sti
	call	ChkSendXOFF
rcvch5:
	pop	cx
	pop	di
	pop	es
	ret
RcvChar endp



;--------------------------------------------------------------------
; SetInt - set the interrupt vector for cl = IRQ to the cs:si handler
; (as abstracted from PS/2 tech ref)

SetInt	proc	near
	xor	ax,ax
	mov	es,ax
	mov	ch,0
	mov	di,08h
	cmp	cl,8
	jb	setint1
	mov	di,70h
	sub	cl,8
setint1:
	add	di,cx
	shl	di,1
	shl	di,1
	cli
	mov	ax,es:[di]
	mov	cs:[si+2],ax
	mov	cx,es:[di+2]
	mov	cs:[si+4],cx
	mov	es:[di],si
	mov	es:[di+2],cs
	mov	es,cx
	mov	di,ax
	cmp	byte ptr es:[di],0cfh
	je	setint2
	cmp	word ptr es:[di+6],424BH
	je	setint3
	mov	ax,es
	cmp	ax,0f000h
	jne	setint3
	cmp	di,word ptr es:[0ff01h]
	jne	setint3
setint2:
	or	byte ptr cs:[si+8],80h
setint3:
	sti
	ret
SetInt	endp



;--------------------------------------------------------------------
; ResetInt - unlink the interrupt vector for cl = IRQ from the cs:si handler
; (if somebody following us has taken the same vector without interrupt
; sharing link, then we just reset it to our predecessor, otherwise we
; gracefully unlink ourselves)

ResetInt proc	near
	xor	ax,ax
	mov	es,ax
	mov	ch,0
	mov	di,08h
	cmp	cl,8
	jb	rstint1
	mov	di,70h
	sub	cl,8
rstint1:
	add	di,cx
	shl	di,1
	shl	di,1
	push	es
	push	di
	mov	cx,cs
	cmp	cx,es:[di+2]
	jne	rstint2
	cmp	si,es:[di]
	je	rstintx 		; it's ok we're first in the chain
rstint2:
;--------------------------------------------------------------------
; follow the chain til we get to us or to a non-sharing handler
;--------------------------------------------------------------------
	les	di,es:[di]
rstint3:
	cmp	word ptr es:[di+6],424bh
	jne	rstintx 		; not a sharer
	cmp	cx,es:[di+4]
	jne	rstint4
	cmp	si,es:[di+2]
	je	rstint5 		; found ourselves linked with sharers
rstint4:
	test	byte ptr es:[di+8],80h
	jnz	rstintx 		; came to first w/o finding ourselves
	les	di,es:[di+2]
	jmp	rstint3
rstint5:
	cli
	mov	ax,cs:[si+2]
	mov	es:[di+2],ax
	mov	ax,cs:[si+4]
	mov	es:[di+4],ax
	sti
	pop	di
	pop	es
	ret
rstintx:
	pop	di
	pop	es
	cli
	mov	ax,cs:[si+2]
	mov	es:[di],ax
	mov	ax,cs:[si+4]
	mov	es:[di+2],ax
	sti
	ret
Resetint endp



;--------------------------------------------------------------------
; ChkMSR - handle delta MSR interrupt - calling ChkXmit if
; there was a change in DSR or CTS

ChkMSR	proc   near
	call	InputMSR
	test	[bx].protocol,DSR_DTR
	jz	chkmsr1
	mov	ah,al
	and	ah,DDSR or DSR
	cmp	ah,DDSR or DSR
	je	chkmsr2
chkmsr1:
	test	[bx].protocol,CTS_RTS
	jz	chkmsr3
	mov	ah,al
	and	ah,DCTS or CTS
	cmp	ah,DCTS or CTS
	jne	chkmsr3
chkmsr2:
	call	ChkXmit
chkmsr3:
	ret
ChkMSR	endp



;--------------------------------------------------------------------
; PollPort - check all the various conditions on the off chance that
; we might have somehow missed an interrupt

PollPort proc	near
	call	InputLSR
	test	al,DATA_READY
	jz	poll1
	call	RcvChar
poll1:
	test	[bx].status,IS_OUTPUT or SEND_XOFF or SEND_XON
	jz	poll2
	call	InputLSR
	test	al,XMIT_EMPTY
	jz	poll2
	call	ChkXmit
	ret
poll2:
	call	InputMSR
	mov	dx,[bx].port
	add	dx,IIR-DATA_REG
	input	al,dx			; clear pending missed iir if needed
	test	al,01h
	jz	PollPort
	ret
PollPort endp



;--------------------------------------------------------------------
; Interrupt - interrupt handler for the ports
; ds:bx -> port rec
;
; checks all the various conditions on the UART

handler dw	ChkMSR, ChkXmit, RcvChar, InputLSR

Interrupt proc	near
	push	di
	push	cx
	push	bx
	mov	bx,offset DGROUP:ports
	mov	cl,al			; source irq
;--------------------------------------------------------------------
; poll all the ports looking for activity on this irq
;--------------------------------------------------------------------
int1:
	mov	dx,[bx].port
	or	dx,dx
	jz	int2
	cmp	cl,[bx].irq
	jne	int2
	add	dx,IIR-DATA_REG
	input	al,dx
	test	al,01h
	jz	int3
int2:
	add	bx,type portrec
	cmp	bx,offset DGROUP:endports
	jb	int1
	stc				; flag no events found for any port
	pop	bx
	pop	cx
	pop	di
	ret
int3:
;--------------------------------------------------------------------
; if found, collect all the events for all the ports on this irq
;--------------------------------------------------------------------
	mov	ah,0
	mov	di,ax
	call	handler[di]
int4:
	mov	dx,[bx].port
	add	dx,IIR-DATA_REG
	input	al,dx
	test	al,01h
	jz	int3
	mov	cl,[bx].irq
int5:
	add	bx,type portrec
	cmp	bx,offset DGROUP:endports
	jnb	int7
int6:
	cmp	[bx].port,0
	je	int5
	cmp	cl,[bx].irq
	jne	int5
	jmp	int4
int7:
	pop	bx
	pop	cx
	pop	di
	clc
	ret
Interrupt endp



;--------------------------------------------------------------------
; there should be MAX_PORTS instances of the following
	assume	cs:SERIAL_TEXT,ds:nothing,es:nothing,ss:nothing



;--------------------------------------------------------------------
; first port

port1	label	near
	jmp	short port1b

prior1	dd	0
	dw	424bh
first1	db	0
	jmp	short reset1
	db	7 dup(0)
reset1	proc	far
	ret
reset1	endp

port1b:
	sti
	push	ds
	push	dx
	push	ax
	mov	ax,DGROUP
	mov	ds,ax
	assume	ds:DGROUP

;--------------------------------------------------------------------
; it was our interrupt
;--------------------------------------------------------------------
	mov	al,[ports].irq
	call	Interrupt
	jnc	port1e
	test	first1,80h
	jnz	port1e
port1d:
	pop	ax
	pop	dx
	pop	ds
	jmp	prior1
port1e:
	mov	al,20h			; EOI - end of interrupt
	cmp	[ports].irq,8
	jb	port1f
	output	0a0h,al 		; slave 8259 first
port1f:
	output	20h,al
	pop	ax
	pop	dx
	pop	ds
	iret
	assume	ds:nothing



;--------------------------------------------------------------------
; second port

port2	label	near
	jmp	short port2b

prior2	dd	0
	dw	424bh
first2	db	0
	jmp	short reset2
	db	7 dup(0)
reset2	proc	far
	ret
reset2	endp

port2b:
	sti
	push	ds
	push	dx
	push	ax
	mov	ax,DGROUP
	mov	ds,ax
	assume	ds:DGROUP

;--------------------------------------------------------------------
; it was our interrupt
;--------------------------------------------------------------------
	mov	al,[ports+type portrec].irq
	call	Interrupt
	jnc	port2e
	test	first2,80h
	jnz	port2e
port2d:
	pop	ax
	pop	dx
	pop	ds
	jmp	prior2
port2e:
	mov	al,20h			; EOI - end of interrupt
	cmp	[ports+type portrec].irq,8
	jb	port2f
	output	0a0h,al 		; slave 8259 first
port2f:
	output	20h,al
	pop	ax
	pop	dx
	pop	ds
	iret
	assume	ds:nothing



;--------------------------------------------------------------------
; third port

port3	label	near
	jmp	short port3b

prior3	dd	0
	dw	424bh
first3	db	0
	jmp	short reset3
	db	7 dup(0)
reset3	proc	far
	ret
reset3	endp

port3b:
	sti
	push	ds
	push	dx
	push	ax
	mov	ax,DGROUP
	mov	ds,ax
	assume	ds:DGROUP

;--------------------------------------------------------------------
; it was our interrupt
;--------------------------------------------------------------------
	mov	al,[ports+2*type portrec].irq
	call	Interrupt
	jnc	port3e
	test	first3,80h
	jnz	port3e
port3d:
	pop	ax
	pop	dx
	pop	ds
	jmp	prior3
port3e:
	mov	al,20h			; EOI - end of interrupt
	cmp	[ports+2*type portrec].irq,8
	jb	port3f
	output	0a0h,al 		; slave 8259 first
port3f:
	output	20h,al
	pop	ax
	pop	dx
	pop	ds
	iret
	assume	ds:nothing



;--------------------------------------------------------------------
; fourth port

port4	label	near
	jmp	short port4b

prior4	dd	0
	dw	424bh
first4	db	0
	jmp	short reset4
	db	7 dup(0)
reset4	proc	far
	ret
reset4	endp

port4b:
	sti
	push	ds
	push	dx
	push	ax
	mov	ax,DGROUP
	mov	ds,ax
	assume	ds:DGROUP

;--------------------------------------------------------------------
; it was our interrupt
;--------------------------------------------------------------------
	mov	al,[ports+3*type portrec].irq
	call	Interrupt
	jnc	port4e
	test	first4,80h
	jnz	port4e
port4d:
	pop	ax
	pop	dx
	pop	ds
	jmp	prior4
port4e:
	mov	al,20h			; EOI - end of interrupt
	cmp	[ports+3*type portrec].irq,8
	jb	port4f
	output	0a0h,al 		; slave 8259 first
port4f:
	output	20h,al
	pop	ax
	pop	dx
	pop	ds
	iret
	assume	ds:nothing



SERIAL_TEXT ends



	end
