/*
============================================================================
	CFMENU.PRO

	Certainty Factors Menu

Implements a popup-menu in which you can specify a rating for each choice
The Up and Down arrow keys, PgUp, PgDn, Home, and End can be used to
move the selector.
The Left and Right arrow keys are used to change the rating for each choice.
F10 will end the menu.
Esc aborts menu selection and make the predicate fail.

The menu will scroll more than DISPLAYROWS choices are provided.

 The arguments to certainmenu are:

   menu(ROW,COL,DISPLAYROWS,WINDOWATTR,FRAMEATTR,
               STRINGLIST,HEADER,STARTCHOICE,SELECTION)

   ROW and COL determine the position of the window
   WATTR and FATTR determine the attributes for the window
   	and its frame - if FATTR is zero there
   	will be no frame around the window.
   DISPLAYROWS determines how many rows should be displayed on the screen
   STRINGLIST is the list of items on the menu
   HEADER is the text to appear in the top of the menu window
   STARTCHOICE determines where the bar should be placed.

  Ex: certainmenu(5,5,7,7,5,["high","medium","low"],
				"Did the component emit heat?",0,CHOICE)


	Copyright (C) 1990 Prolog Development Center
============================================================================
*/

include "tdoms.pro"
include "tpreds.pro"

PREDICATES
ifndef globdecl
  certainmenu(ROW,COL,ROW,ATTR,ATTR,STRINGLIST,STRING,INTEGER,INTEGERLIST)
enddef

  certainmenuinit(ROW,COL,ROW,ATTR,ATTR,STRINGLIST,STRING,ROW,COL,ROW,COL,ROW)
  certainmenu1(KEY,ROW,COL,ROW,COL,ATTR,STRINGLIST,ROW,ROW,ROW,ROW,INTEGERLIST,INTEGERLIST)
  certainmenu2(KEY,KEY,ROW,ROW,ROW,ROW,ROW,ROW,INTEGERLIST,INTEGERLIST) 
  certainmenu3(ROW,ROW,ROW,ROW,ROW,ROW) 
  wr_part_if_changed(ROW,ROW,ROW,COL,STRINGLIST,INTEGERLIST)
  write_part_list(ROW,ROW,ROW,ROW,COL,STRINGLIST,INTEGERLIST)
  write_cval(ROW,INTEGER)

  replacecertainty(ROW,INTEGER,INTEGERLIST,INTEGERLIST)
  replacecertainty1(ROW,INTEGER,INTEGERLIST,INTEGERLIST)
  leftright(ROW,KEY,KEY,INTEGERLIST,INTEGERLIST)
  leftright1(KEY,KEY)
  leftright2(KEY)
  gen_zero_list(INTEGER,INTEGERLIST)
  placecursor(ROW,ROW,INTEGERLIST)
  getcol(ROW,INTEGERLIST,COL)
  testreadkey(KEY,KEY)

CLAUSES
  certainmenu(ROW,COL,MAXH,WATTR,FATTR,STRINGLIST,HEADER,STCHOICE,OUTLIST):-
	certainmenuinit(ROW,COL,MAXH,WATTR,FATTR,STRINGLIST,HEADER,AROW,ACOL,HEIGHT,LEN,NOOFROW),
	STOFFSET=STCHOICE-1,
	certainmenu3(NOOFROW,HEIGHT,0,STOFFSET,BASE,OFFSET),
	N=NOOFROW,
	gen_zero_list(N,INLIST),
	certainmenu1(fkey(999),AROW,ACOL,HEIGHT,LEN,WATTR,STRINGLIST,NOOFROW,-1,BASE,OFFSET,INLIST,OUTLIST),
	removewindow,
	removewindow.

  certainmenuinit(ROW,COL,MAXH,WATTR,FATTR,STRINGLIST,HEADER,AROW,ACOL,HEIGHT,NOOFCOL,NOOFROW):-
	maxlen(STRINGLIST,0,MAXSTRLEN),MAXNOOFCOL=MAXSTRLEN+22,
	str_len(HEADER,HEADLEN),
	HEADL1=HEADLEN+4,
	max(HEADL1,MAXNOOFCOL,NOOFCOL),
	listlen(STRINGLIST,N), N > 0 , NOOFROW=N,
	min(NOOFROW,MAXH,HEIGHT),
	adjframe(FATTR,HEIGHT,NOOFCOL,HH1,HH2),
	adjustwindow(ROW,COL,HH1,HH2,AROW,ACOL),
	makewindow(82,WATTR,0,"",AROW,ACOL,HH1,HH2),
	makewindow(81,WATTR,FATTR,"",AROW,ACOL,HH1,HH2),
	gotowindow(82),
	HHH1=HH1-1,
	field_str(HHH1,3,2,"NO"),field_str(HHH1,17,3,"YES"),
	str_len(HEADER,HEADLEN),
	field_str(0,2,HEADLEN,HEADER),
	gotowindow(81).

  certainmenu1(esc,_,_,_,_,_,_,_,_,_,_,_,[]):-!.
  certainmenu1(fkey(10),_,_,_,_,_,_,_,_,_,_,LIST,LIST):-!.
  certainmenu1(TESTKEY,ROW,COL,H,W,ATTR,STRINGLIST,NOOFROW,OLDBASE,BASE,OFFS,INLIST,OUTLIST):-!,
	wr_part_if_changed(OLDBASE,BASE,H,W,STRINGLIST,INLIST),
	INDEX=BASE+OFFS,
	placecursor(OFFS,INDEX,INLIST),
	testreadkey(TESTKEY,KEY),
	certainmenu2(KEY,NEXTKEY,H,NOOFROW,BASE,OFFS,BASE1,OFFS1,INLIST,OUTLIST1),
	certainmenu1(NEXTKEY,ROW,COL,H,W,ATTR,STRINGLIST,NOOFROW,BASE,BASE1,OFFS1,OUTLIST1,OUTLIST). 

  testreadkey(fkey(999),KEY):-!,readkey(KEY).
  testreadkey(KEY,KEY).

  certainmenu2(fkey(10),fkey(10),_,_,B,O,B,O,LIST,LIST):-!.
  certainmenu2(esc,esc,_,_,_,_,0,0,_,[]):-!.
  certainmenu2(home,fkey(999),_,_,_,_,0,0,LIST,LIST):-!.
  certainmenu2(end,fkey(999),H,LEN,_,_,B1,O1,LIST,LIST):-!,
	O1=H-1,B1=LEN-H.
  certainmenu2(up,fkey(999),H,LEN,B,O,B1,O1,LIST,LIST):-!,
	OO=O-1,certainmenu3(LEN,H,B,OO,B1,O1).
  certainmenu2(down,fkey(999),H,LEN,B,O,B1,O1,LIST,LIST):-!,
	OO=O+1,certainmenu3(LEN,H,B,OO,B1,O1).
  certainmenu2(pgup,fkey(999),H,LEN,B,O,B1,O1,LIST,LIST):-!,
	OO=O-H+1,certainmenu3(LEN,H,B,OO,B1,O1).
  certainmenu2(pgdn,fkey(999),H,LEN,B,O,B1,O1,LIST,LIST):-!,
	OO=O+H-1,certainmenu3(LEN,H,B,OO,B1,O1).
  certainmenu2(left,OUTKEY,_,_,B,O,B,O,INLIST,OUTLIST):-!,
	ROW=B+O,leftright(ROW,left,OUTKEY,INLIST,OUTLIST).
  certainmenu2(right,OUTKEY,_,_,B,O,B,O,INLIST,OUTLIST):-!,
	ROW=B+O,leftright(ROW,right,OUTKEY,INLIST,OUTLIST).
  certainmenu2(_,KEY,_,_,B,O,B,O,LIST,LIST):-readkey(KEY).

  certainmenu3(_,H,B,O,B,O):-O>=0, O<H, !.
  certainmenu3(LEN,H,B1,O1,B2,O2):-O1+B1>=LEN,!, O2=H-1, B2=LEN-H.
  certainmenu3(_,_,B1,O1,0,0):-O1+B1<0,!.
  certainmenu3(_,_,B1,O1,B2,0):-O1<0, !, B2=B1+O1.
  certainmenu3(_,H,B1,O1,B2,O2):-O1>=H, O2=H-1, B2=B1+O1-O2.

  wr_part_if_changed(B,B,_,_,_,_)  :- !.
  wr_part_if_changed(OLDB,B,H,W,STRL,CERTAINL) :-
	SCROLL=B-OLDB, scroll(SCROLL,0),
	write_part_list(0,B,0,H,W,STRL,CERTAINL).

  write_part_list(_,_,H,H,_,_,_) :- !.
  write_part_list(I,B,R,H,W,[_|T],[_|T1]) :- I<B,!, I1=I+1,
		write_part_list(I1,B,R,H,W,T,T1).
  write_part_list(I,B,R,H,W,[STR|T],[CVAL|T1]):-
		LEN=W-22,
		field_str(R,22,LEN,STR),
		write_cval(R,CVAL),
		scr_char(R,10,''),
		R1=R+1,
		write_part_list(I,B,R1,H,W,T,T1).

  write_cval(_,0):-!.
  write_cval(ROW,CVAL):-CVAL>0,!,
	COL=CVAL+10,
	scr_char(ROW,COL,''),
	CVAL1=CVAL-1,
	write_cval(ROW,CVAL1).
  write_cval(ROW,CVAL):-
	COL=CVAL+10,
	scr_char(ROW,COL,''),
	CVAL1=CVAL+1,
	write_cval(ROW,CVAL1).

  placecursor(ROW,INDEX,LIST):-
	getcol(INDEX,LIST,COL),
	cursor(ROW,COL).

  getcol(0,[N|_],COL):-!,COL=N+10.
  getcol(N,[_|T],COL):-N1=N-1,getcol(N1,T,COL).

  leftright(ROW,INKEY,OUTKEY,INLIST,OUTLIST):-
	leftright1(INKEY,OUTKEY),
	cursor(_,COL),NEWcertainty=COL-10,
	replacecertainty(ROW,NEWcertainty,INLIST,OUTLIST).

  leftright1(left,OUTKEY):-
	cursor(R,COL),COL>10,!,COL1=COL-1,
	cursor(R,COL1),scr_char(R,COL,' '),
	leftright2(OUTKEY).
  leftright1(left,OUTKEY):-
	cursor(_,COL),COL=0,!,leftright2(OUTKEY).
  leftright1(left,OUTKEY):-!,
	cursor(R,COL),COL1=COL-1,
	cursor(R,COL1),scr_char(R,COL1,''),
	leftright2(OUTKEY).
  leftright1(right,OUTKEY):-
	cursor(R,COL),COL<10,!,COL1=COL+1,
	cursor(R,COL1),scr_char(R,COL,' '),
	leftright2(OUTKEY).
  leftright1(right,OUTKEY):-
	cursor(_,COL),COL>=20,!,leftright2(OUTKEY).
  leftright1(right,OUTKEY):-!,
	cursor(R,COL),COL1=COL+1,
	cursor(R,COL1),scr_char(R,COL1,''),
	leftright2(OUTKEY).
  leftright1(KEY,KEY).

  leftright2(OUTKEY):-
	readkey(KEY),
	leftright1(KEY,OUTKEY).

  replacecertainty(ROW,NEWcertainty,INLIST,OUTLIST):-
	replacecertainty1(ROW,NEWcertainty,INLIST,OUTLIST),!.
  replacecertainty(_,_,LIST,LIST). % Dont waste gstack if equal 

  replacecertainty1(0,NEWcertainty,[H|T],[NEWcertainty|T]):-!,
	H><NEWcertainty.
  replacecertainty1(ROW,NEWcertainty,[H|T],[H|T1]):-
	ROW1=ROW-1,
	replacecertainty1(ROW1,NEWcertainty,T,T1).

  gen_zero_list(0,[]):-!.
  gen_zero_list(N,[0|L]):-
  	N1=N-1,
  	gen_zero_list(N1,L).

                       
