/*
===========================================================================
	FILENAME.PRO

	Read filename, extended read filename
		
        readfilename(ROW,COL,ATTR,ATTR,EXTENSION,OLDFILENAME,NEWFILENAME)
	readfilename(ROW,COL,ATTR,ATTR,EXTENSION,OLDFILENAME,
							NEWFILENAME,EXIST)
           

  	Copyright (c) 1990 Prolog Development Center
===========================================================================
*/


include "tdoms.pro"
include "tpreds.pro"
include "ui\\lineinp.pro"


CONSTANTS
 new_file	= 0
 existing_file  = 1
 dont_care	= 2

PREDICATES
ifndef globdecl
  readfilename(ROW,COL,ATTR,ATTR,STRING,STRING,STRING)
  readfilename(ROW,COL,ATTR,ATTR,STRING,STRING,STRING,INTEGER)	
  newext(string,string,string)
enddef
  readfilename1(ROW,COL,ATTR,ATTR,STRING,STRING,STRING)
  readfilename2(ROW,COL,ATTR,ATTR,STRING,STRING)
  default_if_no_ext(STRING, STRING, STRING)
  requires_browser(STRING,STRING,STRING)
  matches_constraint(INTEGER,STRING)
  only_yn(char)
  
CLAUSES

  readfilename(Row,Col,Wattr,Fattr,Ext,Old,Filename):-
	  !,readfilename(Row,Col,Wattr,Fattr,Ext,Old,Filename,dont_care).
	  
  readfilename(Row,Col,Wattr,Fattr,Ext,Old,Filename,Exist):-
	upper_lower(OldFile,Old),
	upper_lower(Extent, Ext),
	format(Text, "File name (.%): ", Extent),
	lineinput_repeat(Row,Col,40,Wattr,Fattr,Text,OldFile,Name),
	readfilename1(Row,Col,Wattr,Fattr,Ext,Name,Fname),
	default_if_no_ext(Fname,Extent, NameExt),	
	filenamepath(Filename, "", Nameext), % adds full path, as from dir/3
	matches_constraint(Exist,Filename),!,
	removewindow.

	
  readfilename1(Row,Col,Wattr,Fattr,Ext,InputName,Filename):-
	requires_browser(InputName,Ext,Mask),!,
	Row2=Row+3,
	readfilename2(Row2,Col,Wattr,Fattr,Mask,Filename).
  readfilename1(_,_,_,_,_,Name,Name).

  readfilename2(Row,Col,Wattr,Fattr,Mask,Filename):-
	adjustwindow(Row,Col,10,50,Row1,Col1),
	makewindow(81,Wattr,Fattr,"",Row1,Col1,10,50),
	stlnstatus(off),
	dir("",Mask,Filename),!,
	removewindow.
  readfilename2(_,_,_,_,_,_):-
  	removewindow,
  	fail.

  default_if_no_ext(Filename, DefaultExt, NewName) :-  % user typed name
  	filenameext(Filename, _, Extent),	       % with no extension	
  	Extent="",				       % so add default	
  	!,
	frontchar(DefaultExtent, '.', DefaultExt),
	filenameext(NewName, Filename, DefaultExtent).
  default_if_no_ext(Filename, _, Filename).


  requires_browser("",Ext,Mask):-!,   % if empty string,
  	concat("*.",Ext,Mask).	      %  use *.<default>	
  requires_browser(File,_,File) :-    % else if it has wildcard characters	
  	searchchar(File,'*',_),! ;    %  use as is	
  	searchchar(File,'?',_).
  
  
  matches_constraint(existing_file, Filename) :-
	not(existfile(Filename)),!,
  	expandwindow,
  	cursor(2,21),
  	write(" File Not Found! "),
  	readchar(_),
  	compresswindow,
	!,
	fail.
  matches_constraint(new_file, Filename) :-
  	existfile(Filename),
	expandwindow,
	cursor(2,2),
	write(" File already exists. Overwrite(y/n) "),
	only_yn(Response),
	compresswindow,
	!,
	Response='y'.
  matches_constraint(_,_).


  only_yn(Response) :-
  	readchar(Char),
  	upper_lower(Char,Response),
  	searchchar("yn\027",Response,_),!.
  only_yn(Resp) :-
  	beep, 
  	only_yn(Resp).
  	
  newext(Filename, Ext, NewFilename) :-
	frontchar(EXTENT, '.', Ext),
	filenameext(Newfilename, Filename, Extent).

 /***********************************************************************/
 /*		SET/CHANGE DIRECTORY	   				*/
 /***********************************************************************/

PREDICATES
ifndef globdecl
  setdir(ROW,COL,ATTR,ATTR)
enddef
  newdisk(STRING)

CLAUSES
  setdir(ROW,COL,WATTR,FATTR):-
	disk(DISK),
	lineinput_repeat(ROW,COL,45,WATTR,FATTR,"Directory: ",DISK,NEWDISK),
	newdisk(NEWDISK),!,
	removewindow.
  setdir(_,_,_,_).

  newdisk(DISK):-
	trap(disk(DISK),_,fail),!.
  newdisk(_):-	
	makewindow(1,7,7,"",10,30,4,45),
	write(">> Error in directory name"),nl,
	write("   - Press any key"),
	readkey(_),
	removewindow,
	fail.


