/*
============================================================================
	LONGMENU.PRO

 	Implements a popup-menu with more than 23 possible choices
	
  The up and down arrow keys and pgup, pgdn, home, and end  can be used to
  move the bar.
  RETURN or F10 will select an indicated item.
  Pressing Esc aborts menu selection and returns a zero.

  The arguments to longmenu are:
  menu(ROW,COL,MAXROWS,WINDOWATTR,FRAMEATTR,
		STRINGLIST,HEADER,STARTCHOICE,SELECTION)

	ROW and COL determine the position of the window
	WATTR and FATTR determine the attributes for the window
		and its frame - if FATTR is zero there
		will be no frame around the window.
	MAXROWS determines how many rows should be displayed on the screen
	STRINGLIST is the list of items on the menu
	HEADER is the text to appear in the top of the menu window
	STARTCHOICE determines where the bar should be placed.

  Ex:	 longmenu(5,5,7,7,5,[a,b,c,d,e,f,g,h,i,j,k,l,m,n],letters,0,CHOICE)

	Copyright (C) 1990 Prolog Development Center
============================================================================
*/

include "tdoms.pro"
include "tpreds.pro"

PREDICATES
ifndef globdecl
  longmenu(ROW,COL,ROW,ATTR,ATTR,STRINGLIST,STRING,INTEGER,INTEGER)
enddef
  longmenuinit(ROW,COL,ROW,ATTR,ATTR,STRINGLIST,STRING,ROW,COL,ROW,COL,ROW)
  longmenu1(SYMBOL,ROW,COL,ROW,COL,ATTR,STRINGLIST,ROW,ROW,ROW,ROW,ROW,ROW)
  longmenu2(SYMBOL,ROW,ROW,STRINGLIST,ROW,ROW,ROW,ROW,KEY) 
  longmenu3(ROW,ROW,ROW,ROW,ROW,ROW) 
  longmenu4(INTEGER,ROW,ROW,ROW,ROW)
  longmenu_wr_if_chng(ROW,ROW,ROW,COL,STRINGLIST)
  longmenu_wrlist(ROW,ROW,ROW,ROW,COL,STRINGLIST)

CLAUSES
  longmenu(ROW,COL,MAXH,WATTR,FATTR,STRINGLIST,HEADER,STCHOICE,CHOICE) :-
	longmenuinit(ROW,COL,MAXH,WATTR,FATTR,STRINGLIST,HEADER,AROW,ACOL,HEIGHT,LEN,NOOFROW),
	STOFFSET=STCHOICE-1,
	longmenu3(NOOFROW,HEIGHT,0,STOFFSET,BASE,OFFSET),
	longmenu1(cont,AROW,ACOL,HEIGHT,LEN,WATTR,STRINGLIST,NOOFROW,-1,BASE,OFFSET,BASE1,OFFS1),
	CHOICE = BASE1 + OFFS1+1,
	removewindow.

  longmenuinit(ROW,COL,MAXH,WATTR,FATTR,STRINGLIST,HEADER,AROW,ACOL,HEIGHT,NOOFCOL,NOOFROW):-
	maxlen(STRINGLIST,0,MAXNOOFCOL),
	str_len(HEADER,HEADLEN),
	HEADL1=HEADLEN+4,
	max(HEADL1,MAXNOOFCOL,NOOFCOL),
	listlen(STRINGLIST,N), N > 0 , NOOFROW=N,
	min(NOOFROW,MAXH,HEIGHT),
	adjframe(FATTR,HEIGHT,NOOFCOL,HH1,HH2),
	adjustwindow(ROW,COL,HH1,HH2,AROW,ACOL),
	makewindow(81,WATTR,FATTR,HEADER,AROW,ACOL,HH1,HH2).

  longmenu1(cont,ROW,COL,H,W,ATTR,STRINGLIST,NOOFROW,OLDBASE,BASE,OFFS,BASE2,OFFS2) :-!,
	longmenu_wr_if_chng(OLDBASE,BASE,H,W,STRINGLIST),
	reverseattr(ATTR,REV),
	field_attr(OFFS,0,W,REV),
	cursor(OFFS,0),
	readkey(KEY),
	longmenu2(STOP,H,NOOFROW,STRINGLIST,BASE,OFFS,BASE1,OFFS1,KEY),
	field_attr(OFFS,0,W,ATTR),
	longmenu1(STOP,ROW,COL,H,W,ATTR,STRINGLIST,NOOFROW,BASE,BASE1,OFFS1,BASE2,OFFS2). 
  longmenu1(esc,_,_,_,_,_,_,_,_,OB,OF,OB,OF):-!.
  longmenu1(_,_,_,H,W,ATTR,STRINGLIST,_,OLDBASE,OB,OF,OB,OF):-
	longmenu_wr_if_chng(OLDBASE,OB,H,W,STRINGLIST),
	reverseattr(ATTR,REV),
	field_attr(OF,0,W,REV).


  longmenu2(esc,_,_,_,_,_,0,-1,esc)	:-!.
  longmenu2(stop,_,_,_,B,O,B,O,fkey(10)):-!.
ifdef help_inuse
  longmenu2(cont,_,_,_,B,O,B,O,fkey(1))	:-!,help.
enddef
  longmenu2(selection,_,_,_,B,O,B,O,cr)	:-!.
  longmenu2(selection,H,_,LIST,B,_,B1,O1,char(C)):-
	tryletter(C,LIST,CH),!,longmenu4(CH,H,B,B1,O1).
  longmenu2(cont,_,_,_,_,_,0,0,home)	:-!.
  longmenu2(cont,H,LEN,_,_,_,B1,O1,end)	:-!, O1=H-1, B1 = LEN-H. 
  longmenu2(cont,H,LEN,_,B,O,B1,O1,up)	:-!, OO=O-1,longmenu3(LEN,H,B,OO,B1,O1).
  longmenu2(cont,H,LEN,_,B,O,B1,O1,down):-!, OO=O+1,longmenu3(LEN,H,B,OO,B1,O1).
  longmenu2(cont,H,LEN,_,B,O,B1,O1,pgup):-!, OO=O-H+1,longmenu3(LEN,H,B,OO,B1,O1).
  longmenu2(cont,H,LEN,_,B,O,B1,O1,pgdn):-!, OO=O+H-1,longmenu3(LEN,H,B,OO,B1,O1).
  longmenu2(cont,_,_,_,B,O,B,O,_).

  longmenu3(_,H,B,O,B,O)	:-O>=0, O<H, !.
  longmenu3(LEN,H,B1,O1,B2,O2)	:-O1+B1>=LEN,!, O2=H-1, B2=LEN-H.
  longmenu3(_,_,B1,O1,0,0)	:-O1+B1<0,!.
  longmenu3(_,_,B1,O1,B2,0)	:-O1<0, !, B2=B1+O1.
  longmenu3(_,H,B1,O1,B2,O2)	:-O1>=H, O2=H-1, B2=B1+O1-O2.

  longmenu4(CH,_,B,CH,0)	:-CH<B,!.
  longmenu4(CH,H,B,B1,O1)	:-CH>=B+H,!,O1=H-1,B1=CH-O1.
  longmenu4(CH,_,B,B,O)		:-O=CH-B.

  longmenu_wr_if_chng(B,B,_,_,_)  :- !.
  longmenu_wr_if_chng(OLDB,B,H,W,CL) :-
	SCROLL=B-OLDB, scroll(SCROLL,0),
	longmenu_wrlist(0,B,0,H,W,CL).
      
  longmenu_wrlist(_,_,H,H,_,_) :- !.
  longmenu_wrlist(I,B,R,H,W,[_|T]) :- I<B,!, I1=I+1,
		longmenu_wrlist(I1,B,R,H,W,T).
  longmenu_wrlist(I,B,R,H,W,[STR|T]) :-
		field_str(R,0,W,STR),
		R1=R+1,
		longmenu_wrlist(I,B,R1,H,W,T).


/****************************************************************/
/* 			longmenu_leave				*/
/* As longmenu but the window is not removed on return		*/
/****************************************************************/

ifndef globdecl
PREDICATES
  longmenu_leave(ROW,COL,ROW,ATTR,ATTR,STRINGLIST,STRING,INTEGER,INTEGER)
enddef

CLAUSES
  longmenu_leave(ROW,COL,MAXH,WATTR,FATTR,STRINGLIST,HEADER,STCHOICE,CHOICE):-
	longmenuinit(ROW,COL,MAXH,WATTR,FATTR,STRINGLIST,HEADER,AROW,ACOL,HEIGHT,LEN,NOOFROW),
	STOFFSET=STCHOICE-1,
	longmenu3(NOOFROW,HEIGHT,0,STOFFSET,BASE,OFFSET),
	longmenu1(cont,AROW,ACOL,HEIGHT,LEN,WATTR,STRINGLIST,NOOFROW,-1,BASE,OFFSET,BASE1,OFFS1),
	CHOICE = BASE1 + OFFS1+1.


/****************************************************************/
/* 			longmenu_repeat				*/
/****************************************************************/

PREDICATES
ifndef globdecl
  nondeterm longmenu_repeat(ROW,COL,ROW,ATTR,ATTR,STRINGLIST,STRING,INTEGER,INTEGER)
enddef
  nondeterm longmenu_repeat1(ROW,COL,ROW,COL,ATTR,STRINGLIST,ROW,ROW,ROW,ROW,INTEGER)
  nondeterm longmenu_repeat2(ROW,COL,ROW,COL,ATTR,STRINGLIST,ROW,ROW,ROW,ROW,INTEGER,INTEGER)

CLAUSES
  longmenu_repeat(ROW,COL,MAXH,WATTR,FATTR,STRINGLIST,HEADER,STCHOICE,CHOICE):-
	longmenuinit(ROW,COL,MAXH,WATTR,FATTR,STRINGLIST,HEADER,AROW,ACOL,HEIGHT,LEN,NOOFROW),
	STOFFSET=STCHOICE-1,
	longmenu3(NOOFROW,HEIGHT,0,STOFFSET,BASE,OFFSET),
	longmenu_repeat1(AROW,ACOL,HEIGHT,LEN,WATTR,STRINGLIST,NOOFROW,-1,BASE,OFFSET,CHOICE).
  longmenu_repeat(_,_,_,_,_,_,_,_,_):-removewindow,fail.

  longmenu_repeat1(AROW,ACOL,HEIGHT,LEN,WATTR,STRINGLIST,NOOFROW,OLDBASE,BASE,OFFSET,CHOICE):-
	longmenu1(cont,AROW,ACOL,HEIGHT,LEN,WATTR,STRINGLIST,NOOFROW,OLDBASE,BASE,OFFSET,BASE1,OFFS1),
	CH = BASE1 + OFFS1+1,
	longmenu_repeat2(AROW,ACOL,HEIGHT,LEN,WATTR,STRINGLIST,NOOFROW,BASE,BASE1,OFFS1,CH,CHOICE).

  longmenu_repeat2(_,_,_,_,_,_,_,_,_,_,CH,CH):-CH><0.
  longmenu_repeat2(AROW,ACOL,HEIGHT,LEN,WATTR,STRINGLIST,NOOFROW,OLDB,BASE,OFFS,CH,CH1):-
	CH><0,
	longmenu_repeat1(AROW,ACOL,HEIGHT,LEN,WATTR,STRINGLIST,NOOFROW,OLDB,BASE,OFFS,CH1).



/****************************************************************/
/* 			longmenu_mult				*/
/* Allows a multiple number of selections.			*/
/*								*/
/* Each selection is made by pressing RETURN. All selections    */
/* then activated by pressing F10.				*/
/****************************************************************/

PREDICATES
ifndef globdecl
  longmenu_mult(ROW,COL,ROW,ATTR,ATTR,STRINGLIST,STRING,INTEGERLIST,INTEGERLIST)
enddef
  multlongmenu1(SYMBOL,ROW,COL,ROW,COL,ATTR,STRINGLIST,ROW,ROW,ROW,ROW,INTEGERLIST,INTEGERLIST)
  longmenu_highlight(ROW,ROW,COL,INTEGERLIST,ATTR)
  longmenu_selection(INTEGER,INTEGERLIST,INTEGERLIST,ROW,ROW,COL,ATTR)
  longmenu_trydel(INTEGER,INTEGERLIST,INTEGERLIST,ROW,ROW,COL,ATTR)
  longmenu_rowattr(ROW,ROW,COL,ATTR)

CLAUSES
  longmenu_mult(ROW,COL,MAXH,WATTR,FATTR,STRINGLIST,HEADER,STARTCH,CHOOSED) :-
	longmenuinit(ROW,COL,MAXH,WATTR,FATTR,STRINGLIST,HEADER,AROW,ACOL,HEIGHT,LEN,NOOFROW),
	multlongmenu1(cont,AROW,ACOL,HEIGHT,LEN,WATTR,STRINGLIST,NOOFROW,-1,0,0,STARTCH,CHOOSED),
	removewindow.

  multlongmenu1(stop,_,_,_,_,_,_,_,_,_,_,CHOOSED,CHOOSED).
  multlongmenu1(esc,_,_,_,_,_,_,_,_,_,_,_,[]).
  multlongmenu1(selection,ROW,COL,H,W,ATTR,STRINGLIST,NOOFROW,OLDBASE,BASE,OFFS,CHIN,CHOUT) :-
	SELECTION=BASE+OFFS+1,
	longmenu_selection(SELECTION,CHIN,NEWCHIN,BASE,NOOFROW,W,ATTR),
	multlongmenu1(cont,ROW,COL,H,W,ATTR,STRINGLIST,NOOFROW,OLDBASE,BASE,OFFS,NEWCHIN,CHOUT).
  multlongmenu1(cont,ROW,COL,H,W,ATTR,STRINGLIST,NOOFROW,OLDBASE,BASE,OFFS,CHIN,CHOUT) :-
	longmenu_wr_if_chng(OLDBASE,BASE,H,W,STRINGLIST),
	reverseattr(ATTR,REV),
	longmenu_highlight(BASE,H,W,CHIN,REV),
	cursor(OFFS,0),
	readkey(KEY),
	longmenu2(STOP,H,NOOFROW,STRINGLIST,BASE,OFFS,BASE1,OFFS1,KEY),
	multlongmenu1(STOP,ROW,COL,H,W,ATTR,STRINGLIST,NOOFROW,BASE,BASE1,OFFS1,CHIN,CHOUT). 


  longmenu_highlight(_,_,_,[],_).
  longmenu_highlight(BASE,MAXROW,LEN,[H|T],ATTR):-
	ROW=H-BASE-1,
	longmenu_rowattr(ROW,MAXROW,LEN,ATTR),
	longmenu_highlight(BASE,MAXROW,LEN,T,ATTR).

  longmenu_rowattr(ROW,MAXROW,LEN,ATTR):-
	ROW>=0, ROW<MAXROW,!,
	field_attr(ROW,0,LEN,ATTR).
  longmenu_rowattr(_,_,_,_).

  longmenu_trydel(SELECTION,[SELECTION|REST],REST,BASE,MAXROW,LEN,ATTR):-!,
	ROW=SELECTION-BASE-1,
	longmenu_rowattr(ROW,MAXROW,LEN,ATTR).
  longmenu_trydel(SELECTION,[H|REST],[H|REST1],BASE,MAXROW,LEN,ATTR):-
	longmenu_trydel(SELECTION,REST,REST1,BASE,MAXROW,LEN,ATTR).

  longmenu_selection(SELECTION,OLDCHIN,NEWCHIN,BASE,MAXROW,LEN,ATTR):-
	longmenu_trydel(SELECTION,OLDCHIN,NEWCHIN,BASE,MAXROW,LEN,ATTR),!.
  longmenu_selection(SELECTION,OLDCHIN,[SELECTION|OLDCHIN],_,_,_,_).
