/*
=============================================================================
	PULLDOWN.PRO
	
	Pulldown Menu predicates 
	 
	  pulldown/4 (old style)
	  
	  pulldown/5 (new version)
	
	
	Note: for pulldown/5, a pulldown curtain definition with no
	      selectable entries (eg, just bars and separators)	
	      will behave as a top-level choice with no pulldown defined	


	Copyright (C) 1990 Prolog Development Center
=============================================================================
*/

include"tdoms.pro"
include"tpreds.pro"


ifndef globdecl
CONSTANTS

%% comment this out if you don't need compatibility with
%%  the old pulldown/4

 support_oldstyle = 1
 

DOMAINS
topelem = 	curtain(string,pdwlist,attr); 
	   	elem(string,control,control)
toplist = 	topelem*


pdwelem = 	elem(string,control,control); 
		separator(string); 
		bar
pdwlist = 	pdwelem*

control = 	top_menu; 
		sub_menu; 
		quit

topelem_x = elem(string,col,single_or_curtain)
toplist_x = topelem_x*

single_or_curtain =
		single(control,control);
		curtain(pdwlist,attr,width,count)

width, count = 	integer

updown = 	up; down
stopgo = 	stop; go

ifdef support_oldstyle
  MENUELEM =  curtain(STRING,STRINGLIST)
  MENULIST =  MENUELEM*
enddef

enddef

DATABASE - pdwstate
determ pdwstate(row,count,updown)
stringlists(integer,stringlist)


PREDICATES

ifndef globdecl
ifdef support_oldstyle
 pulldown(attr, menulist, integer, integer)
enddef

pulldown(attr,attr,toplist,integer,integer)
pdwaction(integer,integer)
enddef

width_and_count(toplist,width,count,stringlist)
getword(topelem,string)
make_x_list(toplist,toplist_x,col,col,col,integer)
extraspace(col,col,col)
maxwidth(pdwlist,col,col,count,count,stringlist)
maketopmenu(toplist_x,attr)
pdwkeyact(key,row,row,count,updown,attr,toplist_x,count,stopgo)
trypdwaction(row,count,control,control,stopgo)
updatepdwstate(row,count,control,stopgo)
getpos(char,integer,integer)
newverpos(row,row,integer,count,pdwlist)
isleft_or_right(key,integer)
isup_or_down(key,integer)
getcontrol(single_or_curtain,control,control,row)
makepdwwindow(col,row,row,single_or_curtain)
writepdwlist(pdwlist,row,attr,width,row)
fillspace(string,count,row,width,col)
nextpos(integer,integer,integer,integer)
pdwmovehor(count,count,attr,toplist_x)
pdwmovever(row,row,width,attr)
findtopelem(count,toplist_x,topelem_x)
findpdwelem(row,pdwlist,pdwelem)


no_choices_in(pdwlist)
start_row(attr, row, integer)
line_ver(row,row,col)
line_hor(col,col,row)
left_corner(row,col,char)
right_corner(row,col,char)
intense(attr,attr)
intensefirstupper(row,col,attr,string)
intenseletter(row,col,attr,string)
changepdwstate(pdwstate)

ifdef support_oldstyle
 menulist_toplist(menulist,toplist,attr)
 strlist_pdwlist(stringlist, pdwlist)
enddef

CLAUSES

width_and_count([Elem|T1],Width,Count,[Word|T2]):-
	getword(Elem,Word),
	width_and_count(T1,W1,C1,T2),
	str_len(Word,Len),
	Width = W1+Len,
	Count = C1+1.
width_and_count([],0,0,[]).

getword(curtain(Word,_,_),Word).
getword(elem(Word,_,_),Word).


no_choices_in([]):-!.
no_choices_in([elem(_,_,_)|_]) :-!, fail.
no_choices_in([_|T]) :- no_choices_in(T).

make_x_list([curtain(Word,Pdwlist,_)|T],X_List,Curcol,Space,Extra,Index):- 
	no_choices_in(Pdwlist),
	!,
	make_x_list([elem(Word,top_menu,quit)|T],
			X_List,Curcol,Space,Extra,Index).
make_x_list([curtain(Word,Pdwlist,Attr)|T1],
	    [elem(Word,Col,curtain(Pdwlist,Attr,Width,Count))|T2],
	    Curcol,Space,Extra,Index):-
	extraspace(Extra,Nextra,Extraspace),
	Col = Curcol+Space+Extraspace,
	maxwidth(Pdwlist,W1,0,Count,0,Stringlist),
	Width = W1+2,
	assert(stringlists(Index,Stringlist)),
	str_len(Word,Len),
	Ncurcol = Col+Len,
	Nindex = Index+1,
	make_x_list(T1,T2,Ncurcol,Space,Nextra,Nindex).
make_x_list([elem(Word,Success_act,Fail_act)|T1],
	    [elem(Word,Col,single(Success_act,Fail_act))|T2],
	    Curcol,Space,Extra,Index):-
	extraspace(Extra,Nextra,Extraspace),
	Col = Curcol+Space+Extraspace,
	str_len(Word,Len),
	Ncurcol = Col+Len,
	Nindex = Index+1,
	make_x_list(T1,T2,Ncurcol,Space,Nextra,Nindex).
make_x_list([],[],_,_,_,_).


extraspace(0,0,0):- !.
extraspace(Extra,Nextra,1):- Nextra = Extra-1.


maxwidth([elem(Word,_,_)|T1],Max,Oldmax,Count,Oldcount,[Word|T2]):-
	str_len(Word,Len),
	Len >= Oldmax, !,
	C = Oldcount+1,
	maxwidth(T1,Max,Len,Count,C,T2).
maxwidth([elem(Word,_,_)|T1],Max,Oldmax,Count,Oldcount,[Word|T2]):- !,
	C = Oldcount+1,
	maxwidth(T1,Max,Oldmax,Count,C,T2).
maxwidth([_|T1],Max,Oldmax,Count,Oldcount,[""|T2]):-
	C = Oldcount+1,
	maxwidth(T1,Max,Oldmax,Count,C,T2).
maxwidth([],Width,Width,Count,Count,[]).


maketopmenu([elem(Word,Col,_)|T],Attr):-
	str_len(Word,Len),
	field_str(0,Col,Len,Word),
	intenseletter(0,Col,Attr,Word),
	maketopmenu(T,Attr).
maketopmenu([],_).


ifdef support_oldstyle
menulist_toplist([curtain(Top, [])|T],
	  	 [elem(Top,top_menu,quit)|T2], Attr) :-!,
	menulist_toplist(T,T2, Attr).	
menulist_toplist([curtain(Top, Choices)|T],
	  	 [curtain(Top, PdwList, Attr)|T2], Attr) :-
	strlist_pdwlist(Choices, PdwList),
	!, menulist_toplist(T, T2, Attr).	
menulist_toplist(_,[],_).

strlist_pdwlist([Choice|Tail],[elem(Choice, top_menu, quit)|Tail2]) :-
	!, strlist_pdwlist(Tail, Tail2).
strlist_pdwlist(_,[]).


pulldown(Attr, MenuList, Choice, Subchoice) :-
	menulist_toplist(MenuList, TopList, Attr),
	pulldown(Attr,Attr, Toplist, Choice, Subchoice).
enddef
	

pulldown(Attr,Fattr,Toplist,Choice,Subchoice):-
	textmode(_,Cols),
	width_and_count(Toplist,Width,Count,Topstringlist),
	start_row(Fattr, TopRow, TopSize),
	assert(stringlists(0,Topstringlist)),
	Space = (Cols-Width-2) div (Count+1),
	Extra = (Cols-Width-2) - (Count+1) * Space,
	make_x_list(Toplist,Toplist_x,0,Space,Extra,1),
	makewindow(81,Attr,FAttr,"",0,0,TopSize,Cols),
	maketopmenu(Toplist_x,Attr),
	changepdwstate(pdwstate(0,1,up)),
	pdwmovehor(1,1,Attr,Toplist_x),
	repeat,
	  pdwstate(Verpos,Horpos,Updown),
	  readkey(Key),
	  pdwkeyact(Key,TopRow,Verpos,Horpos,Updown,Attr,Toplist_x,Count,Stopgo),
	Stopgo = stop, !,
	removewindow(81,1),
	removewindow(91,1),
	pdwstate(Subchoice,Choice,_),
	retractall(pdwstate(_,_,_)),
	retractall(stringlists(_,_)).


pdwkeyact(Key,TopRow,_,Horpos,Updown,Attr,Toplist,Maxhorpos,go):-
	isleft_or_right(Key,Increment), !,
	removewindow(91,1),	% even though it may not be present
	nextpos(Horpos,Nhorpos,Increment,Maxhorpos),
	pdwmovehor(Horpos,Nhorpos,Attr,Toplist),
	changepdwstate(pdwstate(0,Nhorpos,Updown)),
	Updown = down,	% make new curtain only if previous was down
	pdwkeyact(down,TopRow,0,Nhorpos,up,0,Toplist,0,_).
pdwkeyact(down,TopRow,0,Horpos,_,_,Toplist,_,go):- !,
	findtopelem(Horpos,Toplist,elem(_,Col,Curtain)),
	makepdwwindow(Col,Nverpos,TopRow,Curtain),
	cursor(Nverpos,1),
	changepdwstate(pdwstate(Nverpos,Horpos,down)).
pdwkeyact(Key,_,Verpos,Horpos,down,_,Toplist,_,go):-
	isup_or_down(Key,Increment), !,
	findtopelem(Horpos,Toplist,elem(_,_,Curtain)),
	Curtain = curtain(Pdwlist,Attr,Width,Length),
	nextpos(Verpos,Verpos1,Increment,Length),
	newverpos(Verpos1,Nverpos,Increment,Length,Pdwlist),
	pdwmovever(Verpos,Nverpos,Width,Attr),
	changepdwstate(pdwstate(Nverpos,Horpos,down)).
pdwkeyact(cr,TopRow,0,Horpos,Updown,_,Toplist,_,go):-
	pdwkeyact(down,TopRow,0,Horpos,Updown,0,Toplist,0,_), !.
pdwkeyact(cr,_,Verpos,Horpos,_,_,Toplist,_,Stopgo):-
	findtopelem(Horpos,Toplist,elem(_,_,Element)),
	getcontrol(Element,Success_act,Fail_act,Verpos),
	trypdwaction(Verpos,Horpos,Success_act,Fail_act,Stopgo).
pdwkeyact(char(C),TopRow,_,Horpos,up,Attr,Toplist,_,Stopgo):- !,
	getpos(C,0,Nhorpos),
	pdwmovehor(Horpos,Nhorpos,Attr,Toplist),
	pdwkeyact(cr,TopRow,0,Nhorpos,up,0,Toplist,0,Stopgo).
pdwkeyact(char(C),TopRow,Verpos,Horpos,_,_,Toplist,_,Stopgo):-
	getpos(C,Horpos,Nverpos),
	findtopelem(Horpos,Toplist,elem(_,_,Curtain)),
	Curtain = curtain(_,Attr,Width,_),
	pdwmovever(Verpos,Nverpos,Width,Attr),
	pdwkeyact(cr,TopRow,Nverpos,Horpos,down,0,Toplist,0,Stopgo).
pdwkeyact(esc,_,_,Horpos,down,_,_,_,go):-
	removewindow(91,1),
	changepdwstate(pdwstate(0,Horpos,up)).


trypdwaction(Verpos,Horpos,Success_act,_,Stopgo):-
	pdwaction(Horpos,Verpos), !,
	updatepdwstate(Verpos,Horpos,Success_act,Stopgo).
trypdwaction(Verpos,Horpos,_,Fail_act,Stopgo):-
	updatepdwstate(Verpos,Horpos,Fail_act,Stopgo).

updatepdwstate(_,Horpos,top_menu,go):-
	removewindow(91,1),	% even though it may not be present
	changepdwstate(pdwstate(0,Horpos,up)).
updatepdwstate(Verpos,Horpos,sub_menu,go):-
	pdwstate(_,_,Updown),
	changepdwstate(pdwstate(Verpos,Horpos,Updown)).
updatepdwstate(Verpos,Horpos,quit,stop):-
	changepdwstate(pdwstate(Verpos,Horpos,up)).


getpos(C,Index,Pos):-
	stringlists(Index,Slist), !,
	tryletter(C,Slist,Pos1),
	Pos = Pos1+1.


newverpos(Verpos,Verpos,_,_,Pdwlist):-
	findpdwelem(Verpos,Pdwlist,elem(_,_,_)), !.
newverpos(Verpos,Nverpos,Increment,Length,Pdwlist):-
	nextpos(Verpos,Verpos1,Increment,Length),
	newverpos(Verpos1,Nverpos,Increment,Length,Pdwlist).


isleft_or_right(right,1).
isleft_or_right(left,-1).

isup_or_down(down,1).
isup_or_down(up,-1).


getcontrol(curtain(Pdwlist,_,_,_),Success_act,Fail_act,Verpos):-
	findpdwelem(Verpos,Pdwlist,elem(_,Success_act,Fail_act)).
getcontrol(single(Success_act,Fail_act),Success_act,Fail_act,_).


start_row(0, 1, 1):-!.
start_row(_, 2, 3).

makepdwwindow(Column,Verpos,TopRow,curtain(Pdwlist,Attr,Width,Length)):-
	Noofrows = Length+2, 
	Noofcols = Width+2,
	adjustwindow(TopRow,Column,Noofrows,Noofcols,Row,Col),
	reverseattr(Attr,Rev),
	makewindow(91,Attr,0,"",Row,Col,Noofrows,Noofcols),
	cursor(1,1),
	Endrow = Noofrows-1,
	Endcol = Noofcols-1,
	line_hor(1,Endcol,0), 
	line_hor(1,Endcol,Endrow),
	line_ver(1,Endrow,0), 
	line_ver(1,Endrow,Endcol),
	scr_char(Endrow,0,''),
	scr_char(Endrow,Endcol,''),
	left_corner(TopRow,Column,Lcorn), 
	scr_char(0,0,Lcorn),
	Rcol = Col+Endcol,
	right_corner(TopRow,Rcol,Rcorn), 
	scr_char(0,Endcol,Rcorn),
	Decapwidth = Width-1,
	writepdwlist(Pdwlist,1,Attr,Decapwidth,Verpos),
	field_attr(Verpos,1,Width,Rev).


writepdwlist([elem(Word,_,_)|T],Row,Attr,Width,Row):-
	field_str(Row,2,Width,Word),
	intenseletter(Row,2,Attr,Word),
	Nrow = Row+1,
	writepdwlist(T,Nrow,Attr,Width,_).
writepdwlist([separator(String)|T],Row,Attr,Width,Verpos):-
	Fillwidth = Width+1,
	str_len(String,Len),
	fillspace(String,Len,Row,Fillwidth,1),
	Endcol = Width+2,
	scr_char(Row,0,''),
	scr_char(Row,Endcol,''),
	Nrow = Row+1,
	writepdwlist(T,Nrow,Attr,Width,Verpos).
writepdwlist([bar|T],Row,Attr,Width,Verpos):-
	writepdwlist([separator("")|T],Row,Attr,Width,Verpos).
writepdwlist([],_,_,_,0).


fillspace(String,Len,Row,Width,Col):-
	field_str(Row,Col,Width,String),
	Nwidth = Width-Len,
	Nwidth > 0, !,
	Ncol = Col+Len,
	fillspace(String,Len,Row,Nwidth,Ncol).
fillspace(_,_,_,_,_).


nextpos(Pos,1,+1,Pos):-!.	/* Wrap right-to-left / bottom-to-top */
nextpos(1,Max,-1,Max):-!.	/* Wrap left-to-right / top-to-bottom */
nextpos(Pos,Npos,Increment,_):- Npos = Pos+Increment.


pdwmovehor(Horpos,Nhorpos,Attr,Toplist):-
	findtopelem(Horpos,Toplist,elem(Word1,Col1,_)),
	str_len(Word1,Len1),
	findtopelem(Nhorpos,Toplist,elem(Word2,Col2,_)),
	str_len(Word2,Len2),
	field_attr(0,Col1,Len1,Attr),
	intenseletter(0,Col1,Attr,Word1),
	reverseattr(Attr,Rev),
	field_attr(0,Col2,Len2,Rev),
	cursor(0,Col2).


pdwmovever(Verpos,Nverpos,Width,Attr):-
	field_attr(Verpos,1,Width,Attr),
	field_str(Verpos,2,Width,Word),
	intenseletter(Verpos,2,Attr,Word),
	reverseattr(Attr,Rev),
	field_attr(Nverpos,1,Width,Rev),
	cursor(Nverpos,1).


findtopelem(1,[Elem|_],Elem):- !.
findtopelem(N,[_|Tail],Elem):- NN = N-1, findtopelem(NN,Tail,Elem).

findpdwelem(1,[Elem|_],Elem):- !.
findpdwelem(N,[_|Tail],Elem):- NN = N-1, findpdwelem(NN,Tail,Elem).



line_ver(R1,R2,C):-
	R2>R1,!, R=R1+1,
	scr_char(R1,C,''),
	line_ver(R,R2,C).
line_ver(_,_,_).


line_hor(C1,C2,R):-
	C2>C1,!, C=C1+1,
	scr_char(R,C1,''),
	line_hor(C,C2,R).
line_hor(_,_,_).


left_corner(1,_,'') :- !.  % no top frame
left_corner(_,0,'') :- !.  % top frame, but left edge	
left_corner(_,_,'').       % normal line join

 
right_corner(1, _,''):-!.  % no top frame
right_corner(_,Col,'') :-    %  top frame, but right edge	
	textmode(_,Cols),   
	Cols=Col+1,!.
right_corner(_,_,'').	    % normal join	


intense(ATTR,ATTR1):-
	bitxor(ATTR,$08,ATTR1).

intensefirstupper(ROW,COL,ATTR,WORD):-
	frontchar(WORD,CH,_),
	CH>='A', CH<='Z',!,scr_attr(ROW,COL,ATTR).
intensefirstupper(ROW,COL,ATTR,WORD):-
	frontchar(WORD,_,REST),COL1=COL+1,
	intensefirstupper(ROW,COL1,ATTR,REST).

intenseletter(ROW,COL,ATTR,WORD):-
	intense(ATTR,INTENS),
	intensefirstupper(ROW,COL,INTENS,WORD),!.
intenseletter(ROW,COL,ATTR,_):-
	intense(ATTR,INTENS),
	scr_attr(ROW,COL,INTENS).


changepdwstate(Newstate):- 
	retractall(pdwstate(_,_,_)), 
	assert(Newstate).
