/*
============================================================================
	TREEMENU.PRO

 	In order to use the treemenu:
  	  1) the SELECTOR domain must be defined.
		note that this need not be an integer as in the examples
		it could be any domain appropriate to the problem
  	  2) clauses for treeaction must be defined.

	Copyright (C) 1990 Prolog Development Center
============================================================================
*/

include "tdoms.pro"
include "tpreds.pro"

CONSTANTS
  % Comment out any of the following constants to avoid
  % the compilation of superflous predicates

  right_in_use = 1
  left_in_use = 1
  down_in_use = 1
  up_in_use = 1
  cursor_mode_in_use = 1
  mouse_mode_in_use = 1

  % Colors used in drawing the tree
  node_color = 106
  highlight_node = 44
  path_color =  114
  path_node_color = 234
  leaf_color = node_color

  % the number of characters between the node string and the branch (>=0)
  line_before_branch = 1

  % the number of characters between branch and the node string (>0)
  line_after_branch  = 2

  % the number of characters between the node string and the branch (>=0)
  line_before_branch_ud = 1

  % the number of characters between branch and the node string (>=0)
  line_after_branch_ud  = 1

  % Scroll amounts
  row_scroll = 4
  col_scroll = 10
  

/*****************************************************************************
	Domains definitions for the treemenu
*****************************************************************************/

DOMAINS
ifndef globdecl
  TREE = tree(STRING,SELECTOR,TREELIST)
  TREELIST = TREE*
  MOVEMODE = cursor; mouse
  DIR = left ; right ; up ; down
enddef

  ROWCOL  = INTEGER
  TREE2 = tree2(ROW,COL,STRING,SELECTOR,TREE2LIST,BRANCHINFO)
  TREE2LIST = TREE2*
  BRANCHLRLIST = ROW*
  BRANCHUDLIST = col*
  BRANCH2 = branch_lr(ROW,CHAR); branch_ud(COL,CHAR)
  BRANCH2LIST = BRANCH2*
  BRANCHINFO =	branchinfo_lr(COL,BRANCH2LIST);
		branchinfo_ud(COL,ROW,BRANCH2LIST);
		nil


DATABASE - treemenu
  treewindow(row,col)
  current_node(integerlist,row,col,selector)
  mode(movemode)


/*****************************************************************************
	Extra local constants
*****************************************************************************/

CONSTANTS
% Do NOT change any of the following definitions.
ifdef right_in_use
	lr_in_use = 1
elsedef
   ifdef left_in_use
   	lr_in_use = 1
   enddef
enddef

ifdef up_in_use
	ud_in_use = 1
elsedef
   ifdef down_in_use
   	ud_in_use = 1
   enddef
enddef


/*****************************************************************************
	Updating the local database
*****************************************************************************/

PREDICATES
  newupdate(TREEMENU)

CLAUSES
  newupdate(treewindow(_,_)):-retract(treewindow(_,_)),fail.
  newupdate(current_node(_,_,_,_)):-retract(current_node(_,_,_,_)),fail.
  newupdate(mode(_)):-retract(mode(_)),fail.
  newupdate(X):-assert(X).


/*****************************************************************************
	Predicates used to compile the tree, common to all directions
*****************************************************************************/

PREDICATES
  branch_parms(branchlrlist,row,row,integer)
  branch_parms(branchudlist,col,col,integer)
  list_len(tree2list,integer,integer)
  list_len(branch2list,integer,integer)
  corner_middle(dir,char)

CLAUSES
  branch_parms([Max],Max,Max,1):-!.
  branch_parms([P|Rest],Max,Sum,Number):-
	branch_parms(Rest,Max,Sum0,Number0),
	Number = Number0 + 1,
	Sum = Sum0 + P.

  list_len([],X,X):-!.
  list_len([_|Rest],N,Len):-
	N2=N+1,
  	list_len(Rest,N2,Len).

  corner_middle(right,''):-!.
  corner_middle(left,''):-!.
  corner_middle(down,''):-!.
  corner_middle(up,'').


/*****************************************************************************
	Predicates used to compile the tree, common to LEFT and RIGHT.
*****************************************************************************/

ifdef lr_in_use
PREDICATES
  mk_branchlist_lr(dir,branchlrlist,branch2list,row,row,row)
  mk_branchlist1_lr(dir,branchlrlist,branch2list,row,row,row)
  inc_row(tree2,row,row)
  corner_lr(dir,row,row,row,row,char)

CLAUSES
  mk_branchlist_lr(_,[R],[branch_lr(R,'')],_,_,_):-!.
  mk_branchlist_lr(Dir,Branches,Result,Rmin,Rmid,Rmax):-
 	mk_branchlist1_lr(Dir,Branches,Result,Rmin,Rmid,Rmax).


  mk_branchlist1_lr(_,[],[],_,_,_):-!.
  mk_branchlist1_lr(Dir,[R1,R3|Brest],
                    [branch_lr(R1,Ch1),
		     branch_lr(Rmid,Ch2),
                     branch_lr(R3,Ch3) | Res],
 		    Rmin,Rmid,Rmax):-
	Rmid < R3,
	Rmid > R1,!,
	corner_lr(Dir,Rmid,Rmin,Rmax,R1,Ch1),
	corner_middle(Dir,Ch2),
	corner_lr(Dir,Rmid,Rmin,Rmax,R3,Ch3),
	mk_branchlist1_lr(Dir,Brest,Res,Rmin,Rmid,Rmax).
  mk_branchlist1_lr(Dir,[R|Brest],[branch_lr(R,Ch)|Res],Rmin,Rmid,Rmax):-
	corner_lr(Dir,Rmid,Rmin,Rmax,R,Ch),
        mk_branchlist1_lr(Dir,Brest,Res,Rmin,Rmid,Rmax).


  corner_lr(right,_,RMIN,_,RMIN,''):-!.
  corner_lr(right,_,_,RMAX,RMAX,''):-!.
  corner_lr(right,RMIDT,_,_,RMIDT,''):-!.
  corner_lr(right,_,_,_,_,''):-!.
  corner_lr(left,_,RMIN,_,RMIN,''):-!.
  corner_lr(left,_,_,RMAX,RMAX,''):-!.
  corner_lr(left,RMIDT,_,_,RMIDT,''):-!.
  corner_lr(left,_,_,_,_,''):-!.


  inc_row(tree2(R,_,_,_,_,_),R,R2):-R2 = R + 2,!.
  inc_row(_,R,R1):-R1 = R + 1.
enddef


ifdef ud_in_use
/*****************************************************************************
	Predicates used to compile the tree, common to UP and DOWN.
*****************************************************************************/

PREDICATES
  mk_branchlist_ud(dir,branchudlist,branch2list,col,col,col)
  mk_branchlist1_ud(dir,branchudlist,branch2list,col,col,col)
  corner_ud(dir,col,col,col,col,char)
  dec_col(tree2,col,col)

CLAUSES
  mk_branchlist_ud(_,[C],[branch_ud(C,'')],_,_,_):-!.
  mk_branchlist_ud(Dir,Branches,Result,Rmin,Rmid,Rmax):-
 	mk_branchlist1_ud(Dir,Branches,Result,Rmin,Rmid,Rmax).


  mk_branchlist1_ud(_,[],[],_,_,_):-!.
  mk_branchlist1_ud(Dir,[C1,C3|Brest],
 	            [branch_ud(C1,Ch1),
                     branch_ud(Cmid,Ch2),
                     branch_ud(C3,Ch3) | Res],
 		    Cmin,Cmid,Cmax):-
	Cmid < C1,
	Cmid > C3,!,
	corner_ud(Dir,Cmid,Cmin,Cmax,C1,Ch1),
 	corner_middle(Dir,Ch2),
 	corner_ud(Dir,Cmid,Cmin,Cmax,C3,Ch3),
 	mk_branchlist1_ud(Dir,Brest,Res,Cmin,Cmid,Cmax).
  mk_branchlist1_ud(Dir,[C|Brest],[branch_ud(C,Ch)|Res],Cmin,Cmid,Cmax):-
	corner_ud(Dir,Cmid,Cmin,Cmax,C,Ch),
        mk_branchlist1_ud(Dir,Brest,Res,Cmin,Cmid,Cmax).

  corner_ud(down,_,MIN,_,MIN,''):-!.
  corner_ud(down,_,_,MAX,MAX,''):-!.
  corner_ud(down,MIDT,_,_,MIDT,''):-!.
  corner_ud(down,_,_,_,_,''):-!.
  corner_ud(up,_,MIN,_,MIN,''):-!.
  corner_ud(up,_,_,MAX,MAX,''):-!.
  corner_ud(up,MIDT,_,_,MIDT,''):-!.
  corner_ud(up,_,_,_,_,'').

  dec_col(tree2(_,C,_,_,_,_),C,C2):-C2 = C - 2,!.
  dec_col(_,C,C1):-C1 = C + 1.
enddef


/*****************************************************************************
	Predicates used to compile the tree RIGHT
*****************************************************************************/

ifdef right_in_use
PREDICATES
  mk_tree_right(tree,tree2,row,col,row,row)
  mk_treelist_right(treelist,tree2list,branchlrlist,row,col,row)

CLAUSES
  mk_tree_right(tree(Name,Select,[]),tree2(Maxrow,Curcol,Name,Select,[],nil),
	        Maxrow,Curcol,Maxrow,Maxrow):-!.
  mk_tree_right(tree(Name,Select,Treelist),tree2(Rmid,Curcol,Name,Select,
  		Restree,branchinfo_lr(Branchcol,Branchlist)),
	        Maxrow,Curcol,Resrow,Rmid):-
        str_len(Name,Len),    /* Adjustment done here */
        Branchcol = Curcol + Len + line_before_branch,
        Colafterbranch = Branchcol + (line_after_branch+1),
        mk_treelist_right(Treelist,Restree,Branches,Maxrow,Colafterbranch,Resrow),
        Branches = [Rmin|_],
        branch_parms(Branches,Rmax,Rowsum,Number),
        Rmid = Rowsum div Number,
        mk_branchlist_lr(right,Branches,Branchlist,Rmin,Rmid,Rmax),!.


  mk_treelist_right([Hd|Tl],[Rhd|Rtl],[Rmid|Btl],Maxrow,Curcol,Resrow):-
        mk_tree_right(Hd,Rhd,Maxrow,Curcol,Resrow1,Rmid),
 	inc_row(Rhd,Resrow1,Maxrow2),
 	mk_treelist_right(Tl,Rtl,Btl,Maxrow2,Curcol,Resrow).
  mk_treelist_right([],[],[],Maxrow,_,Maxrow).
enddef


/*****************************************************************************
	Predicates used to compile the tree LEFT
*****************************************************************************/

ifdef left_in_use
PREDICATES
  mk_tree_left(tree,tree2,row,col,row,row)
  mk_treelist_left(treelist,tree2list,branchlrlist,row,col,row)

CLAUSES
  mk_tree_left(tree(Name,Select,[]),
	       tree2(Maxrow,Curcoln,Name,Select,[],nil),
	       Maxrow,Curcol,Maxrow,Maxrow):-!,
	str_len(Name,Len),
	Curcoln = Curcol - Len + 1.

  mk_tree_left(tree(Name,Select,Treelist),
               tree2(Rmid,Namecol,Name,Select,Restree,
	       branchinfo_lr(Branchcol,Branchlist)),
	       Maxrow,Curcol,Resrow,Rmid):-
	str_len(Name,Len),    /* Adjustment done here */
	Namecol = Curcol - Len + 1,
	Branchcol = Namecol - line_before_branch - 1,
	Curcolnew = Branchcol - (line_after_branch+1),
	mk_treelist_left(Treelist,Restree,Branches,Maxrow,Curcolnew,Resrow),
        Branches = [Rmin|_],
        branch_parms(Branches,Rmax,Rowsum,Number),
        Rmid = Rowsum div Number,
        mk_branchlist_lr(left,Branches,Branchlist,Rmin,Rmid,Rmax),!.


  mk_treelist_left([Hd|Tl],[Rhd|Rtl],[Rmid|Btl],Maxrow,Curcol,Resrow):-
        mk_tree_left(Hd,Rhd,Maxrow,Curcol,Resrow1,Rmid),
 	inc_row(Rhd,Resrow1,Maxrow2),
 	mk_treelist_left(Tl,Rtl,Btl,Maxrow2,Curcol,Resrow).
  mk_treelist_left([],[],[],Maxrow,_,Maxrow).
enddef


/*****************************************************************************
	Predicates used to compile the tree DOWNWARDS
*****************************************************************************/

ifdef down_in_use
PREDICATES
  mk_tree_down(tree,tree2,row,col,col,col)
  mk_treelist_down(treelist,tree2list,branchudlist,row,col,col)

CLAUSES
  mk_tree_down(tree(Name,Select,[]),
	       tree2(Currow,Rescol,Name,Select,[],nil),
	       Currow,Maxcol,Rescol,Midcol):-!,
	str_len(Name,Len),
	Rescol = Maxcol - Len + 1,
	Midcol = Maxcol -  Len div 2.
  mk_tree_down(tree(Name,Select,Treelist),
	        tree2(Currow,Namecol,Name,Select,Restree,
	        branchinfo_ud(Midcol,Branchrow,Branchlist)),
	        Currow,Maxcol,Rescol,Midcol):-
        str_len(Name,Len),    % Adjustment done here
	Branchrow = Currow + line_before_branch_ud + 1,
	Currownew = Branchrow + line_after_branch_ud + 1,
	mk_treelist_down(Treelist,Restree,Branches,Currownew,Maxcol,Rescol),
        Branches = [Cmax|_],
        branch_parms(Branches,Cmin,Colsum,Number),
        Midcol = Colsum div Number,
        Namecol = Midcol - Len div 2,
        mk_branchlist_ud(down,Branches,Branchlist,Cmin,Midcol,Cmax),!.


  mk_treelist_down([Hd|Tl],[Rhd|Rtl],[Midcol|Btl],Currow,Maxcol,Rescol):-
        mk_tree_down(Hd,Rhd,Currow,Maxcol,Rescol1,Midcol),
        dec_col(Rhd,Rescol1,Maxcol2),
        mk_treelist_down(Tl,Rtl,Btl,Currow,Maxcol2,Rescol).
  mk_treelist_down([],[],[],_,Maxcol,Maxcol).
enddef


/*****************************************************************************
	Predicates used to compile the tree UPWARDS
*****************************************************************************/

ifdef up_in_use
PREDICATES
  mk_tree_up(tree,tree2,row,col,col,col)
  mk_treelist_up(treelist,tree2list,branchudlist,row,col,col)

CLAUSES
  mk_tree_up(tree(Name,Select,[]),
	     tree2(Currow,Rescol,Name,Select,[],nil),
	     Currow,Maxcol,Rescol,Midcol):-!,
	str_len(Name,Len),
	Rescol = Maxcol - Len + 1,
	Midcol = Maxcol -  Len div 2.
  mk_tree_up(tree(Name,Select,Treelist),
	     tree2(Currow,Namecol,Name,Select,Restree,
	     branchinfo_ud(Midcol,Branchrow,Branchlist)),
	     Currow,Maxcol,Rescol,Midcol):-
	str_len(Name,Len),    % Adjustment done here
	Branchrow = Currow - line_before_branch_ud - 1,
	Currownew = Branchrow - line_after_branch_ud - 1,
	mk_treelist_up(Treelist,Restree,Branches,Currownew,Maxcol,Rescol),
        Branches = [Cmax|_],
        branch_parms(Branches,Cmin,Colsum,Number),
        Midcol = Colsum div Number,
        Namecol = Midcol - Len div 2,
        mk_branchlist_ud(up,Branches,Branchlist,Cmin,Midcol,Cmax),!.


  mk_treelist_up([Hd|Tl],[Rhd|Rtl],[Midcol|Btl],Currow,Maxcol,Rescol):-
        mk_tree_up(Hd,Rhd,Currow,Maxcol,Rescol1,Midcol),
        dec_col(Rhd,Rescol1,Maxcol2),
        mk_treelist_up(Tl,Rtl,Btl,Currow,Maxcol2,Rescol).
  mk_treelist_up([],[],[],_,Maxcol,Maxcol).
enddef


/*****************************************************************************
		Miscalaneous predicates used to, scroll etc.
*****************************************************************************/

PREDICATES
  init_startpos(dir,row,col,selector)
  adjust_view(row,col,row,col)
  scroll_view(row,row,col,col)
  adjust_row(row,row,row,row,row)
  adjust_col(col,col,col,col,col)
  set_mode(movemode)
  window_dimensions(ROW,COL)
  wd(INTEGER,ROW,ROW, COL,COL)
  
CLAUSES
  adjust_view(Row,Col,Srowmarg,Scolmarg):-
        treewindow(Strow,Stcol),
        window_dimensions(Rnum1, Cnum1),
        adjust_row(Srowmarg,Row,Wrow,Strow,Rnum1),
        adjust_col(Scolmarg,Col,Wcol,Stcol,Cnum1),
        scroll_view(Strow,Wrow,Stcol,Wcol),!.


  scroll_view(R,R,C,C):-!.
  scroll_view(_,Wrow,_,Wcol):- newupdate(treewindow(Wrow,Wcol)).

  adjust_row(M,N,W,S,_):-
  	N < S + M,!,
  	W = N - M,
  	Sa = W - S ,
  	scroll(Sa,0).
  adjust_row(M,N,W,S,A):-
  	N > S + A - M,!,
  	W = N - A + M,
  	Sa = W - S,
  	scroll(Sa,0).
  adjust_row(_,_,W,W,_).

  adjust_col(M,N,W,S,_):-
  	N < S + M,!,
  	W = N - M,
  	Sa =  W - S,
  	scroll(0,Sa).
  adjust_col(M,N,W,S,A):-
  	N > S + A - M,!,
  	W = N - A + M,
  	Sa = W - S ,
  	scroll(0,Sa).
  adjust_col(_,_,W,W,_).

  set_mode(mouse):-
 	newupdate(mode(mouse)).
  set_mode(cursor):-
 	newupdate(mode(cursor)).

  init_startpos(_,Row,Col,Select):-
  	newupdate(current_node([1],Row,Col,Select)),
  	clearwindow,
  	fail.
  init_startpos(right,Row,Col,_):-
  	Startrow = Row - 11,
  	Startcol = Col - 10,
  	newupdate(treewindow(Startrow,Startcol)), fail.
  init_startpos(left,Row,Col,_):-
  	Startrow = Row - 11,
  	Startcol = Col - 60,
  	newupdate(treewindow(Startrow,Startcol)), fail.
  init_startpos(up,Row,Col,_):-
  	Startrow = Row - 18,
  	Startcol = Col - 35,
  	newupdate(treewindow(Startrow,Startcol)), fail.
  init_startpos(down,Row,Col,_):-
  	Startrow = Row - 4,
  	Startcol = Col - 35,
  	newupdate(treewindow(Startrow,Startcol)), fail.
  init_startpos(_,Row,Col,_):-
  	mode(mouse),!,
  	treewindow(Strow,Stcol),!,
  	R = Row - Strow,
  	C = Col - Stcol,
  	cursor(R,C).
  init_startpos(_,_,_,_).


  window_dimensions(High, Wide) :-
  	makewindow(_,_,FrameAttr,_,_,_,Rows, Cols),
  	wd(FrameAttr,Rows, High, Cols, Wide).
  	
  wd(0, Rows, Rows, Cols, Cols):-!.
  wd(_, Rows, High, Cols, Wide) :- 
  	High=Rows-2, 
  	Wide=Cols-2.
  
/*****************************************************************************
   INVOKE_KEY specifies which keys should terminate
   drawing, and invoke action.

   Use TREEACTION to invoke user specified action
   Make clause fail, after action, to continue drawing
*****************************************************************************/

PREDICATES
  invoke_key(key)
ifndef globdecl
  treeaction(key,selector)
enddef

CLAUSES
  invoke_key(cr):-!.
  invoke_key(fkey(10)).


/*****************************************************************************
		Predicates used in cursor mode
*****************************************************************************/

ifdef cursor_mode_in_use
PREDICATES
  transform_dir(dir,key,key)
  find_new_node(key,integerlist,tree2list,integerlist,row,col,selector)
  find_new_node_top(key,integerlist,tree2list,integerlist,row,col,selector)
  find_branch(integer,tree2list,tree2)
  nondeterm new_index(key,integerlist,tree2list,integerlist,row,col,selector)
  nondeterm downmost(integerlist,tree2list,integer,integerlist,row,col,selector)
  upmost(integerlist,tree2list,integer,integerlist,row,col,selector)
  ver_key(key)
  nondeterm iterate(integer,integer,integer,integer)
  treekeyact1(KEY,tree2,dir,row,col)

CLAUSES
  transform_dir(left,right,left):-!.
  transform_dir(left,left,right):-!.
  transform_dir(up,left,down):-!.
  transform_dir(up,right,up):-!.
  transform_dir(up,up,right):-!.
  transform_dir(up,down,left):-!.
  transform_dir(down,up,left):-!.
  transform_dir(down,down,right):-!.
  transform_dir(down,left,down):-!.
  transform_dir(down,right,up):-!.
  transform_dir(_,Key,Key).

  find_branch(1,[Head|_],Head):-!.
  find_branch(N,[_|Rest],Branch):-
        N1 = N - 1,
	find_branch(N1,Rest,Branch).

  find_new_node_top(A1,A2,A3,A4,A5,A6,A7):-
  	find_new_node(A1,A2,A3,A4,A5,A6,A7).


  find_new_node(_,_,_,_,_,_,_):-
  	keypressed,
  	readkey(_),
  	fail.
  find_new_node(left,[P1,_],Treelist,[P1],Row,Col,Select):-!,
	find_branch(P1,Treelist,tree2(Row,Col,_,Select,_,_)).
  find_new_node(right,[P],Treelist,[P,Newinteger],Row,Col,Select):-
	find_branch(P,Treelist,tree2(_,_,_,_,Tlnew,_)),!,
	list_len(Tlnew,0,N),
	Newinteger = (N div 2) + (N mod 2),
	find_branch(Newinteger,Tlnew,tree2(Row,Col,_,Select,_,_)).
  find_new_node(pgup,[P],Treelist,[P,1],Row,Col,Select):-!,
	find_branch(P,Treelist,tree2(_,_,_,_,[tree2(Row,Col,_,Select,_,_)|_],_)).
  find_new_node(pgdn,[P],Treelist,[P,N],Row,Col,Select):-
	find_branch(P,Treelist,tree2(_,_,_,_,Tlnew,_)),!,
	list_len(Tlnew,0,N),
	find_branch(N,Tlnew,tree2(Row,Col,_,Select,_,_)).
  find_new_node(home,[_],[tree2(Row,Col,_,Select,_,_)|_],[1],Row,Col,Select):-!.
  find_new_node(end,[_],Treelist,[N],Row,Col,Select):-!,
	list_len(Treelist,0,N),
	find_branch(N,Treelist,tree2(Row,Col,_,Select,_,_)).
  find_new_node(Key,Poslist,Treelist,Newintl,Row,Col,Select):-
	ver_key(Key),!,
	new_index(Key,Poslist,Treelist,Newintl,Row,Col,Select),!.
  find_new_node(Key,[Ph|Ptl],Treelist,[Ph|Pres],Row,Col,Select):-
	find_branch(Ph,Treelist,tree2(_,_,_,_,Tlnew,_)),
	find_new_node(Key,Ptl,Tlnew,Pres,Row,Col,Select).


  iterate(_,_,_,_):-
  	keypressed,
  	readkey(_),
        fail.
  iterate(Old,1,Old,Old):-!, fail.
  iterate(1,-1,_,1):-!, fail.
  iterate(Old,Quan,_,New):-New = Old + Quan.
  iterate(Old,Quan,Len,New):-!,
	Old1 = Old + Quan,
	iterate(Old1,Quan,Len,New).


  new_index(up,[P],Tl,[P1],Row,Col,Select):-!,
	P1 = P - 1,
	find_branch(P1,Tl,tree2(Row,Col,_,Select,_,_)).
  new_index(down,[P],Tl,[P1],Row,Col,Select):-!,
	P1 = P + 1,
	find_branch(P1,Tl,tree2(Row,Col,_,Select,_,_)).
  new_index(Key,[Ph|Pt],Tl,[Ph|Pr],Row,Col,Select):-
	find_branch(Ph,Tl,tree2(_,_,_,_,Tl2,_)),
	new_index(Key,Pt,Tl2,Pr,Row,Col,Select),!.
  new_index(up,[Ph|Pt],Tl,[Ph1|Pr],Row,Col,Select):-!,
	list_len(Tl,0,L),
	iterate(Ph,-1,L,Ph1),
	find_branch(Ph1,Tl,tree2(_,_,_,_,Tl2,_)),
	list_len(Tl2,0,Len),
	downmost(Pt,Tl2,Len,Pr,Row,Col,Select).
  new_index(down,[Ph|Pt],Tl,[Ph1|Pr],Row,Col,Select):-
  	list_len(Tl,0,L),
	iterate(Ph,1,L,Ph1),
	find_branch(Ph1,Tl,tree2(_,_,_,_,Tl2,_)),
	list_len(Tl2,0,L2),
	upmost(Pt,Tl2,L2,Pr,Row,Col,Select).


  downmost(_,_,_,_,_,_,_):-
  	keypressed,
  	readkey(_),
  	fail.
  downmost(_,[],_,_,_,_,_):-!, fail.
  downmost([_],Tl,L,[L],Row,Col,Select):-!,
	find_branch(L,Tl,tree2(Row,Col,_,Select,_,_)).
  downmost([_|Pt],Tl,L,[L|Rp],Row,Col,Select):-
	find_branch(L,Tl,tree2(_,_,_,_,Tl2,_)),
	list_len(Tl2,0,L2),
	downmost(Pt,Tl2,L2,Rp,Row,Col,Select),!.
  downmost([_|Pt],Tl,L,[N|Rp],Row,Col,Select):-
  	iterate(L,-1,L,N),
	find_branch(N,Tl,tree2(_,_,_,_,Tl2,_)),
	list_len(Tl2,0,L2),
	downmost(Pt,Tl2,L2,Rp,Row,Col,Select).


  upmost(_,_,_,_,_,_,_):-
  	keypressed,
  	readkey(_),
  	fail.
  upmost(_,[],_,_,_,_,_):-!,fail.
  upmost([_],[tree2(Row,Col,_,Select,_,_)|_],_,[1],Row,Col,Select):-!.
  upmost([_|Pt],[tree2(_,_,_,_,Tl,_)|_],L,[1|Rp],Row,Col,Select):-
  	upmost(Pt,Tl,L,Rp,Row,Col,Select),!.
  upmost([_|Pt],Tl,L,[N|Rp],Row,Col,Select):-
       	iterate(1,1,L,N),
	find_branch(N,Tl,tree2(_,_,_,_,Tl2,_)),
        list_len(Tl2,0,L2),
	upmost(Pt,Tl2,L2,Rp,Row,Col,Select),!.


  ver_key(up):-!.
  ver_key(down).


ifdef mouse_mode_in_use
  treekeyact1(fkey(2),Tree,Dir,_,_):-!,
	set_mode(mouse),
	Tree = tree2(Row,Col,_,Select,_,_),
	init_startpos(Dir,Row,Col,Select),
	fail.
enddef
  treekeyact1(ctrlhome,tree2(Row,Col,_,Select,_,_),_,Sr,Sc):-
	newupdate(current_node([1],Row,Col,Select)),
	adjust_view(Row,Col,Sr,Sc),!,
        fail.
  treekeyact1(Key,Tree,Dir,Sr,Sc):-
	current_node(Pl,_,_,_),
	transform_dir(Dir,Key,Newkey),
	find_new_node_top(Newkey,Pl,[Tree],Nl,Row,Col,Select),
	newupdate(current_node(Nl,Row,Col,Select)),
	adjust_view(Row,Col,Sr,Sc),!,
	fail.
enddef


/*****************************************************************************
		Predicates used in mouse mode
*****************************************************************************/

ifdef mouse_mode_in_use
PREDICATES
  treekeyact2(KEY,tree2,dir,row,col)
  modify_view(row,col,row,col)

CLAUSES
ifdef cursor_mode_in_use
  treekeyact2(fkey(2),Tree,Dir,_,_):-!,
	set_mode(cursor),
	Tree = tree2(R,C,_,S,_,_),
	init_startpos(Dir,R,C,S),
	fail.
enddef
  treekeyact2(_,_,_,_,_):-
	current_node(_,R,C,S),
	newupdate(current_node([],R,C,S)),
	fail.
  treekeyact2(right,_,_,Sr,Sc):-modify_view(0,1,Sr,Sc).
  treekeyact2(left,_,_,Sr,Sc):-modify_view(0,-1,Sr,Sc).
  treekeyact2(ctrlleft,_,_,Sr,Sc):-modify_view(0,-10,Sr,Sc).
  treekeyact2(ctrlright,_,_,Sr,Sc):-modify_view(0,10,Sr,Sc).
  treekeyact2(up,_,_,Sr,Sc):-modify_view(-1,0,Sr,Sc).
  treekeyact2(down,_,_,Sr,Sc):-modify_view(1,0,Sr,Sc).
  treekeyact2(pgup,_,_,Sr,Sc):-modify_view(-5,0,Sr,Sc).
  treekeyact2(pgdn,_,_,Sr,Sc):-modify_view(5,0,Sr,Sc).


  modify_view(Radj,Cadj,Srowmarg,Scolmarg):-
        treewindow(Strow,Stcol),
        cursor(R,C),
        Row = Strow + R + Radj,
        Col = Stcol + C + Cadj,
        window_dimensions(Rnum, Cnum),
        adjust_row(Srowmarg,Row,Wrow,Strow,Rnum),
        adjust_col(Scolmarg,Col,Wcol,Stcol,Cnum),
        scroll_view(Strow,Wrow,Stcol,Wcol),
        Newrow = R + Radj - Wrow + Strow,
        Newcol = C + Cadj - Wcol + Stcol,
        cursor(Newrow,Newcol),!,
        fail.
enddef


/*****************************************************************************
		Key stroke interpreter and TREEMENU
*****************************************************************************/

PREDICATES
  treekeyact(KEY,tree2,dir,row,col)

CLAUSES
  treekeyact(esc,_,_,_,_):-!. /* Terminate and fail */
  treekeyact(Key,_,_,_,_):-
	invoke_key(Key),
	current_node(_,_,_,Select),!,
	treeaction(Key,Select).

ifdef cursor_mode_in_use
  treekeyact(Key,Tree,Dir,Sr,Sc):-
	mode(cursor),!,
	treekeyact1(Key,Tree,Dir,Sr,Sc).
enddef

ifdef mouse_mode_in_use
  treekeyact(Key,Tree,Dir,Sr,Sc):-
	treekeyact2(Key,Tree,Dir,Sr,Sc).
enddef


/*****************************************************************************
		Low level predicates used to draw the compiled tree
*****************************************************************************/

PREDICATES
  treedrop(INTEGER,STRING,STRING)
  tree_writetext(INTEGERLIST,INTEGER,ROW,COL,STRING,SELECTOR)
  writetext(ROW,COL,INTEGER,STRING,SELECTOR)
  treeline_ver(ROW,ROW,COL)
  treeline_ver_high(ROW,ROW,COL)
  treeline_ver1(ROW,ROW,COL)
  treeline_ver1_high(ROW,ROW,COL)
  treeline_hor(COL,COL,ROW)
  treeline_hor_high(COL,COL,ROW)
  treeline_hor1(COL,COL,ROW)
  scr_ch(ROW,COL,CHAR)
  scr_ch_high(ROW,COL,CHAR)
  set_attr(ROW,COL,INTEGER,INTEGER,ROW,COL,SELECTOR)
  tree_branch_parms(INTEGER,INTEGER,INTEGER,INTEGER,INTEGER)
  
CLAUSES
  tree_writetext([P],P,Row,Col,Name,Select):-
	writetext(Row,Col,path_node_color,Name,Select) ,!.
  tree_writetext(_,_,Row,Col,Name,Select):-
	writetext(Row,Col,node_color,Name,Select).


  treeline_ver(R1,R2,C):-
  	treewindow(STROW,STCOL), 
	window_dimensions(ROWS, COLS),
  	C>=STCOL, 
	C<STCOL+COLS, 
	R1<STROW+ROWS, 
	R2>=STROW,!,
	SLROW=STROW+ROWS,
	max(STROW,R1,STROW1), 
	min(R2,SLROW,SLROW1),
	CC=C-STCOL, 
	RR1=STROW1-STROW, 
	RR2=SLROW1-STROW,
	treeline_ver1(RR1,RR2,CC).
  treeline_ver(_,_,_).


  treeline_ver1(R1,R2,_):-R2<=R1,!.
  treeline_ver1(R1,R2,C):-
	R=R1+1,
	scr_char(R1,C,''),
	treeline_ver1(R,R2,C).


  treeline_ver_high(R1,R2,C):-
  	treewindow(STROW,STCOL), 
  	window_dimensions(ROWS, COLS),
  	C>=STCOL, 
	C<STCOL+COLS, 
	R1<STROW+ROWS, 
	R2>=STROW,!,
	SLROW=STROW+ROWS,
	max(STROW,R1,STROW1), 
	min(R2,SLROW,SLROW1),
	CC=C-STCOL, 
	RR1=STROW1-STROW, 
	RR2=SLROW1-STROW,
	treeline_ver1_high(RR1,RR2,CC).
  treeline_ver_high(_,_,_).


  treeline_ver1_high(R1,R2,_):-R2<=R1,!.
  treeline_ver1_high(R1,R2,C):-
	R=R1+1,
	field_attr(R1,C,1,path_color),
	scr_char(R1,C,''),
	treeline_ver1_high(R,R2,C).


  treeline_hor(C1,C2,R):-
  	treewindow(STROW,STCOL), 
	window_dimensions(ROWS, COLS),
	SLCOL=STCOL+COLS,
  	R>=STROW, 
	R<STROW+ROWS, 
	C1<SLCOL, 
	C2>STCOL,!,
	max(STCOL,C1,STCOL1), 
	min(C2,SLCOL,SLCOL1),
	RR=R-STROW, 
	CC1=STCOL1-STCOL, 
	CC2=SLCOL1-STCOL,
	treeline_hor1(CC1,CC2,RR).
  treeline_hor(_,_,_).


  treeline_hor_high(C1,C2,R):-
  	treewindow(STROW,STCOL), 
	window_dimensions(ROWS, COLS),
	SLCOL=STCOL+COLS,
  	R>=STROW, 
	R<STROW+ROWS, 
	C1<SLCOL, 
	C2>STCOL,!,
	max(STCOL,C1,STCOL1), 
	min(C2,SLCOL,SLCOL1),
	RR=R-STROW, 
	CC1=STCOL1-STCOL, 
	CC2=SLCOL1-STCOL,
	LEN = CC2 - CC1,
	field_attr(RR,CC1,LEN,path_color),
	treeline_hor1(CC1,CC2,RR).
  treeline_hor_high(_,_,_).


  treeline_hor1(C1,C2,_):-C2<=C1,!.
  treeline_hor1(C1,C2,R):-
	C=C1+1,
	scr_char(R,C1,''),
	treeline_hor1(C,C2,R).


  scr_ch(R,C,CH):-
  	treewindow(STROW,STCOL), 
  	window_dimensions(ROWS, COLS),
  	R>=STROW, 
	R<STROW+ROWS, 
	C>=STCOL, 
	C<STCOL+COLS, !,
	RR=R-STROW, 
	CC=C-STCOL,
	scr_char(RR,CC,CH).
  scr_ch(_,_,_).


  scr_ch_high(R,C,CH):-
  	treewindow(STROW,STCOL), 
  	window_dimensions(ROWS,COLS),
  	R>=STROW, 
	R<STROW+ROWS, 
	C>=STCOL, 
	C<STCOL+COLS, !,
	RR=R-STROW, 
	CC=C-STCOL,
	scr_attr(RR,CC,path_color),
	scr_char(RR,CC,CH).
  scr_ch_high(_,_,_).


  treedrop(0,TEXT,TEXT):-!.
  treedrop(N,TEXT,TEXT1):-frontstr(N,TEXT,_,TEXT1).


  writetext(ROW,COL,ATTR,TEXT,Select):-
  	str_len(TEXT,LEN),
  	treewindow(STROW,STCOL), 
  	window_dimensions(ROWS, COLS),
  	ROW>=STROW, 
	ROW<STROW+ROWS, 
	COL<STCOL+COLS, 
	COL+LEN>STCOL,
  	REALROW=ROW-STROW,
  	RSTART=COL-STCOL,
  	REND=RSTART+LEN,
	VST=STCOL-STCOL, 
	VEND=VST+COLS,
	max(RSTART,VST,RSTART1), 
	min(REND,VEND,REND1),
	REALLEN=REND1-RSTART1,
	DROP=RSTART1-RSTART,
	treedrop(DROP,TEXT,TEXT1),
	field_str(REALROW,RSTART1,REALLEN,TEXT1),
	set_attr(REALROW,RSTART1,REALLEN,ATTR,Row,Col,Select) ,!.
  writetext(_,_,_,_,_).


  set_attr(Srow,Scol,Len,_,Row,Col,Select):-
	 mode(mouse),
	 cursor(R,C),
	 Srow = R,
	 C >= Scol,
	 C < Scol + Len,!,
	 newupdate(current_node([1],Row,Col,Select)),
	 field_attr(Srow,Scol,Len,highlight_node).
  set_attr(Srow,Scol,Len,_,Row,Col,_):-
  	 mode(cursor),
  	 current_node(_,Row,Col,_),!,
  	 field_attr(Srow,Scol,Len,highlight_node),
         cursor(Srow,Scol).
  set_attr(Srow,Scol,Len,Attr,_,_,_):-field_attr(Srow,Scol,Len,Attr).


  tree_branch_parms(P2,Num,Num,Begin,End):-!,
	Mid = Num div 2 + Num mod 2,
	min(P2,Mid,Begin),
	max(P2,Mid,End).
  tree_branch_parms(P2,Num,_,P2,Mid):-
	Mid = Num div 2 + Num mod 2,
	P2 < Mid,!.
  tree_branch_parms(P2,Num,_,Mid,P21):-
	Mid = Num div 2 + Num mod 2,
	P21 = P2 + 1.


  	

/*****************************************************************************
		Drawing predicates used to the LEFT and RIGHT
*****************************************************************************/

ifdef lr_in_use
PREDICATES
  tree_writebranches_lr(col,branch2list)
  tree_writebranches_high_lr(col,branch2list,integer,integer,integer)

CLAUSES
  tree_writebranches_lr(C,[branch_lr(R1,Ch1)]):-!,
  	scr_ch(R1,C,Ch1).
  tree_writebranches_lr(C,[branch_lr(R1,Ch1)|Rest]):-
  	Rest = [branch_lr(R2,_)|_],!,
  	scr_ch(R1,C,Ch1),
	R11 = R1 + 1,
	treeline_ver(R11,R2,C),
  	tree_writebranches_lr(C,Rest).

  tree_writebranches_high_lr(C,[branch_lr(R1,Ch1),branch_lr(R2,Ch2)],_,End,Count):-
  	End = Count + 1,!,
  	scr_ch_high(R1,C,Ch1),
	R11 = R1 + 1,
	treeline_ver_high(R11,R2,C),
  	scr_ch_high(R2,C,Ch2).
  tree_writebranches_high_lr(C,[branch_lr(R1,Ch1),branch_lr(R2,Ch2)|Rest],Begin,End,Count):-
  	End = Count+1,Begin><End,
  	Rest = [branch_lr(R3,_)|_],!,
  	scr_ch_high(R1,C,Ch1),
	R11 = R1 + 1,
	treeline_ver_high(R11,R2,C),
  	scr_ch_high(R2,C,Ch2),
	R21 = R2 + 1,
	treeline_ver(R21,R3,C),
	Newcount = End + 1,
	tree_writebranches_high_lr(C,Rest,Begin,End,Newcount).
  tree_writebranches_high_lr(C,[branch_lr(R1,Ch1)|Rest],Begin,End,Count):-
  	Count = Begin,Count=End,!,
  	Rest = [branch_lr(R2,_)|_],
  	scr_ch_high(R1,C,Ch1),
	R11 = R1 + 1,
	treeline_ver(R11,R2,C),
	Newcount = Count + 1,
	tree_writebranches_high_lr(C,Rest,Begin,End,Newcount).
  tree_writebranches_high_lr(C,[branch_lr(R1,Ch1)|Rest],Begin,End,Count):-
  	Count >= Begin,
  	Count < End,!,
  	Rest = [branch_lr(R2,_)|_],
  	scr_ch_high(R1,C,Ch1),
	R11 = R1 + 1,
	treeline_ver_high(R11,R2,C),
	Newcount = Count + 1,
	tree_writebranches_high_lr(C,Rest,Begin,End,Newcount).

  tree_writebranches_high_lr(C,[branch_lr(R1,Ch1)],_,_,_):-!,
  	scr_ch(R1,C,Ch1).
  tree_writebranches_high_lr(C,[branch_lr(R1,Ch1)|Rest],Begin,End,Count):-
  	Rest = [branch_lr(R2,_)|_],
  	scr_ch(R1,C,Ch1),
	R11 = R1 + 1,
	treeline_ver(R11,R2,C),
	Newcount = Count + 1,
	tree_writebranches_high_lr(C,Rest,Begin,End,Newcount).
enddef


/*****************************************************************************
		Drawing predicates used to draw UP and DOWN
*****************************************************************************/

ifdef ud_in_use
PREDICATES
  tree_writebranches_ud(row,branch2list)
  tree_writebranches_high_ud(row,branch2list,integer,integer,integer)
  find_middle(tree2,col)

CLAUSES
  tree_writebranches_ud(R,[branch_ud(C,Ch)]):-!,
  	scr_ch(R,C,Ch).
  tree_writebranches_ud(R,[branch_ud(C1,Ch1)|Rest]):-
  	Rest = [branch_ud(C2,_)|_],!,
  	scr_ch(R,C1,Ch1),
	C21 = C2 + 1,
	treeline_hor(C21,C1,R),
  	tree_writebranches_ud(R,Rest).

  tree_writebranches_high_ud(R,[branch_ud(C1,Ch1),branch_ud(C2,Ch2)],_,End,End1):-
  	End = End1 + 1, !,
  	scr_ch_high(R,C1,Ch1),
	C21 = C2 + 1,
	treeline_hor_high(C21,C1,R),
  	scr_ch_high(R,C2,Ch2).
  tree_writebranches_high_ud(R,[branch_ud(C1,Ch1),branch_ud(C2,Ch2)|Rest],Begin,End,Count):-
  	End = Count + 1, Begin><End, !,
  	Rest = [branch_ud(C3,_)|_],
  	scr_ch_high(R,C1,Ch1),
	C21 = C2 + 1,
	treeline_hor_high(C21,C1,R),
  	scr_ch_high(R,C2,Ch2),
	C31 = C3 + 1,
	treeline_hor(C31,C2,R),
	Newcount = End + 1,
	tree_writebranches_high_ud(R,Rest,Begin,End,Newcount).
  tree_writebranches_high_ud(R,[branch_ud(C1,Ch1)|Rest],Begin,End,Count):-
  	Count = Begin, Count = End, !,
  	Rest = [branch_ud(C2,_)|_],
  	scr_ch_high(R,C1,Ch1),
	C21 = C2 + 1,
	treeline_hor(C21,C1,R),
	Newcount = Count + 1,
	tree_writebranches_high_ud(R,Rest,Begin,End,Newcount).
  tree_writebranches_high_ud(R,[branch_ud(C1,Ch1)|Rest],Begin,End,Count):-
  	Count >= Begin,
  	Count < End,!,
  	Rest = [branch_ud(C2,_)|_],
  	scr_ch_high(R,C1,Ch1),
	C21 = C2 + 1,
	treeline_hor_high(C21,C1,R),
	Newcount = Count + 1,
	tree_writebranches_high_ud(R,Rest,Begin,End,Newcount).
  tree_writebranches_high_ud(R,[branch_ud(C1,Ch1)],_,_,_):-!,
  	scr_ch(R,C1,Ch1).
  tree_writebranches_high_ud(R,[branch_ud(C1,Ch1)|Rest],Begin,End,Count):-
  	Rest = [branch_ud(C2,_)|_],
  	scr_ch(R,C1,Ch1),
	C21 = C2 + 1,
	treeline_hor(C21,C1,R),
	Newcount = Count + 1,
	tree_writebranches_high_ud(R,Rest,Begin,End,Newcount).

  find_middle(tree2(_,_,_,_,_,branchinfo_ud(Midcol,_,_)),Midcol):-!.
  find_middle(tree2(_,Startcol,Name,_,_,_),Midcol):-
  	str_len(Name,Len),
  	Midcol = Startcol + Len - 1 - (Len div 2).
enddef


/*****************************************************************************
		Predicates used to draw the compiled tree to the RIGHT
*****************************************************************************/

ifdef right_in_use
PREDICATES
  tree_write_right(tree2,integerlist,integer,row,col)
  tree_write_right2(integerlist,integer,row,col,string,tree2list,col,branch2list,row,col)
  tree_writelist_right(integerlist,integer,col,col,tree2list,row,col)

CLAUSES
  tree_write_right(Tree,_,_,Scrow,Sccol):-
 	keypressed,
 	mode(mouse),!,
 	readkey(Key),
 	treekeyact(Key,Tree,right,Scrow,Sccol).
  tree_write_right(tree2(Row,Col,Name,Select,[],_),Pl,P,_,_):-!,
	tree_writetext(Pl,P,Row,Col,Name,Select).
  tree_write_right(tree2(Row,Col,Name,Select,Tl,branchinfo_lr(Bcol,Branches)),Pl,P,Sr,Sc):-
	tree_writetext(Pl,P,Row,Col,Name,Select),
	tree_write_right2(Pl,P,Row,Col,Name,Tl,Bcol,Branches,Sr,Sc).

  tree_write_right2([P|Rest],P,Row,Col,Name,Tl,Bcol,Branches,Sr,Sc):-
  	Rest = [P2|_],!,
	str_len(Name,Len),
	Colaftername = Col + Len,
	treeline_hor_high(Colaftername,Bcol,Row),
	Colafterbranch = Bcol + (line_after_branch + 1),
	Bcol1 = Bcol + 1,
	list_len(Branches,0,Bnum),
	list_len(Tl,0,Tnum),
	tree_branch_parms(P2,Bnum,Tnum,Begin,End),
	tree_writebranches_high_lr(Bcol,Branches,Begin,End,1),
	tree_writelist_right(Rest,1,Bcol1,Colafterbranch,Tl,Sr,Sc) ,!.
  tree_write_right2(_,_,Row,Col,Name,Tl,Bcol,Branches,Sr,Sc):-
	str_len(Name,Len),
	Colaftername = Col + Len,
	treeline_hor(Colaftername,Bcol,Row),
	Colafterbranch = Bcol + (line_after_branch + 1),
	Bcol1 = Bcol + 1,
	tree_writebranches_lr(Bcol,Branches),
	tree_writelist_right([],0,Bcol1,Colafterbranch,Tl,Sr,Sc) ,!.


  tree_writelist_right(_,_,_,_,[],_,_):-!.
  tree_writelist_right([],_,C1,C2,[T|Rest],Scrow,Sccol):-!,
  	T = tree2(R,_,_,_,_,_),
  	treeline_hor(C1,C2,R),
        tree_write_right(T,[],0,Scrow,Sccol),
        tree_writelist_right([],0,C1,C2,Rest,Scrow,Sccol).
  tree_writelist_right(Pl,P,C1,C2,[T|Rest],Scrow,Sccol):-
  	Pl = [P|_],!,
  	T = tree2(R,_,_,_,_,_),
  	treeline_hor_high(C1,C2,R),
        tree_write_right(T,Pl,P,Scrow,Sccol),
        tree_writelist_right([],0,C1,C2,Rest,Scrow,Sccol).
  tree_writelist_right(Pl,P,C1,C2,[T|Rest],Scrow,Sccol):-
  	T = tree2(R,_,_,_,_,_),
  	treeline_hor(C1,C2,R),
  	P1 = P + 1,
        tree_write_right(T,[],0,Scrow,Sccol),
        tree_writelist_right(Pl,P1,C1,C2,Rest,Scrow,Sccol).
enddef


/*****************************************************************************
		Predicates used to draw the compiled tree to the LEFT
*****************************************************************************/

ifdef left_in_use
PREDICATES
  tree_write_left(tree2,integerlist,integer,row,col)
  tree_write_left2(integerlist,integer,row,col,tree2list,col,branch2list,row,col)
  tree_writelist_left(integerlist,integer,col,col,tree2list,row,col)

CLAUSES
  tree_write_left(Tree,_,_,Scrow,Sccol):-
 	keypressed,
 	mode(mouse),!,
 	readkey(Key),
 	treekeyact(Key,Tree,left,Scrow,Sccol).
  tree_write_left(tree2(Row,Col,Name,Select,[],_),Pl,P,_,_):-
	tree_writetext(Pl,P,Row,Col,Name,Select),!.
  tree_write_left(tree2(Row,Col,Name,Select,Tl,branchinfo_lr(Bcol,Branches)),Pl,P,Sr,Sc):-
	tree_writetext(Pl,P,Row,Col,Name,Select),
	tree_write_left2(Pl,P,Row,Col,Tl,Bcol,Branches,Sr,Sc).

  tree_write_left2([P|Rest],P,Row,Col,Tl,Bcol,Branches,Sr,Sc):-
  	Rest = [P2|_],!,
	Bcol1 = Bcol + 1,
	treeline_hor_high(Bcol1,Col,Row),
	Colafterbranch = Bcol - line_after_branch,
	list_len(Branches,0,Bnum),
	list_len(Tl,0,Tnum),
	tree_branch_parms(P2,Bnum,Tnum,Begin,End),
	tree_writebranches_high_lr(Bcol,Branches,Begin,End,1),
	tree_writelist_left(Rest,1,Colafterbranch,Bcol,Tl,Sr,Sc) ,!.
  tree_write_left2(_,_,Row,Col,Tl,Bcol,Branches,Sr,Sc):-
	Bcol1 = Bcol + 1,
	treeline_hor(Bcol1,Col,Row),
	Colafterbranch = Bcol - line_after_branch,
	tree_writebranches_lr(Bcol,Branches),
	tree_writelist_left([],0,Colafterbranch,Bcol,Tl,Sr,Sc) ,!.


  tree_writelist_left(_,_,_,_,[],_,_):-!.
  tree_writelist_left([],_,C1,C2,[T|Rest],Scrow,Sccol):-!,
  	T = tree2(R,_,_,_,_,_),
  	treeline_hor(C1,C2,R),
        tree_write_left(T,[],0,Scrow,Sccol),
        tree_writelist_left([],0,C1,C2,Rest,Scrow,Sccol).
  tree_writelist_left(Pl,P,C1,C2,[T|Rest],Scrow,Sccol):-
  	Pl = [P|_],!,
  	T = tree2(R,_,_,_,_,_),
  	treeline_hor_high(C1,C2,R),
        tree_write_left(T,Pl,P,Scrow,Sccol),
        tree_writelist_left([],0,C1,C2,Rest,Scrow,Sccol).
  tree_writelist_left(Pl,P,C1,C2,[T|Rest],Scrow,Sccol):-
  	T = tree2(R,_,_,_,_,_),
  	treeline_hor(C1,C2,R),
  	P1 = P + 1,
        tree_write_left(T,[],0,Scrow,Sccol),
        tree_writelist_left(Pl,P1,C1,C2,Rest,Scrow,Sccol).
enddef


/*****************************************************************************
		Predicates used to draw the compiled tree UPwards
*****************************************************************************/

ifdef up_in_use
PREDICATES
  tree_write_up(tree2,integerlist,integer,row,col)
  tree_write_up2(integerlist,integer,row,tree2list,col,row,branch2list,row,col)
  tree_writelist_up(integerlist,integer,row,row,tree2list,row,col)

CLAUSES
  tree_write_up(Tree,_,_,Scrow,Sccol):-
 	keypressed,
 	mode(mouse),!,
 	readkey(Key),
 	treekeyact(Key,Tree,up,Scrow,Sccol).
  tree_write_up(tree2(Row,Col,Name,Select,[],_),Pl,P,_,_):-
	tree_writetext(Pl,P,Row,Col,Name,Select),!.
  tree_write_up(tree2(Row,Col,Name,Select,Tl,branchinfo_ud(Midcol,Brow,Branches)),Pl,P,Sr,Sc):-
	tree_writetext(Pl,P,Row,Col,Name,Select),
	tree_write_up2(Pl,P,Row,Tl,Midcol,Brow,Branches,Sr,Sc).

  tree_write_up2([P|Rest],P,Row,Tl,Midcol,Brow,Branches,Sr,Sc):-
  	Rest = [P2|_],!,
	Brow1 = Brow + 1,
	treeline_ver_high(Brow1,Row,Midcol),
	Rowafterbranch = Brow - line_after_branch_ud,
	list_len(Branches,0,Bnum),
	list_len(Tl,0,Tnum),
	tree_branch_parms(P2,Bnum,Tnum,Begin,End),
	tree_writebranches_high_ud(Brow,Branches,Begin,End,1),
	tree_writelist_up(Rest,1,Rowafterbranch,Brow,Tl,Sr,Sc) ,!.
  tree_write_up2(_,_,Row,Tl,Midcol,Brow,Branches,Sr,Sc):-
	Brow1 = Brow + 1,
	treeline_ver(Brow1,Row,Midcol),
	Rowafterbranch = Brow - line_after_branch_ud,
	tree_writebranches_ud(Brow,Branches),
	tree_writelist_up([],0,Rowafterbranch,Brow,Tl,Sr,Sc) ,!.


  tree_writelist_up(_,_,_,_,[],_,_):-!.
  tree_writelist_up([],_,R1,R2,[T|Rest],Scrow,Sccol):-!,
  	find_middle(T,C),
  	treeline_ver(R1,R2,C),
        tree_write_up(T,[],0,Scrow,Sccol),
        tree_writelist_up([],0,R1,R2,Rest,Scrow,Sccol).
  tree_writelist_up(Pl,P,R1,R2,[T|Rest],Scrow,Sccol):-
  	Pl = [P|_],!,
  	find_middle(T,C),
  	treeline_ver_high(R1,R2,C),
        tree_write_up(T,Pl,P,Scrow,Sccol),
        tree_writelist_up([],0,R1,R2,Rest,Scrow,Sccol).
  tree_writelist_up(Pl,P,R1,R2,[T|Rest],Scrow,Sccol):-
  	find_middle(T,C),
  	treeline_ver(R1,R2,C),
  	P1 = P + 1,
        tree_write_up(T,[],0,Scrow,Sccol),
        tree_writelist_up(Pl,P1,R1,R2,Rest,Scrow,Sccol).
enddef


/*****************************************************************************
		Predicates used to draw the compiled tree DOWNwards
*****************************************************************************/

ifdef down_in_use
PREDICATES
  tree_write_down(tree2,integerlist,integer,row,col)
  tree_write_down2(integerlist,integer,row,tree2list,col,row,branch2list,row,col)
  tree_writelist_down(integerlist,integer,row,row,tree2list,row,col)

CLAUSES
  tree_write_down(Tree,_,_,Scrow,Sccol):-
 	keypressed,
 	mode(mouse),!,
 	readkey(Key),
 	treekeyact(Key,Tree,down,Scrow,Sccol).
  tree_write_down(tree2(Row,Col,Name,Select,[],_),Pl,P,_,_):-
	tree_writetext(Pl,P,Row,Col,Name,Select),!.
  tree_write_down(tree2(Row,Col,Name,Select,Tl,branchinfo_ud(Midcol,Brow,Branches)),Pl,P,Sr,Sc):-
	tree_writetext(Pl,P,Row,Col,Name,Select),
	tree_write_down2(Pl,P,Row,Tl,Midcol,Brow,Branches,Sr,Sc).

  tree_write_down2([P|Rest],P,Row,Tl,Midcol,Brow,Branches,Sr,Sc):-
  	Rest = [P2|_],!,
	Brow1 = Brow + 1,
	Row1 = Row + 1,
	treeline_ver_high(Row1,Brow,Midcol),
	Rowafterbranch = Brow1 + line_after_branch_ud,
	list_len(Branches,0,Bnum),
	list_len(Tl,0,Tnum),
	tree_branch_parms(P2,Bnum,Tnum,Begin,End),
	tree_writebranches_high_ud(Brow,Branches,Begin,End,1),
	tree_writelist_down(Rest,1,Brow1,Rowafterbranch,Tl,Sr,Sc),!.
  tree_write_down2(_,_,Row,Tl,Midcol,Brow,Branches,Sr,Sc):-
	Brow1 = Brow + 1,
	Row1 = Row + 1,
	treeline_ver(Row1,Brow,Midcol),
	Rowafterbranch = Brow1 + line_after_branch_ud,
	tree_writebranches_ud(Brow,Branches),
	tree_writelist_down([],0,Brow1,Rowafterbranch,Tl,Sr,Sc),!.


  tree_writelist_down(_,_,_,_,[],_,_):-!.
  tree_writelist_down([],_,R1,R2,[T|Rest],Scrow,Sccol):-!,
  	find_middle(T,C),
  	treeline_ver(R1,R2,C),
        tree_write_down(T,[],0,Scrow,Sccol),
        tree_writelist_down([],0,R1,R2,Rest,Scrow,Sccol).
  tree_writelist_down(Pl,P,R1,R2,[T|Rest],Scrow,Sccol):-
  	Pl = [P|_],!,
  	find_middle(T,C),
  	treeline_ver_high(R1,R2,C),
        tree_write_down(T,Pl,P,Scrow,Sccol),
        tree_writelist_down([],0,R1,R2,Rest,Scrow,Sccol).
  tree_writelist_down(Pl,P,R1,R2,[T|Rest],Scrow,Sccol):-
  	find_middle(T,C),
  	treeline_ver(R1,R2,C),
  	P1 = P + 1,
        tree_write_down(T,[],0,Scrow,Sccol),
        tree_writelist_down(Pl,P1,R1,R2,Rest,Scrow,Sccol).
enddef


/*****************************************************************************
	Predicates used to compile and draw the tree
*****************************************************************************/

PREDICATES
  mk_tree(dir,tree,tree2)
  write_tree(dir,tree2,integerlist)

CLAUSES
ifdef right_in_use
  mk_tree(right,Tree,Compiled):-
 	mk_tree_right(Tree,Compiled,0,0,_,_).
enddef
%-----------------
ifdef left_in_use
  mk_tree(left,Tree,Compiled):-
	mk_tree_left(Tree,Compiled,0,0,_,_).
enddef
%-----------------
ifdef down_in_use
  mk_tree(down,Tree,Compiled):-
	mk_tree_down(Tree,Compiled,0,0,_,_).
enddef
%-----------------
ifdef up_in_use
  mk_tree(up,Tree,Compiled):-
	mk_tree_up(Tree,Compiled,0,0,_,_).
enddef


%-----------------
ifdef right_in_use
  write_tree(right,Tree,Path):-
	tree_write_right(Tree,Path,1,row_scroll,col_scroll).
enddef
%-----------------
ifdef left_in_use
  write_tree(left,Tree,Path):-
	tree_write_left(Tree,Path,1,row_scroll,col_scroll).
enddef
%-----------------
ifdef down_in_use
  write_tree(down,Tree,Path):-
	tree_write_down(Tree,Path,1,row_scroll,col_scroll).
enddef
%-----------------
ifdef up_in_use
  write_tree(up,Tree,Path):-
	tree_write_up(Tree,Path,1,row_scroll,col_scroll).
enddef


/*****************************************************************************
		Key stroke interpreter and TREEMENU
*****************************************************************************/

PREDICATES
ifndef globdecl
  treemenu(dir,tree,selector)
  treemenu(dir,tree,selector,movemode,integerlist)
  maketree(dir,tree)
enddef

CLAUSES
  treemenu(Dir,Tree,Choice):-
  	treemenu(Dir,Tree,Choice,mouse,[]).

  treemenu(Dir,Tree,Choice,Initmode,Path):-
 	mk_tree(Dir,Tree,Comptree),
 	Comptree = tree2(Row,Col,_,Select,_,_),
 	set_mode(Initmode),
 	init_startpos(Dir,Row,Col,Select),
 	repeat,
	write_tree(Dir,Comptree,Path),
	readkey(KEY),
	treekeyact(KEY,Comptree,Dir,row_scroll,col_scroll),!,
	not(KEY=esc),
	current_node(_,_,_,Choice),!.

  maketree(Dir,Tree):-
 	mk_tree(Dir,Tree,Comptree),
 	Comptree = tree2(Row,Col,_,Select,_,_),
 	set_mode(mouse),
 	init_startpos(Dir,Row,Col,Select),
	write_tree(Dir,Comptree,[]).
