/*
==========================================================================
  	WINDOWS.PRO
								 
  	Generic Windowing Operations 
                       (used in Parser-Generator Demos)
  									 
  	Top-level predicates: window_ChangeFrame/1			 
  			      window_ChangeFrame/4			 
  			      window_Make/5				 
  			      window_RemoveAll/0			 
  			      window_Shift/1				 
  			      window_WaitForUserToPressAKey/0		 
  									 
  	Abbreviations used in this file: attr	= attribute		 
  					 col 	= column		 
  					 max 	= maximum		 
  									 
	Copyright (C) 1990 Prolog Development Center
===========================================================================
*/

CONSTANTS
  cols_used_by_frame	= 2  	
  double_border		= "ɻȼͺ"
  leftmost_col		= 0
  max_window_number	= 255
  no_frame		= 0
  rows_used_by_frame	= 2
  single_border		= "ڿĳ"
  topmost_row		= 0
  
include "misc\\colors.pre"

DOMAINS
  ATTR		= INTEGER
  COLS		= INTEGER
  DESCRIPTION	= full; half(LOCATION)
  FRAME_CHARS	= STRING
  HEADER	= STRING
  JUSTIFICATION	= center; left_justify; right_justify
  LOCATION	= bottom; left; right; top
  ROWS		= INTEGER
  TEXT		= STRING
  WINDOW_NUMBER	= INTEGER

PREDICATES
		allow_for_window_frame(ATTR, ROWS, COLS, ROWS, COLS)
		change_frame(ROWS, JUSTIFICATION, TEXT, ATTR)
		change_frame_aux(ROWS, COLS, TEXT, ATTR)
  nondeterm	generate_numbers(WINDOW_NUMBER, WINDOW_NUMBER, WINDOW_NUMBER)
		get_window_info(DESCRIPTION, ROWS, COLS, ROWS, COLS)
		window_ChangeFrame(FRAME_CHARS)
		window_ChangeFrame(LOCATION, JUSTIFICATION, TEXT, ATTR)
		window_Make(WINDOW_NUMBER, ATTR, ATTR, HEADER, DESCRIPTION)
		window_RemoveAll
		window_Shift(WINDOW_NUMBER)
		window_WaitForUserToPressAKey
  
CLAUSES
  allow_for_window_frame(no_frame, ROWS, COLS, ROWS, COLS) :- !.
  allow_for_window_frame(_, ROWS1, COLS1, ROWS2, COLS2) :-
  	ROWS2 = ROWS1 - rows_used_by_frame,
  	COLS2 = COLS1 - cols_used_by_frame.

  change_frame(START_ROW, center, TEXT, ATTR) :-
  	makewindow(_, _, _, _, _, _, _, COLS),
  	str_len(TEXT, LENGTH_OF_TEXT),
  	START_COL = trunc((COLS - LENGTH_OF_TEXT) / 2),
	change_frame_aux(START_ROW, START_COL, TEXT, ATTR).
  change_frame(START_ROW, left_justify, TEXT, ATTR) :-
	START_COL = 1,
	change_frame_aux(START_ROW, START_COL, TEXT, ATTR).
  change_frame(START_ROW, right_justify, TEXT, ATTR) :-
  	makewindow(_, _, _, _, _, _, _, COLS),
  	str_len(TEXT, LENGTH_OF_TEXT),
  	START_COL = COLS - LENGTH_OF_TEXT - 1,
	change_frame_aux(START_ROW, START_COL, TEXT, ATTR).

  change_frame_aux(START_ROW, START_COL, TEXT, ATTR) :-
  	str_len(TEXT, LENGTH_OF_TEXT),
	expandwindow,
  	field_str(START_ROW, 
		  START_COL,
  		  LENGTH_OF_TEXT, 
  		  TEXT),
	field_attr(START_ROW, 
		   START_COL,
		   LENGTH_OF_TEXT, 
		   ATTR),
  	compresswindow.

  generate_numbers(INDEX, INDEX, _).
  generate_numbers(INDEX, START, END) :-
  	START < END,
  	NEW_START = START + 1,
  	generate_numbers(INDEX, NEW_START, END).

  get_window_info(full, topmost_row, leftmost_col, SCREEN_ROWS, SCREEN_COLS) :-
      % Use entire screen.
      	textmode(SCREEN_ROWS, SCREEN_COLS).
  get_window_info(half(bottom), WINDOW_ROWS, leftmost_col, WINDOW_ROWS, SCREEN_COLS) :-
      % Use bottom half of screen.
	textmode(SCREEN_ROWS, SCREEN_COLS),
	WINDOW_ROWS = trunc(SCREEN_ROWS / 2).
  get_window_info(half(left), topmost_row, leftmost_col, SCREEN_ROWS, WINDOW_COLS) :-
      % Use left half of screen.
	textmode(SCREEN_ROWS, SCREEN_COLS),
	WINDOW_COLS = trunc(SCREEN_COLS / 2).
  get_window_info(half(right), topmost_row, WINDOW_COLS, SCREEN_ROWS, WINDOW_COLS) :-
      % Use right half of screen.
	textmode(SCREEN_ROWS, SCREEN_COLS),
	WINDOW_COLS = trunc(SCREEN_COLS / 2).
  get_window_info(half(top), topmost_row, leftmost_col, WINDOW_ROWS, SCREEN_COLS) :-
      % Use top half of screen.  		
	textmode(SCREEN_ROWS, SCREEN_COLS),
	WINDOW_ROWS = trunc(SCREEN_ROWS / 2).

  window_ChangeFrame(FRAME_CHARS) :-
  	makewindow(_, _, FRAME_ATTR, HEADER, _, _, _, _, _, POSITION, _),
  	framewindow(FRAME_ATTR, HEADER, POSITION, FRAME_CHARS).
  	
  window_ChangeFrame(bottom, JUSTIFICATION, TEXT, ATTR) :-
  	makewindow(_, _, _, _, _, _, ROWS, _),
  	START_ROW = ROWS - 1,
	change_frame(START_ROW, JUSTIFICATION, TEXT, ATTR).
  window_ChangeFrame(top, JUSTIFICATION, TEXT, ATTR) :-
	change_frame(topmost_row, JUSTIFICATION, TEXT, ATTR).
  
  window_Make(WINDOW_NUMBER, WINDOW_ATTR, FRAME_ATTR, HEADER, TYPE) :-
	get_window_info(TYPE, START_ROW, START_COL, WINDOW_ROWS, WINDOW_COLS),	
	makewindow(WINDOW_NUMBER,
		   WINDOW_ATTR,
		   FRAME_ATTR,
		   HEADER,
		   START_ROW,
		   START_COL,
		   WINDOW_ROWS,
		   WINDOW_COLS).

  window_RemoveAll :-	
      % Try to remove every legal WINDOW_NUMBER.
  	generate_numbers(WINDOW_NUMBER, 1, max_window_number),	
	trap(shiftwindow(WINDOW_NUMBER), _, fail),
  	trap(removewindow, _, fail),
	fail.
  window_RemoveAll.

  window_Shift(WINDOW_NUMBER) :-
	window_ChangeFrame(single_border),
	shiftwindow(WINDOW_NUMBER),
	window_ChangeFrame(double_border).

  window_WaitForUserToPressAKey :-
	makewindow(_, _, FRAME_ATTR, _, _, _, ROWS1, COLS1),
	allow_for_window_frame(FRAME_ATTR, ROWS1, COLS1, ROWS2, COLS2),
	START_ROW = ROWS2 - 1,
	
      % Save attr & text of last row of window.		
	field_attr(START_ROW, leftmost_col, COLS2, OLD_ATTR),
	field_str(START_ROW, leftmost_col, COLS2, OLD_TEXT),
	
      % Prompt user, and wait for the reply.
	HIGH_INTENSITY_ATTR = OLD_ATTR + 8,
 	field_attr(START_ROW, leftmost_col, COLS2, HIGH_INTENSITY_ATTR),
	field_str(START_ROW, leftmost_col, COLS2, "Press any key."),
	readchar(_),
	
      % Replace saved attr & text.
	field_attr(START_ROW, leftmost_col, COLS2, OLD_ATTR),
	field_str(START_ROW, leftmost_col, COLS2, OLD_TEXT).

