/*
===========================================================================   									 
	XCALC2.PRO
		
  	Calculator Demo of Parser Generator			 
  	---------------------------------------------------------	 
  	Common domains/predicates/etc. and main goal 
  									 
  	Abbreviations used in this file: 				 
  		cursortok	= token and cursor			 
  		par		= parser				 
  		pro		= prolog				 
  		sca		= scanner				 
   		tok		= token					 
   		tokl		= list of cursortoks			 
   		ui		= user interface			 

	Copyright (C) 1990 Prolog Development Center  									 
===========================================================================
*/

check_determ

CONSTANTS
  dialog_window	= 1

DOMAINS
  CURSOR	= INTEGER
  CURSORTOK 	= t(TOK, CURSOR)
  MESSAGE	= STRING
  RESULT	= REAL
  SOURCE	= STRING
  TOKL 		= CURSORTOK*

include "xcalc.dom"		% Parser domains (created by Parser Generator).

DATABASE
  determ	error(MESSAGE, CURSOR)

PREDICATES
  expect(CURSORTOK, TOKL, TOKL)
  is_a_deeper_error(CURSOR)
  replace_error(MESSAGE, CURSOR)
  syntax_error(MESSAGE, TOKL)

				% Contained Predicates
				% --------------------
include "windows.pro"		% window_...
include "xcalc2.sca"		% scan/3
include "xcalc.par"		% s_expr/3
include "xcalc.run"		% calculate/2
include "xcalc2.ui"		% user_interface/0


CLAUSES
  expect(TOK, [TOK|L], L) :- !.
  expect(t(TOKEN, _), [t(_, CURSOR)|_], _) :-
  	is_a_deeper_error(CURSOR),
	string_token(STRING, TOKEN),
  	format(MESSAGE, "Expected %.", STRING),
  	replace_error(MESSAGE, CURSOR),
  	fail.
  expect(t(TOKEN, _), [], _) :-
	string_token(STRING, TOKEN),
  	format(MESSAGE, "Expected % (premature end of text).", STRING),
  	replace_error(MESSAGE, end_of_text),
  	fail.

  is_a_deeper_error(CURSOR) :-
  	error(_, OLD_CURSOR),
  	!,
  	CURSOR > OLD_CURSOR.
  is_a_deeper_error(_).
  	
  replace_error(MESSAGE, CURSOR) :-
  	retractall(error(_, _)),
  	assert(error(MESSAGE, CURSOR)).
  
  syntax_error(PROD, [t(_, CURSOR)|_]) :-
  	is_a_deeper_error(CURSOR),
	format(MESSAGE, "Syntax error in %.", PROD),
	replace_error(MESSAGE, CURSOR).
  syntax_error(PROD, []) :-
	format(MESSAGE, "Syntax error in % (premature end of text).", PROD),
  	replace_error(MESSAGE, end_of_text).  	

GOAL
  window_Make(dialog_window, 
  	      white_on_blue, 
	      white_on_black,
  	      " Simple Calculator (with improved error reporting)", 
	      full),
  window_ChangeFrame(bottom, 
  		     center, 
  		     " Legal operators include *, /, +, -, ^, and parentheses. ",
  		     blue_on_white),
  user_interface,
  removewindow.  





