/*
============================================================================
	XCALCULA.PRO							 
  									 
  	XCALCULA: Programming Language Demo of Parser Generator		 
  	-----------------------------------------------------------	 
  	Common domains/predicates/etc. and main goal 
  									 
  	Abbreviations used in this file: 				 
  		cs		= calcula source			 
  		cursortok	= token and cursor			 
  		eq		= equal to				 
  		ge		= greater than or equal to		 
  		gt		= greater than				 
  		le		= less than or equal to			 
  		lt		= less than				 
  		ne		= not equal to				 
  		op		= operator 	 			 
  		par		= parser				 
  		pro		= prolog				 
   		psf		= prolog source file			 
  		sca		= scanner				 
   		tok		= token					 
   		tokl		= list of cursortoks			 
   		ui		= user interface			 
  									 
  	Copyright (C) 1990 Prolog Development Center  			 					 
===========================================================================
*/

check_determ

CONSTANTS
  calcula_source_window		= 1
  dialog_window			= 2

DOMAINS
  CURSOR	= INTEGER
  CURSORTOK 	= t(TOK, CURSOR)
  MESSAGE	= STRING
  OP		= eq; ne; lt; gt; le; ge
  RESULT	= REAL
  SOURCE	= STRING
  TOKL 		= CURSORTOK*

include "xcalcula.dom"		% Parser domains (created by Parser Generator).

DATABASE
  determ	error(MESSAGE, CURSOR)

PREDICATES
  expect(CURSORTOK, TOKL, TOKL)
  is_a_deeper_error(CURSOR)
  replace_error(MESSAGE, CURSOR)
  syntax_error(MESSAGE, TOKL)

				% Contained Predicates
				% --------------------
include "windows.pro"		% window_...
include "xcalcula.sca"		% scan/3
include "xcalcula.par"		% s_program/3
include "xcalcula.run"		% program_...
include "xcalcula.cs"		% calcula_source_...
include "xcalcula.psf"		% prolog_source_file_...
include "xcalcula.ui"		% user_interface/0

CLAUSES
  expect(TOKEN, [TOKEN|TAIL], TAIL) :- !.
  expect(t(TOKEN, _), [t(_, CURSOR)|_], _) :-
  	is_a_deeper_error(CURSOR),
	string_token(STRING, TOKEN),
  	format(MESSAGE, "Expected %.", STRING),
  	replace_error(MESSAGE, CURSOR),
  	fail.
  expect(t(TOKEN, _), [], _) :-
	string_token(STRING, TOKEN),
  	format(MESSAGE, "Expected % (premature end of text).", STRING),
	calcula_source_Retrieve(SOURCE),
	str_len(SOURCE, END_OF_TEXT),
  	replace_error(MESSAGE, END_OF_TEXT),
  	fail.

  is_a_deeper_error(CURSOR) :- error(_, OLD_CURSOR), !,	CURSOR > OLD_CURSOR.
  is_a_deeper_error(_).
  	
  replace_error(MESSAGE, CURSOR) :-
  	retractall(error(_, _)),
  	assert(error(MESSAGE, CURSOR)).

  syntax_error(PRODUCTION, [t(_, CURSOR)|_]) :-
  	is_a_deeper_error(CURSOR),
	format(MESSAGE, "Syntax error in %.", PRODUCTION),
  	replace_error(MESSAGE, CURSOR).
  syntax_error(PRODUCTION, []) :-
	format(MESSAGE, 
	       "Syntax error in % (premature end of text).", 
	       PRODUCTION),
	calcula_source_Retrieve(SOURCE),
	str_len(SOURCE, END_OF_TEXT),
  	replace_error(MESSAGE, END_OF_TEXT).  	

GOAL
  calcula_source_Load,
  window_Make(calcula_source_window, 
  	      white_on_blue, 
  	      white_on_black, 
  	      " Source ", 
  	      half(top)),
  calcula_source_Retrieve(SOURCE),
  write(SOURCE),
  window_Make(dialog_window, 
  	      white_on_red, 
  	      white_on_black, 
  	      " Dialog ", 
  	      half(bottom)),
  user_interface,
  window_RemoveAll,
  calcula_source_Save.
  
