/*
==========================================================================
	XCLUB.PRO					
  									 
  	Demo of screen handler: Common interest group		 	 
  									 
	Copyright (C) 1990 Prolog Development Center  									 
===========================================================================
*/

check_determ 

/************************************************************************
 *			Domains						*
 ************************************************************************/

include "tdoms.pro"

DOMAINS	
  ADDRESS	= STRING
  AGE		= INTEGER
  FIELD_NAME	= SYMBOL
  FILE		= textfile
  INTEREST	= SYMBOL
  INTERESTS	= INTEREST*
  NAME		= STRING
  SEX		= m or f
  TYPE		= int(); str(); real()


/************************************************************************
 *			Database Predicates				*
 ************************************************************************/

DATABASE 
  field(FIELD_NAME, TYPE, ROW, COL, LEN) 	
  person(NAME, ADDRESS, AGE, SEX, INTERESTS)
  txtfield(ROW, COL, LEN, STRING)			
  value(FIELD_NAME, STRING)			
  windowsize(ROW, COL)


/************************************************************************
 *			Include Tools					*
 ************************************************************************/

include "tpreds.pro"
include "ui\\menu.pro"
include "ui\\status.pro"
include "ui\\lineinp.pro"
include "ui\\filename.pro"
include "screen\\scrhnd.pro"				% Or vscrhnd.pro


/************************************************************************
 *			Support Predicates				*
 ************************************************************************/

PREDICATES
  		get_person(DBASEDOM)
		list_db
  nondeterm	member(INTEREST, INTERESTS)  
		my_write(DBASEDOM)
		process(INTEGER)
		put_person(DBASEDOM)
		string_interests(STRING, INTERESTS)
		string_sex(STRING, SEX)
		
CLAUSES
  get_person(person(NAME, ADDRESS, AGE2, SEX2, INTERESTS2)) :-
	value(name, NAME),
	value(address, ADDRESS),
	value(age, AGE1), 
	str_int(AGE1, AGE2),
	
	value(sex, SEX1), 
	string_sex(SEX1, SEX2),
	
	value(interests, INTERESTS1), 
	string_interests(INTERESTS1, INTERESTS2),
	!.

  list_db :-
	person(NAME, ADDRESS, AGE, SEX, INTERESTS),
	my_write(person(NAME, ADDRESS, AGE, SEX, INTERESTS)),
	fail.
  list_db.

  member(ELEMENT, [ELEMENT|_]).
  member(ELEMENT, [_|TAIL]) :- member(ELEMENT, TAIL).

  my_write(X) :- write(X), nl.

  process(0).
  process(1) :-
	changestatus("Type in a name for the database."),
	readfilename(10, 10, 7, 7, dba, "xclub.dba", FILE),
	openwrite(textfile, FILE),
	writedevice(textfile),
	list_db,
	closefile(textfile).
  process(2) :-
	retractall(value(_, _)),
	createwindow(off),
	changestatus(" Input new person'details.   Move cursor with arrows.      F10:end"),
	scrhnd(off, KEY),
	not(KEY = esc),
	get_person(PERSON),
	assert(PERSON),
	fail.
  process(2) :- removewindow.
  process(3) :-
	retractall(value(_, _)),
	createwindow(off),
	changestatus(" To find old record, give a name and press F10."),
	scrhnd(off, KEY1),
	not(KEY1 = esc),
	value(name, NAME),
	person(NAME, ADDRESS, AGE, SEX, INTERESTS),
	put_person(person(NAME, ADDRESS, AGE, SEX, INTERESTS)),
	changestatus("Now you can modify the data. Press F10 to finish."),
	scrhnd(off, KEY2),
	not(KEY2 = esc),
	retract(person(NAME, ADDRESS, AGE, SEX, INTERESTS)),
	get_person(PERSON),
	asserta(PERSON),
	removewindow,
	!.
  process(3) :- removewindow.
  process(4) :-
	retractall(value(_, _)),
	createwindow(off),
	changestatus(" Type some interests (separated by commas) and then press F10."),
	scrhnd(off, KEY1),
	not(KEY1 = esc),
	value(interests, USER_INTERESTS1), 
	string_interests(USER_INTERESTS1, USER_INTERESTS2),
	person(NAME, ADDRESS, AGE, SEX, PERSON_INTERESTS),
	member(INTEREST, USER_INTERESTS2),
	member(INTEREST, PERSON_INTERESTS),
	put_person(person(NAME, ADDRESS, AGE, SEX, PERSON_INTERESTS)),
	changestatus(" To inspect each matching entry, press F10 repeatedly."),
	scrhnd(off, KEY2),
	not(KEY2 = esc),
	fail.
  process(4) :- removewindow.
  process(5) :-	clearwindow, list_db.

  put_person(person(NAME, ADDRESS, AGE1, SEX1, INTERESTS1)) :-
	retractall(value(_, _)),
	assert(value(name, NAME)),
	assert(value(address, ADDRESS)),
	str_int(AGE2, AGE1),
	assert(value(age, AGE2)), 
	string_sex(SEX2, SEX1),	
	assert(value(sex, SEX2)),
	string_interests(INTERESTS2, INTERESTS1),
	assert(value(interests, INTERESTS2)).

  string_interests("", []) :- !.
  string_interests(STRING, LIST) :-
  	bound(STRING),
  	fronttoken(STRING, ",", REST_OF_STRING),
  	!,
  	string_interests(REST_OF_STRING, LIST).
  string_interests(STRING, [INTEREST|TAIL]) :-
  	bound(STRING),
  	fronttoken(STRING, INTEREST, REST_OF_STRING),
  	!,
  	string_interests(REST_OF_STRING, TAIL).
  string_interests(STRING, [INTEREST]) :- 
  	bound(INTEREST),
  	!,
  	INTEREST = STRING.
  string_interests(STRING, [INTEREST|TAIL]) :-
  	bound(INTEREST),
	string_interests(REST_OF_INTERESTS, TAIL),
	format(STRING, "%,%", INTEREST, REST_OF_INTERESTS).

  string_sex("m", m).
  string_sex("f", f).
  

/************************************************************************
 *			Field Actions					*
 ************************************************************************/

  field_action(_) :- fail.
  

/************************************************************************
 *			Field Values					*
 ************************************************************************/

  field_value(FIELD_NAME, VALUE) :- value(FIELD_NAME, VALUE), !.


/************************************************************************
 *			No Input Fields					*
 ************************************************************************/

  noinput(_) :- fail.


/************************************************************************
 *			Goal						*
 ************************************************************************/
 
GOAL
  textmode(ROWS, COLS),
  WINDOW_ROWS = ROWS - 1,
  makewindow(77, 36, 0, "", 0, 0, WINDOW_ROWS, COLS),
  makestatus(112, ""),
  consult("xclub.scr"),
  consult("xclub.dba"),
  repeat,
  changestatus(" Select an option."),
  menu(10, 25, 71, 23,
	["Save new database",
	 "Input new person",
	 "Update an entry",
	 "Find people with your interests",
	 "List database"],
	 "CHOICES",
	 4, 
	 CHOICE),
  process(CHOICE),
  CHOICE = 0,			% Terminate if Esc was pressed.
  !.
