/*
===========================================================================
	XCOMMU

	Demonstation Serial Communications package

	Copyright (C) 1990 Prolog Development Center
===========================================================================
*/

project  "XCOMMU"

CONSTANTS
 cfg_file="xcommu.dat"

 inputBufSize  = 1000
 outputBufSize = 1000
 menu_esc      = 27

 cook_incoming_Cr=1

GLOBAL DOMAINS
 setting = baud; parity; stop; data
 file = capture_file; infile; outfile

 include "tdoms.pro"
 include "serial\\comglobs.pre"

 include "tpreds.pro"
 include "ui\\menu.pro"
 include "ui\\status.pro"
 include "ui\\filename.pro"


/*  these values are recorded and re-read when the system exits/restarts */

DATABASE - configuration
determ	port(integer)
determ	protocol(integer)
determ	full_duplex
determ	phone_no(string)
determ	capture_file(string)
	value(setting, integer)

DATABASE - internal_use
 value_text(setting, integer, string)
 value_name(setting,string)
 last_selected(integer)
 determ capture_on
PREDICATES
 main
 main_aux
 main_exit(integer)
 nondeterm menu_item(string)
 do_choice(integer)
 bits_list(integer,integerlist,integerlist)
 list_bits(integerlist,integer,integer)
 close_open(integer,integer)
 dial
 dial(string)
 dial_format(string)
 reset_protocol
 reset_port_parameter(setting)
 effect_change_in(setting,integer,integer)
 get_text(string,string,string)
 tx_str(String)
 hang_up
 load_config
 do_menu
 terminal_mode
 capture_toggle
 upload_file
 download_file
 remove_style(integer)
 chk_rdch
 chk_wrch
 rdch_crlf_rs232(char)
 see_if_menu(char)
 cook_13(char)
 wrch_crlf(char)
 trans_ch(char)
 init_port
 may_log(char)
 log_write(Char)
CLAUSES

% Returns menu text reflecting current state of each setting
%  these MUST remain in this order, else the menu choice number
%  will be wrong
menu_item(ComText) :-
	port(ComPort),
	format(ComText, "Port     COM%",ComPort).
menu_item(SettingText) :-
	value_name(Setting,_),	 % this generates, keeps in order
	value(Setting,Value),	 %  no matter who last asserta'd
	value_text(Setting, Value, SettingText).
menu_item("Duplex   Full") :- full_duplex.
menu_item("Duplex   Half") :- not(full_duplex).
menu_item("Protocol").
menu_item("Capture    On") :- capture_on.
menu_item("Capture   Off") :- not(capture_on).
menu_item("Dial").
menu_item("Hang Up").
menu_item("Upload").
menu_item("Download").
menu_item("Quit").


do_choice(1) :-
	retract(port(1)),!,
	close_open(1,2),
	assert(port(2)).
do_choice(1) :-
	retract(port(_)),!,
	close_open(2,1),
	asserta(port(1)).
do_choice(2) :- reset_port_parameter(baud),!.
do_choice(3) :- reset_port_parameter(data),!.
do_choice(4) :- reset_port_parameter(stop),!.
do_choice(5) :- reset_port_parameter(parity),!.
do_choice(6) :- retract(full_duplex),!.
do_choice(6) :- !, assert(full_duplex).
do_choice(7) :- reset_protocol,!.
do_choice(8) :- capture_toggle,!.
do_choice(9) :- dial,!.
do_choice(10) :- hang_up,!.
do_choice(11) :- upload_file,!.
do_choice(12) :- download_file,!.
do_choice(13) :- !, exit(0).
do_choice(_).

%%% default settings in case no config_file found
%%% ==============================================
port(1).
value(baud,b2400).
value(data, data8).
value(stop, stop1).
value(parity, parity_none).
protocol(0).
phone_no("").
capture_file("COMM.LOG").
last_selected(1).

%% text for the sub-menus
%% ======================
value_text(baud, b110,		"Baud      110").
value_text(baud, b150,		"Baud      150").
value_text(baud, b300,		"Baud      300").
value_text(baud, b600,		"Baud      600").
value_text(baud, b1200, 	"Baud     1200").
value_text(baud, b2400, 	"Baud     2400").
value_text(baud, b4800, 	"Baud     4800").
value_text(baud, b9600, 	"Baud     9600").
value_text(baud, b19200,	"Baud    19200").
value_text(baud, b38400,	"Baud    38400").
value_text(data, data5, 	"Data Bits   5").
value_text(data, data6, 	"Data Bits   6").
value_text(data, data7, 	"Data Bits   7").
value_text(data, data8, 	"Data Bits   8").
value_text(stop, stop1, 	"Stop Bits   1").
value_text(stop, stop2, 	"Stop Bits   2").
value_text(parity, parity_none, "Parity   None").
value_text(parity, parity_odd,	"Parity   Odd").
value_text(parity, parity_even, "Parity   Even").

value_name(baud, "Baud Rate").
value_name(data, "Data Bits").
value_name(stop, "Stop Bits").
value_name(parity,"Parity").

reset_port_parameter(Setting) :-
	value(Setting, OldValue),
	ListVal=OldValue+1,
	findall(Text, value_text(Setting, _, Text), Choices),
	value_name(Setting, Name),
	menu(8,10,7,7, Choices, Name, ListVal, NewListVal),
	NewListVal>0,
	NewValue=NewListVal-1,
	retractall(value(Setting, _)),
	asserta(value(Setting, NewValue)),
	port(Port),
	effect_change_in(Setting, Port, NewValue),!.

effect_change_in(baud, Port, NewBaud) :-!, %can be done without closing
	set_baudrate_RS232(Port, NewBaud).
effect_change_in( _, Port, _) :-	   % all others, close, reopen
	close_open(Port,Port).

reset_protocol :-
	protocol(OldP),
	bits_list(OldP, [1,2,3],WasList),
	menu_mult(10,10,7,7,
		["XON/XOFF",
		 "RTS/CTS" ,
		 "DSR/DTR"],"Protocol",WasList, NewList),
	NewList=[_|_],
	list_bits(NewList, 0, New),
	retractall(protocol(_)),
	asserta(protocol(New)),
	port(Port),
	close_open(Port,Port).

bits_list(Bits, [H|T], [H|T2]) :-
	H2=H-1,
	bitleft(1,H2,M),
	bitand(Bits, M, Res),
	Res<>0,!,
	bits_list(Bits, T, T2).
bits_list(Bits, [_|T], T2) :- !, bits_list(Bits,T,T2).
bits_list(_,[],[]).

list_bits([H|T], In, Res) :-
	H2=H-1,
	bitleft(1,H2, R),
	In2=In+R,
	!, list_bits(T,In2,Res).
list_bits(_, X,X).

dial :-
	phone_no(Phone),
	get_text(" Phone Number ", Phone, NewPhone),
	retractall(phone_no(_)),
	asserta(phone_no(NewPhone)),
	dial(NewPhone).

dial(PhoneNo) :-
	dial_format(F),
	format(Codes, F, PhoneNo),
	nl,
	write(Codes),
	tx_str(Codes).


hang_up :-
	port(Port),
	delOutbuf_RS232(Port),
	sleep(100),
	tx_str("+++"),
	sleep(100),
	tx_str("ATH0\r\n").

tx_Str(S) :-
	port(Port),
	frontchar(S,C,Rest),
	txch_RS232(Port,C),
	!, tx_Str(Rest).
tx_Str("").


close_open(Old, New) :-
	closeRS232(Old),
	value(baud,BaudRate),
	protocol(Protocol),
	value(parity,Parity),
	value(data,DataBits),
	value(stop,StopBits),!,
	openRS232(New, inputBufSize, outputBufSize, BaudRate, Parity, DataBits, StopBits, Protocol).

main:-	trap(main_aux, Code, main_exit(Code)),!.
main.

main_Exit(_) :-
	port(Port),
	closeRS232(Port),
	fail.
main_exit(Code) :-
	removewindow(2,1),
	removewindow(81,1),
	removewindow(3,1),
	removewindow(1,1),
	Code=0,!,
	save(cfg_file, configuration).
main_exit(Error) :-
	exit(Error).

main_aux :-
	textmode(Rows, Columns),
	Offset = (Columns - 30)/2,
	High = Rows -1, ROWLim=ROWS-2,
	makewindow(1, 7, 7, " PDC-Comm ",0,0,High,Columns),
	expandwindow(),
	cursor(RowLim, Offset),
	write("ESC: toggle Menu/Terminal Mode"),
	compresswindow(),
	cursor(0,0),
	load_config,
	init_port,
	terminal_mode.

init_port :-
	port(New),
	value(baud,BaudRate),
	protocol(Protocol),
	value(parity,Parity),
	value(data,DataBits),
	value(stop,StopBits),!,
	openRS232(New, inputBufSize, outputBufSize, BaudRate, Parity, DataBits, StopBits, Protocol).

load_config :-
	existfile(cfg_file),!,
	retractall(_, configuration),
	consult(cfg_file,configuration).
load_config.

do_menu :-
	makewindow(3, 7,  0, "", 2, 2, 15, 15),
	repeat,
	last_selected(LastOn),
	findall(Text, menu_item(Text), MenuList),

	menu_leave(2,2,7,7,MenuList, "Option",LastOn, NewChoice),
	do_choice(NewChoice),
	retractall(last_selected(_)),
	asserta(last_selected(NewChoice)),
	remove_style(NewChoice),
	!,terminal_mode.

remove_style(0) :- !,
	removewindow(81, 1),
	removewindow(3,1).
remove_style(_) :-
	removewindow(81, 0),!,
	fail.

%% change here if not hayes-modem type dialing
dial_format("ATDT %s\r\n").

get_text(Prompt, Default, Returned) :-
	makewindow(88, 7,7,Prompt, 10,10, 3, 40),
	edit(Default,Returned,"","","",0,"",1,1,1,1,_,
				RetStatus,0,0,0,1,0,"",""), %CR exits
	removewindow(88,1),
	RetStatus=0.

capture_toggle :-
	retract(capture_on),!,
	closefile(capture_file).
capture_toggle :-
	capture_file(FileName),
	get_text("Capture To", FileName, Capture), % fails if user esc
	assert(capture_on),
	openwrite(capture_file, CAPTURE).


terminal_mode :-
	chk_rdch,
	chk_wrch,
	terminal_mode.

chk_rdch :-
	rdch_CRLF_RS232(CH),!,
	write(CH),
	may_log(CH).
chk_rdch.

may_log(CH) :-
	capture_on,!,
	writedevice(SaveIt),
	writedevice(capture_file),
	log_write(CH),
	writedevice(SaveIt).
may_log(_).


ifdef  cook_incoming_Cr
 log_write(13):-!,nl.
enddef
 log_Write(CH) :- write(CH).


chk_wrch :-
	cursor(X,Y),
	cursor(X,Y),
	not(keypressed),!.
chk_wrch :-
	readchar(CH),
	see_if_menu(CH),
	wrch_CRLF(CH).

 rdch_CRLF_RS232(CH) :-
	port(Port),
	rxch_RS232(Port,CH),
	cook_13(CH).

ifdef cook_incoming_cr	    % The program transmitting to this one
  cook_13(13) :-!,	    % may send either CR or CRLF (like XTERM)
	nl.		    % discarding the LF, and echoing CR as NL
  cook_13(10) :-!, fail.
enddef
  cook_13(_).

  wrch_CRLF(menu_esc) :-!.
  wrch_CRLF('\013') :- not(full_duplex), nl, fail.
  wrch_CRLF('\013') :- !,
	trans_ch('\013'),
	trans_ch('\010').
  wrch_CRLF(CH) :-
	not(full_duplex),
	write(CH),		% local echo
	fail.
  wrch_CRLF(CH) :- trans_ch(CH).

  trans_ch(CH) :-
	port(Port),
	txch_RS232(Port,CH),!.
  trans_ch(CH) :- trans_ch(CH).


  %% hook the menu escape key, call the settings menu
  see_if_menu(menu_esc) :- !, do_menu.
  see_if_menu(_).


/*
===========================================================================
  Xmodem file transfer
===========================================================================
*/

CONSTANTS
 ack=6
 nak=21
 soh=1
 eot=4

 binary_mode = 0

 buffer_size = 256
 block_size  = 128
 packet_size = 131
 fill_byte   = 26

 port_no=2

% TIMING CONSTANTS: all times in centiseconds

% initial nak-ing to start transmitter (up to 9 nak 10 sec apart)
  initial_nak_retries = 9
  initial_soh_wait    = 1000
  first_nak_wait      = 1000

% consecutive retries on erroneous block
  max_naks	     = 9

% can wait 20 seconds for a new block to start
  wait_for_block     = 2000

% or once started, can wait 20 seconds to finish
  wait_in_block      = 2000

% sender wait for ack 30 seconds
  ack_wait	     = 3000


DOMAINS
  receipt = reply_was(char);
		timed_out

  xmit_status = open(string,string);
		close_report;
		progress(integer, real);
		error

DATABASE - xmodem
 determ total_errors(INTEGER)

PREDICATES
  send_file(string)
  send_bytes(real,integer)
  send_block(integer,integer, integer,string)
  send_case(receipt, integer, real, integer, string)
  wait_first_nak

  receive_file(string)
  receive_blocks(char,integer,integer)
  wait_new_header(integer,integer,real)
  valid_start(char)

  nak_until_initial_soh(integer)
  nak_until_case(receipt,integer)

  wait_for_reply(receipt, integer)
  wait_for(integer,integer)
  wait_aux(integer,real)

  complement(char,char)
  fill_if_less_than_128(integer,char)
  lessor_of(real,real,real)
  checksum(string,integer,char)
  checksum_aux(integer,integer,integer,char,char)

  message(string)
  report(xmit_status)
  report_on(xmit_status)

CLAUSES
  upload_file:-
	makestatus(7,"** Enter name of file to transmit **"),
	readfilename(8,20,7,7,dat,"", Filename, existing_file),
	report(open("Sending",Filename)),
	send_file(Filename),
	report(close_report),
	removewindow(83,1),
	!.
  upload_file :-
	removewindow(83,1).

  /* Receive a file using Packaging */
  download_file:-
	makestatus(7, "** Enter name of file to receive **"),
	readfilename(8,20,7,7,dat,"",Filename,new_file),
	report(open("Receiving",Filename)),
	receive_file(Filename),
	report(close_report),
	removewindow(83,1),
	!.
  download_file :-
	removewindow(83,1).

/***********************************************************************
   Receive file predicates
 ***********************************************************************/

receive_file(Filename) :-
	port(Port),
	delInbuf_RS232(Port),		% be sure no leftovers or noise
	nak_until_initial_soh(initial_nak_retries),  % mating dance
	openwrite(outfile, Filename),		     % really ready now
	filemode(outfile, binary_mode), 	     %	so open file
	writedevice(outfile),
	receive_blocks(soh,max_naks,1), 	     %	tro until eot
	writedevice(screen),
	closefile(outfile),!.			     %	or aborted...
receive_file(_).

receive_blocks(eot,_, _) :-!,
	port(Port),
	txCh_RS232(Port,ack).	% end of transmission
receive_blocks(soh,0,_ ) :-	% nak limit exceeded for this block!
	!,
	message("Receive aborted...Too many errors").
receive_blocks(soh, _, Expecting) :-
	report(progress(Expecting, 0)),
	wait_for(packet_size, wait_in_block),	% dont even check until
	port(Port),
	rxCH_RS232(Port, BlockNo),		% all there
	BlockNo=Expecting mod 256,		% such that all fails here
	rxCH_RS232(Port,Complement),		% are either
	complement(Expecting,Complement),	%  block took too long
	rxBin_RS232(Port,Block,block_size),	%   or had bad data
	rxCH_RS232(Port,CheckSum),
	checksum(Block,block_size,Checksum),	% last check OK
	txCh_RS232(Port,ack),			% Ack the xmitter
	writeblock(block_size,Block),		%  write to file
	!,
	Next=Expecting+1,
	marktime(wait_for_block,TimeAllowed),	% expect next block
	wait_new_header(max_naks, Next, TimeAllowed).
receive_blocks(_, Retry, Expect) :-		% failed checks or timeout
	port(Port),
	delInbuf_RS232(Port),			%  empty in case timeout
	txCH_RS232(Port,nak),			% nak & retry
	R2=Retry-1,
	report(error),
	marktime(wait_for_block, TimeAllowed),
	wait_new_header(R2, Expect,TimeAllowed).

wait_new_header(_, Expect,_) :- 		% discard anything
	port(Port),
	rxCh_RS232(Port,Header),		% until valid header
	valid_start(Header),
	!, receive_blocks(Header, max_naks, Expect).
wait_new_header(Retry,Expect,EndTime) :-	% or if time expires, fail.
	not(timeout(EndTime)),
	wait_new_header(Retry,Expect,EndTime).

valid_start(soh).
valid_start(eot).

/***************************************************************************
   send file predicates
****************************************************************************/
send_file(Filename) :-
	trap(openread(infile, Filename),_,fail),
	filemode(infile,binary_mode),
	filepos(infile,0,2),		% end of file
	filepos(infile,FileSize,0),	%  = Filesize
	filepos(infile,0,0),		% reposition at start
	readdevice(infile),
	wait_first_nak,
	send_bytes(FileSize, 1),	% send Size bytes, starting block no
	readdevice(keyboard),
	closefile(infile),!.
send_file(File) :-
	write("Cant read ",File),!.

send_bytes(0, _):-
	port(Port),
	txCh_RS232(Port, eot),!.	% no more bytes to send, notify done
send_bytes(Remaining, BlockNo) :-
	report(progress(BlockNo, Remaining)),
	lessor_of(Remaining, block_size, DataSize),	% block or leftover
	readblock(DataSize, Block),
	send_block(max_naks, DataSize, BlockNo, Block), % (re)send until ack
	Next=BlockNo+1,
	Rem2=Remaining-DataSize,
	!, send_bytes(Rem2, Next).
send_bytes(_,_).			% catch send_block fail

send_block(0 , _,_, _) :- !,
	message("Retry count exceeded, too many errors!"),
	fail.
send_block( Retry , Size, BlockNo, Block) :-
	complement(BlockNo, CBkNo),		%
	port(Port),
	txCH_RS232(Port, soh),			% checking for naks here
	txCH_RS232(Port, BlockNo),		% is silly...its all buffered
	txCH_RS232(Port, CBkNo),		%  so check after sent
	txBin_RS232(Port, Block,Size),		%  if early nak, delete
	checksum(Block,Size, CheckSum), 	%  outbuffer
	fill_if_less_than_128(Size, CheckSum),	% fills and sends, or sends
	wait_for_reply(Reply, ack_wait ),
	send_case(Reply, Retry, Size, BlockNo, Block).

send_case(timed_out,_,_,_,_):-			% cant get ack, give up!
	message("Timeout waiting for ack"),
	!, fail.
send_case(reply_was(ack),_,_,_ ,_) :-!. 	% ok!
send_case(reply_was(nak),Retry, Size, No, Block) :- % receive asked for retry
	Retry2= Retry-1,
	report(error),
	!, send_block(Retry2, Size, No, Block).
%send_case(reply_was(_) , R, S, P, B) :-  % received something else ...
%	--> invalid reply..	   .	  % XMODEM not well defined here...


%% they also serve who only sit and wait...
%%
wait_for_reply(reply_was(Byte),Wait) :-
	wait_for(1, Wait),
	port(Port),
	rxCH_RS232(Port,Byte),
	!.
wait_for_reply(timed_out,_).

wait_for(Incoming, Interval) :-
	marktime(Interval, EndTime),
	wait_aux(Incoming, EndTime).

wait_aux(Expecting,_) :-
	port(Port),
	queuesize_RS232(Port, Inqueue, _),
	Inqueue>=Expecting,!.
wait_aux(Expecting, Time) :-
	not(timeout(Time)),
	wait_aux(Expecting,Time).

%
% fill out the last block to 128 sent, then send checksum
%  fill_byte constant specifies fill character (normally CTRL-Z)
%  herein lies the source of XMODEM's habit of rounding file sizes up
%  to the next multiple of 128 bytes... a vestigial bit of CPM...

fill_if_less_than_128(128, CheckSum) :-!,
	port(Port),
	txCH_RS232(Port,CheckSum).
fill_if_less_than_128(LastSent, CheckSum) :-
	port(Port),
	txCH_RS232(Port,fill_byte),
	Last2=LastSent+1,
	Check2=(CheckSum+fill_byte) mod 256,
	!, fill_if_less_than_128(Last2,  Check2).


% Compute/test 1s complement of a byte
%
complement(Byte, CompByte) :-
	bitxor(Byte,255,Comp),
	CompByte=Comp.

%
% Compute/test the 1 byte checksum on the binary block
%
checksum(Ptr, Size, CheckSum) :-
	ptr_dword(Ptr,Seg,Off),
	checksum_aux( Size, Seg, Off, 0, CheckSum).

checksum_aux(0, _, _, Sum, Sum):-!.
checksum_aux(N, Seg, Off, Accum, Total) :-
	membyte(Seg,Off,Byte),
	A2=(Accum+Byte) mod 256,
	N2=N-1,
	Off2=Off+1,
	checksum_aux(N2, Seg, Off2, A2, Total).


nak_until_initial_soh(0) :-!,
	message("No transmit...Receiver timeout!"),
	fail.
nak_until_initial_soh(RetryNo) :-
	port(Port),
	txCH_RS232(Port,nak),			% nak and...
	wait_for_reply(Reply,initial_soh_wait), % wait up to 10 seconds
	nak_until_case(Reply,RetryNo).		%  for a reply

nak_until_case(reply_was(soh),_) :- !.
nak_until_case(reply_was(_),Retry) :-
	message("Reply but not soh!"),
	report(error),
	port(Port),
	delInbuf_RS232(Port),		 % out of whack, flush & re-sync
	R2 = Retry -1,
	!,nak_until_initial_soh(R2).
nak_until_case(timed_out, Retry) :-	 % eg, 1 nak every 10 seconds until...
	R2=Retry-1,
	report(error),
	nak_until_initial_soh(R2).

wait_first_nak :-
	message("Waiting for nak from receiver..."),
	wait_for_reply(Reply,first_nak_wait),
	Reply=reply_Was(nak),!.
wait_first_nak :- wait_first_nak.

lessor_of(A,B,A) :- A<B,!.
lessor_of(_, B, B).

message(String) :-
	writedevice(SaveD),
	writedevice(screen),
	write(String),
	nl,
	writedevice(SaveD).

report(open(Mode,Filename)) :-!,
	format(Title," %: % ", Mode, Filename),
	makewindow(2, 7,7, Title, 10, 4, 4, 55),
	retractall(total_errors(_)),
	asserta(total_errors(0)).

report(close_report) :- !,
	cursor(0,0), write("Press a key to continue"), readchar(_),
	removewindow(2,1).

report(X) :-
	shiftwindow(Was),
	writedevice(Saved),
	writedevice(screen),
	gotowindow(2),
	report_on(X),
	gotowindow(Was),
	writedevice(Saved).

report_on(progress(Block, Remaining)) :-
	cursor(0,0),
	write(""),
	cursor(1,3),
	writef("Block %-6   Remaining %-6  Errors (none)", Block, Remaining).

report_on(error) :-
	retract(total_errors(T)),!,
	T2=T+1,
	asserta(total_errors(T2)),
	cursor(1, 36),
	writef("Errors %-6", T2).

GOAL
	main.
