/*
==========================================================================
	XLABEL.PRO					
  									 
  	Demo of screen handler: Label printing			 	 
  									 
	Copyright (C) 1990 Prolog Development Center
===========================================================================
*/



check_determ


/************************************************************************
 *			Domains						*
 ************************************************************************/

include "tdoms.pro"

DOMAINS
  FIELD_NAME	= SYMBOL
  FILE		= myprinter
  TYPE		= int(); str(); real()


/************************************************************************
 *			Database Predicates				*
 ************************************************************************/

DATABASE
  dbstrike
  field(FIELD_NAME, TYPE, ROW, COL, LEN)
  font(INTEGER)
  label(STRING)
  printer(STRING)
  txtfield(ROW, COL, LEN, STRING)		
  value(FIELD_NAME, STRING)		
  windowsize(ROW, COL)


/************************************************************************
 *			Include Tools					*
 ************************************************************************/

include "tpreds.pro"
include "ui\\menu.pro"
include "ui\\status.pro"
include "ui\\lineinp.pro"
include "ui\\filename.pro"
include "screen\\scrhnd.pro"		% Or vscrhnd.pro


/************************************************************************
 *			Support Predicates				*
 ************************************************************************/

PREDICATES
		change(DBASEDOM)
 		display_label
		file_name(STRING)
  nondeterm	font_text(INTEGER, STRING)
  nondeterm 	for(INTEGER, INTEGER, INTEGER)
		get_nth_element(INTEGER, STRINGLIST, STRING)
  nondeterm 	member(STRING, STRINGLIST)
		print_label(STRINGLIST)
  		print_labels
		printer_code(STRING)
  nondeterm 	printers(STRING)
		read_file(STRING, STRING)
		set_printer_codes
		string_stringlist(STRING, STRINGLIST)
		write_n(INTEGER, CHAR)

CLAUSES
  change(value(FIELD_NAME, _)) :- retract(value(FIELD_NAME, _)), fail.
  change(label(_)) :- retract(label(_)), fail.
  change(font(_)) :- retract(font(_)), fail.
  change(printer(_)) :- retract(printer(_)), fail.
  change(label(LABEL)) :- !, assert(label(LABEL)), display_label.
  change(X) :- assert(X).

  display_label:-
	label(LABEL),!,
	shiftwindow(5),
	window_str(LABEL),
	shiftwindow(1).

  file_name(FILE_NAME) :-
	value(file, CURRENT_FILE_NAME),
	!,
	cursor(ROW, COL),
	R1 = ROW + 2,
	readfilename(R1, COL, 23, 23, lbl, CURRENT_FILE_NAME, FILE_NAME),
	change(value(file, FILE_NAME)).

  font_text(1, "Fast").  
  font_text(2, "Medium").  
  font_text(3, "Quality").

  for(I, I, _).
  for(I, A, B) :- B > A, A1 = A + 1, for(I, A1, B).

  get_nth_element(1, [HEAD|_], HEAD) :- !.
  get_nth_element(N, [_|TAIL], ELEMENT) :-
  	N > 1,
  	NEW_N = N - 1,
  	get_nth_element(NEW_N, TAIL, ELEMENT).

  member(ELEMENT, [ELEMENT|_]).
  member(ELEMENT, [_|TAIL]) :- member(ELEMENT, TAIL).

  print_label(LIST) :-
	value(indent, INDENT1),
	str_int(INDENT1, INDENT2),
	member(LINE, LIST),
	write_n(INDENT2, ' '),
	write(LINE),
	nl,
	fail.
  print_label(LIST) :-
	listlen(LIST, LEN),
	value(labellines, LABEL_LINES1),
	str_int(LABEL_LINES1, LABEL_LINES2),
	!,
	SKIP = LABEL_LINES2 - LEN,
	write_n(SKIP, '\n').

  print_labels :-
	label(LABEL),
	string_stringlist(LABEL, LIST),
	printer(PRINTER),
	value(number, NUMBER_OF_LABELS1),
	str_int(NUMBER_OF_LABELS1, NUMBER_OF_LABELS2),
	openwrite(myprinter, PRINTER),
	writedevice(myprinter),
	set_printer_codes,
	for(I, 1, NUMBER_OF_LABELS2),
		print_label(LIST),
	I = NUMBER_OF_LABELS2,
	!,
	closefile(myprinter).

  printer_code("") :- !.
  printer_code(CODE) :-
	frontchar(CODE, '\\', REST),
	fronttoken(REST, NUM, RESTCODE),
	str_int(NUM, CHI),
	char_int(CH, CHI),
	write(CH),
	!,
	printer_code(RESTCODE).
  printer_code(CODE) :-	
  	frontchar(CODE, CH, REST), 
  	write(CH), 
  	printer_code(REST).

  printers(prn).  
  printers(com1).  
  printers(com2).

  read_file(FILE_NAME, LABEL) :- file_str(FILE_NAME, LABEL), !.
  read_file(FILE_NAME, _) :-
  	makewindow(1, 23, 23, "", 5, 20, 4, 45),
	write(">> File not found: ", FILE_NAME),
	readkey(_),
	removewindow,
	fail.

  set_printer_codes :- value(initcode, INIT), printer_code(INIT), fail.
  set_printer_codes :- 
  	dbstrike, 
  	value(dbstrikecode, DBSTRIKE),
  	printer_code(DBSTRIKE),
  	fail.
  set_printer_codes :-
	font(FONT1),
	str_int(FONT2, FONT1),
	concat("font", FONT2, FIELD_NAME),
	value(FIELD_NAME, FONT_CODE),
	printer_code(FONT_CODE),
	fail.
  set_printer_codes.

  string_stringlist("", []) :- !.
  string_stringlist(STRING, [H|T]) :-
	searchchar(STRING, '\n', N),
	frontstr(N, STRING, H, R),
	frontchar(R, _, R1),!,
	string_stringlist(R1, T).
  string_stringlist(STRING, [STRING]).


  write_n(0, _) :- !.
  write_n(N, CH) :- N > 0, write(CH), N1 = N - 1, write_n(N1, CH).

/************************************************************************
 *			Field Actions					*
 ************************************************************************/

  field_action(dbstrike) :- retract(dbstrike), !.
  field_action(dbstrike) :- assert(dbstrike).

  field_action(dir) :- cursor(ROW, COL), setdir(ROW, COL, 23, 23).

  field_action(edit) :-
	label(LABEL),
	shiftwindow(5),
	edit(LABEL, LABEL1, "edit", "", "", 0, "", RET), 
	shiftwindow(1),
	refreshstatus,
	RET >< 1,
	!,
	change(label(LABEL1)).
  field_action(edit) :- display_label.

  field_action(file) :- file_name(_).

  field_action(font) :-
	cursor(ROW, COL),
	findall(X, font_text(_, X), LIST),
	menu(ROW, COL, 23, 23, LIST,"Choose font", 0, FONT),
	FONT >< 0,
	change(font(FONT)).

  field_action(load) :-
	cursor(ROW, COL),
	R1 = ROW + 2,
	readfilename(R1, COL, 23, 23, lbl, "", FILE_NAME),
	read_file(FILE_NAME, LABEL),
	change(value(file, FILE_NAME)),
	change(label(LABEL)).

  field_action(print) :- print_labels.

  field_action(printer) :-
	cursor(ROW, COL),
	findall(X, printers(X), LIST),
	menu(ROW, COL, 23, 23, LIST, "Choose printer", 0, NR),
	get_nth_element(NR, LIST, PRINTER),
	change(printer(PRINTER)).

  field_action("save") :-
	file_name(FILE_NAME),
	label(LABEL),
	!,
	file_str(FILE_NAME, LABEL).

  field_action(saveconfig) :- save("xlabel.dba").

/************************************************************************
 *			Field Values					*
 ************************************************************************/

  field_value(dir, DISK) :- !, disk(DISK).
  field_value(dbstrike, on) :- dbstrike, !.
  field_value(dbstrike, off) :- !.
  field_value(font, FONT) :-  font(NR), font_text(NR, FONT), !.
  field_value(printer, X) :-  printer(X), !.
  field_value(FIELD_NAME, VAL) :- value(FIELD_NAME, VAL), !.


/************************************************************************
 *			No Input Fields					*
 ************************************************************************/

  noinput(dbstrike).
  noinput(dir).		
  noinput(edit).	
  noinput(font).
  noinput(load).	
  noinput(print).	
  noinput(printer).	
  noinput("save").	
  noinput(saveconfig).


/************************************************************************
 *			Goal						*
 ************************************************************************/
 
GOAL	
  textmode(ROWS, COLS),
  ROWS1 = ROWS - 16,
  makewindow(5, 66, 67, "LABEL", 15, 0, ROWS1, COLS),
  makewindow(1, 32, 33, "Label Printing", 0, 0, 15, COLS),
  makestatus(23, " Move the cursor with the arrow keys and select by pressing RETURN"),
  consult("xlabel.dba"),
  display_label,
  scrhnd(off, _).

