/*
============================================================================
	XMCURSED.PRO
	
	Demo of techniques for mouse cursor definition
	
	Copyright (C) 1990 Prolog Development Center
============================================================================
*/		



include "BGI.PRE"

include "MOUSE\\MOUSE.PRE"
include "MOUSE\\MOUSE.PRO"

include "MOUSE\\MUSTMOUS.PRO"
include "GRAPHICS\\BGIFIND.PRO"

CONSTANTS

% screen mask(32) + cursor_mask(32) + hotspot(X,Y)(4) + style(2)
  cursor_size=70

% selections
  test_cursor 	= 0
  set_bits	= 1
  clear_bits	= 2
  new		= 3
  saveit	= 4
  prev		= 5
  next	 	= 6
  done		= 7
 
% cursor styles
 outline  = 0
 normal   = 1
 inverted = 2
 opaque   = 3 
 
DOMAINS
 db_selector = cursors

DATABASE - cursor_data

 cursor_reload
 cursor_style(integer)
 hot_spot(integer,integer) 

DATABASE - cursor_prog
 
 current_ref(ref)
 
 style_zone(integer,integer,integer,integer, integer)
 menu(integer, integer)
 cursor_box(integer,integer, integer,integer, integer,integer, 
 			integer,integer, integer,integer)
 text(integer, string)
 style_text(integer, string)
 write_mode
 
PREDICATES
 nondeterm repeat
 nondeterm repeat_cursor
 nondeterm for(integer,integer,integer)
 nondeterm neighbor_bit(integer,integer,integer,integer)

	main
	main2(string)
	main3_graphic_mode
	gr_mode_exit

	  
	show_bitmap(string)
	show_fatbit(integer,integer, symbol)
	fatbit(symbol, integer,integer,integer,integer)
 	bit_state(integer, integer, symbol)
 	result_state(integer, symbol)
 	new_bitmap(integer, string)

	process_mousepress(integer,integer, string, integer)
 	do_selection(Integer, string)
 	
	pixel_fat(integer,integer,integer,integer,integer)
	fat_region(integer,integer,integer,integer,integer,integer)

	set_screen_mask(integer,string)
	simple_mask_Style(integer,integer,integer)
	xor_old_hot
	set_bit(integer,integer,integer,string)

	set_bit_word(integer, integer, integer, symbol)
	check_refresh_hot(integer, integer)
	stroke_hot(integer,integer)
	cursor_postfix(String,integer,integer,integer)
	current_cursor(string)
	save_cursor(string)
	next_cursor
	prev_cursor
	new_cursor
	open_cursor_lib
	just_fail_if(integer,integer)
	screen_Setup
	bleep
GOAL
 main.
 
CLAUSES	
	
 main :- 
 	must_have_mouse,
 	bgi_find(BGI_Path),!,
 	textmode(Rows,Cols),
 	main2(BGI_Path),
 	textmode(Rows,Cols).
 main.
  	 
 
 main2(BGI_PATH) :-
	trap(initgraph(detect,0,_,_,BGI_PATH),Err, no_bgi_err(Err)),
	trap(main3_graphic_mode, _, gr_mode_exit),!.
 main2(_).
	 	 

 main3_graphic_mode :-
 	open_cursor_lib,
	screen_setup,
	repeat,
	current_cursor(Bitmap),
	repeat_cursor,
 	  mouse_get(X,Y,Buttons),
	  Buttons <> 0,
 	  process_mousepress(X, Y, BitMap, Buttons),
 	fail.
 
 	
 gr_mode_exit :-
 	db_close(cursors),
 	closegraph.
 	 	
 

%% set/clear a fat bit, or move the hot-spot...

process_mousepress(X,Y,BitMap, Button) :-
	pixel_fat(Button, X,Y, Line, Col),!,
	set_bit(Button, Line, Col, Bitmap).
	

%% make a menu selection
process_mousepress(X,Y,Cursor,_) :-
	menu(Right, High),
	X<=Right,!,
	Choice = trunc(Y/High),
	do_selection( Choice, Cursor),
	mouse_waitrelease.
	

%% change the mask style
process_mousepress(X, Y, Cursor,_) :-
	style_zone(SLeft, Stop, Sright, Sbot, Shigh),
	X>Sleft,
	X<Sright,
	Y>Stop,
	Y<Sbot,!,
	mouse_waitrelease,
	Style = trunc((Y - Stop)/Shigh),
 	retractall(cursor_style(_)),
 	asserta(cursor_style(Style)),
 	set_screen_mask(Style, Cursor),
 	save_cursor(Cursor),
 	do_Selection(test_cursor, Cursor).
	
process_mousepress(X,Y,_,_) :-  % else scribble in margins to see		
	mouse_off,
	putpixel(X,Y,white),	% how the hot-spot works
	mouse_on.


%% based on which button, test to see if in cursor region, 
%%   if so, return the virtual (Fat-bit-matrix) row, col
%%
pixel_fat(rightbutton, X, Y, Vrow, Vcol):-
	cursor_box(HX, Hy, Xbase, Ybase, Lr, Lb, Xsep, Ysep, BoxWide, BoxHigh),
	X>=Hx, Y>=Hy, X<Lr, Y<Lb,!, % in fat bit zone
	Vcol = (X - Xbase + Xsep/2 - 1)/(BoxWide + Xsep),
	Vrow = (Y - Ybase + Ysep/2 - 1)/(BoxHigh + Ysep).
pixel_fat(leftbutton, X, Y, Vrow, Vcol):-
	cursor_box(_, _, Xbase, Ybase, Lr, Lb, Xsep, Ysep, BoxWide, BoxHigh),
	X>=Xbase, Y>=Ybase, X<Lr, Y<Lb,!, % in fat bit zone
	Vcol = trunc((X - Xbase + Xsep/2 - 1)/(BoxWide + Xsep)),
	Vrow = trunc((Y - Ybase + Ysep/2 - 1)/(BoxHigh + Ysep)).
			
%% given a fat-bit row,col return the corresponding pixel-region
fat_region(VCol, VRow, Left, Top, Right, Bottom) :-
	cursor_box(_,_, Xbase, Ybase, _,_, Xsep, Ysep, BoxWide, BoxHigh),!,
	Left   = Xbase + Vcol * (BoxWide + Xsep),
	Top    = Ybase + Vrow * (BoxHigh + Ysep),
	Right  = Left + BoxWide - 1,
	Bottom = Top  + BoxHigh - 1.

show_bitmap(Map) :-
	ptr_dword(Map, Seg, Off),
	mouse_off,
	for(Line , 0, 15),
	   for(Col, 0, 15),
	   	Bit = 15 - Col,
	   	Offset = Off + 32 + 2 * Line,
	   	memword(Seg, Offset, Word),
	   	bit_state(Word, Bit, State),
	   	show_fatbit(Line, Col, State),
	   Col=15,
	Line=15,
	mouse_on,
	hot_spot(X, Y),!,
	stroke_hot(X,Y).

show_fatbit( Vrow, Vcol, State) :-
	fat_region(Vcol, Vrow, Left, Top, Right, Bottom),
	fatbit(State, Left, Top, Right, Bottom).

fatbit(on, L, T, R, B) :- 
	setfillstyle(solid_fill, white),	
	bar(L,T,R,B).
fatbit(off, L, T, R, B) :- 
	setfillstyle(solid_fill, black),	
	bar(L,T,R,B).

bit_state(Word, Bit, State) :-
	bitleft(1, Bit, Mask),
	bitand(Word, Mask, Result),
	result_state(Result, State).
	
result_state(0, off) :-!.
result_state(_, on).


%% make a new bitmap, using Fillvalue
new_bitmap(FillValue, Map) :-
	str_len(Map, cursor_size),
	ptr_dword(Map, Seg, Off),
	for(Line, 0, 15),
	Offset = Off + 32 + 2 * Line,
	memword(Seg, Offset, FillValue),
	Line = 15,
	!,
	set_screen_mask(outline,Map),
	cursor_postfix(Map, -1,-1, outline). % default these	

%% place the hot spot...
set_bit(rightbutton, Row, Col, _) :-
	retract(hot_spot(OldCol, OldRow)),!,
	asserta(hot_spot(Col, Row)),
	stroke_hot(OldCol,OldRow),
	stroke_hot(Col, Row),
	mouse_waitrelease.

%% set or clear a cursor fat bit	
set_bit(leftbutton, Row, Col, BitMap) :-
	Bit = 15 - Col,
	ptr_dword(BitMap, Seg, Off),
	Offset = Off + 32 + 2 * Row,
	memword(Seg, Offset, Word),
	set_bit_word(Word, Bit, NewWord, State),
	memword(Seg, Offset, NewWord),
	mouse_off,
	show_fatbit(Row, Col, State),
	mouse_on,
	check_refresh_hot(Col,Row).


set_bit_word(Word, Bit, NewWord, on) :-
	write_mode, !,
	bitleft(1, Bit, Mask),
	bitor(Word, Mask, NewWord).
	
set_bit_word(Word, Bit, NewWord, off) :-
	bitleft(1, Bit, Mask),
	bitnot(Mask,  NotMask),
	bitand(Word, NotMask, NewWord).

check_refresh_hot(X,Y) :-
	hot_spot(X,Y),!,
	stroke_hot(X,Y).
check_refresh_hot(_,_).

stroke_hot(X,Y) :-
	fat_region(X,Y, L,T,R,B),
	Cx= L+( R - L ) / 2,
	Cy= T+( B - T ) / 2,
	settextjustify(center_text,center_text),
	settextstyle(gothic_font,0,0),	% need a stroked-font for
	setwritemode(xor_put),		%  the xor_put...
	mouse_off,
	outtextxy(Cx,Cy,"*"),
	mouse_on.


%% given a pointer to a cursor definition, set
%%  the screen mask area to define the appropriate 
%%  cursor style
%%       
%%%   outline style
 set_screen_mask(outline,Mask):-
	set_screen_mask(normal, Mask),	 % start off as normal
 	ptr_dword(Mask, Seg, Off),	 
	for(Line, 0, 15),		 % then find 1- pixel wide "outline", 
	  CM_Offset = Off+32+Line*2,	 % and set to forced background
	  memword(Seg, CM_Offset, Cursor),
 	  for(Shift, 0, 15),
 	    bitleft(1,Shift, BitTest),
 	    bitand(Cursor, BitTest, State),
 	    State<>0, 				% for each non-zero bit
	    neighbor_bit(Line,Shift,NLine, NBit), %  for each neighbor
	    CM_Off2 = Off + 32 + Nline * 2,	  %  of that bit that is ...
	    memword(Seg, CM_Off2, CurWord2),	
	    bitleft(1, Nbit, Ntest),
	    bitand(CurWord2, Ntest, Zero),
	    Zero = 0,				% not set
	    SM_Off = CM_Off2 - 32,		% 
	    memword(Seg, Sm_Off, SM_Word),	%  turn off the corresponding
	    bitnot(Ntest, AllBut),
	    bitand(SM_Word, AllBut, SM_Word2),	%  screen mask bit
	    memword(Seg, SM_Off, SM_Word2),
	fail.  				
 set_screen_mask(outline,_):-!.

%% simpler styles
 set_screen_mask(Style, BitMap):-
 	ptr_dword(BitMap, Seg, Off),
 	for(Line, 0, 15),
 	 SM_Offset = Off + 2 * Line,
 	 CM_Offset = SM_Offset + 32,
 	 memword(Seg, CM_Offset, Cursor_Word),
 	 simple_mask_Style(Style, Screen_Word, Cursor_Word),
 	 memword(Seg, SM_Offset, Screen_Word),
 	Line = 15,
 	!.

 simple_mask_Style(inverted, $FFFF, _):-!.
 simple_mask_Style(opaque,   $0000, _):-!.
 simple_mask_Style(normal,   Mask, Word) :-!, 
 	bitxor(Word, $FFFF, Mask).
 

%% process a menu selection
 do_selection(next,_) :- !,next_cursor.
 do_selection(prev,_) :- !,prev_cursor.
 do_selection(new,_)  :- !,new_cursor.
 do_selection(saveit,Cursor) :-!, save_cursor(Cursor).
 do_selection(clear_bits,_) :-!, retractall(write_mode).
 do_selection(set_bits,_) :- write_mode,!.
 do_selection(set_bits,_) :-!, asserta(write_mode).	
 do_selection(test_cursor,Cursor) :-
	hot_spot(HotX,HotY),!,
	ptr_dword(Cursor, Seg, Off),
	mouse_off,
 	mouse_set_graphics_cursor(HotX,HotY, Seg, Off), 
 	mouse_on.
 do_selection(done,_):- 
 	exit(0). 		% exit to surrounding trap	
   	
 neighbor_bit(Line, Bit, Line, B1) :- 
 	Bit<15, B1 = Bit + 1; 
 	Bit>0,  B1 = Bit - 1.
 neighbor_bit(Line, Bit, L1, Bit) :- 
	Line>0,  L1 = Line - 1;
	Line<15, L1 = Line + 1.

%% set/read the hot spot & style values from the end of the cursor data
 cursor_postfix(BitMap, HotX, HotY, Style):-
 	ptr_dword(BitMap, Seg, Off),
 	Xoff = Off + 64,
 	Yoff = Off + 66,
 	Soff = Off + 68,
 	memword(Seg, Xoff, HotX),
 	memword(Seg, Yoff, HotY),
 	memword(Seg, Soff, Style).

 current_cursor(Bitmap) :-
 	current_ref(Ref),!,
 	ref_bin(cursors,Ref,Bitmap,_),
	xor_old_hot,
 	cursor_postfix(BitMap, HotX, HotY, Style),
	retractall(_, cursor_data),
	asserta(hot_spot(HotX, HotY)),
	asserta(cursor_Style(Style)),
	show_bitmap(BitMap).
 
 
 xor_old_hot:-
 	hot_spot(X,Y),!,
 	stroke_hot(X,Y).
 xor_old_hot.
 	
 	
 save_cursor(Bitmap) :-
 	current_ref(Ref),
	hot_spot(HX, HY),
	cursor_style(Style),!,
	cursor_postfix(BitMap, HX, HY, Style),
 	bin_replace(cursors, Ref, BitMap, cursor_size).
  
 next_cursor :-
 	current_ref(Ref),
 	chain_next(cursors, Ref, Next),
 	retract(current_ref(_)),!,
 	asserta(current_ref(Next)),
 	asserta(cursor_reload).
 next_cursor :- bleep.
 
 prev_cursor :-
 	current_ref(Ref),
 	chain_prev(cursors, Ref, Next),
 	retract(current_ref(_)),!,
 	asserta(current_ref(Next)),
 	asserta(cursor_reload).
 prev_cursor :- bleep.
 
 bleep :- sound(11,60).
 
 new_cursor:-
 	current_ref(Ref),
 	chain_prev(cursors, Ref, Before),
 	new_bitmap(0, BitMap),
 	chain_bininsertafter(cursors, cursors, Before, BitMap, cursor_size,NewRef),
 	retract(current_ref(_)),!,
 	asserta(current_ref(NewRef)),
 	asserta(cursor_reload).
 new_cursor:-
 	new_Bitmap(0,BitMap),
 	chain_bininserta(cursors, cursors,BitMap,cursor_size,Ref),
 	retractall(current_ref(_)),
 	asserta(current_ref(Ref)),
 	asserta(cursor_reload).
 	
%% open existing cursor database
%%  or create a new one 
 open_cursor_lib :-
 	trap(db_open(cursors, "cursor.dat", in_file), 
					Err, just_fail_if(Err, 7002)),
	chain_first(cursors, cursors, Ref),!,
	asserta(current_ref(Ref)).
		
 open_cursor_lib :-
 	db_create(cursors, "cursor.dat", in_file),
 	new_cursor.
 
 just_fail_if(X,X) :-!,fail.      
 just_fail_if(Err,_):- exit(Err). % else exit to surrounding trap, if any


 screen_setup :-
	getmaxx(Xmax),
	CenterX = Xmax/2,
	getmaxy(Ymax),
 	Hper = Ymax/8,
 	textwidth(" Save Cursor ", Right),
	settextjustify(center_text,center_text),
 
     %% draw the main menu	
 	for(I,0,7),
  	Top = I * Hper,
 	Bottom = Top + Hper - 2,
 	rectangle(0, Top, Right, Bottom),
	X  = Right / 2,
	Y  = Top + Hper / 2, 
 	text(I,Label),
 	outtextxy(X, Y, Label),
 	I = 7,!,
 	asserta(menu(Right, Hper)),
 	
      %% Draw the cursor box and hot-spot zone	
 	getaspectratio(Xasp, Yasp),
 	Xsep = 2,
 	Ysep = (Xasp/Yasp) * Xsep,
 	BoxHigh = round(0.6 * Ymax / 16) - 2,
 	BoxWide = round((Yasp/Xasp) * BoxHigh),
	Xbase = Xmax/2 - 7*(BoxWide + Xsep),
	Ybase = Ymax/2 - 7*(BoxHigh + Ysep),
	CLeft = Xbase - Xsep,
	CTop  = Ybase - YSep,
	CRight= Xbase + 16 * (BoxWide + Xsep),
	CBottom= Ybase + 16 * (BoxHigh + Ysep),
	rectangle(CLeft, CTop, CRight, CBottom),
	HotLeft = CLeft - 2*(BoxWide + Xsep),
	HotTop  = CTop  - 2*(BoxHigh + Ysep),
	setlinestyle(dotted_line, 0, norm_width),
	rectangle(Hotleft, Hottop, CRight, CBottom),
	setlinestyle(solid_line, 0, norm_width),
 	asserta(cursor_box(HotLeft, HotTop, Xbase, Ybase, 
 	                    Cright, Cbottom, Xsep, YSep, BoxWide, BoxHigh)),
	
      %% Draw the style boxes		
	SLeft  = CRight + 4 * BoxWide, 
	textwidth(" Inverted ", Style_wide),
	SRight = Style_wide + Sleft,
	ScenterX = Sleft + Style_wide / 2,
	Sper   = (CBottom - Ctop) / 4,
	for(SI, 0, 3),
	STop = CTop + Si * Sper,
	SBot = STop + Sper - 2,
	rectangle(SLeft, STop, Sright, Sbot),
 	ScenterY = STop + Sper / 2 ,
	style_text(Si, Stext),
	outtextxy(SCenterX, SCenterY, Stext),
	Si = 3,!,
	StitleY = Ctop - BoxHigh,
	outtextxy(SCenterX, StitleY, "Style"),
	asserta(style_zone(SLeft, Ctop, Sright, Cbottom, Sper)),
	
      %% Button help prompts	
 	LBY = 0.9 * Ymax,
	textheight("S",High),
	
	RBY = LBY + High + 2,
 	outtextxy(CenterX, LBY, "Left Button  - Set/Clear Bits"),
 	outtextxy(CenterX, RBY, "Right Button - Place Hot Spot"),

      %% Title 	
 	settextstyle(triplex_font, horiz_dir, 2),
 	TitleY = 0.08 * Ymax,
 	outtextxy(CenterX, TitleY, "Cursor Editor").

%% default database data
 	
 text(0,"Test Cursor"). 	
 text(1,"Set Bits").
 text(2,"Clear Bits").
 text(3,"New Cursor").
 text(4,"Save Cursor").
 text(5,"Prev Cursor").
 text(6,"Next Cursor").
 text(7,"Exit").  	

 style_text(0, "Outline").
 style_text(1, "Normal").
 style_text(2, "Inverted").
 style_text(3, "Opaque").

% write_mode.

%% acts like repeat if cursor_reload flag not set, else
%%  allows backtrack to previous repeat, which lets the cursor
%%  be changed to a new value...

 repeat_cursor.
 repeat_cursor :-
 	not(cursor_reload),
 	repeat_cursor.
 	
 
	
for(I,I,_).
for(I,J,K) :-!,
	J<K,
	J2=J+1,
	for(I,J2,K).			
repeat.
repeat :- repeat.		

		
write_mode.