/*
=====================================================================
	XMOUPICK.PRO
	
	Demonstrates picking a drawing pattern with the mouse.
	
	Copyright 1990 Prolog Development Center
=====================================================================
*/

include "TDOMS.PRO"
include "TPREDS.PRO"

include "BGI.PRE"
include "MOUSE\\MOUSE.PRE"
include "MOUSE\\MOUSE.PRO"
include "MOUSE\\MUSTMOUS.PRO"
include "GRAPHICS\\BGIFIND.PRO"


GLOBAL PREDICATES
	pick_pattern(Integer) - (o)

DATABASE - patterns
	pattern_(Integer,Integer,Integer,Integer,Integer)

PREDICATES
	draw_patterns(Integer)
	poll_mouse(Integer)
	in_pattern(Integer,Integer,Integer)
	change_pattern(Integer)
	button_or_break(integer)	
CLAUSES
	pick_pattern(Pattern) :-
		setviewport(0,0,159,99,1),
		setcolor(2), 
		rectangle(0,0,159,99),
		setfillstyle(1,2), 
		bar(0,0,159,99),
		draw_patterns(0), 
		mouse_on,
		poll_mouse(Pattern), 
		!, mouse_off.
	
	
	draw_patterns(12) :-
		setfillstyle(1,4), 
		bar(20,85,140,93),
		settextjustify(1,1), 
		setcolor(2),
		outtextxy(80,90,"Exit"),
		retractall(pattern_(12,_,_,_,_)),
		assert(pattern_(12,20,85,140,93)), !.
	draw_patterns(Pattern) :-
		Left = 20 + ((Pattern mod 3) * 45), 
		Right = Left + 30,
		Top = 5 + (Pattern mod 4) * 20, 
		Bot = Top + 15,
		setfillstyle(Pattern,3), 
		bar(Left,Top,Right,Bot),
		rectangle(Left,Top,Right,Bot),
		retractall(pattern_(Pattern,_,_,_,_)),
		assert(pattern_(Pattern,Left,Top,Right,Bot)),
		Next = Pattern + 1,
		draw_patterns(Next).


	poll_mouse(Pattern) :-
		getmaxx(MaxX), 
		XRatio = MaxX / 640,
		repeat, 
		mouse_get(X,Y,Button),
		Button >< 0, 
		XPos = X * XRatio,
		in_pattern(XPos,Y,Pattern), 
		mouse_waitrelease,
		!.
	
	
	button_or_break(Button):- Button<>0,!.
	button_or_break( _ ) :- breakpressed, exit(0).

	
	in_pattern(X,Y,Pattern) :-
		pattern_(Pattern,Left,Top,Right,Bot),
		X >= Left, 
		X <= Right, 
		Y >= Top, 
		Y <= Bot, 
		!.
	
	change_pattern(12) :- !.
	change_pattern(Pattern) :-
		getmaxx(MaxX), 
		getmaxy(MaxY),
		setviewport(0,0,MaxX,MaxY,1),
		clearviewport,
		setfillstyle(Pattern,7),
		bar(0,0,MaxX,MaxY).

GOAL    textmode(SaveRow,SaveCol),
	must_have_mouse,
	bgi_find(Bgi_Path),
	trap(initgraph(detect,0,_,_,BGI_PATH),Err, no_bgi_err(Err)),
	setbkcolor(1),
	repeat, 
		pick_pattern(Pattern),
		change_pattern(Pattern),
	Pattern = 12, 
	closegraph,
	textmode(SaveRow,SaveCol).

