/*
============================================================================
	XMOUZART.PRO
                                                                            
	Mouse Demonstration Program
                                                      
	Copyright (C) 1990 Prolog Development Center
============================================================================
*/

check_determ

include "bgi.pre"

include "mouse\\mouse.pre"
include "mouse\\mouse.pro"
include "mouse\\mustmous.pro" 
include "graphics\\images.pre"
include "graphics\\images.pro"
include "graphics\\graph.pre"
include "graphics\\graph.pro"
include "graphics\\bgifind.pro"

CONSTANTS
  wind_numb_info = 1
  wind_numb_disp = 2
  wind_numb_cmds = 3
  wind_numb_colr = 4
  wind_numb_pivt = 5
  wind_numb_cnfm = 6

  wind_name_info = " Information "
  wind_name_disp = " Display "
  wind_name_cmds = " Commands "
  wind_name_colr = " Color "
  wind_name_pivt = " Pivot "
  wind_name_cnfm = " Confirm "


DATABASE
  cmd_text(INTEGER,STRING)
  window_title(integer,string)
  
/*****************************************************************************
                               MOUSE CALL

  mouse(WINDOW,WinX, WinY, MouseX, MouseY,BUTTONS) - (O,O,O,O,O,O)

    WINDOW  : Name of the window containing the mouse
    WinX    : pixel value of the mouse in window local coords
    WinY    : pixel value of the mouse in window local coords
    MouseX  : pixel value of the mouse cursor location
    MouseY  : pixel value of the mouse cursor location
    BUTTONS : 0 = None,1 = Left, 2 = Right, 3 = Both.


*****************************************************************************/

PREDICATES
  mouse(STRING,INTEGER,INTEGER,INTEGER,INTEGER,INTEGER)
  in_window(integer,integer,integer, integer, string)
CLAUSES
  mouse(NAME,WinX,WinY,MouseX,MouseY,BUTTONS):-
   	mouse_get(MouseX,MouseY,BUTTONS),
	in_window(MouseX,MouseY, WinX, WinY, Name),!.

  in_window(X, Y, Wx, Wy, WindowName) :-
	window_(Number,_,Left,_,Right,Bottom,Top), 
	Y>=Top, Y<Bottom, X>Left, X<Right,
	window_title(Number, WindowName),!,
	Wx=X-Left, Wy=Y-Top. 


/*************************************************************************** 
		make a skull shaped cursor
*****************************************************************************/

PREDICATES
  make_skull

CLAUSES
  make_skull:-
	MASK="\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\240\15\248\31\252\31\198\99\70\98\70\98\124\62\124\62\248\31\240\15\224\7\1\1\32\4\224\7\224\7\192\3",
	ptr_dword(MASK,SEG,OFF),
	NOFF1=OFF+54,
	NOFF2=OFF+55,
	membyte(SEG,NOFF1,0), 
	membyte(SEG,NOFF2,0),
	mouse_set_graphics_cursor(12,8,SEG,OFF).


/*****************************************************************************
******************************************************************************
*****************************                    *****************************
*****************************    Demo Program    *****************************
*****************************                    *****************************
******************************************************************************
*****************************************************************************/

DATABASE - demo
  determ display_info.
  determ color_selected(INTEGER)
  determ pivot_start(INTEGER,INTEGER)


/*****************************************************************************
		DATABASE AND FLAG INITIALIZATION
*****************************************************************************/

CLAUSES
  pivot_start(0,0).
  color_selected(blue).

/*****************************************************************************
		Create the various windows on the screen
*****************************************************************************/

PREDICATES
  outline_screen
  outline_cmds
  outline_scle
  outline_wrld
  outline_info
  show_quadrant(INTEGER,INTEGER,INTEGER)
  nondeterm for(INTEGER,INTEGER,INTEGER)  
CLAUSES
  outline_screen:-
	getmaxx(MaxX), 
	getmaxy(MaxY),
	InfoHigh=trunc(MaxY/5),
        DispHigh=MaxY-InfoHigh,
	DispWide=trunc(MaxX*(63/80)),
	CmdWide = MaxX - DispWide, 
	CMDHigh = MaxY*(10/25), 
	ColrY = CmdHigh + InfoHigh,
	ColrHigh = MaxY*(4/25),
	PivY = ColrY + ColrHigh,
	PivHigh = MaxY-PivY,	
	make_Gwindow(1,7,7,wind_name_info,0,0,InfoHigh,MaxX),
	make_Gwindow(2,7,12,wind_name_disp,InfoHigh,0,DispHigh,DispWide),
	make_Gwindow(3,7,7,wind_name_cmds,InfoHigh,DispWide,CmdHigh,CMDWide),
	make_Gwindow(4,7,7,wind_name_colr,ColrY,DispWide,ColrHigh,CMDWide),
	make_Gwindow(5,7,7,wind_name_pivt,PivY,DispWide,PivHigh,CmdWide),
	outline_info,
	outline_cmds,
	outline_scle,
	outline_wrld.

  outline_info:-
	shift_Gwindow(wind_numb_info),
	getdrivername(Adapter), 
	getgraphmode(Cmode),
	getmodename(CMode,Mode),
	setcolor(lightgray),
	format(Text, "Adapter: % Mode: %", Adapter,Mode),
	show_text([Text]).


  outline_scle:-
	shift_Gwindow(wind_numb_colr),
	getviewsettings(Left,T, Right,B,_),
	Y1 = 3, 
	Y2 = (B - T)/2+1, 
	Per = (Right - Left)/15, 
	for(Color,1,15),
	  X1 = (Color - 1)*Per + 1, 
	  X2 = X1 + Per - 2,
	  setfillstyle(solid_fill,COlor),
	  bar(X1,Y1, X2, Y2),
	Color = 15,!.
   

   outline_cmds:-
	shift_Gwindow(wind_numb_cmds),
	getviewsettings(VLeftX, VTopY, VRightX, VBotY, _),
	Yper = (VBotY - VTopY)/7,
	XMax = (VrightX - VLeftX),
        X = Xmax div 2,
	setcolor(black),
	settextjustify(center_text,center_text),
	for(I, 0, 6),
  	   cmd_text(I, Text),
	   Y = I * YPer + YPer/2 +1, 
	   Yb1 = 1+I * Yper,
           Yb2 = Yb1 + Yper,
           Color = lightgray + I mod 2,
	   setfillstyle(solid_fill, Color),		
	   bar(0, Yb1, Xmax, Yb2), 
	   outtextxy(X, Y, Text),
	I = 6,!,
	settextjustify(left_text,top_text).

   cmd_text(0,"Display Info").
   cmd_text(1,"Clear Display").
   cmd_text(2,"Mouse Off").
   cmd_text(3,"Mouse On").
   cmd_text(4,"Cursor: Arrow").
   cmd_text(5,"Cursor: Skull").
   cmd_text(6,"Exit").
	
   window_title(wind_numb_info,wind_name_info).
   window_title(wind_numb_disp,wind_name_disp).
   window_title(wind_numb_cmds,wind_name_cmds).
   window_title(wind_numb_colr,wind_name_colr).
   window_title(wind_numb_pivt,wind_name_pivt).
   window_title(wind_numb_cnfm,wind_name_cnfm).

  outline_wrld:-
	shift_Gwindow(wind_numb_pivt),
	getviewsettings(Left,Top, Right,B,_),
	MaxX = Right - Left, MaxY = B - Top,
	MIDDLE_Y = MaxY div 2,
	MIDDLE_X = MAxX div 2,
	setcolor(red),
	line(MIDDLE_X,0, Middle_X,MaxY),
	line(0,Middle_Y,MaxX, Middle_Y),
	drawpoly([0,0, 0,MaxY, MaxX,MaxY, MaxX,0, 0,0]),
	show_quadrant(MaxX,MaxY,magenta),
	make_Scale(wind_numb_disp, 0, MaxX, 0, MaxY).	
	
  show_quadrant(_,_,COLOR) :- 
	setcolor(COLOR), 
	fail.
  show_quadrant(MaxX,MaxY,_):-
	pivot_start(0,0), !,
	MIDDLE_Y = MaxY div 2,
	MIDDLE_X = MaxX div 2,
	line(0,0,MIDDLE_X,MIDDLE_Y),
	line(MIDDLE_X,0,0,MIDDLE_Y).

  show_quadrant(MaxX,MaxY,_):-
	pivot_start(_,0), !,
	MIDDLE_Y = MaxY div 2,
	MIDDLE_X = MaxX div 2,
	line(MaxX,0,MIDDLE_X,MIDDLE_Y),
	line(MIDDLE_X,0,MaxX,MIDDLE_Y).

  show_quadrant(MaxX,MaxY,_):-
	pivot_start(0,_), !,
	MIDDLE_Y = MaxY div 2,
	MIDDLE_X = MaxX div 2,
	line(0,MaxY,MIDDLE_X,MIDDLE_Y),
	line(0,MIDDLE_Y,MIDDLE_X,MaxY).

  show_quadrant(MaxX,MaxY,_):-
	MIDDLE_Y = MaxY div 2,
	MIDDLE_X = MaxX div 2,
	line(MaxX,MaxY,MIDDLE_X,MIDDLE_Y),
	line(MaxX,MIDDLE_Y,MIDDLE_X,MaxY).


/* ************************************************************************** */
/* ****              CONFIRM EXIT                                        **** */
/* ************************************************************************** */

PREDICATES
  confirm_exit
  get_window_release(STRING, INTEGER, INTEGER).

CLAUSES
  confirm_exit:-
	image_open,
	shift_Gwindow(wind_numb_disp),
	getviewsettings(Left,Top,Right,B,_),
	WinY = Top+(B-Top)/2,
	WinX = Left+(Right-Left)/2,
	High = 64, 
	Wide = 200,
        WinR = WinX + Wide,
	WinB = WinY + High,
	CenX = Wide/2,
	CenY = High/2,	
	maxviewport,
	mouse_off,
	image_hold(under_window,WinX,WinY,WinR,WinB), 
	mouse_on,
	make_Gwindow(wind_numb_cnfm, 7,7,"Confirm Exit", WinY, WinX, High, Wide),
	settextstyle(gothic_font, horiz_dir, 1),
	settextjustify(center_text,center_text),
	outtextxy(CenX, CenY, "Yes      No"),	
	settextstyle(default_font, horiz_dir, 1),
	mouse_waitrelease,
	get_window_release(wind_name_cnfm, Wx, _),
	Wx<CenX,!.

   confirm_exit:-
	mouse_off,
	window_(wind_numb_cnfm,_,Left,Top,_,_,_),!,
	remove_Gwindow(wind_numb_cnfm),
	maxviewport, Left2 = Left-1,
	image_recall(under_window, Left2, Top, copy_put),
	mouse_on,
	image_close,
	fail.


  get_window_release(WINDOW, Wx, Wy):-
	repeat,
	mouse(W, Wx, Wy, _, _, BUTTONS),
	W=WINDOW,
	BUTTONS <> 0, !.


/*****************************************************************************
	TO DISPLAY OR NOT TO DISPLAY,THAT IS THE QUESTION
*****************************************************************************/

PREDICATES
  toggle_display_flag.

CLAUSES
  toggle_display_flag:-
	display_info, !,
	retractall(display_info).

  toggle_display_flag:-
	assert(display_info).


/*****************************************************************************
	CHANGE color FLAG TO NEW VALUE
*****************************************************************************/

PREDICATES
  change_color(INTEGER)

CLAUSES
  change_color(Wx):-
	shift_Gwindow(wind_numb_colr),
  	getviewsettings(Left,T,Right,B,_),
	Y1 = B - T - (B-T)/2 + 5, 
	Y2 = B - T - 2, 
	X2 = Right - Left - 1,
	Per = (Right-Left)/15,
        Color = trunc(Wx/Per)+1,
	mouse_off,
	setfillstyle(slash_fill, COLOR),
	bar(1, Y1, X2, Y2), 
	mouse_on,
	retractall(color_selected(_)),
	assert(color_selected(COLOR)).


/*****************************************************************************
	SET START POINT FOR LINE DRAWING
*****************************************************************************/

PREDICATES
  activate_quadrant(INTEGER,INTEGER,INTEGER,INTEGER).
  activate_quadrant(INTEGER,INTEGER,INTEGER,INTEGER,INTEGER,INTEGER).
  
CLAUSES
  activate_quadrant(X,Y,Xmax,Ymax) :-
	shift_Gwindow(Was),
	shift_Gwindow(wind_numb_disp),
	getviewsettings(Left,Top,Right,B,_),
	shift_Gwindow(Was),
	Xd=Right-Left, Yd = B-Top, 
	activate_quadrant(X,Y, Xmax,Ymax, Xd, Yd).
	
  activate_quadrant(X,Y,Xmax,Ymax,_,_):-
	Y < Ymax div 2,
	X < Xmax div 2, !,
	show_quadrant(Xmax,Ymax,black),
	retractall(pivot_start(_,_)),
	assert(pivot_start(0,0)),
	show_quadrant(Xmax,Ymax,magenta).

  activate_quadrant(X,Y,Xmax,Ymax,Xd,_):-
	Y < Ymax div 2,
	X >= Xmax div 2, !,
	show_quadrant(Xmax,Ymax,black),
	retractall(pivot_start(_,_)),
	assert(pivot_start(Xd,0)),
	show_quadrant(Xmax,Ymax,magenta).

  activate_quadrant(X,_,Xmax,Ymax,_,Yd):-
	X < Xmax div 2, !,
	show_quadrant(Xmax,Ymax,black),
	retractall(pivot_start(_,_)),
	assert(pivot_start(0,Yd)),
	show_quadrant(Xmax,Ymax,magenta).

  activate_quadrant(_,_,Xmax,Ymax,Xd,Yd):-
	show_quadrant(Xmax,Ymax,black),
	retractall(pivot_start(_,_)),
	assert(pivot_start(Xd,Yd)),
	show_quadrant(Xmax,Ymax,magenta).


/*****************************************************************************
	PERFORM ACTION FROM MOUSE POSITION AND BUTTONS
*****************************************************************************/

PREDICATES
  update(STRING,INTEGER,INTEGER,INTEGER,INTEGER,INTEGER).
  do_cmd(integer)
CLAUSES
  /* ******** DISPLAY MOUSE POSITION IN THE "Information" WINDOW */
  update(WINDOW_NAME,Wx, Wy, Mx, My ,BUTTONS):-
	display_info,
	mouse_get_mickey_count(Dx,Dy), % update only if mouse was moved
	Dx+Dy<>0,
	shift_Gwindow(wind_numb_info),
	color_selected(COLOR),
	storage(STACK,HEAP,TRAIL),
	format(LINE0,"Stack : %  Heap : % Trail : %  Color %    ",STACK,HEAP,TRAIL,COLOR),
	format(Line2," (%3.0,%3.0) in %-10         (%3.0,%3.0)           %   ",Wx,Wy,Window_Name,Mx,My,Buttons),
	show_text([Line0, 
	"            Window                    Screen          Buttons   ",Line2]),
	fail.

  /* No buttons pressed means no actions */
  update(_,_,_,_,_,0):- !.

  /* Select new color */
  update(wind_name_colr,Wx,_,_,_,_):-
	change_color(Wx).

  /* Select new quadrant */
  update(wind_name_pivt,Wx,Wy,_,_,_):-
	shift_Gwindow(wind_numb_pivt),
	mouse_off,
	getviewsettings(L,T,Right,B,_),
	Mx=Right-L, My=B-T,
	activate_quadrant(WX, WY,Mx,My),
	mouse_on.

  /* Draw a line */
  update(wind_name_disp,WX,WY,_,_,_):-
	shift_Gwindow(wind_numb_disp),
	shift_scale(wind_numb_disp),
	color_selected(COLOR),
	setcolor(COLOR),
	pivot_start(X,Y),
	mouse_off,
	line(X,Y,WX, WY), % scaled to pivot window values
	mouse_on,
	shift_Gwindow(wind_numb_info).

  % ******** MOUSE BUTTON PRESSED IN THE "Commands" WINDOW   ********
  % ******** ACTIVATE COMMAND ACCORDING TO THE LINE SELECTED ********

  update(wind_name_cmds, _, Wy,_,_,_):-
	shift_Gwindow(wind_numb_cmds),
	getviewsettings(_,T,_,B,_),
	Per = (B-T) div 7,
	Cmd = Wy div Per,
	do_cmd(Cmd).
 
 /* Display Info command */
  do_cmd(0) :-
	toggle_display_flag,
	mouse_waitrelease.

  /* Clear Display command */
  do_cmd(1) :-
	shift_Gwindow(wind_numb_disp),
	clear_Gwindow.

  /* Mouse off command */
  do_cmd(2) :-	  
	mouse_off.

  /* Mouse on command */
  do_cmd(3) :-
	mouse_on.
	

  /* Cursor: arrow command */
  do_cmd(4) :-	
	mouse_get(X,Y,_),
	mouse_init(_),
	mouse_set(X,Y),
	mouse_on.

  /* Cursor: skull command */
  do_cmd(5) :-
	make_skull.

  /* Exit command */
  do_cmd(6) :-
	confirm_exit,
	exit(0).



  for(I,I,_).
  for(I,J,K) :-
	J<K,
	J2=J+1,
	for(I,J2,K).


  maxviewport :-
	getmaxx(Mx),
 	getmaxy(My),
	setviewport(0,0, Mx, My, clip).
	



/*****************************************************************************
		DO FOREVER,MAIN PROGRAM LOOP
*****************************************************************************/

PREDICATES
  main	
  top_level
  restore_screen(INTEGER,INTEGER)
CLAUSES
  top_level:-
	shift_Gwindow(wind_numb_info),
	attribute(lightgray),
	mouse_on,
	repeat,
	mouse(NAME,WinX, WinY, Mx, My,Buttons),
	update(NAME,WinX, WinY, Mx, My,Buttons),
	fail.

  
 repeat.
 repeat:-repeat.


 main :-
	textmode(Srow,Scol),
	must_have_mouse,
	bgi_find(BGI_Path),
	trap(initgraph(detect,0,_,_,BGI_Path), Err, no_bgi_err(Err)),
	outline_screen,
	trap(top_level, _, restore_screen(Srow, Scol)),!.
 main.
	

 restore_screen(Row, Col) :-
	closegraph,
	textmode(Row,Col).
 
 
/*****************************************************************************
		MAIN PROGRAM GOAL
*****************************************************************************/

GOAL	main

		
