/*
=====================================================================
	XOBJDEMO.PRO
	
	Object-Oriented Graphic User Interface Demo
	
	Copyright (C) 1990 Prolog Development Center
=====================================================================
*/

project "XOBJDEMO"

include "XOBJDEMO.PRE"

include "GRAPHICS\\OOPGRAPH.PRO"


include "MOUSE\\MUSTMOUS.PRO"
include "GRAPHICS\\BGIFIND.PRO"


/*
---------------------------------------------------------------------
	SAVE & CONSULT SETUP
---------------------------------------------------------------------
*/


PREDICATES
	
	%bgi_demo
	save_GUI
	consult_GUI
	clear_GUI

CLAUSES

	consult_GUI :-
		sBar_Consult("SCROLL.GUI"),
	        gWindows_Consult("GWINDOWS.GUI"),
		dial_Consult("DIALS.GUI"),
	       	drawlistView_Consult("DRAWVIEW.GUI"),
		object_Consult("OBJECTS.GUI").

	save_GUI :-
		sBar_Save("SCROLL.GUI"),
        	gWindows_Save("GWINDOWS.GUI"), 
		dial_Save("DIALS.GUI"),
	       	drawlistView_Save("DRAWVIEW.GUI"),
		object_Save("OBJECTS.GUI").

	clear_GUI :-
	  	sBar_Clear,
        	gWindows_Clear,
		dial_Clear,
	       	drawlistView_Clear,
		object_Clear.


/*
---------------------------------------------------------------------
	MAIN LOOP
---------------------------------------------------------------------
*/

PREDICATES

	main
	main(MouseX,MouseY,MouseButton)
	mainact(Integer)
	read_Mouse(Integer,Integer,Integer)
	gWindow_Init

CLAUSES

	gWindow_Init :-
		getMaxX(MaxX), 
		getMaxY(MaxY),
		gWindow_initviewport(0,0,MaxX,MaxY,clip).


	read_Mouse(MouseX,MouseY,Buttons) :-
  		mouse_on, 
  		repeat,
  		mouse_get(MouseX,MouseY,Buttons),
		Buttons >< 0, !.


	main :- repeat,
        	gWindow_Init,
		getmaxx(X), 
		getmaxy(Y),
		setviewport(0,0,X,Y,0),
		read_mouse(MouseX,MouseY,Buttons),
		main(MouseX,MouseY,Buttons),
		fail, !.


	main(MX,MY,Buttons) :-
        	gWindow_abs_to_rel(MX,MY,MouseX,MouseY),
		object_Get(Object,Garea),
		region_Inside(MouseX,MouseY,Garea), !,
	        gWindow_setmaxviewport,
		message(Object,mousepressed(MouseX,MouseY,Buttons)).

	main(MouseX,MouseY,_) :-
		settextstyle(default_font,horiz_dir,1),
        	Menu = ["New Window","Load","Save","Clear","Exit"],
		gmenu(MouseX,MouseY,white,green,blue,Menu,Choice),
        	mainact(Choice),
		fail.


	mainact(1) :- !,
        	drawlistView_Create(_,colors(blue,white,white,red)).

	mainact(2) :- !, 
		clear_GUI, 
		maxviewport,
		mouse_off, 
		clearviewport, 
		mouse_on,
		consult_GUI, 
		screen_Redraw.
	mainact(3) :- !, save_GUI.
	mainact(4) :- !, 
		maxviewport, 
		mouse_off,
		clearviewport, 
		mouse_on, 
		clear_GUI.
	mainact(5) :- !, exit(0).

/*
---------------------------------------------------------------------
	INSTALL THE DIAL DEMO
---------------------------------------------------------------------
*/

PREDICATES

	install_dial_demo
	adj_region(region,region, real)
CLAUSES

	install_dial_demo :-
		getmaxy(MaxY),
		getaspectratio(Xa,Ya),
		Shigh = round(15*(Xa/Ya))+1,

		S1Bot = 15 + Shigh, 
		adj_region(region(40,15,200,S1Bot) , Bar1Reg, MaxY),		
		sBar_Create(Bar1,horz,Bar1Reg,style(7,5,0),10),

		S2Bot = 105 + Shigh,
		adj_region(region(335,105,585,S2Bot), Bar2Reg, MaxY),
		sBar_Create(Bar2,horz,Bar2Reg,style(7,5,0),10),
	
		adj_region(region(1,45,15,190), Bar3Reg, MaxY),
		sBar_Create(Bar3,vert,Bar3Reg,style(7,5,0),10),

		adj_region(region(250,100,265,195), Bar4Reg, MaxY),
		sBar_Create(Bar4,vert,Bar4Reg,style(7,5,0),10),

		adj_region(region(300,100,315,195), Bar5Reg, MaxY),
		sBar_Create(Bar5,vert,Bar5Reg,style(7,5,0),10),

		
		setcolor(lightcyan),
		D1Y = 70/199 * MaxY,			
		dial_Create(Dial1,half,120,D1Y,5,0,100,25,"HALF",
			[band(blue,25,50),
			 band(green,50,75),
			 band(red,75,100)]),

		setcolor(cyan),
		D2Y = 50/199 * MaxY,
		dial_Create(Dial2,hstrip,480,D2Y,5,0,200,40,"HSTRIP",
			[band(green,0,40),
			 band(green,80,120),
			 band(red,160,200)]),
	
		setcolor(lightcyan),
		D3Y = 140/199 * MaxY,	
		dial_Create(Dial3,full,130,D3Y,5,0,100,25,"FULL",
			[band(blue,0,25),
			 band(green,25,50),
			 band(blue,50,75),
			 band(red,75,100)]),
		
		setcolor(lightblue),
		D4Y = 50/199 * MaxY,
		dial_Create(Dial4,vstrip,300,D4Y,5,0,50,10,"VSTRIP",
			[band(green,0,10),
   			 band(green,20,30),
			 band(red,40,50)]),

		setcolor(brown),
		D5Y = 160/199 * MaxY,
		dial_Create(Dial5,pedal,460,D5Y,5,0,50,25,"PEDAL",
			[band(blue,0,25),
			 band(red,25,50)]),
	
		sBar_Install_WhenSet(Bar1,dial(Dial1),0),
		sBar_Install_WhenSet(Bar2,dial(Dial2),0),
		sBar_Install_WhenSet(Bar3,dial(Dial3),0),
		sBar_Install_WhenSet(Bar4,dial(Dial4),0),
		sBar_Install_WhenSet(Bar5,dial(Dial5),0).



	adj_region(region(Left, Top, Right, Bottom), 
		       region(Left, Top2, Right, Bottom2), MaxY) :-
		Top2=(Top/199)*MaxY,
		Bottom2 = (Top2-Top) + Bottom.
		
		
/*
---------------------------------------------------------------------
	RUN
---------------------------------------------------------------------
*/

PREDICATES

	run

CLAUSES

	run :-  install_dial_demo,
		main.

PREDICATES
	start
	bgi_demo
	bgi_demo_exit
	
CLAUSES

	start :- 
		must_have_mouse,
		bgi_demo, 
		fail.
	start.

	bgi_demo :-
		bgi_find(BGI_PATH),
		trap(initgraph(detect,0,_,_,BGI_PATH),Err, no_bgi_err(Err)),
		mouse_on,
		gwindow_init,
		trap(run,_,bgi_demo_exit).

	bgi_demo_exit :-
		closegraph.

goal
start
