/*
===========================================================================
 	XPOLLSND.PRO

	Demo of polled transmission with 'time out'
		(sender copy)

	Copyright (C) 1990 Prolog Development Center
===========================================================================
*/

Project "xpollsnd"
CONSTANTS
  port=1
 sender_copy=1
  
GLOBAL DOMAINS
  FILE = df

include "SERIAL\\COMGLOBS.PRE"
 
PREDICATES
  main
  poll
  send_str(STRING)
  send_ch_CRLF(CHAR,INTEGER)
  send_ch(CHAR,INTEGER)
  receive_str()
  receive_ch(CHAR,INTEGER)
  wait_ok(INTEGER,INTEGER,INTEGER)
  tx_status(INTEGER)
  check_status(INTEGER,INTEGER,STRING)
  wait_buffer_empty
  wait_empty(real)
  wait_for_break
GOAL
  main.
  
  
CLAUSES

  main :- trap(poll,_,fail),!.
  main :- closeRS232(port).	% be sure it closes no matter what

  poll :-	
	openRS232(port,256,256,b1200,parity_none,data8,stop1,xon_xoff),

ifdef sender_copy
	wait_for_break,
	send_str("Hello all our readers\n"),
	wait_buffer_empty,
	sendbreak_RS232(port,100),
elsedef			
	openwrite(df,"poll.log"),
	sendbreak_RS232(port,100),
	receive_str,
	closefile(df),
enddef
	
	closeRS232(port).


  /* Transmit a string  */
  send_str(""):-!.
  send_str(S):-
  	frontchar(S,CH,S2),
	write(CH), 
	send_ch_CRLF(CH,50),
	send_str(S2).

  send_ch_CRLF('\10',I):-
  	!,send_ch('\13',I), 
  	send_ch('\10',I).
  send_ch_CRLF(CH,I):-send_ch(CH,I).

  send_ch(CH,_):-txch_RS232(port,CH),!.
  send_ch(CH,I):-
	status_RS232(port,Status),!,
	wait_ok(Status,I,I2), send_ch(CH,I2).

  /* Receive a string and copy it to a file */
  receive_str():-
	receive_ch(CH,50),!,
	write(CH),
	writedevice(FP), 
	writedevice(df), 
	write(CH), 
	writedevice(FP),
  	receive_str().
  receive_str().

  receive_ch(CH,_):-
  	rxch_RS232(port,CH), 
  	CH<>'\013', !.		% discard these
  receive_ch(CH,_):-		
  	rxch_RS232(port,CH), !.
  receive_ch(CH,I):-
	status_RS232(port,Status), !,
	Status<>16,
	wait_ok(Status,I,I2), 
	receive_ch(CH,I2).

  /* Time out */
  wait_ok(_,I,I2):-
  	I > 0, 
  	I2=I-1, 
  	sleep(10),!.
  wait_ok(Status,_,50):-
  	tx_status(Status).

  /* De-mask status value */
  tx_status(0):-!.
  tx_status(Status):-
  	shiftwindow(WD), shiftwindow(1),
  	check_status(Status,1, "Input Characters have beenlost"),
  	check_status(Status,2,  "Parity Error"),
  	check_status(Status,4,  "Overrun detected"),
  	check_status(Status,8,  "Framing error detected"),
  	check_status(Status,16, "Break signal detected"),
  	check_status(Status,32, "An Xoff has been received"),
  	check_status(Status,64, "An Xon has been received"),
  	check_status(Status,128,"An Xoff has been transmitted"),
  	check_status(Status,256,"An Xon has been transmitted"),
  	check_status(Status,512,"Input buffer empty when attemptto read"),
  	check_status(Status,1024,"Output buffer full when attemptto write"),
  	check_status(Status,2048,"Ring indicator"),
  	check_status(Status,4096,"Carrier detected"),
  	write("\nPress Space to continue or Esc to abort"), 
  	readchar(Ch),
  	shiftwindow(2), shiftwindow(3),
  	shiftwindow(WD),CH<>'\27'.

  check_status(Status,BitMask,Mess):-
  	bitand(Status,BitMask,V), 
  	V<>0, !, 
  	nl, write(Mess).
  check_status(_,_,_).

  wait_empty(TimeMark) :-
    	queuesize_RS232(port, _, OutBuf),
  	OutBuf>0,!,
  	not(timeout(Timemark)),
  	wait_empty(Timemark).
  wait_empty(_).
  	
  wait_buffer_empty:-
  	marktime(3000, Mark),
  	wait_empty(Mark),!.
  wait_buffer_empty:-
  	write("Timed out waiting for receiver!").
  
  
  wait_for_break :-
  	status_RS232(port,Stat), 
  	bitand(Stat,16,BreakBit),
  	BreakBit<>0,!.
  wait_for_break :- wait_for_break.
  
