/*
===========================================================================
	XPRNTOOL.PRO

	Demo of printer tools: PRNTOOLS.PRO

	Copyright (C) 1990 Prolog Development Center
===========================================================================
*/

check_determ

DOMAINS
  FILE = prn_file

include "tdoms.pro"
include "tpreds.pro"
include "ui\\status.pro"
include "ui\\menu.pro"
include "ui\\lineinp.pro"
include "ui\\filename.pro"
include "misc\\prntools.pro"


/*****************************************************************************
	STORED PRINTER SETUP PARAMETERS
*****************************************************************************/

DATABASE - prndat
  determ margin(INTEGER)
  determ topmargin(INTEGER)
  determ pagelength(INTEGER)
  determ printer_init(STRING)
  determ printdest(STRING)
  determ charconv(STRING)  
  determ header(STRING)  
  determ footer(STRING)  


CLAUSES
/*****************************************************************************
	DEFAULT VALUES
*****************************************************************************/
	margin(2).
	topmargin(3).
	pagelength(66).
	printer_init("").
	printdest(con).
	charconv("Y").
	header("").
	footer("").
	
	

/*****************************************************************************
	MODIFY PRINTER SETUP
*****************************************************************************/

PREDICATES
  printer_setup(INTEGER)         	

CLAUSES
  printer_setup(1):-
  	printer_init(Init),
	lineinput(16,40,60,7,23," Initialization sequence: ",Init,Newinit),
	retractall(printer_init(_)),
	assert(printer_init(Newinit)).
  printer_setup(2):-
  	pagelength(Len),
  	str_int(Sl,Len),
	lineinput(17,40,40,7,23," Page Length: ",Sl,Nsl),
	str_int(Nsl,Nlen),
	retractall(pagelength(_)),
	assert(pagelength(Nlen)).
  printer_setup(3):-
  	margin(Len),
  	str_int(Sl,Len),
	lineinput(18,40,40,7,23," Margin: ",Sl,Nsl),
	str_int(Nsl,Nlen),
	retractall(margin(_)),
	assert(margin(Nlen)).
  printer_setup(4):-
  	topmargin(Len),
  	str_int(Sl,Len),
	lineinput(19,40,40,7,23," Top Margin: ",Sl,Nsl),
	str_int(Nsl,Nlen),
	retractall(topmargin(_)),
	assert(topmargin(Nlen)).
  printer_setup(5):-
  	printdest(Dest),
  	lineinput(20,40,40,7,23," Print Destination: ",Dest,Ndest),
	retractall(printdest(_)),
	assert(printdest(Ndest)).
  printer_setup(6):-
  	charconv(Conv),
  	lineinput(21,40,40,7,23," Character conversion (Y/N): ",Conv,Chconv),
  	upper_lower(Chconvu,Chconv),
        retractall(charconv(_)),
        assert(charconv(Chconvu)).
  printer_setup(7):-
  	footer(Footer),
  	lineinput(20,40,40,7,23," Footer: ",Footer,NFooter),
	retractall(footer(_)),
	assert(footer(Nfooter)).
  printer_setup(8):-
	save("xprn.dat",prndat).


/*****************************************************************************
	Edit
*****************************************************************************/

PREDICATES
  chksave(INTEGER,STRING,STRING,STRING)
  ed
  ed(STRING)

CLAUSES
  chksave(0,FILENAME,TXT,TXT1):-!,
	TXT><TXT1,
	file_str(FILENAME,TXT1).
  chksave(1,FILENAME,TXT,TXT1):-
	TXT><TXT1,
	lineinput(5,70,40,66,66,"Skip the changes (y/n) ? ","",ANS),
	upper_lower(ANS,ANS1),
	ANS1="n",!,
	file_str(FILENAME,TXT1).
  chksave(_,_,_,_).


  ed(FILENAME):-
	existfile(FILENAME),!,
	file_str(FILENAME,TXT),
	textmode(RR,CC),
	makewindow(1,2,2,"",0,0,RR,CC),
	edit(TXT,TXT1,FILENAME,"","",0,"",RET),
	chksave(RET,FILENAME,TXT,TXT1),
	removewindow.
  ed(FILENAME):-
	lineinput(5,70,40,66,66,"Create new file (y/n) ? ","",ANS),
	upper_lower(ANS,ANS1),
	ANS1="y",!,
	textmode(RR,CC),
	makewindow(1,2,2,"",0,0,RR,CC),
	edit("",TXT1,FILENAME,"","",0,"",RET),
	chksave(RET,FILENAME,"\1",TXT1),
	removewindow.
  ed(_).


  ed:-	readfilename(17,26,23,20,txt,"",FILENAME),
	FILENAME><"",
	ed(FILENAME).


/*****************************************************************************
	Initialize and close print
*****************************************************************************/

DOMAINS
  PRNMODE = prn; scrn	% Add your new alternatives here !

PREDICATES
  prepare_print(PRNMODE)
  end_print(PRNMODE)
  wait_key(PRNMODE)

CLAUSES
  prepare_print(prn):-
	margin(MARGIN),
	topmargin(TOPMARGIN),
	pagelength(PAGELENGTH),
	charconv(CHARCONV),
	printer_init(INITCODE),
	printdest(PRINTDEST),
	init_printer(MARGIN,TOPMARGIN,PAGELENGTH,CHARCONV,INITCODE,PRINTDEST),!.
  prepare_print(scrn):-
  	textmode(RR,CC),
  	RR1=RR-1, 
  	RR2=RR1-6,
  	makestatus(112," Use BREAK to stop printing"),
  	makewindow(20,7,23," SCREEN PRINT ",0,0,RR1,CC),
	init_printer(0,0,RR2,"N","",con),!.


  end_print(prn):-
  	close_printer,!.
  end_print(scrn):-
  	close_printer,
	shiftwindow(20),
  	removewindow,
  	removestatus,!.

  wait_key(scrn):-!,
  	write(">> Print ended. Press any Key"),readkey(_).
  wait_key(_).

/***************************************************************************
	Make footers and headers
****************************************************************************/

PREDICATES
  datestr(STRING)

CLAUSES
  datestr(DATO):-
	date(Y,M,D),
	format(DATO,"Date: %.%.%",D,M,Y).

  make_footer:-
        footer(Footer),
        write_str("\n----------------------------------------------------------------------------\n"),
	get_pageno(PAGENO),
	format(LASTLINE,"%-68Page %3\n",Footer,PAGENO),
	write_str(LASTLINE).

  make_header:-
        header(Header),
	datestr(Date),
        format(Line,"\n%-60 %\n",Header,Date),
        write_str(Line),
        write_str("----------------------------------------------------------------------------\n\n").


/*****************************************************************************
	Print a text file
*****************************************************************************/

PREDICATES
  print_file(PRNMODE,STRING)

CLAUSES
  print_file(PRNMODE,TXT):-
	prepare_print(PRNMODE),
	trap(write_str(TXT),_,end_print(PRNMODE)),!,
	wait_key(PRNMODE),
	end_print(PRNMODE).
  print_file(_,_).


/*****************************************************************************
	Actions
*****************************************************************************/

PREDICATES
  main_process(INTEGER)
  get_prnmode(INTEGER,PRNMODE)

CLAUSES
  get_prnmode(1,scrn).
  get_prnmode(2,prn).


  main_process(1):-!,ed.

  main_process(2):-
	readfilename(17,26,23,20,txt,"XPRNTOOL.PRO",FILENAME),
	existfile(FILENAME),
	file_str(FILENAME,TXT),
	retractall(header(_)),assert(header(FILENAME)),
	menu_repeat(18,39,7,23,
  		[" Screen print ",
  		 " Printer "],
  	        " Print mode ",1,CH),
  	get_prnmode(CH,PRNMODE),
  	print_file(PRNMODE,TXT),
  	fail.


  main_process(3):-
  	menu_repeat(14,27,7,23,
  		[" Initialization",
  		 " PageLength",
  		 " Margin",
  		 " Top Margin",
  		 " Print Destination  ",
  		 " Character Conversion ",
  		 " Footer ",
  		 " Save setup"],
  	         " Printer setup",1,Ch),
  	printer_setup(Ch),
  	fail.
  main_process(_).


/*****************************************************************************
	Testing menu
*****************************************************************************/

PREDICATES
  demo_main

CLAUSES
  demo_main:-	
  	trap(consult("xprn.dat",prndat),_,fail),
  	fail.
  demo_main :-
    	menu_repeat(12,23,7,23,
	    [" Edit/Create text files",
	     " Print a textfile",
  	     " Printer Setup",
	     " End"],
	     " Make a selection",1,Ch),
	main_process(Ch),
	Ch=4,!,
	removewindow.

/*****************************************************************************
	The GOAL
*****************************************************************************/

GOAL	textmode(RR,CC),
	makewindow(1,2,2,"Demo of PRNTOOLS.PRO",0,0,RR,CC),
	write("\nPRNTOOLS.PRO contains some usefull predicates to control output to a printer"),
	write("\nThe facilities contains:\n"),
	write("\n\t- Automatic pagebreak"),
	write("\n\t- Leftmargin and topmargin"),
	write("\n\t- Headers and Foothers"),
	write("\n\t- Character conversions to the printer"),
	write("\n\t- Couting line and page numbers"),
	write("\n\t- Printer initialization"),
	demo_main.
