/*
===========================================================================
	XREPORT.PRO					 
  									 
  	Demo of screen handler: Report Generation		 	 
  									 
  	The report generator makes it possible to design a page 	 
  	layout as if it were to be a screen layout, and then print 	 
  	the designed page.						 
  									 
  	Copyright (C) 1990 Prolog Development Center
===========================================================================
*/

check_determ


/************************************************************************
 *			Domains						*
 ************************************************************************/

include "tdoms.pro"

DOMAINS
  FIELD_NAME	= SYMBOL
  TYPE		= int(); str(); real()


/************************************************************************
 *			Database Predicates				*
 ************************************************************************/

DATABASE
  delivery
  field(FIELD_NAME, TYPE, ROW, COL, LEN)
  payment(STRING)
  txtfield(ROW, COL, LEN, STRING)		
  value(FIELD_NAME, STRING)		
  warranty
  windowsize(ROW, COL)


/************************************************************************
 *			Include Tools					*
 ************************************************************************/

include "tpreds.pro"
include "screen\\report.pro"


/************************************************************************
 *			Support Predicates				*
 ************************************************************************/

PREDICATES
  		get_nth_element(INTEGER, STRINGLIST, STRING)
		price(FIELD_NAME, REAL)
  nondeterm	product(STRING, STRING, REAL)
		stringlist_string(STRINGLIST, STRING)

CLAUSES
  delivery.

  get_nth_element(1, [HEAD|_], HEAD) :- !.
  get_nth_element(N, [_|TAIL], ELEMENT) :-
  	N > 1,
  	NEW_N = N - 1,
  	get_nth_element(NEW_N, TAIL, ELEMENT).

  payment("Cheque").

  price(FIELD_NAME, PRICE) :- 
  	value(FIELD_NAME, CODE), 
  	product(CODE, _, PRICE),
  	!.
  price(_, 0).

  product("1111", "Washing Machine", 200.35).
  product("2222", "Dishwasher", 239.67).
  product("3333", "Fridge and Freezer", 456.78).  
  product("4444", "Radio", 456.78).  
  product("5555", "Television", 456.78).  

  stringlist_string([], "").
  stringlist_string([HEAD|TAIL], STRING) :- 
  	stringlist_string(TAIL, REST_OF_STRING), 
  	concat(HEAD, REST_OF_STRING, STRING).

  value("name", "J B Gruntfuttock").
  value("street", "3 Railway Terrace").
  value("city", "Seattle").
  value("state", "Washington").
  value("tel", "222 333 4444").
  value("item1", "1111").
  value("item2", "2222").
  value("item3", "3333").
  value("money", "1000.00").


/************************************************************************
 *			Field Values					*
 ************************************************************************/

  field_value(change, CHANGE2) :-
	value(money, MONEY1),
	!,
	str_real(MONEY1, MONEY2),
        price(item1, PRICE1),
        price(item2, PRICE2),
        price(item3, PRICE3),
        CHANGE1 = MONEY2 - (PRICE1 + PRICE2 + PRICE3),
        str_real(CHANGE2, CHANGE1).

  field_value(date, DATE) :- 
  	!,
  	date(YEAR, MONTH, DAY), 
  	format(DATE, "%/%/%", YEAR, MONTH, DAY).

  field_value(delivery, yes) :- delivery, !.
  field_value(delivery, no) :- !.

  field_value(make1, DESC) :- 
	value(item1, CODE), 
	product(CODE, DESC, _), !.

  field_value(make2, DESC) :- 
	value(item2, CODE), 
	product(CODE, DESC, _), !.

  field_value(make3, DESC) :- 
	value(item3, CODE), 
	product(CODE, DESC, _), !.


  field_value(payment, PAYMENT) :- payment(PAYMENT), !.

  field_value(price1, PRICE2) :- 
  	value(item1, CODE),
  	product(CODE, _, PRICE1),
	!,
	str_real(PRICE2, PRICE1).

  field_value(price2, PRICE2) :-
	value(item2, CODE),
	product(CODE, _, PRICE1),
	!,
	str_real(PRICE2, PRICE1).

  field_value(price3, PRICE2) :-
	value(item3, CODE),
	product(CODE, _, PRICE1),
  	!,
	str_real(PRICE2, PRICE1).

  field_value(time, TIME) :-
  	!,
	time(HOUR, MINUTE, SECOND, _),
	format(TIME, "%:%:%", HOUR, MINUTE, SECOND).

  field_value(total, TOTAL2) :-
  	!,
        price(item1, PRICE1),
        price(item2, PRICE2),
        price(item3, PRICE3),
        TOTAL1 = PRICE1 + PRICE2 + PRICE3,
        str_real(TOTAL2, TOTAL1).
 
  field_value(warranty, yes) :- warranty, !. 
  field_value(warranty, no) :- !. 

  field_value(FIELD_NAME, VALUE) :- value(FIELD_NAME, VALUE), !.


/************************************************************************
 *			Goal						*
 ************************************************************************/
 
GOAL
  textmode(ROWS, COLS),
  consult("xreport.scr"),
  makewindow(1, 31, 0, "", 0, 0, ROWS, COLS),
  /* writedevice(printer), */
  report(20).

