/*
===========================================================================
	XSHOP.PRO					
  									 
  	Demo of screen handler: Completing a pro-forma invoice	 	 
  									 
	Copyright (C) 1990 Prolog Development Center
===========================================================================
*/

check_determ


/************************************************************************
 *			Domains						*
 ************************************************************************/

include "tdoms.pro"

DOMAINS
  FIELD_NAME	= SYMBOL
  TYPE		= int(); str(); real()


/************************************************************************
 *			Database Predicates				*
 ************************************************************************/

DATABASE
  delivery
  field(FIELD_NAME, TYPE, ROW, COL, LEN)
  payment(STRING)
  txtfield(ROW, COL, LEN, STRING)		
  value(FIELD_NAME, STRING)		
  warranty
  windowsize(ROW, COL)
  

/************************************************************************
 *			Include Tools					*
 ************************************************************************/

include "tpreds.pro"
include "ui\\menu.pro"
include "ui\\status.pro"
include "ui\\lineinp.pro"
include "screen\\vscrhnd.pro"				% Or vscrhnd.pro


/************************************************************************
 *			Support Predicates				*
 ************************************************************************/

PREDICATES
		get_nth_element(INTEGER, STRINGLIST, STRING)
		let_user_select_product(FIELD_NAME)
		price(FIELD_NAME, REAL)
  nondeterm	product(STRING, STRING, REAL)

CLAUSES
  get_nth_element(1, [HEAD|_], HEAD) :- !.
  get_nth_element(N, [_|TAIL], ELEMENT) :-
  	N > 1,
  	NEW_N = N - 1,
  	get_nth_element(NEW_N, TAIL, ELEMENT).

  let_user_select_product(FIELD_NAME) :-
  	retractall(value(FIELD_NAME, _)),
	cursor(ROW, COL),
	findall(X, product(X, _, _), CODE_LIST),
	findall(X, product(_, X, _), DESCRIPTION_LIST),
	menu(ROW, 
	     COL, 
	     23, 
	     23,
	     DESCRIPTION_LIST,
	     "Please select the product",
	     1,
	     CHOICE),
	get_nth_element(CHOICE, CODE_LIST, CODE),
	assert(value(FIELD_NAME, CODE)).


  payment(cash).

  price(FIELD_NAME, PRICE):- 
  	value(FIELD_NAME, CODE), 
  	product(CODE, _, PRICE),
	!.
  price(_, 0).

  product("1111", "Washing Machine", 200.35).
  product("2222", "Dishwasher", 239.67).
  product("3333", "Fridge and Freezer", 456.78).  
  product("4444", "Radio", 45.78).  
  product("5555", "Television", 256.78).  


CLAUSES
/************************************************************************
 *			Field Actions					*
 ************************************************************************/

  field_action(delivery) :- retract(delivery), !.
  field_action(delivery) :- assert(delivery).

  field_action(item1) :- let_user_select_product(item1).

  field_action(item2) :- let_user_select_product(item2).

  field_action(item3) :- let_user_select_product(item3).

  field_action(payment) :-
  	retractall(payment(_)),
	cursor(ROW, COL),
	LIST=["Cash", "Check", "Credit card"],
	menu(ROW,
	     COL,
	     23,
	     23,
	     LIST,
	     "Please select method of payment",
	     1,
	     CHOICE),
	get_nth_element(CHOICE, LIST, METHOD),
	assert(payment(METHOD)). 

  field_action(warranty) :- retract(warranty), !.
  field_action(warranty) :- assert(warranty).


/************************************************************************
 *			Field Values					*
 ************************************************************************/

  field_value(change, CHANGE2) :-
	value(money, MONEY1),
	!,
	str_real(MONEY1, MONEY2),
        price(item1, PRICE1),
        price(item2, PRICE2),
        price(item3, PRICE3),
        CHANGE1 = MONEY2 - (PRICE1 + PRICE2 + PRICE3),
        str_real(CHANGE2, CHANGE1).

  field_value(date, DATE) :- 
  	!,
  	date(YEAR, MONTH, DAY), 
  	format(DATE, "%/%/%", YEAR, MONTH, DAY).

  field_value(delivery, yes) :- delivery, !.
  field_value(delivery, no) :- !.

  field_value(make1, DESC) :- value(item1, CODE), product(CODE, DESC, _), !.

  field_value(make2, DESC) :- value(item2, CODE), product(CODE, DESC, _), !.

  field_value(make3, DESC) :- value(item3, CODE), product(CODE, DESC, _), !.

  field_value(payment, PAYMENT) :- payment(PAYMENT), !.

  field_value(price1, PRICE2) :- 
  	value(item1, CODE),
  	product(CODE, _, PRICE1),
	!,
	str_real(PRICE2, PRICE1).

  field_value(price2, PRICE2) :-
	value(item2, CODE),
	product(CODE, _, PRICE1),
	!,
	str_real(PRICE2, PRICE1).

  field_value(price3, PRICE2) :-
	value(item3, CODE),
	product(CODE, _, PRICE1),
  	!,
	str_real(PRICE2, PRICE1).

  field_value(time, TIME) :-
  	!,
	time(HOUR, MINUTE, SECOND, _),
	format(TIME, "%:%:%", HOUR, MINUTE, SECOND).

  field_value(total, TOTAL2) :-
  	!,
        price(item1, PRICE1),
        price(item2, PRICE2),
        price(item3, PRICE3),
        TOTAL1 = PRICE1 + PRICE2 + PRICE3,
        str_real(TOTAL2, TOTAL1).
 
  field_value(warranty, yes) :- warranty, !. 
  field_value(warranty, no) :- !. 

  field_value(FIELD_NAME, VALUE) :- value(FIELD_NAME, VALUE), !.


/************************************************************************
 *			No Input Fields					*
 ************************************************************************/

  noinput(change).	
  noinput(date).	
  noinput(delivery).	
  noinput(make1).		
  noinput(make2).
  noinput(make3).	
  noinput(payment).	
  noinput(price1).	
  noinput(price2).
  noinput(price3).
  noinput(time).		
  noinput(total).
  noinput(warranty).


/************************************************************************
 *			Goal						*
 ************************************************************************/
 
GOAL	
  consult("xshop.scr"),
  textmode(ROWS, COLS),
  WINDOW_ROWS = ROWS - 1,
  makewindow(1, 66, 66, "Sales Transaction Record", 0, 0, WINDOW_ROWS, COLS),
  makestatus(112, " Fill in the sales record and press F10 when finished."),
  scrhnd(off, _).

