/*
==========================================================================
	XTERM.PRO

	Terminal Emulation program (PC <---> PC or PC <---> Mainframe)
	==========================

	Everything we get from the serial port will be shown in
	the Receive window, and every character typed from the keyboard
	will be transmitted through the serial port.
	
	This process will be aborted when the user presss Esc.

	MUST BE COMPILED AS PROJECT

	Copyright (C) 1990 Prolog Development Center
===========================================================================
*/

check_determ
project "xterm"

CONSTANTS

 port=1

 cook_incoming_cr=1 
 

include "serial\\comglobs.pre"

  
PREDICATES
  main
  run
  terminal
  chk_rdch
  chk_wrch
  rdch_CRLF_RS232(Char)
  wrch_CRLF(Char)
  trans_ch(Char)
  cook_13(char)


GOAL 
  main.

CLAUSES
  
  main :-
  	trap(run,_,fail),!.
  main :- 
  	closeRS232(port).  % just to be sure


  run :-
	textmode(ROWS,COLS),
	ROW1=ROWS div 2,
	makewindow(2, 42,36," Transmit window ", 0,0,ROW1,COLS),
        makewindow(3, 63,5," Receive window ", ROW1,0,ROW1,COLS),
	InputBufSize	=	10,		% Size of input buffer
	OutputBufSize	=	10,		% Size of output buffer 
	BaudRate	=	b1200,		% bits per second
	Parity		=	parity_none, 
	WordLength	=	data8,	
	StopBits	=	stop1,	
  	Protocol	= 	no_protocol,	
  	
  	openRS232(port, InputBufSize, OutputBufSize, BaudRate, Parity,
  		  WordLength, StopBits, Protocol),

	terminal, !. % terminal must fail for program to terminate

  run :-		% Must be sure the port gets closed! 
  	closeRS232(port),
  	removewindow(2,1),
  	removewindow(3,1).


/****************************************************************/
/*								*/
/*		TERMINAL MODE					*/
/*		Transmission without time out			*/
/*								*/
/****************************************************************/

  terminal :- 
  	chk_rdch, 
  	chk_wrch,
  	terminal.

  chk_rdch :- 
  	rdch_CRLF_RS232(CH),!,
	gotowindow(3),
  	write(CH).
  chk_rdch.

  chk_wrch :- 
  	gotowindow(2), 
  	cursor(R,C),
  	cursor(R,C),
  	not(keypressed),!.
  chk_wrch :- 
  	readchar(CH),
  	CH<>'\027', 
  	wrch_CRLF(CH).
  
  rdch_CRLF_RS232(CH) :-
  	rxch_RS232(port,CH),
  	cook_13(CH).
  	
ifdef cook_incoming_cr      % The program transmitting to this one
  cook_13(13) :- 	    % may send either CR or CRLF (like XTERM)
  	!,gotowindow(3),    % at end of line. This works for either by	
  	nl.		    % discarding the LF, and echoing CR as NL	
  cook_13(10) :-!, fail. 
enddef
  cook_13(_).
  
  
  wrch_CRLF('\013') :- !,
  	nl, 
  	trans_ch('\013'), 
  	trans_ch('\010').
  wrch_CRLF(CH) :- 
  	write(CH), 
  	trans_ch(CH).
  
  trans_ch(CH) :- txch_RS232(port,CH),!.
  trans_ch(CH) :- trans_ch(CH).


