/*
===========================================================================
	XTRAVEL.PRO					
  									 
  	Demo of screen handler: Will you reach your destination in 	 
  	time?								 
  									 
 	Copyright (C) 1990 Prolog Development Center			 
===========================================================================
*/

check_determ


/************************************************************************
 *			Domains						*
 ************************************************************************/

include "tdoms.pro"

DOMAINS
  FIELD_NAME	= SYMBOL
  TYPE		= int(); str(); real()

/************************************************************************
 *			Database Predicates				*
 ************************************************************************/

DATABASE
  field(FIELD_NAME, TYPE, ROW, COL, LEN)
  txtfield(ROW, COL, LEN, STRING)		
  value(FIELD_NAME, STRING)		
  windowsize(ROW, COL)
  

/************************************************************************
 *			Include Tools					*
 ************************************************************************/

include "tpreds.pro"
include "ui\\menu.pro"
include "ui\\status.pro"
include "screen\\scrhnd.pro"		% Or vscrhnd.pro


/************************************************************************
 *			Support Predicates				*
 ************************************************************************/

PREDICATES
  pad_to_two_chars_if_appropriate(STRING, STRING)
  parse_time(STRING, INTEGER, INTEGER, INTEGER)
  parse_time_aux(INTEGERLIST, INTEGER, INTEGER, INTEGER)
  string_integerlist(STRING, INTEGERLIST)
  time_diff(STRING, STRING, REAL)

CLAUSES
  pad_to_two_chars_if_appropriate(STRING_IN, STRING_OUT) :-
  	str_len(STRING_IN, LENGTH),
  	LENGTH < 2,
  	!,
  	concat("0",STRING_IN, STRING_OUT).		
  pad_to_two_chars_if_appropriate(STRING, STRING).

  parse_time(TIME, HOUR, MINUTE, SECOND):-
	string_integerlist(TIME, INTEGERLIST),
        parse_time_aux(INTEGERLIST, HOUR, MINUTE, SECOND).
        
  parse_time_aux([HOUR,MIN,SEC], HOUR, MIN, SEC):-!.
  parse_time_aux([HOUR,MIN], HOUR, MIN, 0) :-!.
  parse_time_aux([HOUR], HOUR, 0, 0).
   
    
  string_integerlist("",[]):-!.
  string_integerlist(STRING, [HEAD|TAIL]):-
	fronttoken(STRING, FRONTTOKEN, REST_OF_STRING),
	str_int(FRONTTOKEN, HEAD),
	!,
	string_integerlist(REST_OF_STRING, TAIL).
  string_integerlist(STRING, LIST):-
	fronttoken(STRING, _, REST),
	string_integerlist(REST, LIST).

  time_diff(TIME1, TIME2, TIME_DIFF):-
	parse_time(TIME1, HOUR1, MINUTE1, SECOND1),
	parse_time(TIME2, HOUR2, MINUTE2, SECOND2),
	TOTAL_SECONDS1 = HOUR1 * 3600.0 + MINUTE1 * 60 + SECOND1,
	TOTAL_SECONDS2 = HOUR2 * 3600.0 + MINUTE2 * 60 + SECOND2,
	TIME_DIFF = TOTAL_SECONDS1 - TOTAL_SECONDS2.


CLAUSES
/************************************************************************
 *			Field Actions					*
 ************************************************************************/

  field_action(current_time):-
  	makewindow(1, 7, 7, "Set new time", 5, 5, 8, 40),
  	write("Hours:\t"),
  	readint(HOURS),
  	nl,
  	write("Minutes:\t"),
  	readint(MINUTES),
  	nl,
  	write("Seconds:\t"),
  	readint(SECONDS),
  	time(HOURS, MINUTES, SECONDS, 0),
  	fail.
  field_action(current_time):- removewindow.  	

  field_action(speed):-
  	makewindow(1, 7, 7, "Arvid's vindue", 5, 5, 8, 40),
  	write("Hej"),
  	sound(100, 300),
  	readchar(_),
  	removewindow.


/************************************************************************
 *			Field Values					*
 ************************************************************************/

  field_value(current_time, CURRENT_TIME) :-
  	!,
	time(HOURS1, MINUTES1, SECONDS1, HUNDREDTHS),

	SECONDS2 = SECONDS1 + ((HUNDREDTHS + 50) div 100),
	str_int(SECONDS3, SECONDS2),
	pad_to_two_chars_if_appropriate(SECONDS3, SECONDS4),

	str_int(MINUTES2, MINUTES1),
	pad_to_two_chars_if_appropriate(MINUTES2, MINUTES3),

	str_int(HOURS2, HOURS1),
	pad_to_two_chars_if_appropriate(HOURS2, HOURS3),

	format(CURRENT_TIME, "%:%:%", HOURS3, MINUTES3, SECONDS4).

  field_value(speed, SPEED2):-!,
	field_value(current_time, CURRENT_TIME),

	value(arrival_time, ARRIVAL_TIME),

	time_diff(ARRIVAL_TIME, CURRENT_TIME, TIME_DIFF),

	value(miles, MILES1),
	!,
	str_real(MILES1, MILES2),
	SPEED1 = MILES2 / (TIME_DIFF / 3600),
	str_real(SPEED2, SPEED1).

  field_value(flip, FLIP2):-
	retract(value(flip, FLIP1)),
	!,
	time(_, _, _, HUNDREDTHS), 
	FREQUENCY = HUNDREDTHS * 100 + 10,
	sound(9, FREQUENCY),
	frontchar(FLIP1, FRONTCHAR, REST_OF_FLIP),
	format(FLIP2, "%s%c", REST_OF_FLIP, FRONTCHAR),
	assert(value(flip, FLIP2)).
  field_value(flip, FLIP) :-
  	!,
  	FLIP = "                            ",
  	assert(value(flip, FLIP)).

  field_value(FIELD_NAME, VALUE) :- value(FIELD_NAME, VALUE), !.


/************************************************************************
 *			No Input Fields					*
 ************************************************************************/

  noinput(speed).	 	
  noinput(current_time).
  noinput(flip).


/************************************************************************
 *			Goal						*
 ************************************************************************/
 
GOAL 
  consult("xtravel.scr"), 
  createwindow(off), 
  scrhnd(off, _).

