/*
===========================================================================
	XTREE.PRO

	Demo of treemenu

	Copyright (C) 1990 Prolog Development Center
===========================================================================
*/


/* 
   To see how a menu tree is drawn, try the following trace 
   with the editor window turned off. 

   shorttrace scr_char field_attr field_str
*/

check_determ
code=3800

DOMAINS
  SELECTOR = STRING

include "UI\\STATUS.PRO"
include "UI\\MENU.PRO"
include "UI\\TREE.PRO"


/****************************************************************
		A network in a relational database
****************************************************************/

DATABASE - network
  r(STRING,STRING)

CLAUSES
  r("start","start_problem").
  r("start","overheating").
  r("start","smell").
  r("start","vibration").
  r("start","start_last").
  r("start_problem","gasoline_help").
  r("start_problem","electrical_system").
  r("start_problem","alternative").
  r("electrical_system","battery_flat").
  r("electrical_system","wet_weather").
  r("battery_flat","another").
  r("battery_flat","recharge").
  r("battery_flat","push_start").
  r("start_last","bad_running").
  r("start_last","brakes").
  r("start_last","wiper").
  r("start_last","lights").
  r("start_last","horn").
  r("bad_running","bad_idling").
  r("bad_running","lack_of_power").



/****************************************************************
	Building a tree from a network: 
	The tree is only built to the depth given by an integer
****************************************************************/

PREDICATES
  collect_tree(STRING,INTEGER,TREE)
  nondeterm collect_tree1(STRING,INTEGER,TREE)

CLAUSES
  collect_tree(NODE,1,tree(NODE,NODE,[])):-!.
  collect_tree(NODE,N,tree(NODE,NODE,TREEL)):-
	findall(T,collect_tree1(NODE,N,T),TREEL).

  collect_tree1(NODE,N,TREE):-
	N1=N-1,
	r(NODE,NODE1),
	collect_tree(NODE1,N1,TREE).


/****************************************************************
	Tree action
****************************************************************/

CLAUSES
  treeaction(_,SELECT):-
	cursor(R,C),
	makewindow(_,_,_,_,SR,SC,_,_),
	RR=R+SR, CC=C+SC,
	menu(RR,CC,32,33,[continue,select],SELECT,0,CH),
	CH=2.


/****************************************************************
	The demo
****************************************************************/

PREDICATES
  test
  show(dir)

CLAUSES
  show(DIRECTION):-
	collect_tree("start",9,TREE), % Collect tree to level 9
	treemenu(DIRECTION,TREE,CHOICE,cursor,[1,1,2,1,2]),
	clearwindow,
	write("Choice= ",CHOICE),nl.

 test:-	
 	write("\nYou are invited to make a selection from a tree menu.\n\n"),
	write("The menu will be shown with the tree drawn in a total of\n"),
	write("four different attitudes.\n\n"),
	write("Each time you press F10 or Esc, or make a selection by pressing\n"),
	write("RETURN while the cursor is in the selection field, the menu will reappear\n"),
	write("with the tree drawn in a new attitude.\n\n"),
	write("You can move around the tree by using the cursor keys. F2 can be used\n"),
	write("to switch between mousemode and cursormode.\n\n"),
	write("  - In mousemode you can move the cursor whereever you want.\n"),
	write("  - In cursormode you the cursor will jump from node to node\n"),
	write("\n"),
	write("There are many parameters to select how the tree is drawn\n"),
	write("these are defined in the top of the file TREE.PRO\n"),
	write("\n"),
	write("\n"),
	write(".. Press the space bar to begin ......"),
	readkey(_),
	makestatus(112," Arrows : move around   F2 : Switch mode   F10 or Return  : Select node"),
	fail.

 test:-	show(right),fail.
 test:-	show(left),fail.
 test:-	show(down),fail.
 test:-	show(up),fail.
 test.

GOAL	
  textmode(RR,CC),RR1=RR-1,
  makewindow(1,7,23," Tree Menu Demo ",0,0,RR1,CC),
  test.
